; collide.asm - foe collision detection & handling


	.dbreg $04

;*******************************************************************************
;*
;* Foe <-> Tarzan collision handlers
;*
;* in: x = index into foe tables (must be preserved)
;*
;*******************************************************************************

MonkeyBite:
	lda	#12
	sta	v_hitmonkey	; set flag so Tarzan will fall

HurtTzan:
	lda	#1
	sta 	v_damageflag

Ignore:
	rtl

BossBite:
	lda	v_priestdead
	bne	Ignore

	lda	v_foex,x	; report hit & position to Tarzan
	sta	v_bosshit

	lda	#1
	sta	v_damageflag
	sta	v_slapos

	rtl

FireballHit:
	lda	v_foeframe,x
	cmp	#3
	bcs	Ignore

	lda	#ExplodeFB
	sta	v_foeai,x

	lda	v_playerx	; align explosion over player
	sta	v_foex,x

	bra	BossBite

	rtl

ApeSlap:
	lda	v_foeframe,x
	cmp	#18
	bcc	HurtTzan
	beq	.biteim

	cmp	#19
	beq	.biteim

	cmp	#20
	bcs	HurtTzan
.biteim
	bra	BossBite

	rtl

EOLHit:
	lda	v_foemode,x	; Get eol mode
	beq	.exiteol	; eol not active yet

	lda	#1
	sta	v_eolhit

	stz	v_foemode,x
.exiteol
	rtl

PitHit:
	lda	#1
	sta	v_foemode,x	; Indicate pit collision top

	inx
	inx

	sta	v_foemode,x	; Indicate pit collision bottom

	dex
	dex

	lda	#$0100	; Preload
	sta	v_pitopen

	ora	v_foemode,x
	sta	v_foemode,x

	cpx	#0
	beq	.exit	; 1st pit - we're done

	cpx	#6	; 2nd pit?
	bne	.not2	; No  -

	lda	#$0200	; Yes -
	sta	v_pitopen

	ora	v_foemode,x
	sta	v_foemode,x

	bra	.exit
.not2
	cpx	#12	; 3rd pit
	bne	.not3

	lda	#$0400	; Yes -
	sta	v_pitopen

	ora	v_foemode,x
	sta	v_foemode,x

	bra	.exit
.not3
	cpx	#18	; 3rd pit
	bne	.exit

	lda	#$0800
	sta	v_pitopen

	ora	v_foemode,x
	sta	v_foemode,x
.exit
	rtl

;*******************************************************************************
;*
;* Foe <-> Weapon collision handlers
;*
;* in: x = index into foe tables (must be preserved)
;*
;* Terminate with rtl
;*
;*******************************************************************************

SmallFoeKnifed:	lda	#1	; mark foe hurt by this knife sequence
	sta	v_foeknife
SmallFoeKilled:	lda	#PoofAI    	; switch to smoke sequence
	sta	v_foeai,x
	lda	#POOFSM	; small poof
	sta	v_foetype,x
	lda	#$0400
	sta	v_foedelay,x
	stz	v_foeframe,x	; small poof starts at frame 0
	stz	v_foeactframe,x
	stz	v_foedma,x
	rtl

MedFoeKnifed:	lda	#1	; mark foe hurt by this knife sequence
	sta	v_foeknife
MedFoeKilled:	lda	#PoofAI    	; switch to smoke sequence
	sta	v_foeai,x
	lda	#POOFMD	; medium poof
	sta	v_foetype,x
	lda	#$0400
	sta	v_foedelay,x
	lda	#4	; medium poof starts at frame 4
	sta	v_foeframe,x
	sta	v_foeactframe,x
	stz	v_foedma,x
	rtl

TallFoeKnifed:	lda	v_foey,x	; adjust poof so it is more centered
	sec		; vertically
	sbc	#48
	sta	v_foey,x
	bra	MedFoeKnifed

; A new piranha needs to be generated when one is killed.

PiranhaKnifed:	lda	#1	; mark foe hurt by this knife sequence
	sta	v_foeknife
	lda	#PirPoofAI    	; switch to smoke sequence
	sta	v_foeai,x
	lda	#POOFSM	; small poof
	sta	v_foetype,x
	lda	#$0400
	sta	v_foedelay,x
	stz	v_foeframe,x	; small poof starts at frame 0
	stz	v_foeactframe,x
	stz	v_foedma,x
	lda	v_foey,x	; center smoke poof...
	clc
	adc	#12
	sta	v_foey,x
	rtl

; A new barracuda needs to be generated when one is killed

BarraKnifed:	lda	#1	; mark foe hurt by this knife sequence
	sta	v_foeknife
	lda	#BarPoofAI    	; switch to smoke sequence
	sta	v_foeai,x
	lda	#POOFMD	; medium poof
	sta	v_foetype,x
	lda	#$0400
	sta	v_foedelay,x
	lda	#4
	sta	v_foeframe,x	; medium poof starts at frame 4
	sta	v_foeactframe,x
	stz	v_foedma,x
	lda	v_foey,x	; center smoke poof...
	clc
	adc	#32
	sta	v_foey,x
	rtl

; When a Thrower is killed, there is a possibility that a stone has been
; reserved but has not yet been spawned (this happens when a Thrower is killed
; while picking up a rock).  In this case, the stone needs to be marked free
; so other Throwers can use it.

ThrowerKilled:	ldy	v_foemisc1,x	; get stone foe table index
	bmi	.nostone	; negative value = "stone not reserved"
	lda	#0	; mark stone free
	sta	v_foemisc1,y
.nostone:	jmp	TallFoeKnifed

EelKnifed:
	lda	v_foey,x
	clc
	adc	#48
	sta	v_foey,x
	jmp	MedFoeKnifed

BossKnifed:
	lda	v_bossinvince	; if boss is invincible, don't process
	bne	.exit	; collision

	dec	v_bosshealth	; don't kill boss until hit points run out

	lda	#1	; mark foe hurt by this knife sequence
	sta	v_foeknife

.exit:	rtl

Lamphit:
	lda	#2
	sta	v_foeframe,x
	sta	v_foeactframe,x
	stz	v_foedma,x
	lda	v_foeoff,x
	and	#$4000	; mask off all but flip bit
	beq	.moverite
	lda	#-8
	bra	.exit
.moverite	lda	#8
.exit	sta	v_foevelx,x
	lda	#8
	sta	v_foevely,x
	rtl

;*******************************************************************************
;*
;* Collide - check foe collisions
;*
;* Data bank reg must point to bank containing foedat.asm!
;*
;* in:  x = index into foe tables (preserved)
;*
;*******************************************************************************

BossCollide:
	ldy	v_foetype,x
	lda	foecoll,y
	sta	z_temp0
	lda	foebank,y
	sta	z_temp1
	lda	v_foeframe,x
	asl
	asl
	tay

	lda	v_foex,x	; convert from qixels to pixels...
	lsr
	lsr
	sta	z_temp6
	lda	v_foey,x
	lsr
	lsr
	sta	z_temp7

	lda	v_foeoff,x	; if foe flipped horizontally, flip
	bit	#$4000	; collision box (negate x offsets)
	beq	.noflip

	lda	[z_temp0],y	; flipped left (is now right)
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex1
	ora	#$FF00
.noex1:	eor	#$FFFF	; negate
	inc
	clc
	adc	z_temp6
	sta	v_foerite

	lda	[z_temp0],y	; flipped right (is now left)
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex2
	ora	#$FF00
.noex2:	eor	#$FFFF	; negate
	inc
	clc
	adc	z_temp6
	sta	v_foeleft

	bra	.vert
.noflip:
	lda	[z_temp0],y	; normal left...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex3
	ora	#$FF00
.noex3:	clc
	adc	z_temp6
	sta	v_foeleft

	lda	[z_temp0],y	; normal right...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex4
	ora	#$FF00
.noex4:	clc
	adc	z_temp6
	sta	v_foerite
.vert:
	lda	[z_temp0],y	; top...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex5
	ora	#$FF00
.noex5:	clc
	adc	z_temp7
	sta	v_foetop

	lda	[z_temp0],y	; bottom...
	and	#$00FF
	bit	#$0080
	beq	.noex6
	ora	#$FF00
.noex6:	clc
	adc	z_temp7
	sta	v_foebot

	; check for collision with Tarzan

	lda	v_foerite
	cmp	v_playerleft
	bcc	.exit

	lda	v_playerrite
	cmp	v_foeleft
	bcc	.exit

	lda	v_foebot
	cmp	v_playertop
	bcc	.exit

	lda	v_playerbot
	cmp	v_foetop
	bcc	.exit

	; foe collided with Tarzan

	lda	v_foekillfrm	; if knife frame, hurt foe
	bne	.exit

	lda	v_damageflag	; if Tzan already hurt this cycle, skip
	bne	.exit

	lda	v_playerAI
	cmp	#TZStartSwing1
	beq	.exit

	cmp	#TZSwing1	
	beq	.exit

	cmp	#TZSwing2	
	beq	.exit

	cmp	#TZOffswing1	
	beq	.exit

	jmp	BossBite
.exit
	rtl


Collide:
	; build collision box for foe (foecoll is a table of pointers
	; to collision tables, indexed by sequence id.  v_foeframe * 4
	; is an index into the proper table)

	ldy	v_foetype,x
	lda	foecoll,y
	sta	z_temp0
	lda	foebank,y
	sta	z_temp1
	lda	v_foeframe,x
	asl
	asl
	tay

	lda	v_foex,x	; convert from qixels to pixels...
	lsr
	lsr
	sta	z_temp6
	lda	v_foey,x
	lsr
	lsr
	sta	z_temp7

	lda	v_foeoff,x	; if foe flipped horizontally, flip
	bit	#$4000	; collision box (negate x offsets)
	beq	.noflip

	lda	[z_temp0],y	; flipped left (is now right)
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex1
	ora	#$FF00
.noex1:	eor	#$FFFF	; negate
	inc
	clc
	adc	z_temp6
	sta	v_foerite

	lda	[z_temp0],y	; flipped right (is now left)
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex2
	ora	#$FF00
.noex2:	eor	#$FFFF	; negate
	inc
	clc
	adc	z_temp6
	sta	v_foeleft

	bra	.vert
.noflip:
	lda	[z_temp0],y	; normal left...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex3
	ora	#$FF00
.noex3:	clc
	adc	z_temp6
	sta	v_foeleft

	lda	[z_temp0],y	; normal right...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex4
	ora	#$FF00
.noex4:	clc
	adc	z_temp6
	sta	v_foerite
.vert:
	lda	[z_temp0],y	; top...
	iny
	and	#$00FF
	bit	#$0080
	beq	.noex5
	ora	#$FF00
.noex5:	clc
	adc	z_temp7
	sta	v_foetop

	lda	[z_temp0],y	; bottom...
	and	#$00FF
	bit	#$0080
	beq	.noex6
	ora	#$FF00
.noex6:	clc
	adc	z_temp7
	sta	v_foebot

	; check for collision with Tarzan

	lda	v_foerite
	cmp	v_playerleft
	bcc	.chekdart

	lda	v_playerrite
	cmp	v_foeleft
	bcc	.chekdart

	lda	v_foebot
	cmp	v_playertop
	bcc	.chekdart

	lda	v_playerbot
	cmp	v_foetop
	bcc	.chekdart

	; foe collided with Tarzan

	lda	v_foekillfrm	; if knife frame, hurt foe
	bne	.knife

	lda	v_damageflag	; if Tzan already hurt this cycle, skip
	bne	.exit

	lda	v_playerAI
	cmp	#TZStartSwing1
	beq	.exit

	cmp	#TZSwing1	
	beq	.exit

	cmp	#TZSwing2	
	beq	.exit

	cmp	#TZOffswing1	
	beq	.exit

	ldy	v_foetype,x
	lda	foetzanai,y
	sta	z_temp0
	jmp	(z_temp0)
.knife:
	lda	v_foeknife	; if another foe killed by current knife,
	bne	.exit	; don't kill this one

	ldy	v_foetype,x
	lda	foeknifeai,y
	sta	z_temp0
	jmp	(z_temp0)
.exit
	rtl

.chekdart
	; check for collision with dart

	lda	v_dartactive
	beq	.exit1

	lda	v_foerite
	cmp	v_dartleft
	bcc	.exit1

	lda	v_dartrite
	cmp	v_foeleft
	bcc	.exit1

	lda	v_foebot
	cmp	v_darttop
	bcc	.exit1

	lda	v_dartbot
	cmp	v_foetop
	bcc	.exit1

	; foe collided with dart

.darted
	lda	v_darthit	; if another foe killed by current dart,
	bne	.exit1	; don't kill this one

	lda	#1
	sta	v_darthit

	ldy	v_foetype,x
	lda	foedartai,y
	sta	z_temp0
	jmp	(z_temp0)
.exit1
	rtl


*****************************************************************************


	.dbreg $00
