*****************************************************************************
*
* CONTROL.ASM
*
*****************************************************************************

Readctrl:   
	LAI

	lda	v_DebugMode
	beq	.doscan

	lda	v_priestdead
	bne	.skipscan
.doscan
	jsl	Scanjoys	; Get Controller bits

.skipscan

*****************************************************************************
*
* The procedure for recording a demo segment is as follows:
*
* Set the RECDEMO switch (in EQU.ASM) to 1. Set the PLAYDEMO to 0.
* Set JOYPAD2ENABLE to 1.
*
* Assemble and download. Goto SDEBUG. Set a breakpoint at ".stopitrec".
*
* Run the program and choose a level (via password). You have about 
* 10 seconds play time, but in any event, the recording will stop
* automatically if the storage space has been exceeded.
*
* To stop recording, kill the character, or press any key on Controller #2.
*
* Save the binary image at v_demoptr (ie $7F6000) plus length, which is
* found in the x register (add 2), to some <file>.BIN.
*
* Compress the bin file using the command: pp p d -m 2 -o %1.bin
* 
*	where %1 is the filename.
*
* Include the file in bank 0 somewhere and enjoy.
*
*
*
	.if	ALLOWDEMO
@Dodemo
	LAI

	lda	v_demorecord	; Recording demo data?
	beq	.nodemorecord	; No  -

	lda	z_joy2	; Yes - Get joy2 keys
	ora	v_killdemorec	; Kill demo record?
	beq	.contrecord	;   No  - Continue recording

	lda	v_JOYPAD2ENABLE	; Is joypad2 enabled?
	beq	.contrecord	; No  - Then skip record mode

	ldx	v_demoptr	; Otherwise, get data ptr
	lda	#$FFFF	; Issue "demo data record stop"
	sta	>$7F0000,x

	inx
	inx

	stx	v_demoptr
	sta	>$7F0000,x	; Issue "demo data record stop"

.stopitrec
	bra	.stoprecord	; Stop recording

.contrecord

	ldx	v_demoptr
	lda	z_joy1
	sta	>$7F0000,x

	inx
	inx

	stx	v_demoptr

	cpx	#$8000
	bcc	.nodemorecord

.stoprecord
	lda	#0
	sta	v_demorecord
.nodemorecord
	lda	v_demoplay
	beq	.noplaydemo
.demopause
	lda	z_joy1
	beq	.contdemo

	and	#JOYPADSTART
	bne	.stopdemoplay
.contdemo
	lda	v_demobank
	sta	z_temp1

	lda	v_demoptr
	sta	z_temp0

	lda	[z_temp0]
	cmp	#$FFFF
	beq	.stopdemoplay

	inc	v_demoptr
	inc	v_demoptr

	sta	z_joy1
	
	bra	.noplaydemo
.stopdemoplay
	lda	#1	; Stop demo & return
	sta	v_levelfail

	sta	v_demostate
	sta	v_demoreturn
.noplaydemo

	.endif

*****************************************************************************
*
* Allow game action to be slowed when controller 2 is used
*

	.if	SLOWMOJOY2

	lda	z_joy2	; Get controller bits
	and	#JOYPADR	; R button pushed?
	beq	.nodelay1	; No  - Check for short delay request

	ldx	#60	; Yes - Delay a long time
.dloop
	jsl	Waitfornmi       

	dex	                 
	bne	.dloop           
.nodelay1
	lda	z_joy2  	; Get controller bits
	and	#JOYPADL	; L button pushed?
	beq	.nodelay2	; No  - Then no delay requested

	ldx	#6	; Yes - Delay a short while
.dloop2
	jsl	Waitfornmi       

	dex	                 
	bne	.dloop2          
.nodelay2
	.endif
           
*****************************************************************************
*
*
* If START & SELECT on Joypad #2 both pressed, do next level
*
	.if	JOYPAD2ENABLE
*
****************************************

	lda	v_DebugMode	; Check master debug switch
	beq	.debstrt	; Skip if switch not set

	lda	z_joy2	; Get keys
	tax

	bit	#JOYPADY	; Isolate Y button
	bne	.xcount2	; Check X if set

	stz	v_ycount2	; Otherwise clear repeat flag
.xcount2
	bit	#JOYPADX	; Isolate X button
	bne	.bcount2	; Check B if set

	stz	v_xcount2	; Otherwise clear repeat flag
.bcount2
	and	#JOYPADSTART	; START button pushed?
	beq	.noreset	; No  - No reset then

	txa		; Yes - Get keys back
	and	#JOYPADSELECT	; SELECT button pushed as well?
	bne	.pullret	; Yes -

	lda	v_startcount2	; No  - Then debounce START button
	bne	.debstrt

	lda	v_pause2	; No  - Just START on pad 2
	eor	#1	; Toggle Debug mode
	sta	v_pause2
	sta	v_dboneshot

	lda	#12
	sta	v_startcount2
	
	rtl
.debstrt
	dec	v_startcount2
	bpl	.xdebstrt

	stz	v_startcount2
.xdebstrt
	rtl

.pullret
	lda	z_level
	cmp	#SHIP
	bne	.justpull

	lda	#SHIPHOLDB
	sta	z_level
	asl
	sta	z_levelword
.justpull
	SA		; Need to pull "jsl Readctrl"
	pla
	LA
	pla

	SA		; Need to pull "jsl ProcessLevel"
	pla
	LA
	pla

	jml	Endit


.noreset
	lda	z_joy2	; Process replay level request
	and	#JOYPADB
	beq	.chk4fsrt

	lda	#tarzroar_eff

	jsl	load_Effect

;	SA		; Need to pull level call
;	pla
;	LA
;	pla
;
;	SA		; Need to pull level call
;	pla
;	LA
;	pla
;
;	lda	z_level
;	cmp	#REEF
;	bne	.skpadj1
;
;	dec	z_level
;.skpadj1
;	lda	z_level
;	dec
;	beq	.chk4fsrt
;	
;	sta	z_level
;	asl
;	sta	z_levelword
;
;	lda	#1
;	sta	v_bakup	; Indicate level restart requested
;
;	rtl
.chk4fsrt
	lda	z_joy2
	and	#JOYPADA
	beq	.chk4invince

	SA		; Need to pull level call
	pla
	LA
	pla

	SA		; Need to pull level call
	pla
	LA
	pla

	lda	#1
	sta	v_restart	; Indicate level restart requested

	rtl

.chk4invince
	lda	z_joy2	; Set debug frame display
	and	#JOYPADX	; Want coin num to be frame num?
	beq	Checktilex	; No  -

	lda	v_xcount2	; Yes - but debounce...
	ora	v_frmdsposhot
	bne	Checktilex

	lda	#1
	sta	v_frmdsposhot	; Next blank changes char
	sta	v_xcount2

****************************************

Checktilex:
	lda	z_joy2
	and	#JOYPADY	; Want to display different info?
	beq	Chekaddlife	; No  - Clear flag

	lda	v_ycount2	; Tiletype/status display toggle cleared?
	bne	Chekaddlife	; No  - 

	inc	v_ycount2	; Set usage flag

	lda	v_debugswitch1	; Flip functions
	eor	#1
	sta	v_debugswitch1

****************************************

Chekaddlife:
;	lda	v_DebugMode
;	beq	Chekcolbox
;
;	lda	z_joy1
;	and	#JOYPADL
;	beq	Chekcolbox
;
;	lda	v_lcount
;	bne	Chekcolbox
;
;	inc	v_lcount
;
;	lda	v_numdart
;	cmp	#9
;	bcs	.chklives
;         
;	inc
;	sta	v_numdart
;.chklives
;	lda	v_lives
;	cmp	#9
;	bcs	Chekcolbox
;
;	inc
;	sta	v_lives


****************************************

Chekcolbox:
	lda	z_joy2
	and	#JOYPADSELECT
	beq	Checkselect

	lda	v_selectcount2
	bne	Checkselect

	inc	v_selectcount2
	
	lda	v_plotcbox
	eor	#1
	sta	v_plotcbox

	.endif

*****************************************************************************

Checkselect:
	lda	z_joy1	; Otherwise, get current pad stats
	and	v_JOYPADSELECT	; Select pressed?
	beq	CheckPause	; No  - Skip weapon toggle then

	lda	v_selectcount	; Select button been released?
	bne	CheckPause	; No  - Skip weapon toggle then

	jsl	Canblowdart	; Yes - But can we even blow (swim)?

	bcc	CheckPause	;     No  - 

	lda	v_numdart	;     Yes -
	cmp	#1
	bcs	.adjcp

	bra	CheckPause
.adjcp
	lda	#1
	sta	v_selectcount	; Otherwise, indicate keypressed

	lda	v_weaponsel	; Toggle weapon select state
	eor	#1
	sta	v_weaponsel


****************************************

CheckPause:
	lda	z_joy1	; Otherwise, get current pad stats
	and    	v_JOYPADSTART	; Start pressed?
	beq	.xstrtchk	; No  -

	lda	v_startcount	; Debounce START button
	bne	.xstrtchk	; Skip state change if not debounced

	lda	#1	; Set debounce control
	sta	v_startcount
	sta	v_pauseos

	lda	v_pause	; Otherwise, get current state of pause
	eor	#1	; Toggle 
	sta	v_pause	; 

	beq	.setfull

;	ldy	#$1A01	; fade music for pause
;	lda	#DCOM_FADEOUT

	ldy	#0
	lda	#DCOM_STOP_SONG
	
	jsl	Pass_Driver_Command

	jsl	Waitfornmi

	lda	#6

	SA
	sta	INIDISP
	LA

	lda	#tarzroar_eff

	jsl	load_Effect

	rtl
.setfull
;	ldy	#$1A01	; fade music back in
;	lda	#DCOM_FADEIN

	ldy	v_cursong
	lda	#DCOM_START_SONG
	
	jsl	Pass_Driver_Command

	jsl	Waitfornmi

	SA
	lda	z_inidispshd
	sta	INIDISP
	LA

.xstrtchk
	rtl


*****************************************************************************
*

Endit:
	lda	v_levelfail	; Check for level failure
	bne	.exit	; Failed level - loss of life

	lda	v_lostlife
	bne	.exit

	lda	z_level	; Get level
	sta	v_givepass
	cmp	#SHIP	; Are we at the ship?
	bne	.ldnxt	; No  -

	inc		; Yes - Bypass the Holds
	inc
	inc

	bra	.donxt
.ldnxt
	ldx	z_levelword	
	lda	_nextlevel,x
	bpl	.donxt	; Do next level

	lda	#-1
	sta	z_level	; Restart

	stz	z_levelword

	rtl
.donxt
	sta	z_level
	asl
	sta	z_levelword
.exit
	rtl


Canblowdart:
	sec		; Assume we can blow dart

	php

	lda	v_playerAI
	cmp	#TZFloat		
	beq	.setnodart

	cmp	#TZFloatknife	
	beq	.setnodart

	cmp	#TZSwimturn
	beq	.setnodart

	cmp	#TZSwim
	beq	.setnodart

	cmp	#TZSwimtoFloat
	beq	.setnodart

	cmp	#TZFloaturn
	beq	.setnodart

	cmp	#TZFloat2swim
	beq	.setnodart

	plp

	rtl
.setnodart
	plp

	clc		; Can't blow a dart

	rtl
