*****************************************************************************
*
* DCOM.ASM
*
*
*
* These are the command bytes you can send to
* the sound effect driver:
*
* These numbers are tightly bound to the order of vectors
* in the array DCom_Vectors
*
*****************************************************************************

DCOM_RUN	.equ	2	; Tells Driver it can start playing

DCOM_START_EFFECT .equ	4	; Port 0 specifies the effect number

DCOM_START_SONG	.equ	6	; Port 0 specifies the song number


*****************************************************************************
*
* There are 2 buffers for holding a song's state. 
* These can be used to switch between 2 songs, or to have 1 song briefly 
* interrupt another. 
* The following commands manipulate the active song and backup song buffers:
*
*****************************************************************************

DCOM_SECOND_SONG .equ	8	; Save the first song's state, then start
			; a second song. Port 0 specifies the new
			; song number
DCOM_INTERLUDE	.equ	10	; Save the first song's state, then start
			; a second song. When the second song ends,
			; the first song will automatically be
			; restored. (All players in interlude must
			; stop first.)
DCOM_SWITCH_SONG .equ	12	; Saves the current song's state, restores
			; the old song.
DCOM_RELOAD	.equ	14	; Causes driver to mute and re-start the
			; APU upload procedure. You can load in
			; new music or a whole new driver. You can
			; load into any area at or above $200
DCOM_FADEOUT	.equ	16	; Port 0 specifies how many ticks for each
		   	; volume turndown (must be non-zero)
			; Port 1 specifies the volume turndown for
			; each of those ticks. (non-zero)
			;
			; Once you do a fadedown, you can then start
			; a new song and fade it in: the volumes
			; will stay at 0 until you do a fadein.
			;
			; Examples:
			;   Port 0 = 1  2.5 seconds to go quiet
			;   Port 1 = 1
			;
			;   Port 0 = 1   quiet in 1 tick
			;   Port 1 = 127
			;
			;   Port 0 = 255 10 minutes to go quiet
			;   Port 1 = 1
			;
DCOM_FADEIN	.equ	18	; Same parameters as fadeout, but the
			; volume rises to maximum.
			;
DCOM_STOPFADE	.equ	20	; This command will stop a fade-in or fade-out
			; at it's current volume.
			;
DCOM_STEREOMODE	.equ	22  	; To put into mono mode, make Port 0 = 50
			; To restore stereo mode, make Port 0 = 0
			;
DCOM_NOTEOVERRIDE .equ	24   	; Override the note of the next START_EFFECT.
			; Note used is in Port 0 & 1. (low,high)
			;
DCOM_STOP_SONG	.equ	26	; Stop currentsong without touching the current
			; volume (so effects will still play)
			; Notes are allowed to decay normally.
			;
			; Port 0: if Port 0 == 0, I will stop the
			;      song, and clear the interlude
			;      flag, so that if this was an
			;      interlude, it will NOT restart
			;      the song it interrupted.
			;
			;      if Port 0 != 0, I will leave the
			;      interlude flag alone. (If this
			;      was an interlude, the interlude
			;      will be stopped, and the old music
			;      will be restarted.)

DCOM_STOP_SOUND	.equ	28	; Call to immediately kill selected sounds.
			; If Port1 == KILL_EFFECT, Port0 specifies
			;        the effect number to kill. ALL
			;        channels playing this effect
			;        number will be killed.
			; If Port1 == KILL_ALL_EFFECTS, All channels
			;        playing effects will be stopped,
			;        but all music will continue.
			;        Port 0 parameter is ignored.
			; If Port1 == KILL_ALL_MUSIC, All channels
			;        playing a music sound will be
			;        stopped (but the music will
			;        continue triggering new notes
			;        unless you issue a STOP_SONG
			;        command: this is to cause an
			;        immediate stop to the music
			;        sounds, and is intended to be
			;        used to stop the "normal decay"
			;        documented in STOP_SONG, above.
			;        Port 0 parameter is ignored
			; If Port1 == KILL_ALL_SOUNDS, every active
			;        channel is stopped (but music
			;        can trigger new notes, as in
			;        KILL_ALL_MUSIC.)
			;        Port 0 parameter is ignored.
; Port 1 parameters to DCOM_STOP_SOUND:

KILL_EFFECT	.equ	0

KILL_ALL_EFFECTS .equ	$1

KILL_ALL_MUSIC	.equ	$FF
KILL_ALL_SOUNDS	.equ	$80


DCOM_MAX_COMMAND .equ	28   ; maximum command number we'll accept

