************************************************************************
*			                  			
* EQU.ASM
*			        	          			
************************************************************************

LEVELS     = 2

*
* Picture Processing Unit (PPU) Registers Constants Definition
*


INIDISP	.equ	$2100	; Initial Screen Settings
OBJSEL	.equ	$2101	; Object Size & Data Area Designation

OAMADDL	.equ	$2102	; Object Attrib Mem Addr Access (low)
OAMADDH	.equ	$2103	; Object Attrib Mem Addr Access (high)
OAMDATA	.equ	$2104	; Data for OAM Write

BGMODE	.equ	$2105	; BG Mode & Character Size Settings
MOSAIC	.equ	$2106	; Size & Screen for Mosaic Display

BG1SC	.equ	$2107	; Ptr to BG1 Data & Screen Size Select
BG2SC	.equ	$2108	; Ptr to BG2 Data & Screen Size Select
BG3SC	.equ	$2109	; Ptr to BG3 Data & Screen Size Select
BG4SC	.equ	$210A	; Ptr to BG4 Data & Screen Size Select

BG12NBA	.equ	$210B	; BG1/BG2 Char Data Area Designation
BG34NBA	.equ	$210C	; BG3/BG4 Char Data Area Designation

BG1HOFS	.equ	$210D	; Horizontal Scroll Value for BG1
BG1VOFS	.equ	$210E	; Vertical Scroll Value for BG1
BG2HOFS	.equ	$210F	; Horizontal Scroll Value for BG2
BG2VOFS	.equ	$2110	; Vertical Scroll Value for BG2
BG3HOFS	.equ	$2111	; Horizontal Scroll Value for BG3
BG3VOFS	.equ	$2112	; Vertical Scroll Value for BG3
BG4HOFS	.equ	$2113	; Horizontal Scroll Value for BG4
BG4VOFS	.equ	$2114	; Vertical Scroll Value for BG4

VMAINC	.equ	$2115	; VRAM Addr Increment Val Designation
VMADDL	.equ	$2116	; Address for VRAM Read (low)
VMADDH	.equ	$2117	; Address for VRAM Read (high)
VMDATAL	.equ	$2118	; Data for VRAM Write (Low)
VMDATAH	.equ	$2119	; Data for VRAM Write (high)

M7SEL	.equ	$211A	; Initial Setting for Screen Mode 7
M7A	.equ	$211B	; Matrix Parameter A for Mode 7 Scrn
M7B	.equ	$211C	; Matrix Parameter B for Mode 7 Scrn
M7C	.equ	$211D	; Matrix Parameter C for Mode 7 Scrn
M7D	.equ	$211E	; Matrix Parameter D for Mode 7 Scrn
M7X	.equ	$211F	; Center Position  X for Mode 7 Scrn
M7Y	.equ	$2120	; Center Position  Y for Mode 7 Scrn

CGADD	.equ	$2121	; Address for Color Gen-(CGRAM) Write
CGDATA	.equ	$2122	; Data for Color Gen-(CGRAM) Write

W12SEL	.equ	$2123	; Window Mask Settings for BG1/BG2
W34SEL	.equ	$2124	; Window Mask Settings for BG3/BG4
WOBJSEL	.equ	$2125	; Window Mask Settings for Color/OBJ

WH0	.equ	$2126	; Window Position Designation W1-Left
WH1	.equ	$2127	; Window Position Designation W1-Rght
WH2	.equ	$2128	; Window Position Designation W2-Left
WH3	.equ	$2129	; Window Position Designation W2-Rght

WBGLOG	.equ	$212A	; Mask Window Logic - BG1,BG2,BG3,BG4
WOBJLOG	.equ	$212B	; Mask Window Logic - Color / Object

TM	.equ	$212C	; Main Screen Designation
TS	.equ	$212D	; Sub  Screen Designation
TMW	.equ	$212E	; Window Mask Designator for Main Scrn
TSW	.equ	$212F	; Window Mask Designator for Sub Scrn

CGSWSEL	.equ	$2130
CGADSUB	.equ	$2131
COLDATA	.equ	$2132
SETINI	.equ	$2133
MPYL	.equ	$2134
MPYM	.equ	$2135
MPYH	.equ	$2136
SLHV	.equ	$2137
RDOAMDATA	.equ	$2138
RDVMDATAL	.equ	$2139
RDVMDATAH	.equ	$213A
RDCGDATA	.equ	$213B
OPHCT	.equ	$213C
OPVCT	.equ	$213D
STAT77	.equ	$213E
STAT78	.equ	$213F
APUIO0	.equ	$2140
APUIO1	.equ	$2141
APUIO2	.equ	$2142
APUIO3	.equ	$2143

NMITIMEN	.equ	$4200
WRIO	.equ	$4201
WRMPYA	.equ	$4202
WRMPYB	.equ	$4203
WRDIVL	.equ	$4204
WRDIVH	.equ	$4205
WRDIVB	.equ	$4206
HTIMEL	.equ	$4207
HTIMEH	.equ	$4208
VTIMEL	.equ	$4209
VTIMEH	.equ	$420A
MDMAEN	.equ	$420B
HDMAEN	.equ	$420C
MEMSEL	.equ	$420D
RDNMI	.equ	$4210
TIMEUP	.equ	$4211
HVBJOY	.equ	$4212
RDIO	.equ	$4213
RDDIVL	.equ	$4214
RDDIVH	.equ	$4215
RDMPYL	.equ	$4216
RDMPYH	.equ	$4217
JOY1L	.equ	$4218
JOY1H	.equ	$4219
JOY2L	.equ	$421A
JOY2H	.equ	$421B
JOY3L	.equ	$421C
JOY3H	.equ	$421D
JOY4L	.equ	$421E
JOY4H	.equ	$421F


	; dma registers

DMA0INC	.equ	$4300
DMA0DEST	.equ	$4301
DMA0A1L	.equ	$4302
DMA0A1H	.equ	$4303
DMA0A1B	.equ	$4304
DMA0NUML	.equ	$4305
DMA0NUMH	.equ	$4306
DMA0ADDL	.equ	$4305
DMA0ADDH	.equ	$4306
DMA0BANK	.equ	$4307
DMA0A2L	.equ	$4308
DMA0A2H	.equ	$4309
DMA0LINE	.equ	$430A

DMA1INC	.equ	$4310
DMA1DEST	.equ	$4311
DMA1A1L	.equ	$4312
DMA1A1H	.equ	$4313
DMA1A1B	.equ	$4314
DMA1NUML	.equ	$4315
DMA1NUMH	.equ	$4316
DMA1ADDL	.equ	$4315
DMA1ADDH	.equ	$4316
DMA1BANK	.equ	$4317
DMA1A2L	.equ	$4318
DMA1A2H	.equ	$4319
DMA1LINE	.equ	$431A

DMA2INC	.equ	$4320
DMA2DEST	.equ	$4321
DMA2A1L	.equ	$4322
DMA2A1H	.equ	$4323
DMA2A1B	.equ	$4324
DMA2NUML	.equ	$4325
DMA2NUMH	.equ	$4326
DMA2ADDL	.equ	$4325
DMA2ADDH	.equ	$4326
DMA2BANK	.equ	$4327
DMA2A2L	.equ	$4328
DMA2A2H	.equ	$4329
DMA2LINE	.equ	$432A

DMA3INC	.equ	$4330
DMA3DEST	.equ	$4331
DMA3A1L	.equ	$4332
DMA3A1H	.equ	$4333
DMA3A1B	.equ	$4334
DMA3NUML	.equ	$4335
DMA3NUMH	.equ	$4336
DMA3ADDL	.equ	$4335
DMA3ADDH	.equ	$4336
DMA3BANK	.equ	$4337
DMA3A2L	.equ	$4338
DMA3A2H	.equ	$4339
DMA3LINE	.equ	$433A

DMA4INC	.equ	$4340
DMA4DEST	.equ	$4341
DMA4A1L	.equ	$4342
DMA4A1H	.equ	$4343
DMA4A1B	.equ	$4344
DMA4NUML	.equ	$4345
DMA4NUMH	.equ	$4346
DMA4ADDL	.equ	$4345
DMA4ADDH	.equ	$4346
DMA4BANK	.equ	$4347
DMA4A2L	.equ	$4348
DMA4A2H	.equ	$4349
DMA4LINE	.equ	$434A

DMA5INC	.equ	$4350
DMA5DEST	.equ	$4351
DMA5A1L	.equ	$4352
DMA5A1H	.equ	$4353
DMA5A1B	.equ	$4354
DMA5NUML	.equ	$4355
DMA5NUMH	.equ	$4356
DMA5ADDL	.equ	$4355
DMA5ADDH	.equ	$4356
DMA5BANK	.equ	$4357
DMA5A2L	.equ	$4358
DMA5A2H	.equ	$4359
DMA5LINE	.equ	$435A
    		
DMA6INC	.equ	$4360
DMA6DEST	.equ	$4361
DMA6A1L	.equ	$4362
DMA6A1H	.equ	$4363
DMA6A1B	.equ	$4364
DMA6NUML  	.equ	$4365
DMA6NUMH  	.equ	$4366
DMA6ADDL  	.equ	$4365
DMA6ADDH  	.equ	$4366
DMA6BANK  	.equ	$4367
DMA6A2L	.equ	$4368
DMA6A2H	.equ	$4369
DMA6LINE	.equ	$436A

DMA7INC	.equ	$4370
DMA7DEST	.equ	$4371
DMA7A1L	.equ	$4372
DMA7A1H	.equ	$4373
DMA7A1B	.equ	$4374
DMA7NUML	.equ	$4375
DMA7NUMH	.equ	$4376
DMA7ADDL	.equ	$4375
DMA7ADDH	.equ	$4376
DMA7BANK	.equ	$4377
DMA7A2L	.equ	$4378
DMA7A2H	.equ	$4379
DMA7LINE	.equ	$437A



; Joypad bits

JOYPADB	.equ	$8000	
JOYPADY	.equ	$4000	
JOYPADSELECT	.equ	$2000	
JOYPADSTART	.equ	$1000	
JOYPADUP	.equ	$0800	
JOYPADDOWN	.equ	$0400	
JOYPADLEFT	.equ	$0200	
JOYPADRIGHT	.equ	$0100	
JOYPADA	.equ	$0080	
JOYPADX	.equ	$0040	
JOYPADL	.equ	$0020	
JOYPADR	.equ	$0010	

JOYPADABXY	.equ	$C0C0
JOYARROWS	.equ	$0F00
JOYLEFTRIGHT	.equ	$0300
JOYNOARROWS	.equ	$F0F0
JOYPADLR	.equ	$0030
		   

	; APU comm port definitions

APUPORT0	.equ	$2140
APUPORT1	.equ	$2141
APUPORT2	.equ	$2142
APUPORT3	.equ	$2143

	; SNES & General Program Equates

ZPAGE	.equ	$0000	; $100 bytes long
STACK	.equ	$1FFF	; $0E0 bytes long

BG1MAP	.equ	$7F0000	; bg1 map is upto 16 screens wide x 4 high=64
BG2MAP	.equ	$7F8000	; bg2 map is upto  8 screens wide x 4 high=32

_rowlocs1	.equ	$7FD000	;used by chek4flr
TILETYPE	.equ	$7FD800	;256*4*word tiletypes per level

BG3MAP	.equ	$7FE000	; BG3 is 2x2

SCRATCH	.equ	$7E2000	; Scratch space for decompress (32k)
SCRATCH2	.equ	$7E5000	; Unpack area
SCRATCH3	.equ	$7E8000	; Unpack area (16k)

ANIMSCRATCH	.equ	$7E2000
TRANSARTSCRATCH	.equ	$7F0000
TRANSARTSCRATCH3 .equ	$7F8000

; PPU RAM Sizes - (in bytes)

CGRAMSIZE	.equ	$0200
OAMRAMSIZE	.equ	$0220


; BG and OBJ register initilization values. 
; These point to VRAM areas to
; hold character definitions. all vram addresses in words
; BG1 & BG2 & BG3 character definitions start at $n000

BG1CHAR	.equ	$0000

BG1BASE	.equ	$4000 
BG1BASE1B	.equ	$4400
BG1BASE1C	.equ	$4800
BG1BASE1D	.equ	$4C00

BG2CHAR	.equ	$3000

BG2BASE	.equ	$4800         
BG2BASE2B	.equ	$4C00         

BG3CHAR  	.equ	$5000         

BG3BASE	.equ	$5800     ; 5100
BG3BASE3B	.equ	$5C00     ; 5500
BG3BASE3C	.equ	$6000     ; 5900
BG3BASE3D	.equ	$6400     ; 5D00

OBJBASE	.equ	$6000
OBJBASEA	.equ	$7000


CHRLISTRAM	.equ	SCRATCH+$4100
CHRDECOMP	.equ	SCRATCH+$4180

; extra background equates

CHARMASK16	.equ	$FFF0	; 16 dot wide characters
CHARMASK8	.equ	$FFF8	; 8 dot wide characters
SCREENEDGE	.equ	$0020	; 32 col wide screen wraparound    
OFFSETMASK	.equ	$0FFF	; to avoid negative offsets
OFFSETMASK2	.equ	$3FFF


; scroll stuff

DXMID	.equ	128
DYMID	.equ	128

WALLDN	.equ	1
MOVUP	.equ	1
SCRDN	.equ	1

CEILING	.equ	2
MOVDN	.equ	2
SCRUP	.equ	2

WALLLT	.equ	4
MOVLT	.equ	4
SCRRT	.equ	4

WALLRT	.equ	8
MOVRT	.equ	8
SCRLT	.equ	8


NONE	.equ	0

SHARED	.equ	$0
DIVIDED	.equ	$30

STUNTIME	.equ	200

MAXATTR	.equ	254	;max pointer allowed into attr table

MUSICON	.equ	1
MAXHEALTH	.equ	5

dsptime	.equ	120

*****************************************************************************
*
* Debug & Cheat Enabling Constants
*
*****************************************************************************

SLOWMOJOY2	.equ	0	; Allows slow down from joy pad 2
DEBUGMODE	.equ	0	; Global debug
JOYPAD2ENABLE	.equ	0	; Enable Joypad #2 to work
LOGOSON	.equ	1	; Enable Full Logo \ Title implementation
MAPGUI	.equ	0	; Allow Map to function as level selector
PLOTCBOX	.equ	0	; Plot Tarzans collision box
MUSICOK	.equ	1	; Enable Music & SFX procedures
ALLOWDEMO	.equ	1	; Enable demo code to assemble
RECDEMO	.equ	0	; Record demo keypresses
PLAYDEMO	.equ	0	; Allow demo to play
DEMODELAY	.equ	400	; Time between demo display from main


