; foe.asm - foe initialization, foe driver, foe ai utility routines


; v_foeplot:
;    < 0 = foe dead / not used
;      0 = foe onscreen
;    > 0 = foe offscreen
;
; v_foedma:
;      non-zero = foe dma required
;
; v_foex, v_foey, v_foevelx, and v_foevely are all specified in quarter
; pixels (qixels).



*******************************************************************************
*
* Loadanims
*
*******************************************************************************

Loadanims:
	lda	#$130	; default right clipping boundary
	sta	v_rightedge

	stz	v_slapos	; Ape slap sfx one-shot
	stz	v_hitmonkey
	stz	v_shakescreen
	stz	v_pitopen

	stz	v_bossactive
	stz	v_bossinvince
	stz	v_bossdead

	stz	v_priestdead

	ldx	#34
	lda	#1
.clearwater:
	sta	v_water,x

	dex
	dex

	bpl	.clearwater

	ldx	z_levelword	; do level specific initialization...

	lda	>foelevinit-2,x
	sta	z_temp0

	ldx	#0

	jsr	(z_temp0,x)

	; init foe tables

	ldx	z_levelword
	lda	>foelocs,x
	sta	z_temp0

	lda	#^foelocs
	sta	z_temp1

	ldy	#0	; index into foe data for level
	tyx		; index into foe tables
.loadfoe:
	lda	[z_temp0],y	; get foe type (byte)
	and	#$FF
	cmp	#$FF	; End of the list?
	bne	.notlast	; No  -

	jmp	.foesloaded	; Yes -
.notlast:
	sta	v_foetype,x	; set foe type
	iny

	phy
	txy
	tax
	lda	>ifoeai,x	; set initial ai routine...
	sta	v_foeai,y
	lda	>foepriority,x	; set foe plot ref tarzan
	sta	v_foepriority,y
	lda	>ifoehit,x	; set initial hits...
	sta	v_foehit,y
	lda	>ifoeframe,x	; set initial frame...
	sta	v_foeactframe,y
	sta	v_foeframe,y
	inc	z_temp7	; set initial delay...
	lda	z_temp7
	and	#$0007
	ora	>ifoedelay,x
	sta	v_foedelay,y
	tyx
	ply

	lda	#1	; assume foe offscreen
	sta	v_foeplot,x

	lda	[z_temp0],y	; get foe x coord
	iny
	iny
	sta	v_foex,x
	sta	v_foepos1,x

	lda	[z_temp0],y	; get foe y coord
	iny
	iny
	sta	v_foey,x
	sta	v_foepos2,x

	lda	[z_temp0],y	; get "offset" (flip, priority, palette,
	iny		; and name)
	iny
	sta	v_foeactoff,x
	sta	v_foeoff,x

	lda	[z_temp0],y	; get aux value 1
	iny
	iny
	sta	v_foeaux1,x

	lda	[z_temp0],y	; get aux value 2
	iny
	iny
	sta	v_foeaux2,x

	lda	[z_temp0],y	; get aux value 3
	iny
	iny
	sta	v_foeaux3,x

	stz	v_foevelx,x	; foe initially doesn't move
	stz	v_foevely,x
	stz	v_foedma,x
	stz	v_foemode,x	; foe set to normal mode

	inx
	inx
	jmp	.loadfoe

; all foes have been loaded into the foe tables

.foesloaded:
	; set rest of foes to "unused" (dead)

	lda	z_level
	and	#$FF
	cmp	#CLEARING
	bne	.setmark

	lda	v_apebossdead
	beq	.setmark

	phx

	ldx	#0
	lda	#$FFFF
	sta	v_foeplot,x

	inx
	inx

	sta	v_foeplot,x

	plx
.setmark
	lda	#$FFFF
.mark:
	cpx	#MAXFOE
	bcs	.done

	sta	v_foeplot,x

	inx
	inx

	bra	.mark
.done:
	; load holes into hole table

	ldx	z_levelword	; point to hole data for level (z_temp1
	lda	>holelocs-2,x	; still points to same bank)
	sta	z_temp0

	ldy	#0	; index into hole data for level
	tyx		; index into foe tables
.loadhole:	lda	[z_temp0],y	; load x coord
	bmi	.holesdone	; negative = end of data
	iny
	iny
	sta	v_holex,x
	lda	[z_temp0],y	; load y coord
	iny
	iny
	sta	v_holey,x
	inx
	inx
	bra	.loadhole

.holesdone:	stx	v_nexthole	; upper bound for hole table entries

.noholes:	jsl	LoadFoePals
	jsl	PlotFrontFoes	; needed to set v_plot variables for
	jsl	PlotBackFoes
	jsr	InitFoeVram	; InitFoeVram
	jsr	InitHoleVram
	rtl


*******************************************************************************
*
* Level-Specific Foe Initialization Routines
*
*******************************************************************************

foelevinit:   ;	.dw	.none	; null
	.dw	.jungle1	; Jungle 1
	.dw	.home	; Tarzan's home
	.dw	.clearing	; Ape's clearing
	.dw	.swamp	; Swamp
	.dw	.beach	; Beach
	.dw	.none	; Beach-Bay CANCELLED
	.dw	.bay	; Reef
	.dw	.ship	; Ship
	.dw	.none	; Ship Hold A
	.dw	.none	; Ship Hold B
	.dw	.jungle3	; Jungle 3
	.dw	.downriver	; Downriver
	.dw	.jungle4	; Jungle 4
	.dw	.upriver	; Upriver
	.dw	.waterfall	; Waterfall
	.dw	.ship	; Caverns
	.dw	.street	; Opar - Street
	.dw	.none	; Opar - Wells
	.dw	.temple	; Opar - Temple
	.dw	.shamonboss	; Opar - Shamon
	.dw	.none	; Bonus Level - Exterior
	.dw	.none	; Bonus Level - Interior

.none:	rts

.home:
.jungle3:	pea	|COMPmonkey
	pea	^COMPmonkey
	pea	|monkey
	pea	^monkey
	jsl	UNPACK
	pla
	pla
	pla
	pla

	; fall through...
.jungle1:
.waterfall:	pea	|COMPbird
	pea	^COMPbird
	pea	|bird
	pea	^bird
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.clearing:	pea	|COMPape1
	pea	^COMPape1
	pea	|ape1
	pea	^ape1
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPape2
	pea	^COMPape2
	pea	|ape2
	pea	^ape2
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.swamp:	pea	|COMPtoad
	pea	^COMPtoad
	pea	|toad
	pea	^toad
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPfrog
	pea	^COMPfrog
	pea	|frog
	pea	^frog
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.beach:  	pea	|COMPlion1
	pea	^COMPlion1
	pea	|lion1
	pea	^lion1
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPlion2
	pea	^COMPlion2
	pea	|lion2
	pea	^lion2
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.bay:	pea	|COMPeel
	pea	^COMPeel
	pea	|eel
	pea	^eel
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPbaracuda
	pea	^COMPbaracuda
	pea	|baracuda
	pea	^baracuda
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.ship:
	pea	|COMPpirate
	pea	^COMPpirate
	pea	|pirate
	pea	^pirate
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.downriver:	pea	|COMPcroc
	pea	^COMPcroc
	pea	|croc
	pea	^croc
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPcroc2
	pea	^COMPcroc2
	pea	|croc2
	pea	^croc2
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.jungle4:	pea	|COMPleopard1
	pea	^COMPleopard1
	pea	|leopard1
	pea	^leopard1
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPleopard2
	pea	^COMPleopard2
	pea	|leopard2
	pea	^leopard2
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.upriver:	pea	|COMPstump
	pea	^COMPstump
	pea	|stump
	pea	^stump
	jsl	UNPACK
	pla
	pla
	pla
	pla
	rts

.street:
.temple:	pea	|COMPthrower
	pea	^COMPthrower
	pea	|thrower
	pea	^thrower
	jsl	UNPACK
	pla
	pla
	pla
	pla

	pea	|COMPoparian
	pea	^COMPoparian
	pea	|oparian
	pea	^oparian
	jsl	UNPACK
	pla
	pla
	pla
	pla

.shamonboss
	lda	#FIRE.PALSET	; use palette 5 for fire...
	sta	z_temp0
	lda	#^FIRE.PALSET
	sta	z_temp1
	ldy	#10

	jsl	LoadPal

	rts


*******************************************************************************
*
* LoadPal - load palette
*
* in:  z_temp0/z_temp1 - long pointer to palette data
*      y - palette number * 2
*
*******************************************************************************

LoadPal:
	ldx	paladdr,y
	ldy	#$1E
.copypal:
	lda	[z_temp0],y
	sta	PALETTE2,x

	dex
	dex

	dey
	dey

	bpl	.copypal

	rtl


*******************************************************************************
*
* LoadFoePals
*
*******************************************************************************

LoadFoePals:

.PAL	.equ	OBJECT+$200	; 8 words of temp space

	; Palette 6 defaults to rodent/hole, even if no rodents
	; are present.  This can be overridden by using palette 6
	; for a different foe.

	lda	#RODENT.PALSET
	sta	z_temp0

	ldy	#6*2

	jsl	LoadPal

	; Load foe palettes, only loading a palette once, even if
	; there are multiple foes using the same palette slot.

	ldx	#14	; mark palettes 3-7 as "not loaded" (0)

.clrpaltab:
	stz	.PAL,x
	dex
	dex
	bpl	.clrpaltab

	sta	.PAL,x	; palettes 0-2 are reserved (this code
	sta	.PAL+2,x	; is needed only if foes can share
	sta	.PAL+4,x	; palettes with Tarzan, coins, etc).

	lda	#^foepal	; point to palette segment
	sta	z_temp1

	ldx	#0
.loop:
	lda	v_foeplot,x	; skip unused table entries
	bmi	.next

	lda	v_foeoff,x	; pull palette bits from offset
	and	#$0E00
	xba
	tay

	lda	.PAL,y	; this palette already loaded?
	bne	.next	; yes, look for another foe

	inc		; no, mark it as loaded and load it...
	sta	.PAL,y

	lda	v_foetype,x
	phx
	tax

	lda	>foepal,x
	sta	z_temp0

	jsl	LoadPal

	plx
.next:
	inx
	inx
	cpx	#MAXFOE
	bne	.loop

	; For now, load poof palette into pal 7.

	lda	#WATPOOF.PALSET ; use bluer palette for underwater
	ldy	z_level	; levels and normal palette when on
	cpy	#REEF	; land...
	beq	.water

	cpy	#DOWNRIVER
	beq	.water

	cpy	#UPRIVER
	beq	.water

	lda	#POOF.PALSET
.water:
	sta	z_temp0

	ldy	#7*2

	jmp	LoadPal	; tail recursive call


*******************************************************************************
*
* InitFoeVram - load data for onscreen foes into vram
*
*******************************************************************************

InitFoeVram:
	ldy	#0
.loop:
	lda	v_foeplot,y	; skip unused and offscreen foes
	bne	.next

	ldx	v_foetype,y
	lda	>foebank,x
	sta	z_dreccount
	lda	>foedrec,x
	sta	z_spridrec

	lda	v_foeactframe,y
	asl
	tax

	lda	v_foeactoff,y
	and	#$01FF
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; vram address for download

	phy
	txy		; foe frame
	jsl	Sendframe
	jsl	Senddata
	ply
.next:
	iny
	iny
	cpy	#MAXFOE
	bcc	.loop
	rts


*******************************************************************************
*
* PlotFoes
*
*******************************************************************************

	.dbreg $04

PlotFrontFoes:
	phb		; point to bank containing foe pointer
	pea	#$8484	; tables to make things speedy
	plb
	plb

	ldx	#0
.loop:
	lda	v_foeplot,x	; $ffff = not used
	bmi	.nextfoe

	ldy	v_foetype,x	; skip foes that should be behind
	lda	v_foepriority,x	; Tarzan
	beq	.nextfoe

	ldy	z_attrcount

	lda	#1	; assume foe offscreen
	sta	v_foeplot,x

	lda	v_foey,x   	; clip y...
	lsr
	lsr
	sta	ATTRY,y
	sec
	sbc	z_levely
	cmp	#$FFC0
	bcs	.yokay
	cmp	#$140
	bcs	.nextfoe
.yokay:
	lda	v_foex,x	; clip x...
	lsr
	lsr
	sta	ATTRX,y
	sec
	sbc	z_levelx
	cmp	#$FFD0
	bcs	.xokay
	cmp	v_rightedge	; $130
	bcs	.nextfoe
.xokay:
	stz	v_foeplot,x	; foe onscreen

	lda	v_foeactframe,x
	sta	ATTRFRAME,y

	lda	v_foeactoff,x
	sta	ATTROFF,y

	phx
	lda	v_foetype,x
	tax
	lda	foebank,x
	sta	ATTRBANK,y
	lda	foedrec,x
	sta	ATTRDREC,y
	plx

	iny
	iny
	sty	z_attrcount
.nextfoe:
	inx
	inx
	cpx	#MAXFOE
	bcc	.loop

	plb
	rtl


PlotBackFoes:
	phb		; point to bank containing foe pointer
	pea	#$8484	; tables to make things speedy
	plb
	plb

	ldx	#0
.loop:
	lda	v_foeplot,x	; $ffff = not used
	bmi	.nextfoe

	ldy	v_foetype,x	; skip foes that should be in front of
	lda	v_foepriority,x	; Tarzan
	bne	.nextfoe

	ldy	z_attrcount

	lda	#1	; assume foe offscreen
	sta	v_foeplot,x

	lda	v_foey,x	; clip y...
	lsr
	lsr
	sta	ATTRY,y
	sec
	sbc	z_levely
	cmp	#$FFC0
	bcs	.yokay
	cmp	#$140
	bcs	.nextfoe
.yokay:
	lda	v_foex,x	; clip x...
	lsr
	lsr
	sta	ATTRX,y
	sec
	sbc	z_levelx
	cmp	#$FFD0
	bcs	.xokay
	cmp	v_rightedge	; $130
	bcs	.nextfoe
.xokay:
	stz	v_foeplot,x	; foe onscreen

	lda	v_foeactframe,x
	sta	ATTRFRAME,y

	lda	v_foeactoff,x
	sta	ATTROFF,y

	phx
	lda	v_foetype,x
	tax
	lda	foebank,x
	sta	ATTRBANK,y
	lda	foedrec,x
	sta	ATTRDREC,y
	plx

	iny
	iny
	sty	z_attrcount
.nextfoe:
	inx
	inx
	cpx	#MAXFOE
	bcc	.loop

	plb
	rtl

	.dbreg $00


*******************************************************************************
*
* InitHoleVram - load data for hole into vram (all holes use same image)
*
*******************************************************************************

InitHoleVram:
	lda	#^hole
	sta	z_dreccount
	lda	#hole
	sta	z_spridrec
	ldy	#0	; frame number
	lda	#$1E00+OBJBASE	; vram address
	jsl	Sendframe
	jsl	Senddata
	rts


*******************************************************************************
*
* PlotHoles
*
*******************************************************************************

PlotHoles:
	ldy	v_nexthole
	beq	.exit
	ldx	z_attrcount

.plot:	dey
	dey
	bmi	.done

	lda	v_holey,y	; clip y...
	sta	ATTRY,x
	sec
	sbc	z_levely
	cmp	#$FFC0
	bcs	.yokay
	cmp	#$140
	bcs	.plot
.yokay:
	lda	v_holex,y	; clip x...
	sta	ATTRX,x
	sec
	sbc	z_levelx
	cmp	#$FFD0
	bcs	.xokay
	cmp	#$130
	bcs	.plot
.xokay:
	stz	ATTRFRAME,x
	lda	#^hole
	sta	ATTRBANK,x
	lda	#hole
	sta	ATTRDREC,x

	lda	#$3DE0	; priority = 3, palette = 6, name = $040
	sta	ATTROFF,x	; preload

	lda	z_level
	cmp	#SHIPHOLDA
	beq	.holds

	lda	z_level
	cmp	#SHIPHOLDB
	beq	.holds

	bra	.notholds

.holds
	lda	#$2DE0	; priority = 2, palette = 6, name = $040
	sta	ATTROFF,x	; preload

.notholds
	inx
	inx
	bra	.plot

.done:	stx	z_attrcount
.exit:	rtl


*******************************************************************************
*
* Dofoes - process all foes in foe tables
*
*******************************************************************************

	.dbreg $04	; makes things much simpler

Dofoes:
	LAI

	lda	#8
	sta	v_maxxfercnt

	phb		; point to bank containing foe pointer
	pea	#$8484	; tables to make things speedy
	plb
	plb

	lda	v_foeknife	; if another foe hit by current knife,
	beq	.noknife	; wait until knife sequence ends before
	lda	v_foekillfrm	; allowing another foe to be killed
	sta	v_foeknife
.noknife:
	ldx	#0

.loop
	lda	v_foeplot,x	; skip dead foes
	bmi	nextfoe

	bne	.offscreen	; don't check collisions for offscreen foes

	jsl	Collide	; Compute a collision

	lda	v_foedma,x	; if waiting for dma, don't run ai
	bne	.newframe

.offscreen
	lda	v_foedelay,x	; if delay is zero, run ai routine
	bit	#$00FF
	beq	.doai
	dec		; otherwise wait until zero
	sta	v_foedelay,x
	bra	.updatefoe

.doai
	pha

	lda	v_pause
	ora	v_pause2
	beq	.doai1

	pla

	jmp	nextfoe

.doai1
	pla
	xba		; reset delay counter
	ora	v_foedelay,x
	sta	v_foedelay,x
	jmp	(v_foeai,x)	; call ai routine

@updatefoe
	lda	v_foex,x	; update foe x position
	clc
	adc	v_foevelx,x
	sta	v_foex,x

	lda	v_foey,x	; update foe y position
	clc
	adc	v_foevely,x
	sta	v_foey,x

@nextfoe
	inx
	inx
	cpx	#MAXFOE
	bcc	.loop
	plb
	rtl
@newframe
	lda	v_foeplot,x	; if foe offscreen, don't dma new frame
	bne	.nodma

	lda	v_xfercount	; don't update frame if dma buffer full
	cmp	v_maxxfercnt
	bcc	newframe2

	sta	v_foedma,x	; try again next time
	bra	updatefoe

@newframe2
	ldy	v_foetype,x
	lda	foebank,y
	sta	z_dreccount
	lda	foedrec,y
	sta	z_spridrec

	lda	v_foeframe,x
	sta	v_foeactframe,x
	asl
	tay
	lda	v_foeoff,x
	sta	v_foeactoff,x
	and	#$01FF
	asl
	asl
	asl
	asl
	adc	#OBJBASE
	phx
	jsl	Sendframe
	plx
	stz	v_foedma,x	; mark dma done
	bra	.updatefoe

.nodma:	stz	v_foedma,x	; foe is offscreen, just update variables
	lda	v_foeoff,x	; and be done with it
	sta	v_foeactoff,x
	lda	v_foeframe,x
	sta	v_foeactframe,x
	jmp	.updatefoe



; ai termination routines
;
; WAIT    - just saves ai position (does not do any dma)
; STEP    - switches to frame specified by v_foeframe+1
; SAME    - forces dma of the current frame
; FRAME N - switches to frame N
; RESET   - same as FRAME 0
; FIRST   - switch to initial frame for foe type (from ifoeframe table)
; FORCE   - forces dma of current frame, even if offscreen
; BLAST   - ignore dma limits & force dma of current frame
; TERM    - gets rid of foe

FoeWait:	
	pla
	inc		; point to next instruction
	sta	v_foeai,x

	jmp	updatefoe

FoeStep:	
	inc	v_foeframe,x
FoeSame:	
	pla
	inc		; point to next instruction
	sta	v_foeai,x

	jmp	newframe

FoeForce:	
	stz	v_foeplot,x	; tell driver foe is onscreen
	bra	FoeSame

FoeBlast:	
	pla
	inc		; point to next instruction
	sta	v_foeai,x

	jmp	newframe2

FoeTerm:	
	lda	#$FFFF
	sta	v_foeplot,x

	jmp	nextfoe

FoeFrame0:	
	lda	#0
FoeFrame:	
	sta	v_foeframe,x	; a = frame number

	pla
	inc		; point to next instruction
	sta	v_foeai,x

	jmp	newframe

FoeFirst:	
	ldy	v_foetype,x	; get initial frame for foe type

	lda	ifoeframe,y
	sta	v_foeframe,x

	pla

	inc		; point to next instruction
	sta	v_foeai,x

	jmp	newframe


	.MACRO WAIT
		jsr FoeWait
	.ENDM

	.MACRO STEP
		jsr FoeStep
	.ENDM

	.MACRO SAME
		jsr FoeSame
	.ENDM

	.MACRO FORCE
		jsr FoeForce
	.ENDM

	.MACRO BLAST
		jsr FoeBlast
	.ENDM

	.MACRO TERM
		jmp FoeTerm
	.ENDM

	.MACRO FRAME frm
		lda #\frm
		jsr FoeFrame
	.ENDM

	.MACRO RESET
		jsr FoeFrame0
	.ENDM

	.MACRO FIRST
		jsr FoeFirst
	.ENDM


*******************************************************************************
*
* AI Utility Routines
*
* All routines expect x to be the foe index; it must be preserved.
*
*******************************************************************************

;
; Set horizontal flip bit
;

SetFlipH:
	lda	v_foeoff,x
	and	#$3FFF
	ora	#$4000
	sta	v_foeoff,x
	rts


;
; Clear horizontal flip bit
;

ClearFlipH:
	lda	v_foeoff,x
	and	#$3FFF
	sta	v_foeoff,x
	rts


;
; Toggle horizontal flip bit
;

FlipH:
	lda	v_foeoff,x
	eor	#$4000
	sta	v_foeoff,x
	rts


;
; Return absolute Y distance from foe to Tarzan
;

DistY:
	lda	v_foey,x
	sec
	sbc	v_playery
	bpl	.pos
	eor	#$FFFF
	inc
.pos:	rts


;
; Return absolute X distance from foe to Tarzan
;

DistX:
	lda	v_foex,x
	sec
	sbc	v_playerx
	bpl	.pos
	eor	#$FFFF
	inc
.pos:	rts


;
; Return absolute distance from foe to point contained in foepos1,foepos2
;

Dist:
	lda	v_foepos1,x
	sec
	sbc	v_foex,x
	bpl	.xpos
	eor	#$FFFF
	inc
.xpos:	sta	z_temp0

	lda	v_foepos2,x
	sec
	sbc	v_foey,x
	bpl	.ypos
	eor	#$FFFF
	inc
.ypos:	sta	z_temp1

	cmp	z_temp0
	bcc	.y2smaller
	lda	z_temp0
.y2smaller:	lsr
	eor	#$FFFF
	inc
	adc	z_temp0
	adc	z_temp1
	rts


;
; FoeFloor
;
; Checks if foe is on a walkable tile.  If so, adjusts y coordinate
; to sync with that tile.
;
; out: cf=1 = tile is walkable
;

FoeFloor:
	phx
	lda	v_foex,x
	lsr
	lsr
	tay
	lda	v_foey,x
	lsr
	lsr
	sta	z_temp7
	jsl	Getground
	bpl	.groundbelow
	cmp	#MAXRISE
	bcs	.adjust
.nofloor:	plx
	clc		; cf=0 = no floor
	rts

.groundbelow:	cmp	#MAXDROP
	bcs	.nofloor

.adjust: 	clc
	adc	z_temp7
	asl
	asl
	plx
	sta	v_foey,x
	sec		; cf=1 = floor
	rts


;
; FlipFix
;
; Negates accumulator if foe is flipped.  (The accumulator should initially
; be negative if the foe defaults to facing left, positive if facing right.)
;

FlipFix:
	pha
	lda	v_foeoff,x
	bit	#$4000
	beq	.same
	pla
	eor	#$FFFF
	inc
	rts

.same: 	pla
	rts


;
; AdjustX
;
; Adjust x coordinate in direction of travel.
;
; in:  a = delta x
;

AdjustX:
	jsr	FlipFix
	clc
	adc	v_foex,x
	sta	v_foex,x
	rts


;
; AdjustY
;
; Adjust y coordinate.
;
; in:  a = delta y
;

AdjustY:
	clc
	adc	v_foey,x
	sta	v_foey,x
	rts


;
; NearEdge
;
; Sets carry flag if foe x coord is within N pixels of the boundary
; it is moving toward.
;
; in:  a = dist from edge
;      v_foeaux1 = left limit
;      v_foeaux2 = right limit
;
; out: cf=1 = foe is within specified distance of boundary
;

NearEdge:
	sta	z_temp0
	lda	v_foevelx,x
	bmi	.goingleft

.goingright:	lda	v_foex,x
	clc
	adc	z_temp0
	cmp	v_foeaux2,x
	rts

.goingleft:	lda	v_foeaux1,x
	clc
	adc	z_temp0
	cmp	v_foex,x
	rts


;
; NearOpEdge
;
; Sets carry flag if foe x coord is within N pixels of the boundary it
; is moving away from.
;
; in:  a = dist from edge
;      v_foeaux1 = left limit
;      v_foeaux2 = right limit
;
; out: cf=1 = foe is within specified distance of boundary
;

@NearOpEdge:
	sta	z_temp1
	lda	v_foevelx,x
	bmi	.goingright
	bpl	.goingleft


;
; TowardTzan
;
; Determines if foe is headed in Tarzan's direction.  This routine only
; works if the foe's x velocity is greater than zero (i.e. foe is moving).
;
; (This routine wouldn't be needed--FacingTzan would suffice--if the art for
; all foes initially faced left.)
;
; out:  cf=1 = foe is moving toward Tarzan
;

TowardTzan:
	lda	v_foevelx,x
	bmi	.goingleft
.goingright:
	lda	v_playerx
	cmp	v_foex,x
	rts
.goingleft:
	lda	v_foex,x
	cmp	v_playerx
	rts


;
; FacingTzan
;
; Determines if front of foe is pointed toward Tarzan (assumes all foes
; initially face left).
;
; out:  cf=1 = foe is facing Tarzan
;

FacingTzan:
	lda	v_foeoff,x
	bit	#$4000
	beq	.goingleft
.goingright:
	lda	v_playerx
	cmp	v_foex,x
	rts
.goingleft:
	lda	v_foex,x
	cmp	v_playerx
	rts


;
; TzanInZone
;
; Determines if Tarzan is standing in a foe's horizontal zone
;
; in:   v_foeaux1 = left limit
;       v_foeaux2 = right limit
;
; out:  cf=1 = Tarzan in zone
;

TzanInZone:
	lda	v_playerx
	cmp	v_foeaux1,x
	bcc	.exit
	lda	v_foeaux2,x
	cmp	v_playerx
.exit:	rts


	.dbreg $00

