*****************************************************************************
*
* All foe ai routines expect x to contain the foe table index; it must
* be preserved.
*
* Initially:   	v_foeaux1-3 = aux values from foe definition
*	v_foepos1 = initial foe x coord
*	v_foepos2 = initial foe y coord
*
*****************************************************************************

	.dbreg $04

*****************************************************************************
*
*  Dummy AI handler
*
*****************************************************************************

NoAI:
	SAME
	bra	NoAI


*******************************************************************************
*
* Bird (three types)
*
* v_foeaux1 = appearance frequency (measured in multiples of the bird frame
*             rate)
* v_foeaux2 = maximum y value in qixels (don't want birds on ground)
* v_foeaux3 = bird type table (4 words)
*
* v_foemisc1 = appearance timer
*
*******************************************************************************

BirdAI:
	jsl	Random
	and	#31
	sta	v_foemisc1,x
.initwait:	WAIT
	dec	v_foemisc1,x
	bpl	.initwait
	bra	.newbird

.tryagain:	WAIT

	; choose y coordinate

.newbird:	jsl	Random
	clc
	adc	z_levely
	asl
	asl
	cmp	v_foeaux2,x
	bcs	.tryagain
	sta	v_foey,x

	; choose y velocity

	jsl	Random
	and	#$0E
	tay
	lda	d_birdvely,y
	sta	v_foevely,x

	; choose bird type

	jsl	Random
	and	#6
	tay
	lda	v_foeaux3,x
	sta	z_temp0
	lda	(z_temp0),y
	sta	v_foetype,x

	jsl	Random
	lsr
	bcs	.fromright

	; start bird left of screen

	lda	z_levelx
	sec
	sbc	#32
	asl
	asl
	sta	v_foex,x
	jsr	SetFlipH
	bra	.setvel

	; start bird right of screen

.fromright:	lda	z_levelx
	clc
	adc	#256+32
	asl
	asl
	sta	v_foex,x
	jsr	ClearFlipH

	; choose x velocity

.setvel:	jsl	Random
	and	#$0006
	tay
	lda	d_birdvelx,y
	jsr	FlipFix
	sta	v_foevelx,x

.fly:	lda	v_foey,x
	cmp	v_foeaux2,x
	bcc	.flyokay
         	stz	v_foevely,x
.flyokay:
	FIRST		; frame 0 (fly)
	STEP		; frame 1
	STEP		; frame 2
	STEP		; frame 3
	STEP		; frame 4
	jsr	DistX
	cmp	#1280
	bcs	.remove
	jsr	DistY
	cmp	#1280
	bcs	.remove
	lda	v_foeplot,x
	bne	.fly
	lda	v_foevely,x
	bne	.fly
	jsl	Random
	cmp	#220
	bcc	.flyokay

	lda	v_foevelx,x
	sta	v_foemisc1,x
	STEP		; frame 5 (turn)
	stz	v_foevelx,x
	STEP		; frame 6
	STEP		; frame 7
	lda	v_foemisc1,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
	jsr	FlipH
	jmp	.fly

.remove:	stz	v_foevelx,x	; remove bird from screen
	lda	#-1
	sta	v_foex,x

	jsl	Random	; reset appearance timer
	and	#$0F
	clc
	adc	v_foeaux1,x
	sta	v_foemisc1,x

.wait:	WAIT
	dec	v_foemisc1,x
	bne	.wait
	jmp	.newbird


*******************************************************************************
*
* Butterfly (three types)
*
* The butterfly runs at double speed so jitter can be added
* between frames.
*
* v_foeaux1 = radius of bounding circle in qixels
*
*******************************************************************************

BflyAI:
	ldy   #-4
	jsl	Random
	bmi	.setleft
	jsr	SetFlipH
	ldy	#4
.setleft:	tya
	sta	v_foevelx,x

.flutter:
	jsr	.jitter	; frame 0
	FIRST
	jsr	.jitter
	WAIT

	jsr	.jitter	; frame 1
	STEP
	jsr	.jitter
	WAIT

	jsr	.jitter
	jsr	Dist	; if > than radius, point bfly
	cmp	v_foeaux1,x	; toward center
	bcc	.flutter
	ldy	#4
	lda	v_foex,x
	cmp	v_foepos1,x
	bcc	.goright
	ldy	#-4
.goright:	tya
	sta	v_foevelx,x
	ldy	#4
	lda	v_foey,x
	cmp	v_foepos2,x
	bcc	.godown
	ldy	#-4
.godown:	tya
	sta	v_foevely,x
	jsr	FlipH

	STEP		; frame 2
	WAIT
	STEP		; frame 3
	WAIT
	bra	.flutter

.jitter:
	jsl	Random
	and	#$0003
	asl
	tay
	lda	bflydir,y
	sta	v_foevely,x
	rts


*******************************************************************************
*
* Crab
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
*******************************************************************************

CrabAI:
	ldy	#snap_eff
	jsl	OnscreenSound
	stz	v_foevelx,x

.sleep:	RESET		; frame 0 (stand)
	SAME		; always sleep for at least 2 frames
	jsl	Random
	cmp	#192
	bcc	.sleep
	lda	#-2
	jsr	FlipFix
	sta	v_foevelx,x

.walk:	lda	#84
	jsr	NearEdge
	bcc	.noflip
	jsr	FlipH
	lda	v_foevelx,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
.noflip:
	ldy	#snap_eff
	jsl	OnscreenSound
	FRAME 1		; frame 1 (walk)
	STEP		; frame 2
	lda	v_bossactive	; if boss is active, terminate crab
	beq	.noboss
	TERM
.noboss:	STEP		; frame 3
	ldy	#snap_eff
	jsl	OnscreenSound
	STEP		; frame 4
	STEP		; frame 5
	STEP		; frame 6

	jsl	Random
	cmp	#230
	bcs	CrabAI
	cmp	#210
	bcc	.walk	; keep walking
	lda	#84
	jsr	NearOpEdge
	bcc	.walk
	jsr	FlipH
	lda	v_foevelx,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
	bra	.walk


MadCrabAI:
;	lda	v_playerx	; wait for Tarzan to approach arch
;	cmp	#$63F<<2
;	bcs	.start
;	SAME		; frame 0 (stand)
;	bra	MadCrabAI
;.start:
;	lda	#-5
;	sta	v_foevelx,x
;.left:
;	FRAME 1	; frame 1 (walk)
;	STEP		; frame 2
;	STEP		; frame 3
;	STEP		; frame 4
;	STEP		; frame 5
;	STEP		; frame 6
;	lda	v_foex,x
;	cmp	#$604<<2
;	bcs	.left
;
;	jsr	ClearFlipH
;	lda	#5
;	sta	v_foevelx,x
;.right:
;	FRAME 1	; frame 1 (walk)
;	STEP		; frame 2
;	STEP		; frame 3
;	STEP		; frame 4
;	STEP		; frame 5
;	STEP		; frame 6
;	lda	v_foex,x
;	cmp	#$77E<<2
;	bcc	.right
;
;	stz	v_foevelx,x	; attack finished, go to sleep
;.sleep:	SAME	      	; frame 0 (stand)
;	bra	.sleep


*******************************************************************************
*
* Boss Ape
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
* v_foemisc1 = stength for leap (can leap when reaches N - dropped to zero 
*              after a leap
*
* v_foemisc2 = counts number of consecutive knife hits (after being hit
*              twice, ape comes out of stand into a walk)
*
*******************************************************************************

ApeAI:
	lda	#12
	sta	v_bosshealth
	sta	v_bossoldhealth
	sta	v_bossactive

	lda	#64	; initial strength for leap
	sta	v_foemisc1,x

	stz	v_foemisc2,x

.startwalk
	lda	#-3

	jsr	FlipFix

	sta	v_foevelx,x

	stz	v_foemisc2,x

.walk
	FRAME 3		; walk 0

	lda	v_foemisc1,x	; gain strength for leap
	clc
	adc	#8
	sta	v_foemisc1,x

	STEP		; walk 1
	STEP		; walk 2
	STEP		; walk 3

	jsr	TowardTzan	; don't attack if Tarzan is behind ape

	bcc	.keepwalking

	lda	v_playery	; don't attack if Tarzan is in tree
	cmp	#$360<<2
	bcc	.keepwalking

	jsr	DistX	; try charge (head-butt) if Tarzan is

	cmp	#224	; close
	bcs	.checkleap

	jmp	.charge

.checkleap
	cmp	#400	; if Tarzan is a bit farther away, try
	bcs	.keepwalking	; a leap

	lda	v_foemisc1,x	; can only use leap when enough strength
	cmp	#128	; has been saved up
	bcc	.keepwalking

	lda	#400

	jsr	NearEdge

	bcs	.keepwalking

	stz	v_foemisc1,x	; leap uses up strength

	jmp	.leap

.keepwalking
	STEP		; walk 4
	STEP		; walk 5
	STEP		; walk 6
	STEP		; walk 7

	lda	#192

	jsr	NearEdge

	bcs	.turn

	lda	v_foex,x
	bmi	.turn

	jsr	TowardTzan

	bcs	.walk

	lda	v_playery
	cmp	#$360<<2
	bcc	.walk

	lda	#192

	jsr	NearOpEdge

	bcc	.turn

	jmp	.walk

.turn
	stz	v_foevelx,x
	STEP		; turn 0
	STEP		; turn 1
	STEP		; turn 2
	lda	#192
	jsr	AdjustX
	jsr	FlipH
	jmp	.startwalk

.charge
	stz	v_foevelx,x
	lda	#APE2
	sta	v_foetype,x
	stz	v_foemisc2,x
	ldy	#gorilla1_eff	; roar sound
	jsl	SoundEffect
	RESET		; charge 0
	STEP		; charge 1
	jsl	BossCollide
	lda	#-32
	jsr	AdjustX
	STEP		; charge 2
	jsl	BossCollide
	lda	#-32
	jsr	AdjustX
	STEP		; charge 3
	jsl	BossCollide
.chargedone
	lda	#-16
	jsr	AdjustX
	STEP		; charge 4 (recovery & roar...)
	STEP		; charge 5
	STEP		; charge 6
	STEP		; charge 7
	STEP		; charge 8
	STEP		; charge 9
	STEP		; charge 10
	STEP		; charge 11
	STEP		; charge 12
	STEP		; charge 13
	lda	#APE1
	sta	v_foetype,x

.stand
	RESET
	inc	v_foemisc1,x	; gain strength for leap (faster than
	inc	v_foemisc1,x	; when walking)
	lda	v_foemisc2,x
	cmp	#3
	bcs	.standxwalk
	jsr	FacingTzan
	bcc	.standxwalk
	jsr	DistX
	cmp	#768
	bcs	.standxwalk
	cmp	#192
	bcs	.noslap
	lda	v_playery
	cmp	#$360<<2
	bcs	.slap
.noslap
	jsl	Random
	cmp	#240
	bcc	.stand

.standxwalk
	STEP
	STEP
	lda	#-40
	jsr	AdjustX
	jmp	.startwalk

.slap
	lda	#$0404
	sta	v_foedelay,x
	lda	#APE2
	sta	v_foetype,x
	stz	v_foemisc2,x
	FRAME 14	; slap 0
	STEP		; slap 1
	STEP		; slap 2
	STEP		; slap 3

	jsl	BossCollide

	lda	v_bosshit
	beq	.skipsfxslap

	ldy	#punch1_eff

	jsl	SoundEffect
.skipsfxslap
	STEP		; slap 4
	STEP		; slap 5
	STEP		; slap 6
	STEP		; slap 7
	STEP		; slap 8
	STEP		; slap 9
	STEP		; slap 10
	STEP		; slap 11

	lda	#APE1
	sta	v_foetype,x
	lda	#$0505
	sta	v_foedelay,x
	jmp	.stand

.leap
	stz	v_foevelx,x
	lda	#APE2
	sta	v_foetype,x
	ldy	#gorilla1_eff	; roar sound
	jsl	SoundEffect
	RESET		; charge 0
	STEP		; charge 1
	lda	#$0000
	sta	v_foedelay,x
	lda	#-16
	sta	v_foevely,x
	lda	#-24
	jsr	FlipFix
	sta	v_foevelx,x
.inair
	FRAME 2		; charge 2 (en route)
	lda	v_foevely,x
	inc
	inc
	sta	v_foevely,x
	cmp	#18
	bne	.inair
	stz	v_foevelx,x
	stz	v_foevely,x
	lda	#$0505
	sta	v_foedelay,x
	jmp	.chargedone


@ApeHit
	stz	v_foevelx,x
	stz	v_foevely,x
	inc	v_foemisc2,x
	lda	#$3A0<<2
	sta	v_foey,x
	lda	#$0505
	sta	v_foedelay,x
	lda	#APE1
	sta	v_foetype,x
	FRAME 14
	STEP
	STEP
	STEP
	lda	v_bosshealth
	beq	.apedead
	SAME
	FRAME	16
	FRAME	15
	FRAME 14
	jmp	.stand

.apedead
	lda	#-64
	jsr	AdjustY
	jmp	BossPoofAI


ApeMonitor
	WAIT
	lda	v_bossinvince
	beq	.notinvince
	dec	v_bossinvince
	bra	ApeMonitor

.notinvince
	lda	v_bosshealth
	cmp	v_bossoldhealth
	beq	ApeMonitor
	sta	v_bossoldhealth
	lda	#ApeHit
	sta	v_foeai+2
	lda	#42
	sta	v_bossinvince
	bra	ApeMonitor


*******************************************************************************
*
* Smoke poof handler
*
*******************************************************************************

PoofAI:
	lda	v_foeoff,x	; switch to smoke palette
	and	#$F1FF
	ora	#$0E00
	sta	v_foeoff,x
	stz	v_foevelx,x	; no velocity for smoke...
	stz	v_foevely,x


	BLAST		; 0

	phx

	ldy	#paff_eff

	jsl	SoundEffect

	plx

	STEP		; 1
	STEP		; 2
	STEP		; 3

	TERM

PirPoofAI:
	lda	v_foeoff,x	; switch to smoke palette
	sta	v_foemisc1,x	; (save original v_foeoff for later)
	and	#$F1FF
	ora	#$0E00
	sta	v_foeoff,x
	stz	v_foevelx,x	; no velocity for smoke...
	stz	v_foevely,x

	BLAST		; 0

	phx

	ldy	#paff_eff

	jsl	SoundEffect

	plx

	STEP		; 1
	STEP		; 2
	STEP		; 3

	lda	#PIRANHA
	sta	v_foetype,x
	lda	v_foemisc1,x	; switch back to piranha palette
	sta	v_foeoff,x
	jmp	PiranhaAI	; regenerate a new piranha


BarPoofAI:
	lda	v_foeoff,x	; switch to smoke palette
	sta	v_foemisc1,x	; (save original v_foeoff for later)
	and	#$F1FF
	ora	#$0E00
	sta	v_foeoff,x
	stz	v_foevelx,x	; no velocity for smoke...
	stz	v_foevely,x

	BLAST		; 0

	phx

	ldy	#paff_eff

	jsl	SoundEffect

	plx

	STEP		; 1
	STEP		; 2
	STEP		; 3

	lda	#BARRACUDA
	sta	v_foetype,x
	lda	v_foemisc1,x	; switch back to piranha palette
	sta	v_foeoff,x
	ldy	v_foepos1,x	; mark fish out of zone
	sta	v_water,y	; non-zero value
	jmp	BarracudaAI	; regenerate a new piranha


BossPoofAI:
	lda	#$0404
	sta	v_foedelay,x
	lda	#POOFLG
	sta	v_foetype,x
	lda	v_foeoff,x	; switch to smoke palette
	and	#$F1FF
	ora	#$0E00
	sta	v_foeoff,x

	FRAME 8		; 0

	phx

	ldy	#paff_eff

	jsl	SoundEffect

	plx

	STEP		; 1
	STEP		; 2
	STEP		; 3
	lda	#-1
	sta	v_foey,x
	sta	v_bossdead
	TERM


*******************************************************************************
*
* Tropical fish
*
* v_foeaux1 = index into fish tables (fishleft, fishrite, fishtop, fishbot)
*
*******************************************************************************

FishAI:
	jsl	Random	; get random x velocity
	and	#$0006
	tay
	lda	ifishdir,y
	jsr	FlipFix
	sta	v_foevelx,x

.move:
	jsr	.fishvert
	FIRST		; frame 0
	STEP		; frame 1
	STEP		; frame 2
	STEP		; frame 3
	STEP		; frame 4
	STEP		; frame 5
	stz	v_foevely,x

	; if fish is outside its box, make sure it is headed
	; back inside.

	lda	#1	; z_temp0 = 0 if outside box
	sta	z_temp0
	ldy	v_foeaux1,x
	lda	v_foey,x
	cmp	fishtop,y	; fish box top
	bcs	.y2
	lda	#1
	sta	v_foevely,x
	stz	z_temp0
	bra	.checkx
.y2:	cmp	fishbot,y	; fish box bot
	bcc	.checkx
	lda	#-1
	sta	v_foevely,x
	stz	z_temp0
.checkx:
	lda	v_foex,x
	cmp	fishleft,y	; fish box left
	bcs	.x2
	lda	v_foevelx,x
	bmi	.turn
	bra	.move
.x2:	cmp	fishrite,y	; fish box right
	bcc	.cont
	lda	v_foevelx,x
	bmi	.move
.turn:    	lda	#$0404
	sta	v_foedelay,x

	STEP		; frame 6
	STEP		; frame 7
	STEP		; frame 8

	lda	#$0606
	sta	v_foedelay,x
	lda	v_foevelx,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
	STEP		; frame 9

	jsr	FlipH
.domove:	jmp	.move

.cont:	lda	z_temp0	; if fish was outside box, swim in
	beq	.domove	; same direction
.rand:
	jsl	Random	; otherwise change x direction every
	cmp	#240	; so often
	bcc	.domove
	and	#6
	tay
	lda	fishvelx,y
	sta	v_foevely,x
	bra	.turn

.fishvert:
	jsl	Random	; throw in a little vertical motion
	and	#30	; every so often...
	tay
	lda	fishvely,y
	clc
	adc	v_foevely,x
	cmp	#2
	bpl	.fix1
	cmp	#-2
	bmi	.fix2
	sta	v_foevely,x
            	rts
.fix1:	lda	#2
	sta	v_foevely,x
	rts
.fix2:	lda	#-2
	sta	v_foevely,x
	rts


*******************************************************************************
*
* Slave fish (for schools)
*
* Each slave must be associated with another fish (possibly another slave fish)
*
* v_foeaux1 = foe index of fish to slave to
*
*******************************************************************************

SlishAI:
	ldy	v_foeaux1,x	; convert absolute coords in foex,foey
	lda	v_foex,x	; to relative offsets in foepos1,foepos2
	sec
	sbc	v_foex,y
	sta	v_foepos1,x

	lda	v_foey,x
	sec
	sbc	v_foey,y
	sta	v_foepos2,x
.track:
	ldy	v_foeaux1,x	; get index of master fish
	lda	v_foevelx,y	; take on velocity and delay time
	sta	v_foevelx,x	; of master...
	lda	v_foevely,y
	sta	v_foevely,x
	lda	v_foedelay,y
	sta	v_foedelay,x

	lda	v_foex,y	; add slave offset to master position...
	clc
	adc	v_foepos1,x
	sta	v_foex,x

	lda	v_foey,y
	clc
	adc	v_foepos2,x
	sta	v_foey,x

	lda	v_foeoff,y	; use flip and priority of master
	and	#$F000	; and vram address of slave...
	sta	z_temp0
	lda	v_foeoff,x
	and	#$0FFF
	ora	z_temp0
	sta	v_foeoff,x

	lda	v_foeframe,y	; convert master's frame to the proper
	pha		; frame for the slave, adjusting for
	lda	v_foetype,y	; different fish types (subtract master's
	tay		; initial frame from current frame and
	pla		; add slave's initial frame)
	sec
	sbc	ifoeframe,y
	ldy	v_foetype,x
	clc
	adc	ifoeframe,y
	sta	v_foeframe,x

	SAME
	bra	.track


*******************************************************************************
*
* Moray Eel
*
* v_foeaux1 = index into eel tables (eelleft, eelrite, eeltop, eelbot)
*
*******************************************************************************

EelAI:
	lda	#-4
	sta	v_foevelx,x
.move:
	RESET		; frame 0
	STEP		; frame 1
	lda	v_foevelx,x
	bmi	.samespeed
	jsl	Random
	cmp	#220
	bcc	.samespeed
	ldy	v_foevelx,x
	lda	eelvel-4,y
	sta	v_foevelx,x
	lda	eeldelay-4,y
	sta	v_foedelay,x
.samespeed:
	STEP		; frame 2 (swim)
	STEP		; frame 3
	STEP		; frame 4
	STEP		; frame 5

	; If eel is outside its box, make sure it is headed
	; back inside.

	ldy	v_foeaux1,x

	lda	v_foey,x
	cmp	eeltop,y	; eel box top
	bcs	.y2
	lda	#1
	sta	v_foevely,x
	bra	.checkx
.y2:	cmp	eelbot,y	; eel box bottom
	bcc	.checkx
	lda	#-1
	sta	v_foevely,x
.checkx:
	lda	v_foex,x
	cmp	eelleft,y	; eel box left
	bcs	.x2
	lda	v_foevelx,x
	bmi	.turn
	bra	.cont
.x2:	cmp	eelrite,y	; eel box right
	bcc	.cont
	lda	v_foevelx,x
	bpl	.turn

	; Eel is within box, periodically choose a new y velocity
	; and turn around.
.cont:
	jsl	Random
	cmp	#210
	bcc	.gomove
	and	#14
	tay
	lda	eeldir,y
	sta	v_foevely,x
.turn:
	lda	v_foevelx,x
	sta	v_foemisc1,x
	stz	v_foevelx,x
	FRAME 6		; frame 6 (turn)
	STEP		; frame 7
	STEP		; frame 8
	STEP		; frame 9
	STEP		; frame 10
	STEP		; frame 11
	lda	v_foemisc1,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
	jsr	FlipH
.gomove:	jmp	.move


*******************************************************************************
*
* Active Monkey
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
* A monkey must start with enough room to get up in the direction it is
* facing.
*
*******************************************************************************

MonkeyAI:
	lda	#$0C0C	; tail wagging is slow
	sta	v_foedelay,x
	RESET		; frame 0 (sit)
	STEP		; frame 1
	STEP		; frame 2
	STEP		; frame 3
	jsr	.intruder  	; if Tarzan is on monkey's branch,
	bcs	.stand	; get up
	jsl	Random	; even if Tarzan is not an immediate
	cmp	#220	; threat, stand up occasionally
	bcc	MonkeyAI

.stand:
	lda	#$0505
	sta	v_foedelay,x
	jsr	.intruder	; if Tarzan is threaten'g & behind
	bcc	.noflip	; the monkey then flip before stand'g
	jsr	FacingTzan	; (monkey faces rite, carry is flipped)
	bcc	.noflip
	jsr	FlipH
.noflip:
	STEP		; frame 4 (stand)
	STEP		; frame 5
	STEP		; frame 6
	jsr	.intruder
	bcc	.walk
	jmp	.run

.walk:	lda	#3	; smooth out the recommended speed
	jsr	FlipFix	; of 4 pixel jumps @ 8 frames/sec
	sta	v_foevelx,x

.walkchk	FRAME 7		; frame 7 (walk 1)
	STEP		; frame 8 (2)
	STEP		; frame 9 (3)
	jsl	Random	; sometimes play monkey oo-oo sound
	cmp	#220
	bcc	.nonoise
	ldy	#chimp1_eff
	jsl	OnscreenSound
.nonoise:
	STEP		; frame 10 (4)
	STEP		; frame 11 (5)
	STEP		; frame 12 (6)

	jsr	.intruder	; if Tarzan is threatening, don't sit
	bcs	.nosit	; (sit starts from frame 6 of walk)
      	jsl	Random	; randomly decide to sit
	cmp	#200
	bcc	.nosit

	lda	#144
	jsr	NearEdge
	bcc	.sit

.nosit:
	STEP		; frame 13 (7)

	jsr	.intruder	; if Tarzan has invaded the branch
	bcc	.norun	; and the monkey is facing him,
	jsr	TowardTzan	; try an attack
	bcc	.norun

	lda	#208
	jsr	NearEdge
	bcc	.run

.norun:
	STEP		; last frame of walk (frame 14)

	lda	#144
	jsr	NearEdge
	bcs	.walkturn

	jsr	.intruder	; if Tarzan is behind the monkey and
	bcc	.dowalk	; there's enough room to turn around
	jsr	TowardTzan	; and chase him, turn around
	bcs	.dowalk
	lda	#220	; (need room for walk)
	jsr	NearOpEdge
	bcc	.walkturn
.dowalk:	jmp	.walk

.walkturn:
	stz	v_foevelx,x
	STEP		; frame 15 (walk turn)
	STEP		; frame 16
	STEP		; frame 17
	jsr	FlipH
	jsr	.intruder	; ****
	bcs	.run	; ****
	jmp	.walk

.sit:	stz	v_foevelx,x
	FRAME 26	; frame 26 (sit)
	STEP		; frame 27
	STEP		; frame 28
	jmp	MonkeyAI

.run:	ldy	#squak_eff	; monkey scream
	jsl	SoundEffect
	FRAME 18	; frame 18 (run to walk transition)

.keeprunning:	lda	#5	; speed up for run
	jsr	FlipFix
	sta	v_foevelx,x

	FRAME 19	; frame 19 (run)

	lda	#180
	jsr	NearEdge
	bcs	.runturn

	jsr	.intruder	; if Tarzan is behind the monkey and
	bcc	.noturn	; there's enough room to turn around and
	jsr	TowardTzan	; and chase him, turn around
	bcs	.noturn
	lda	#180	; (need room for full run)
	jsr	NearOpEdge
	bcc	.runturn

.noturn:
	STEP		; frame 20
	STEP		; frame 21
	STEP		; frame 22
	STEP		; frame 23
	STEP		; frame 24

	jsr	.intruder   	; if Tarzan is still on branch, keep
	bcs	.keeprunning	; running

	lda	#162
	jsr	NearEdge
	bcs	.keeprunning	; otherwise keep running

	lda	#3
	jsr	FlipFix
	sta	v_foevelx,x
	STEP		; frame 25 (run to walk transition)
	jmp	.walk

.runturn:	stz	v_foevelx,x
	FRAME 29	; frame 29 (run turn)
	STEP		; frame 30
	STEP		; frame 31
	jsr	FlipH
	jmp	.keeprunning


; .intruder subroutine - set carry if Tarzan has invaded the
; monkey's branch

.intruder:
	jsr	TzanInZone
	bcc	.noint
	jsr	DistY
	cmp	#32<<2
	bcs	.noint
	sec
	rts
.noint:	clc
	rts


*******************************************************************************
*
* Sitting Monkey
*
*******************************************************************************

SitMonkeyAI:
	RESET
	STEP
	STEP
	STEP
	bra	SitMonkeyAI


*******************************************************************************
*
* Drop Monkey
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
* v_foeaux3 = y coord of monkey's branch
*
*******************************************************************************

DropMonkeyAI:
	RESET		; frame 0 (tail wag)
	STEP		; frame 1
	STEP		; frame 2
	STEP		; frame 3

	; determine if this is a good time to jump

	jsr	DistX	; if Tarzan is close to the
	cmp	#256	; monkey's perch, jump
	bcs	.notclose
	jsr	DistY
	cmp	#128
	bcc	.drop

.notclose:	lda	v_foeaux3,x	; if Tarzan is standing on the
	sec	         	; monkey's branch, drop down
	sbc	v_playery
	bpl	.pos
	eor	#$FFFF
	inc
.pos:	cmp	#128
	bcs	DropMonkeyAI

	jsr	TzanInZone
	bcc	DropMonkeyAI

	; jump down to branch

.drop:	lda	#$0404
	sta	v_foedelay,x
	FRAME 32	; frame 32 (jump)
	STEP		; frame 33
	STEP		; frame 34
	STEP		; frame 35
	STEP		; frame 36
	STEP		; frame 37

	lda	v_foey,x
	clc
	adc	#128
	sta	v_foey,x
	lda	#32
	sta	v_foevely,x
	STEP		; frame 38

	lda	#$0101
	sta	v_foedelay,x
.fall:
	jsr	FoeFloor	;cf=1 = floor
	bcs	.land

	SAME		; hold frame 38 for duration of fall
	bra	.fall

.land:	stz	v_foevely,x
	lda	#$0404
	sta	v_foedelay,x
	STEP		; frame 49 (land)
	STEP		; frame 40
	FRAME 28	; frame 28 (last frame of sit)
	jmp	MonkeyAI


*******************************************************************************
*
* Ship Pirates
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
* v_foeaux3 = activation distance (x) in qixels
*
*******************************************************************************

PirateAI:
	SAME
	jsr	DistX   	; stand until Tarzan shows up
	cmp	v_foeaux3,x
	bcs	PirateAI

	STEP		; frame 1 (stand to walk)
	STEP		; frame 2
	STEP		; frame 3

	lda	#-23<<2	; adjust x coord to walking position
	jsr	AdjustX
.walk:
	lda	#-3	; start pirate moving
	jsr	FlipFix
	sta	v_foevelx,x

	FRAME 4	; frame 4 (walk)
	STEP		; frame 5
	STEP		; frame 6

           	jsr	DistY	; shoot if Tarzan is in range and
	cmp	#32<<2	; pirate is walking toward him
	bcs	.dontshoot
	jsr	DistX
	cmp	#128<<2
	bcs	.dontshoot
	jsr	TowardTzan
	bcc	.dontshoot

	ldy	#MAXFOE-2 	; don't shoot if a shot sequence is
	lda	v_foeplot,y	; already in progress
	bpl	.dontshoot
	ldy	#MAXFOE-4
	lda	v_foeplot,y
	bpl	.dontshoot

	lda	#ShotAI	; start gun blast
	sta	v_foeai,y

	lda	#0
	sta	v_foeplot,y	; assume onscreen
	sta	v_foedma,y	; not waiting for dma

	txa	           	; save index of pirate that fired gun
	sta	v_foeaux1,y

	lda	#$0600
	sta	v_foedelay,y
.dontshoot:
	STEP		; frame 7
	STEP		; frame 8
	STEP		; frame 9
	STEP		; frame 10
	STEP		; frame 11

	lda	#144
	jsr	NearEdge
	bcc	.walk

	stz	v_foevelx,x
	STEP		; frame 12
	lda	#64
	jsr	AdjustX
	STEP		; frame 13
	STEP		; frame 14

	jsr	FlipH
	jmp	.walk


ShotAI:
	ldy	#pistol_eff
	jsl	SoundEffect

	ldy	v_foeaux1,x

	lda	#SHOT
	sta	v_foetype,x

	lda	v_foey,y
	sec
	sbc	#42<<2
	sta	v_foey,x

	lda	v_foeoff,y
	and	#$FE00
	ora	#$01E0
	sta	v_foeoff,x

	lda	v_foevelx,y
	sta	v_foevelx,x
	stz	v_foevely,x

	stz	v_foehit,x	; ***** FOR TESTING ONLY!

	jsr	.sync	; first two frames of shot have
	RESET		; flames
	jsr	.sync
	STEP

	jsr	.sync	; rest of shot is smoke
	STEP
	STEP
	STEP
	TERM

.sync:			; keep shot sequence aligned with
	ldy	v_foeaux1,x	; end of gun (horizontally)
	lda	#-20<<2
	jsr	FlipFix
	clc
	adc	v_foex,y
	sta	v_foex,x
	rts



*******************************************************************************
*
* Barracuda
*
* v_foemisc1 = barracuda onscreen indicator
*              0 = barracuda has never been onscreen
*              1 = barracuda was onscreen at one time
* v_foepos1 = index of barracuda zone in v_water
*
*******************************************************************************

NewBarracuda:
	stz	v_foemisc1,x	; mark barracuda offscreen

	; choose y coordinate
	;
	; This routine loops through all vertical water "zones"
	; and determines which empty zone is closest to Tarzan.

	lda	v_playerdir	; project a point in the direction Tarzan
	asl		; is moving to use for proximity
	tay		; calculations...
	lda	v_playery
	clc
	adc	d_barproj,y
	sta	z_temp7
.scan:
	lda	#$FFFF	; z_temp1 = min dist from Tarzan so far
	sta	z_temp1
	ldy	#32
	lda	#$180<<2
.loop:
	sta	z_temp0	; z_temp0 = zone center point (y)
	lda	v_water,y
	beq	.next	; skip occupied zones (0=occupied)
	lda	z_temp0	; compute distance from zone center point
	sec		; to Tarzan...
	sbc	z_temp7
	bpl	.pos	; (make sure it is positive)
	eor	#$FFFF
	inc
.pos:	cmp	z_temp1	; smaller than the best distance?
	bcs	.next	; no, try again
	sta	z_temp1	; yes, make it the new best
	sty	z_temp2	; z_temp2 = index of best zone so far
	lda	z_temp0
	sta	z_temp3	; z_temp3 = closest distance so far
.next:	lda	z_temp0	; move to next zone
	clc
	adc	#96
	dey
	dey
	bpl	.loop

	ldy	z_temp2
	tya
	sta	v_foepos1,x	; save index of best zone
	lda	#0	; mark best zone occupied by foe
	sta	v_water,y
	lda	z_temp3	; position foe in zone
	sta	v_foey,x

	; choose x coordinate

	jsl	Random
	cmp	#192
	bcc	.fromleft

	lsr		; set fish going right to left...
	lsr
	clc
	adc	#256+72
	adc	z_levelx
	asl
	asl
	sta	v_foex,x
	jsr	ClearFlipH
	bra	.setspeed
.fromleft:
	lsr		; set fish going left to right...
	lsr
	sta	z_temp0
	lda	z_levelx
	sec
	sbc	z_temp0
	sbc	#72
	asl
	asl
	sta	v_foex,x
	jsr	SetFlipH

	; choose speed

.setspeed:	jsl	Random
	and	#$000C
	tay
	lda	barracudavel,y
	jsr	FlipFix
	sta	v_foevelx,x
	rts


BarracudaAI:
	jsr	NewBarracuda

.swim:	lda	#$0808
	sta	v_foedelay,x
	jsr	.attack
	RESET		; frame 0 (swim)
	jsr	.attack
	STEP		; frame 1
	jsr	.attack
	STEP		; frame 2
	jsr	.attack
	STEP		; frame 3

	lda	v_foemisc1,x   ; if foe was within screen x boundaries
	beq	.offscreen	; at one time, wait for it to leave

	lda	v_foeplot,x	; if offscreen, time for a new fish
	bne	.newfish

	jsr	TowardTzan	; if fish is onscreen and not facing
	bcs	.swim	; Tarzan, turn around every so often...
	jsl	Random
	cmp	#240
	bcc	.swim

	lda	v_foevelx,x	; turn...
	sta	v_foemisc2,x
	stz	v_foevelx,x
	lda	#$0606
	sta	v_foedelay,x
	STEP		; frame 4 (turn)
	STEP		; frame 5
	STEP		; frame 6
	jsr	FlipH
	lda	v_foemisc2,x
	eor	#$FFFF
	inc
	sta	v_foevelx,x
	jmp	.swim

.newfish:	ldy	v_foepos1,x	; mark fish out of zone
	tya
	sta	v_water,y	; non-zero value
	jmp	BarracudaAI	; generate new barracuda

.offscreen:	lda	z_levelx	; otherwise, wait for it to get there...
	clc
	adc	#256
	sta	z_temp0
	lda	v_foex,x
	lsr
	lsr
	cmp	z_levelx
	bcc	.stilloff
	cmp	z_temp0
	bcs	.stilloff
	lda	#1
	sta	v_foemisc1,x
.stilloff	jmp	.swim

.attack:			; if Tarzan is in the current zone, the
	jsr	TowardTzan	; foe is facing him, and the foe is close,
	bcc	.attackexit	; attack...
	lda	v_playery
	and	#$FFC0
	sta	z_temp0
	lda	v_foey,x
	and	#$FFC0
	cmp	z_temp0
	bne	.attackexit
	jsr	DistX
	cmp	#340
	bcs	.attackexit
	pla		; pop return address
	FRAME 7		; frame 7 (attack)
	STEP		; frame 8
	STEP		; frame 9
	jmp	.swim
.attackexit:	rts


*******************************************************************************
*
* Piranha
*
* v_foeaux1 = # of onscreen swim cycles before getting tired
*
* v_foemisc1 = # of swim cycles taken place onscreen; -1 indicates tired mode
*
*******************************************************************************

NewPiranha:
	stz	v_foemisc1,x	; mark piranha offscreen

	; choose y coordinate

	lda	v_playery
	sec
	sbc	#256
	cmp	#$300<<2
	bcs	.okay
	lda	#$300<<2
.okay:	sta	z_temp0
	jsl	Random
	asl
	clc
	adc	z_temp0
	sta	v_foey,x

	; choose x coordinate

	jsl	Random
	and	#63
	clc
	adc	#256+32
	adc	z_levelx
	asl
	asl
	sta	v_foex,x
	jsr	ClearFlipH

	; choose speed

	jsl	Random
	and	#$000C
	tay
	lda	piranhavel,y
	jsr	FlipFix
	sta	v_foevelx,x
	rts


PiranhaAI:
	jsr	NewPiranha
.swim:
	ldy	#0	; if piranha is far away from Tarzan
	jsr	DistX	; horizontally then y velocity = 0
	cmp	#384	;
	bcs	.setvely
	jsr	DistY	; if piranha is near Tarzan vertically
	cmp	#25	; then y velocity = 0, otherwise move
	bcc	.setvely	; toward Tarzan
	iny
	lda	v_foey,x
	cmp	v_playery
	bcc	.setvely
	ldy	#-1
.setvely:	tya
	sta	v_foevely,x

	jsr	TowardTzan	; if piranha is not facing Tarzan,
	bcc	.turn	; turn around

.justdoit:	jsr	.attack
	RESET		; frame 0 (swim)
	jsr	.attack
	STEP		; frame 1

	lda	v_bossactive	; if in Mr. Crocodile is active, get
	beq	.okay	; outta here...
	jmp	.crocescape
.okay:
	jsr	.attack
	STEP		; frame 2
	jsr	.attack
	STEP		; frame 3

	lda	v_foeplot,x	; if foe is offscreen, check if it
	bne	.offscreen	; should be removed
.timer:	lda	v_foemisc1,x	; if piranha is tired of Tarzan, just
	bmi	.justdoit	; keep swimming
	inc		; otherwise inc onscreen timer
	sta	v_foemisc1,x
	cmp	v_foeaux1,x	; if foe has been onscreen too long,
	bcc	.swim	; switch to "tired" mode...

	lda	#-1	; go to tired mode:  if moving right,
	sta	v_foemisc1,x	; turn around & swim away
	stz	v_foevely,x
	lda	v_foevelx,x
	bpl	.turn
	jsl	Random	; if moving left, only turn around
	cmp	#200	; sometimes
	bcc	.justdoit

.turn:	lda	v_foevelx,x
	sta	v_foemisc2,x
	stz	v_foevelx,x
	FRAME	4	; frame 4 (turn)
	STEP		; frame 5
	STEP		; frame 6
	lda	v_foemisc2,x
	eor	#$FFFF
	inc

	sta	v_foevelx,x
	jsr	FlipH

	lda	v_foemisc1,x	; if in tired mode, just swim blindly
	bmi	.justdoit
.goswim:	jmp	.swim	; otherwise swim & home in on Tarzan

.offscreen:	lda	v_foemisc1,x	; if fish is offscreen but was onscreen
	beq	.goswim	; at one time, replace it with a
	jsr	NewPiranha	; different fish...
	jmp	.swim

.attack:
	lda	v_foemisc1,x	; if piranha is in escape mode, just exit
	bmi	.attackexit
	jsr	DistX	; if piranha is very close to Tarzan,
	cmp	#232	; chomp away...
	bcs	.attackexit
	jsr	DistY
	cmp	#32
	bcs	.attackexit
	lda	v_playery
	cmp	#$300<<2	; don't start attack if too close to
	bcc	.attackexit	; surface
	pla
	FRAME 7		; frame 7 (attack)
	STEP		; frame 8
	STEP		; frame 9
	jmp	.timer
.attackexit:	rts


	; Tarzan is near the crocodile, so get away ASAP.

.crocescape:	stz	v_foevely,x	; no vertical motion
	lda	v_foevelx,x	; if moving left, keep going that way
	bmi	.running
	stz	v_foevelx,x	; otherwise turn around
	FRAME 4
	STEP
	STEP
	jsr	FlipH
.running:	lda	#-8	; speed up
	sta	v_foevelx,x
.gogo:	RESET		; and swim out of there
	STEP
	STEP
	STEP
	lda	v_foeplot,x	; until fish is offscreen
	beq	.gogo
	TERM


*******************************************************************************
*
* Spider
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
*******************************************************************************

SpiderAI:
	; wait until Tarzan is beneath spider

	RESET
	lda	v_playery
	sec
	sbc	v_foey,x
	cmp	#512
	bcs	SpiderAI
	jsr	DistX
	cmp	#32
	bcs	SpiderAI

	; drop onto cavern floor

	lda	#8
	sta	v_foevely,x
	lda	#$0101
	sta	v_foedelay,x
.drop:	SAME		; frame 0 (hang/drop)
	lda	v_foevely,x
	cmp	#16
	bcs	.maxvel
	clc
	adc	#2
	sta	v_foevely,x
.maxvel:
	jsr	FoeFloor	; cf=1 = walkable
	bcc	.drop

	; land safely

	stz	v_foevely,x
	lda	#$0505
	sta	v_foedelay,x

	lda	v_foex,x	; start spider off in Tarzan's direction
	sec
	sbc	v_playerx
	bpl	.okay
	jsr	FlipH
.okay:	STEP		; frame 1 (land)
	STEP		; frame 2

	; walk within zone

.walk:	lda	#-3
	jsr	FlipFix
	sta	v_foevelx,x

	FRAME 3		; frame 3 (walk)
	STEP		; frame 4
	STEP		; frame 5
	STEP		; frame 6

	lda	#72
	jsr	NearEdge
	bcs	.turn

      	jsr	.intruder	; if Tarzan isn't around, keep walking
	bcc	.walk
	jsr	TowardTzan	; if Tarzan is here & spider is headed
	bcs	.walk	; toward him, keep walking

	lda	#72	; if Tarzan is behind spider & there is
	jsr	NearOpEdge	; room to turn around & git him, go ahead
	bcs	.walk

.turn:	stz	v_foevelx,x
	STEP		; frame 7 (turn)
	STEP		; frame 8
	STEP		; frame 9
	jsr	FlipH
	bra	.walk

.intruder:			; set cf if Tarzan has invaded the
	jsr	TzanInZone
	bcc	.noint
	jsr	DistY
	cmp	#64<<2
	bcs	.noint
	sec
	rts
.noint:	clc
	rts


*******************************************************************************
*
* Small Cat
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
* v_foemisc1 = pounce length (0=short, non-zero=long)
*
*******************************************************************************

SmallCatAI:
	STEP		; frame 1 (stand to walk)
	STEP		; frame 2
	STEP		; frame 3
.walk:
	lda	#-3
	jsr	FlipFix
	sta	v_foevelx,x
	FRAME 4		; frame 4 (walk)
	STEP		; frame 5
	STEP		; frame 6
	STEP		; frame 7
	STEP		; frane 8

	jsr	.intruder	; if Tarzan is threatening, cat is facing
	bcc	.nopounce	; him, there is room to pounce 'n walk,
	jsr	TowardTzan	; and Tarzan is close enough to pounce on,
	bcc	.nopounce	; go ahead
	jsr	DistX	; depending on distance from Tarzan,
	cmp	#220	; choose between short and long jumps
	bcc	.checkshort
	cmp	#650
	bcs	.nopounce
	lda	#704+72
	jsr	NearEdge
	bcs	.checkshort
	jsr	DistX
	lda	#1
	sta	v_foemisc1,x	; 1 = long pounce
	jmp	.pounce

.checkshort:	lda	#320+72
	jsr	NearEdge
	bcs	.nopounce
	stz	v_foemisc1,x	; 0 = short pounce
	jmp	.pounce

.nopounce:	STEP		; frame 8 (last frame of walk)

	lda	#108
	jsr	NearEdge
	bcs	.turn

	jsr	.intruder	; if a threatening Tarzan is behind
	bcc	.randturn	; Mr. Cat and there is room to turn
	jsr	TowardTzan	; around and walk, do it
	bcs	.gowalk
	lda	#108
	jsr	NearOpEdge
	bcc	.turn
.gowalk:
	jmp	.walk
.randturn:
	lda	#108	; if Tarzan isn't around, turn every
	jsr	NearOpEdge	; so often
	bcs	.gowalk
	jsl	Random
	cmp	#250
	bcc	.gowalk

.turn:
	stz	v_foevelx,x
	STEP
	STEP
	STEP
	lda	#36
	jsr	AdjustX
	jsr	FlipH
	jmp	.walk


	; pounce sequence
.pounce:
	stz	v_foevelx,x	; stop moving for takeoff
	FRAME 14
	STEP		; frame 15 (crouch)
	STEP		; frame 16
	STEP		; frame 17
	STEP		; frame 18
	STEP		; frame 19
	STEP		; frame 20 (final takeoff)

	lda	#-32
	jsr	FlipFix
	sta	v_foevelx,x
	lda	#-32
	jsr	AdjustX
	lda	#-32
	jsr	AdjustY
	lda	#-8
	sta	v_foevely,x

	STEP		; frame 21 (in air)

	lda	v_foemisc1,x	; if short jump, don't do any mid-air
	beq	.short	; extension
	lda	#-4
	sta	v_foevely,x
	SAME
	lda	#4
	sta	v_foevely,x
	SAME

.short:	lda	#-16
	jsr	FlipFix
	sta	v_foevelx,x
	lda	#16
	jsr	AdjustY
	lda	#8
	sta	v_foevely,x

	STEP		; frame 22 (feet out for landing)

	lda	#16
	jsr	AdjustY
	stz	v_foevelx,x
	stz	v_foevely,x

	STEP		; frame 23 (landing & recovery)
	STEP		; frame 24
	STEP		; frame 25
	STEP		; frame 26

	RESET		; frame 0 (stand)
	SAME
	SAME

	jmp	SmallCatAI

.intruder:			; set cf if Tarzan has invaded the
	jsr	TzanInZone
	bcc	.noint
	jsr	DistY
	cmp	#64<<2
	bcs	.noint
	sec
	rts
.noint:	clc
	rts


*******************************************************************************
*
* Boss crocodile (downriver)
*
* The croc body (CROCBOD) must be in the foe table immediately after the
* croc (CROC)>.
*
* v_foemisc1 = crocodile ai mode, called before each swim cycle
*
*******************************************************************************

CrocAI:
	; wait until Tarzan gets near the end of level

	WAIT		; (this part of the ai could run much less
	lda	v_playerx	; often than currently)
	cmp	#$747<<2
	bcc   CrocAI
	lda	#.chaseai	; start out in chase mode
	sta	v_foemisc1,x
	lda	#$180	; expand right boundary to handle
	sta	v_rightedge	; clipping of giant crododile
	sta	v_bossactive
.swim:
	lda	#-8
	jsr	FlipFix
	sta	v_foevelx,x
	jmp	(v_foemisc1,x)	; call sub-ai routine

.doswim:	FRAME 1	; frame 1 (tail frame 1)
	STEP		; frame 2 (2)
	STEP		; frame 3 (3)
	STEP		; frame 4 (4)
	STEP		; frame 5 (5)
	STEP		; frame 6 (6)
	STEP		; frame 7 (7)
	STEP		; frame 8 (8)
	lda	#320
	jsr	NearEdge
	bcc	.swim

.turn:	lda	#CROC2	; switch to turn sequence (has a
	sta	v_foetype,x	; separate drec)
	lda	#croc2
	stz	v_foevelx,x
	stz	v_foevely,x
	RESET
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	lda	#CROC	; switch back to normal croc
	sta	v_foetype,x
	lda	#8
	jsr	FlipFix
	sta	v_foevelx,x
	jsr	FlipH	; flip croc tail
	txy		; get offset of body
	iny
	iny
	lda	v_foeactoff,y	; and flip croc body
	eor	#$4000
	sta	v_foeactoff,y
	lda	#-32
	jsr	AdjustY
	lda	#-128
	jsr	AdjustX
	jmp	.swim

.chaseai:
	lda	#320	; follow Tarzan around until a good snap
	jsr	NearOpEdge	; at him can be taken...
	bcs	.keepchasing
	jsr	TowardTzan
	bcc	.keepchasing
	lda	v_foey,x
	sec
	sbc	#64
	sbc	v_playery
	bpl	.posy
	eor	#$FFFF
	inc
.posy:	cmp	#128
	bcs	.keepchasing
	jsr	DistX
	cmp	#320
	bcc	.attack
.keepchasing:	lda	#2
	ldy	v_foey,x
	cpy	#$320<<2	; don't get too close to surface,
	bcc	.chasedown	; otherwise the croc will go airborne
	cpy	v_playery	; while turning
	bcc	.chasedown
	lda	#-2
.chasedown:	sta	v_foevely,x
	jmp	.doswim
.attack:	lda	#.escapeai	; switch to escape mode (takes effect
	sta	v_foemisc1,x	; after bite & turn seq ends)
	jmp	.turn

.escapeai:			; move toward swirl box...
	lda	v_foey,x
	cmp	#$348<<2
	bcc	.nope
	cmp	#$382<<2
	bcs	.nope
	lda	v_foex,x
	cmp	#$773<<2
	bcc	.nope
	cmp	#$7B9<<2
	bcc	.doswirl
.nope:	ldy	#2
	lda	v_foey,x
	cmp	#$35D<<2
	bcc	.escdiry
	ldy	#0
	cmp	#$382<<2
	bcc	.escdiry
	ldy	#-2
.escdiry:	tya
	sta	v_foevely,x
	jmp	.doswim

.doswirl:	lda	#.chaseai	; switch to chase mode (takes effect
	sta	v_foemisc1,x	; after swirl ends)
	stz	v_foevelx,x
	stz	v_foevely,x
	lda	#$0303	; speed up animation for swirl
	sta	v_foedelay,x
	FRAME 9	; transition from swim to swirl
.swirl:	FRAME 10	; 1st half of swirl...
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	jsr	FlipH	; 2nd hald of swirl is 1st half
	FRAME 10	; x-flipped...
	STEP
	STEP
	STEP
	STEP
	STEP
	jsl	Random	; randomly decide whether to do another
	cmp	#200	; swirl or stop
	bcc	.endswirl
	STEP		; finish 2nd half of swirl
	STEP
	STEP
	jsr	FlipH
	jmp	.swirl

.endswirl:	FRAME 19
	lda	#$0404	; back to normal swim rate
	sta	v_foedelay,x
	STEP		; transition from swirl to swim
	jsr	FlipH
	lda	#-128
	jsr	AdjustX
	lda	#64
	jsr	AdjustY
	jmp	.swim


; The crocodile body syncs up with the tail when in the swim sequence.
; At all other times, the body is removed from the screen.  The croc body
; frame is kept in vram & does not need to be downloaded when the tail
; frame changes.

CrocBodAI:
	FORCE		; only dowload body once

	; wait for crocodile to become active

.wait:	WAIT
	lda	v_bossactive
	beq	.wait
	stz	v_foedelay,x

	; sync croc body with tail

.sync:	txy		; get pointer to tail foe
	dey
	dey
	lda	v_foetype,y	; only display body when croc is swimming
	cmp	#CROC
	bne	.nobod
	lda	v_foeframe,y
	cmp	#9
	bcs	.nobod

	lda	v_foex,y	; set body to coordinates of tail
	sta	v_foex,x
	lda	v_foey,y
	sta	v_foey,x
	WAIT		; body frame is already in vram
	bra	.sync

.nobod:	lda	#-1000
	sta	v_foex,x
	WAIT
	bra	.sync


*******************************************************************************
*
* Rodent
*
* v_foeaux1 = index into hole table (multiple of 4), specifying which hole
*             this rodent lives in
*
*******************************************************************************

RodentAI:
	; Copy hole x,y coordinates from hole data table to
	; foe variables.

	ldy	z_levelword
	lda	holelocs-2,y
	sta	z_temp0
	ldy	v_foeaux1,x
	lda	(z_temp0),y
	asl
	asl
	sta	v_foemisc1,x	; v_foemisc1 = hole x coord in qixels
	iny
	iny
	lda	(z_temp0),y
	asl
	asl
	sta	v_foemisc2,x	; v_foemisc2 = hole y coord in qixels

	; Wait until either the rat is onscreen or Tarzan has the
	; audacity to get between the rat and its hole.

.wait:	SAME
	lda	v_foeplot,x	; if rodent is onscreen, start running
	beq	.startrun

	jsr	DistY	; must be close to y position of rodent
	cmp	#320
	bcs	.wait

	lda	v_foeoff,x	; if rodent is flipped, it must be
	bit	#$4000	; on the left side of its hole
	bne	.onleft

	lda	v_playerx
	cmp	v_foemisc1,x
	bcc	.wait
	cmp	v_foex,x
	bcs	.wait
	bcc	.startrun
.onleft:
	lda	v_playerx
	cmp	v_foex,x
	bcc	.wait
	cmp	v_foemisc1,x
	bcs	.wait

	; Run toward hole, continually checking if near enough to
	; jump into it.
.startrun:
	lda	#$0303	; speed up for run
	sta	v_foedelay,x
	lda	#-8
	jsr	FlipFix
	sta	v_foevelx,x
.run:
;	lda	v_stundelay,x
;	bne	.freeze
	RESET
	jsr	.checkjump
	STEP
	jsr	.checkjump
	STEP
	jsr	.checkjump
	STEP
	jsr	.checkjump
	STEP
	jsr	.checkjump
	STEP
	jsr	.checkjump
	bra	.run

;.freeze	WAIT
;	dec	v_stundelay,x
;	beq	.fin
;	lda	v_foevelx,x
;	beq	.nos
;	sta	v_foeaux3,x
;	stz	v_foevelx,x
;.nos
;	bra	.run
;.fin	lda	v_foeaux3,x
;	sta	v_foevelx,x
;	stz	v_foeaux3,x
;	bra	.run

.checkjump:
	lda	v_foemisc1,x
	sec
	sbc	v_foex,x
	bpl	.pos
	eor	#$FFFF
	inc
.pos:	cmp	#40
	bcc	.jump
	rts

.jump:	pla
	stz	v_foevelx,x
	lda	v_foemisc1,x
	sta	v_foex,x
	lda	v_foemisc2,x
	sta	v_foey,x
	FRAME	6
	STEP
	STEP
	STEP

	lda	#$0FFF	; remove rodent from screen
	sta	v_foey,x
	lda	#$4040	; keep foe in limbo for a bit before
	sta	v_foedelay,x	; it tries to reset
	lda	v_foepos1,x	; move back to original x coord
	sta	v_foex,x
	SAME

	; Wait until Tarzan is on the opposite side of the hole
	; or moves away from it vertically.

	lda	#$0606	; slow down for waiting
	sta	v_foedelay,x

.inhole:	SAME		; wait for player to get away from
	lda	v_playery	; rodent's starting position
	sec
	sbc	v_foepos2,x
	cmp	#1024
	bcc	.inhole
	jsr	DistX
	cmp	#1024
	bcc	.inhole

 	lda	v_foepos2,x
	sta	v_foey,x
	jmp	.wait


*******************************************************************************
*
* Bat
*
* The flip bit of the bat definition has special meaning:
*      0 = fly toward Tarzan
*      1 = fly away from Tarzan
*
* v_foemisc1 = y distance from Tarzan to drop to
*
*******************************************************************************

BatAI:
	; wait until Tarzan is close

	SAME
	stz	v_foeframe,x

	lda	v_playery
	sec
	sbc	v_foey,x
	cmp	#768
	bcs	BatAI

	; The "open eyes" frame currently isn't used because it
	; requires Tarzan to get too close to the bat before it
	; activates.

	jsr	DistX
;	cmp	#444	; open eyes at this distance
;	bcs	BatAI
	cmp	#444	; attack at this distance
	bcc	BatWake
	inc	v_foeframe,x
	bra	BatAI

BatWake:
	lda	v_foex,x	; if player is to the left of the bat,
	sec		; use original h-flip, otherwise flip it
	sbc	v_playerx
	bpl	.origdir
	jsr	FlipH
.origdir:
	; open wings & drop toward Tarzan

	FRAME 2
	lda	#8
	sta	v_foevely,x
	lda	#$0101
	sta	v_foedelay,x

	jsl	Random
	and	#127
	clc
	adc	#200
	sta	v_foemisc1,x

.drop:	FRAME 3
	lda	v_foevely,x
	cmp	#16
	bcs	.maxvel
	clc
	adc	#2
	sta	v_foevely,x
.maxvel:	jsr	DistY
	cmp	v_foemisc1,x
	bcc	.level
	cmp	#768 	; don't dive if Tarzan has fallen down
	bcc	.drop	; a pit

.level:	lda	#8
	sta	v_foevely,x
	lda	#$0505
	sta	v_foedelay,x
	STEP
	lda	#4
	sta	v_foevely,x
	lda	#32
	jsr	AdjustX
	lda	#-10
	jsr	FlipFix
	sta	v_foevelx,x
	STEP
	stz	v_foevely,x
	STEP

	; fly offscreen
.fly:
	jsl	Random
	and	#$000E
	tay
	lda	batvely,y
	sta	v_foevely,x
	FRAME 8
	FRAME 9
	FRAME 7
	lda	v_foeplot,x
	beq	.fly

	TERM


*******************************************************************************
*
* Slave Bat
*
* Bats can be wired to other bats.  The slaves are automatically triggered when
* the master is triggered.
*
* v_foeaux1 = foe table index of master bat (multiple of 2)
*
*******************************************************************************

SlatAI:
	SAME
	ldy	v_foeaux1,x	; get index of master bat
	lda	v_foeplot,y	; if master is dead, make Tarzan pay
	bmi	.wake

	lda	v_foeframe,y	; if master is attacking, do likewise
	cmp	#2
	bcs	.wake
      	sta	v_foeframe,x	; use same frame as master
	bra	SlatAI

.wake:	jmp	BatWake


*******************************************************************************
*
* Club Swinging Oparian
*
* v_foeaux1 = left limit
* v_foeaux2 = right limit
*
*******************************************************************************

OparianAI:

.move:	lda	#-2
	jsr	FlipFix
	sta	v_foevelx,x

.walk:	RESET
	STEP
	ldy	#whoosh4_eff
	jsl	OnscreenSound
	STEP
	STEP
	STEP
	STEP
	ldy	#whoosh4_eff
	jsl	OnscreenSound
	STEP
	STEP

	lda	#112
	jsr	NearEdge
	bcs	.turn

	jsr	TzanInZone
	bcc	.walk
	jsr	DistY
	cmp	#256
	bcs	.walk
	jsr	TowardTzan
	bcs	.walk
	lda	#112
	jsr	NearOpEdge
	bcs	.walk

.turn:	stz	v_foevelx,x
	STEP
	STEP
	STEP
	jsr	FlipH
	lda	#-64
	jsr	AdjustX
	bra	.move


*******************************************************************************
*
* Rock Throwing Oparian
*
* v_foeaux1 = foe table index of first of 2 consecutive stones
* v_foeaux3 = y value of floor
*
* v_foemisc1 (of thrower) = foe table index of stone, -1 when no stone (this
*      is also used by the thrower death routine)
*
*******************************************************************************

ThrowerAI:
	lda	#-1	; thrower doesn't have a stone yet
	sta	v_foemisc1,x

.wait:	FRAME 1     	; 1
	ldy	v_foeaux1,x	; get foe table index of stone
	lda	v_foemisc1,y	; check 1st stone
	beq	.stonefree	; if free, use it
	iny
	iny
	lda	v_foemisc1,y	; check 2nd stone
	bne	ThrowerAI	; if not free, wait
.stonefree:	jsr	DistX
	cmp	#650
	bcs	.wait
	tya		; save index of stone in v_foemisc1
	sta	v_foemisc1,x
	lda	#1	; mark stone active (v_foemisc1
	sta	v_foemisc1,y	; of stone <> 0)

	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	SAME
	STEP		; 6
	STEP		; 7
	STEP		; 8
	STEP		; 9
	ldy	#whoosh2_eff
	jsl	OnscreenSound
	FRAME	8	; 8
	FRAME	10	; 10

	ldy	v_foemisc1,x	; get foe table index of stone
	txa
	sta	v_foemisc1,y	; save ptr to thrower
	lda	#$FFFF	; stone is initially ofscreen
	sta	v_foey,y
	sta	v_foemisc1,x	; stone now has a life of its own

	lda	#0	; set-up misc stone variables...
	sta	v_foeplot,y
	sta	v_foedma,y
	sta	v_foevelx,y
	sta	v_foevely,y
	sta	v_foedelay,y
	lda	#StoneAI
	sta	v_foeai,y
	lda	#STONE
	sta	v_foetype,y
	lda   v_foeoff,x
	and	#$CE00	; keep palette & flip of thrower, but
	ora	v_foeoff,y	; switch to vram slot & priority of
	sta	v_foeoff,y	; stone
	sta	v_foeactoff,y
	lda	v_foeaux3,x	; copy y value of floor to stone entry
	sta	v_foeaux3,y
	STEP		; 11
	STEP		; 12
	jmp	ThrowerAI



; This is used to reserve a place in the foe table *before* the thrower(s), so
; the stone will have a higher priority when plotted.

StoneDumAI:
	stz	v_foemisc1,x
	TERM


; This is the real ai routine for a falling stone.

StoneAI:
;	lda	_xfercount
;	cmp	#6
;	bcc	.start
;	WAIT
;	bra	StoneAI

.start:	stz	v_foeplot,x

	lda	#$0101
	sta	v_foedelay,x
	ldy	v_foemisc1,x
	lda	#-48
	jsr	FlipFix
	clc
	adc	v_foex,y
	sta	v_foex,x
	lda	#-48
	clc
	adc	v_foey,y
	sta	v_foey,x

	lda	#16
	sta	v_foevely,x
	FRAME 13
	lda	#18
	sta	v_foevely,x
	lda	#14	; make sure falling stone frame is in
	sta	v_foeframe,x	; vram
	BLAST		; 14

.fall:	lda	v_foevely,x
	cmp	#32
	bcs	.max
	inc
	inc
	sta	v_foevely,x
.max:	WAIT		; 14
	lda	v_foey,x	; if stone is offscreen and below Tarzan,
	cmp	v_playery	; get rid of it...
	bcc	.above
	ldy	v_foeplot,x
	bne	.done
.above:	cmp	v_foeaux3,x	; check for collision with floor
	bcc	.fall

.impact:	jsr	FoeFloor
	lda	#$0404
	sta	v_foedelay,x
	stz	v_foevely,x
	ldy	#crashopr_eff	; choose proper crash sound effect for
	lda	z_level	; this level (can be either streets or
	cmp	#OPARSTREET	; temple)
	beq	.street
	ldy	#crash1_eff
.street:	sty	v_shakescreen	; trigger screen shake effect
	jsl	SoundEffect
	STEP		; 15
	STEP		; 16
	STEP		; 17
	STEP		; 18
	STEP		; 19

.done:	stz	v_foemisc1,x	; mark stone finished
	TERM


PileAI:	BLAST
.wait:	WAIT
	bra	.wait


*******************************************************************************
*
* Stump Overlay (upriver)
*
*******************************************************************************

StumpAI:
	stz	v_foeplot,x	; assume onscreen
	SAME
.wait	WAIT
	bra	.wait


*******************************************************************************
*
* Leopard Boss (Jungle 4)
*
*******************************************************************************

LeopardAI:
	lda	#11
	sta	v_bosshealth
	sta	v_bossoldhealth

.sleep:
	WAIT

	lda	v_playerx	; wait for Tarzan to get near the boss
	cmp	#$730<<2
	bcc	.sleep

;	lda	v_playery
;	cmp	#$2E0<<2
;	bcc	.sleep

	sta	v_bossactive	; mark boss active

.startwalk:	
	lda	#-3

	jsr	FlipFix

	sta	v_foevelx,x

.walk:
	FRAME 11	; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6

	lda	v_playery
	cmp	#$2D2<<2
	bcc	.noleap

	jsr	TowardTzan

	bcc	.noleap

	jsr	DistX

	cmp	#$512

	bcs	.noleap

	jmp	.leap

.noleap:
	STEP		; 7
	STEP		; 8
	STEP		; 9
	STEP		; 10
	STEP		; 11

	lda	#216

	jsr	NearEdge

	bcs	.turn

	lda	v_playery
	cmp	#$2D2<<2
	bcc	.walk

	jsr	TowardTzan

	bcc	.behindtzan

	lda	#512

	jsr	NearEdge

	bcs	.walk

	jsr	DistX

	cmp	#512
	bcs	.walk

	jmp	.leap

.behindtzan:
	lda	#216

	jsr	NearOpEdge

	bcs	.walk

.turn:
	stz	v_foevelx,x

	STEP		; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3

	lda	#112

	jsr	AdjustX

	jsr	FlipH

	lda	v_foeactoff+4	; flip leopard body
	eor	#$4000
	sta	v_foeactoff+4

	jmp	.startwalk

.stand:
	FRAME	1

	jsr	FacingTzan

	bcc	.standxwalk

	jsr	DistX

	cmp	#768
	bcs	.standxwalk

	cmp	#256
	bcs	.noswipe

	jmp	.swipe

.noswipe:
	jsl	Random

	cmp	#240
	bcc	.stand

	cmp	#248
	bcc	.standxwalk

	jmp	.scream

.standxwalk:
	FRAME	8
	STEP
	STEP

	jmp	.startwalk


.scream:
	FRAME 1
	FRAME 2		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
	STEP		; 7
	FRAME 6
	FRAME 4
	FRAME 3
	FRAME 2

	jmp	.stand

.swipe:
	FRAME 1
	SAME

	lda	#LEOPARD2
	sta	v_foetype,x

	RESET		; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
	STEP		; 7
	STEP		; 8
	FRAME 2		; 2
	FRAME 9		; 9

	lda	#LEOPARD1
	sta	v_foetype,x

	jmp	.stand

.leap:
	lda	#LEOPARD2
	sta	v_foetype,x

	stz	v_foevelx,x

	FRAME	10	; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6

	lda	#-160

	jsr	AdjustX

	lda	#$0000
	sta	v_foedelay,x

	lda	#-20
	sta	v_foevely,x

	lda	#-24

	jsr	FlipFix

	sta	v_foevelx,x

.goingup:
	FRAME	17  	; leap 7 (en route)

	lda	v_foevely,x
	inc
	inc
	sta	v_foevely,x

	cmp	#2
	bne	.goingup

.goingdown:
	FRAME 18	; 8

	lda	v_foevely,x
	inc
	inc
	sta	v_foevely,x

	cmp	#22
	bne	.goingdown

	stz	v_foevely,x
	stz	v_foevelx,x

	lda	#$0505
	sta	v_foedelay,x

	STEP		; 9
	STEP		; 10
	STEP		; 11
	STEP		; 12

	lda	#LEOPARD1
	sta	v_foetype,x

	jmp	.stand

@LeopardHit:
	stz	v_foevelx,x
	stz	v_foevely,x

	lda	#$310<<2
	sta	v_foey,x

	lda	#$0505
	sta	v_foedelay,x

	lda	#LEOPARD2
	sta	v_foetype,x

	FRAME 23	; 23/0
	STEP		; 24/1
	STEP		; 25/2
	STEP		; 26/3

	lda	v_bosshealth
	beq	.leoparddead

	STEP		; 27/4
	SAME		; 27/4
	SAME		; 27/4
	SAME		; 27/4
	SAME		; 27/4

	jsl	Random

	lsr
	bcc	.recover

	lda	#30	; extend invincibility
	sta	v_bossinvince

	FRAME 26	; 26/3
	FRAME 25	; 25/2
	STEP		; 26/3
	STEP		; 27/3
	SAME		; 27/3
	SAME		; 27/3
	SAME		; 27/3
	SAME		; 27/3
	SAME		; 27/3
.recover
	FRAME 24	; 24/1
	FRAME 23	; 23/0

	lda	#LEOPARD1
	sta	v_foetype,x

	jmp	.stand

.leoparddead:
	lda	#-32

	jsr	AdjustY

	jmp	BossPoofAI


LeopardMonitor:
	WAIT

	lda	v_bossinvince
	beq	.notinvince

	dec	v_bossinvince

	bra	LeopardMonitor

.notinvince:
	lda	v_bosshealth
	cmp	v_bossoldhealth

	beq	LeopardMonitor

	sta	v_bossoldhealth

	lda	#LeopardHit
	sta	v_foeai+2

	lda	#30
	sta	v_bossinvince

	bra	LeopardMonitor

LeopardBodAI:
.sleep:
	WAIT		; wait for leopard to wake up

	lda	v_bossactive
	beq	.sleep

	stz	v_foedelay,x

	FORCE		; keep body in vram

.sync:
	WAIT

	lda	v_foetype+2
	cmp	#LEOPARD1
	bne	.offscreen

	lda	v_foeframe+2
	cmp	#8
	bcs	.offscreen

	lda	v_foex+2
	sta	v_foex,x

	lda	v_foey+2
	sta	v_foey,x

	bra	.sync

.offscreen:
	lda	#-1000
	sta	v_foex,x

	bra	.sync


*******************************************************************************
*
* Boss Lion AI (Beach)
*
*******************************************************************************

LionLeapOK:
	lda	v_playerx
	cmp	#$A9D<<2
	bcs	.nope

	jsr	TowardTzan

	bcc	.nope

	lda	v_foevelx,x	; If the lion is moving to the left,
	bmi	.enoughroom	; there is always enough room to leap

	lda	v_foex,x	; If moving right, need to make sure
	cmp	#$AD0<<2	; he doesn't go crashing into the rocks
	bcs	.nope

.enoughroom:
	jsr	DistX

	cmp	#512
	bcs	.nope

	sec

	rts

.nope:
	clc

	rts


LionAI:
	lda	#9
	sta	v_bosshealth
	sta	v_bossoldhealth

.sleep:
	WAIT

	lda	v_playerx	; wait for Tarzan to get near boss
	cmp	#$9A5<<2

	bcc	.sleep
	sta	v_bossactive	; mark boss active

	phx

	ldy	#lionbch_eff

	jsl	SoundEffect

	plx

	jmp	.leap	; surprise ol' Mr. Tarzan

.startwalk:
	lda	#-3

	jsr	FlipFix

	sta	v_foevelx,x

.walk:
	FRAME 10	; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
	STEP		; 7
	STEP		; 8
	STEP		; 9
	STEP		; 10
	STEP		; 11

	jsr	LionLeapOK

	bcc	.noleap

	jmp	.leap

.noleap:
	lda	#216

	jsr	NearEdge

	bcs	.turn

	jsr	TowardTzan

	bcs	.walk

	lda	v_playerx
	cmp	#$A9D<<2
	bcs	.walk

	lda	#216

	jsr	NearOpEdge

	bcs	.walk

.turn:
	stz	v_foevelx,x

	STEP
	STEP
	STEP
	STEP

	lda	#96

	jsr	AdjustX

	jsr	FlipH

	lda	v_foeactoff+4	; flip lion body
	eor	#$4000
	sta	v_foeactoff+4

	jmp	.startwalk


.stand:
	FRAME	1

	jsr	FacingTzan

	bcc	.standxwalk

	jsr	DistX

	cmp	#768
	bcs	.standxwalk

	cmp	#256
	bcs	.noswipe

	jmp	.swipe

.noswipe:
	jsl	Random

	cmp	#240
	bcc	.stand

	cmp	#248
	bcc	.standxwalk

	jmp	.roar

.standxwalk:	
	FRAME	8
	STEP
	STEP

	jmp	.startwalk

.swipe: 
	lda	#LION2
	sta	v_foetype,x

	ldy	#lionbch_eff

	jsl	SoundEffect

	RESET		; 0/0
	STEP		; 1/1
	STEP		; 2/2
	SAME
	STEP		; 3/3
	STEP		; 4/4
	STEP		; 5/5
	STEP		; 6/6
	STEP		; 7/7
	STEP		; 8/8
	FRAME 1		; 1/1
	FRAME 0		; 0/0

	lda	#LION1
	sta	v_foetype,x

	jmp	.stand

.roar:
	ldy	#lionbch_eff

	jsl	SoundEffect

	FRAME 2		; 2/0
	STEP		; 3/1
	STEP		; 4/2
	STEP		; 5/3
	STEP		; 6/4
	FRAME 5		; 5/3
	FRAME	4	; 4/2
	FRAME	3	; 3/1

	jmp	.stand

.leap:	lda	#LION2
	sta	v_foetype,x
	stz	v_foevelx,x
	FRAME	9	; 0
	ldy	#lionbch_eff
	jsl	SoundEffect
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	lda	#-160
	jsr	AdjustX
	lda	#$0000
	sta	v_foedelay,x
	lda	#-20
	sta	v_foevely,x
	lda	#-24
	jsr	FlipFix
	sta	v_foevelx,x

.goingup:	FRAME	15  	; leap 6 (en route)
	lda	v_foevely,x
	inc
	inc
	sta	v_foevely,x
	cmp	#2
	bne	.goingup

.goingdown:	FRAME 16	; 7
	lda	v_foevely,x
	inc
	inc
	sta	v_foevely,x
	cmp	#22
	bne	.goingdown

	stz	v_foevely,x
	stz	v_foevelx,x
	lda	#$0505
	sta	v_foedelay,x
	STEP		; 8
	STEP		; 9
	STEP		; 10
	lda	#LION1
	sta	v_foetype,x
	jmp	.stand


@LionHit:
	stz	v_foevelx,x
	stz	v_foevely,x

	lda	#$3DF<<2
	sta	v_foey,x

	lda	#$0505
	sta	v_foedelay,x

	lda	#LION2
	sta	v_foetype,x

	FRAME 20	; 20/0
	STEP		; 21/1
	STEP		; 22/2
	STEP		; 23/3

	lda	v_bosshealth
	beq	.liondead

	STEP		; 24/4
	SAME		; 24/4
	SAME		; 24/4
	SAME		; 24/4
	SAME		; 24/4

	jsl	Random

	lsr
	bcc	.recover

	lda	#30	; extend invincibility
	sta	v_bossinvince

	FRAME 23	; 23/3
	FRAME 22	; 22/2
	STEP		; 23/3
	STEP		; 24/3
	SAME		; 24/3
	SAME		; 24/3
	SAME		; 24/3
	SAME		; 24/3
	SAME		; 24/3

.recover
	FRAME 21	; 21/1
	FRAME 20	; 20/0

	lda	#LION1
	sta	v_foetype,x

	jmp	.stand

.liondead:	
	lda	#-32
	jsr	AdjustY
	jmp	BossPoofAI


LionMonitor:
	WAIT
	lda	v_bossinvince
	beq	.notinvince
	dec	v_bossinvince
	bra	LionMonitor

.notinvince:
	lda	v_bosshealth
	cmp	v_bossoldhealth
	beq	LionMonitor

	sta	v_bossoldhealth

	lda	#LionHit
	sta	v_foeai+2

	lda	#30
	sta	v_bossinvince

	bra	LionMonitor



LionBodAI:
.sleep:	WAIT
	lda	v_bossactive	; wait for boss to wake up
	beq	.sleep
	stz	v_foedelay,x

	FORCE		; keep body in vram

.sync:	WAIT
	lda	v_foetype+2
	cmp	#LION1
	bne	.offscreen
	lda	v_foeframe+2
	cmp	#7
	bcs	.offscreen
	lda	v_foex+2
	sta	v_foex,x
	lda	v_foey+2
	sta	v_foey,x
	bra	.sync

.offscreen:	lda	#-1000
	sta	v_foex,x
	bra	.sync



*******************************************************************************
*
* Giant Toad
*
* v_foemisc1 = delay between licks
*
*******************************************************************************

ToadAI:
	stz	v_foemisc1,x	; zero delay

.wait:	RESET		; set for frame 0
	lda	v_foemisc1,x	; get delay value
	beq	.timerzero	; if 0, then go do more stuff
	dec		; else delay = delay - 1
	sta	v_foemisc1,x	; save the new value in delay storage
	bra	.wait	; loop back
.timerzero:	jsr	DistY	; get Y dist from Tzan
	cmp	#24	; is it w/in 24 qixels (6 pixels)?
	bcs	.nolick	; if not, go do something else
	jsr	DistX	; get X dist from Tzan
	cmp	#300	; is it w/in 300 qixels (75 pixels)?
	bcs	.nolick	; if not, go do something else
	jsr	FacingTzan	; else turn are we facing Tzan
	bcs	.lick	; if so, go to lick routine
	bcc	.hop	; else go to hop routine
.nolick:	jsl	Random	; get a random delay value
	cmp	#248	; is it 1 in 248?
	bcc	.wait	; if so go to wait routine

.hop:	FRAME	4	; set to frame 4 of frog anm
	STEP		; do frame 5 (hop)
	STEP		; do frame 6 (hop)
	STEP		; do frame 7 (hop)
	jsr	FlipH	; flip the frog 180
	bra	.wait	; go to wait routine

.lick:	lda	#10	; get lick delay value
	sta	v_foemisc1,x	; save it
	STEP		; set and show frame 1
	STEP		; do frame 2
	STEP		; do frame 3
	bra	.wait	; go to wait routine


*******************************************************************************
*
* Swamp Frog
*
* v_foemisc1 = frog appearance timer
*
*******************************************************************************

FrogAI:
	jsl	Random	; choose random delay value...

	and	#7
	clc
	adc	#5
	sta	v_foemisc1,x

.wait:
	WAIT		; count down till frog appearance...

	dec	v_foemisc1,x
	bne	.wait

	lda	#$390<<2	; position frog just below surface and
	sta	v_foey,x	; dma frame 0...
	
	stz	v_foeframe,x

	FORCE

	lda	#$36A<<2	; set starting y coordinate
	sta	v_foey,x

	jsl	Random	; choose x coordinate from list...

	and	#$000E
	tay

	lda	v_playerx
	cmp	#$400<<2
	bcs	.righthalf

	lda	d_frogx,y

	bra	.setx
.righthalf:	
	lda	d_frogx+8,y
.setx:	
	sta	v_foex,x

	jsl	Random	; randomly flip animation

	lsr
	bcc	.sameflip

	jsr	FlipH
.sameflip:
	SAME		; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6

	lda	#$0FFF<<2
	sta	v_foex,x

	jmp	FrogAI


*******************************************************************************
*
* Dragonfly
*
* frame 0 = fly frame 0
* frame 1 = fly frame 1
* frame 2 = turn
*
*******************************************************************************

Dragonmotion	=	16
Attackspeed	=	20
BottomEdge	=	2688
RightEdge	=	8000
Top_Left	=	256

DragonflyAI:
	lda	#128
	sta	v_foeaux3,x	; delay counter
	lda	v_foemode,x	; are we in attack mode?
	bne	.via
.mainloop
	RESET
	STEP
	dec	v_foeaux3,x
	beq	.attack	; if done hovering, then attack Tzan
	jsr	FacingTzan	; if we're not facing Tzan...
	bcc	.mainloop	; then keep hovering
	jsr	DistY	; get y dist from Tzan
	cmp	#256	; is he 64 pixels away?
	bcs	.mainloop
	jsr	DistX	; get x dist from Tzan
	cmp	#300	; is he 75 pixels away?
	bcs	.mainloop
.via	bra	.newpos
.attack	jsr	DistX
	cmp	#600	; is the player 150 x pixels away?
	BGE	DragonflyAI
	jsr	DistY
	cmp	#600	; is the player 150 y pixels away?
	BGE	DragonflyAI
	lda	#1
	sta	v_foemode,x	; set attack mode
	lda	v_playercentrx	; get Tzan's centr x coord
	asl
	asl
	sta	v_foeaux1,x	; save x coord dest
	cmp	v_foex,x	; where is it in relation to Tzan?
	BLT	.atleft
	lda	#Attackspeed	; move rite vel
	bra	.saveatx
.atleft	lda	#-Attackspeed	; move left vel
.saveatx	sta	v_foevelx,x
	sta	v_foemisc1,x
	lda	v_playercentry	; repeat for y vectors
	asl
	asl
	sta	v_foeaux2,x
	cmp	v_foey,x
	BLT	.atdown
	lda	#Attackspeed
	bra	.saveaty
.atdown	lda	#-Attackspeed
.saveaty	sta	v_foevely,x
	sta	v_foemisc2,x
	lda	v_foeoff,x
	and	#$4000	; mask off all bits except flip
	beq	.atpt_lft
	lda	v_foevelx,x	; foe facing rite
	bmi	.atturn
	bra	.newpos
.atpt_lft	lda	v_foevelx,x	; move right? (A>0)
	bmi	.vector	; if not, turn the foe
.atturn	FRAME 2		; turn around frame
	jsr	FlipH	; flip the foe 180
.vector	brl	.fly
.newpos	jsl	Random	; get a new x coord
	and	#127
	cmp	#64
	BLT	.newpos
	asl
	asl
	sta	v_foeaux1,x
	jsl	Random
	lsr
	bcc	.rite
	lda	#-Dragonmotion
	sta	v_foevelx,x	; set vel for moving left
	sta	v_foemisc1,x
	lda	v_foex,x
	sec
	sbc	v_foeaux1,x	; minus the dffrnc of the new coord
	cmp	#Top_Left
	BLT	.newpos
	bra	.savex
.rite	lda	#Dragonmotion
	sta	v_foevelx,x	; set vel for moving right
	sta	v_foemisc1,x
	lda	v_foex,x
	clc
	adc	v_foeaux1,x	; add the sum of the new coord
	cmp	#RightEdge	; right edge of level
	BGE	.newpos
.savex	sta	v_foeaux1,x	; save the final x coord
.anothery	jsl	Random	; get a new y coord
	and	#127
	cmp	#64
	BLT	.anothery
	asl
	asl
	sta	v_foeaux2,x
	jsl	Random
	lsr
	bcc	.down
	lda	#-Dragonmotion
	sta	v_foevely,x	; set vel for moving up
	sta	v_foemisc2,x
	lda	v_foey,x
	sec
	sbc	v_foeaux2,x	; minus the dffrnc of the new coord
	cmp	#Top_Left
	BLT	.anothery
	bra	.savey
.down	lda	#Dragonmotion
	sta	v_foevely,x	; set vel for moving down
	sta	v_foemisc2,x
	lda	v_foey,x
	clc
	adc	v_foeaux2,x	; add the sum of the new coord
	cmp	#BottomEdge	; bottom edge of level
	BGE	.anothery
.savey	sta	v_foeaux2,x	; save the final y coord
	stz	v_foemode,x	; no attack mode
	lda	v_foeoff,x
	and	#$4000	; mask off all bits except flip
	beq	.pt_lft
	lda	v_foevelx,x	; foe facing rite
	bmi	.turn
	bra	.fly
.pt_lft	lda	v_foevelx,x	; move right?
	bmi	.fly	; if not, turn the foe
.turn	FRAME 2		; turn around frame
	jsr	FlipH	; flip the foe 180
.fly	RESET
	STEP
	stz	v_foeaux3,x	; 0 flg 4 looping 'til final x,y
	stz	v_foevelx,x	; set horiz motion to nil
	lda	v_foemisc1,x
	bmi	.lft	; are we moving left or right?
	lda	v_foex,x	; moving right
	cmp	v_foeaux1,x	; if rite, have we passed end coord?
	BLT	.setflgx
.viax	lda	v_foeaux1,x	; end the end coord
	sta	v_foex,x	; force the final coord
	bra	.nexty
.lft	lda	v_foex,x
	cmp	v_foeaux1,x	; if left, have we passed end coord?
	beq	.nexty	; are we at the end x coord?
	BGE	.setflgx	; we haven't reached the end x coord
	bra	.viax	; we've passed the end x coord
.setflgx	inc	v_foeaux3,x	; set flag - final x not passed
	lda	v_foemisc1,x
	sta	v_foevelx,x	; set horiz motion to moving
.nexty	stz	v_foevely,x	; set vert motion to nil
	lda	v_foemisc2,x
	bmi	.up	; are we moveing up or down?
	lda	v_foey,x
	cmp	v_foeaux2,x	; if down, have we passed end coord?
	BLT	.setflgy
.viay	lda	v_foeaux2,x
	sta	v_foey,x
	bra	.hover
.up	lda	v_foey,x
	cmp	v_foeaux2,x	; are we at the final y coord?
	beq	.hover	; are we at the end y coord?
	BGE	.setflgy	; we haven't reached the end y coord
	bra	.viay	; we've passed the end y coord
.setflgy	inc	v_foeaux3,x	; set flag - final y not found
	lda	v_foemisc2,x
	sta	v_foevely,x	; set vert motion to moving
.hover	lda	v_foeaux3,x	; is the loop flag set?
	bne	.fly	; if so, the
	jmp	DragonflyAI

*******************************************************************************
*
* FloortrigAI
*
* frame 0 = floor trigger up  frame 0
* frame 1 = floor trigger dwn frame 1
*
*******************************************************************************

FloortrigAI:
.collide
;	lda	v_flrtrig1
;	cmp	#1
;	beq	.trigplate

	FRAME 0

	bra	.collide

.trigplate
	FRAME 1

	bra	.collide


*******************************************************************************
*
* EOLevelAI
*
*******************************************************************************

EOLevelAI:
;	lda	z_levelx
;	sec
;	sbc	#32

	stz	v_foex,x

;	lda	z_levely
;	clc
;	adc	#240
	
	stz	v_foey,x
.eolloop
	WAIT

	phx		; Save foe table index

	ldx	z_levelword	; Get current level
	lda	>d_numcoinsget,x ; Get num coins required for bonus

	plx

	cmp	v_coincur	; Get actual num coins collected
	bcc	.eolloop	; Sorry, not enough to trigger EOL

.chk4boss			; Got coins now, But check for boss
	lda	v_foeaux1,x	; Is this a boss level?
	beq	.plotcoords	; No  - Then process EOL animation

	lda	z_levelx	; Otherwise
	sec
	sbc	#32
	sta	v_foex,x

	lda	z_levely
	clc
	adc	#240
	sta	v_foey,x
.bossalive
	WAIT

	lda	z_level
	and	#$FF
	cmp	#CLEARING
	bne	.notapefight

	lda	v_apebossdead
	bne	.plotcoords
.notapefight
	lda	v_bossdead
	bpl	.bossalive
.plotcoords
	lda	v_foeaux2,x
	sta	v_foex,x

	lda	v_foeaux3,x
	sta	v_foey,x

	RESET

	lda	#1
	sta	v_foemode,x

.notaboss
	lda	v_foeplot,x
	bne	.offscreen

	lda	#1
	sta	v_cyclface

	FRAME 0

	jsr	DistY

	cmp	#100<<2
	bcs	.notaboss

	jsr	DistX

	cmp	#100<<2
	bcs	.notaboss

	jsr	FacingTzan

	bcs	.janesface

	jsr	FlipH

	bra	.janesface

.janesface
	STEP		; 1
	STEP		; 2
.adj
;	ldy	#paff_eff
;
;	jsl	SoundEffect

	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
.wait
	WAIT

	jsr	FacingTzan

	bcs	.wait

	jsr	FlipH

;	SAME
	FRAME 2

	bra	.adj

.offscreen
	stz	v_cyclface

	WAIT

	bra	.notaboss

.exit
	rtl

*******************************************************************************
*
* Lamp
*
* frame 0 = Sits on floor
* frame 1 = Starts to fall off edge
* frame 2 = Falling down shaft
* frame 3 = Crushed at bottom
*
* All frames color cycle the 1st 5 colors of the palette. The code that does
* this magic feat, is located in specials.asm
*
*******************************************************************************

TZLampAI:
	lda	v_foeaux3,x	; is the lamp broken?
	bne	.broken
	lda	v_foevely,x	; is the lamp moving?
	bne	.fall
	FRAME 0		; show a still lamp
	bra	TZLampAI
.broken	FRAME 4		; show a broken lamp
	bra	.broken
.fall	inx		; adjust foe index for flicker
	inx
	sta	v_foevely,x	; set flicker y vel same as lamp
	dex
	dex
	lda	v_foevelx,x	; get lamp x vel
	inx
	inx
	sta	v_foevelx,x	; set flicker x vel
	dex
	dex
	FRAME 1		; show a moving lamp
	lda	v_foeoff,x
	eor	#$4000	; flip the lamp around
	sta	v_foeoff,x
	FRAME 1
	lda	#9
	sta	v_foemisc1,x	; loop counter
	stx	v_foemode	; save x index for flash explosion
.fallloop	FRAME 2
	FRAME 3		; show tumbling lamp
	lda	v_foeoff,x
	eor	#$4000	; tumble lamp around
	sta	v_foeoff,x
	lda	v_foevelx,x	; get lamp x vel
	bmi	.left	; if < 0, then moving left
	lda	v_foex,x	; has lamp passed final x coord?
	cmp	v_foeaux1,x
	BLT	.loopmore	; if not, go do more
.noxvel	stz	v_foevelx,x	; if so, 0 lamp x vel (no horiz vel)
	inx
	inx
	stz	v_foevelx,x	; do the same for the flicker
	dex
	dex
.loopmore	dec	v_foemisc1,x	; counter = counter - 1
	bne	.fallloop	; if not 0, then go do more
	beq	.smashlamp	; else, go smash lamp on bottom
.left	lda	v_foex,x
	cmp	v_foeaux1,x	; has lamp passed final x coord?
	beq	.noxvel	; if x coord = final, then set x vel
	BGE	.loopmore	; if haven't reached x coord, do more
	bra	.noxvel	; go set x vel to 0 (no horiz vel)
.smashlamp	lda	v_foeaux1,x	; get final x coord
	sta	v_foex,x	; set lamp final x coord
	inx
	inx
	sta	v_foex,x	; set flicker final x coord
	dex
	dex
	lda	v_foeaux2,x	; get final y coord
	sta	v_foey,x	; set lamp final y coord
	inx
	inx
	sec
	sbc	#32
	sta	v_foey,x	; set flicker final y coord
	dex
	dex
	lda	#1
	sta	v_foeaux3,x	; set flag to indicate broken lamp
	stz	v_foevelx,x
	stz	v_foevely,x	; set lamp x & y vel to 0
	inx
	inx
	stz	v_foevelx,x
	stz	v_foevely,x	; set flicker x & y vel to 0
	dex
	dex
	brl	TZLampAI

*******************************************************************************
*
* Lamp light flickering routine
*
* Cycles routinely.  Will move with lamp if lamp gets hit.
*
*******************************************************************************

FlickerAI:
	RESET
	STEP
	STEP
	STEP

	bra	FlickerAI


*******************************************************************************
*
* Lamp explosion light if lamp hits bottom
*
*******************************************************************************

FlashAI:

	dex
	dex
	dex
	dex

	lda	v_foevely,x	; get the lamp y vel

	inx
	inx
	inx
	inx

	cmp	#0
	bne	.exit	; if lamp moving, skip flash

	dex
	dex
	dex
	dex

	lda	v_foey,x	; get the current y coord

	inx
	inx
	inx
	inx

	cmp	v_foeaux2,x	; are we at the final y coord?
	bne	.exit	; if not, skip flash

	RESET

	ldy	#crashcav_eff

	jsl	SoundEffect

	STEP
	STEP
	STEP
	TERM

.exit
	WAIT

	bra	FlashAI


*******************************************************************************
*
* WatertopAI
*
*******************************************************************************

WatertopAI:
	RESET		; Frame 0
	STEP		; Frame 1
	STEP		; Frame 2
	STEP		; Frame 3

	bra	WatertopAI


*******************************************************************************
*
* WaterdoorAI
*
* frame 0 =
* frame 1 =
* frame 2 =
* frame 3 =
* frame 4 =
* frame 5 =
*
*
*******************************************************************************

WatrdoorAI:
	RESET
	STEP
	STEP
	STEP
	STEP

	bra	WatrdoorAI


*******************************************************************************
*
* BushtrapAI
*
*******************************************************************************

BushtrapAI:
	stz	v_foeplot,x	; assume onscreen
	SAME
.wait	WAIT
	bra	.wait


*******************************************************************************
*
* Jungle pit opening
*
*******************************************************************************

PitbotAI:
PittopAI:
	FIRST

	lda	v_foemode,x	; Get pit status
	beq	PittopAI	; Frame 0 if closed

	STEP		; frame 1
	STEP		; frame 2

	lda	#2
;	ora	v_foemode,x	; Get pit status
	sta	v_foemode,x	; Get pit status

	STEP		; frame 3
	STEP		; frame 4
	STEP		; frame 5

.openpit
	FRAME 5
	bra	.openpit


*******************************************************************************
*
* Leaves billowing from jungle pit
*
*******************************************************************************

LeavesAI:
	dex		; Ajust to coresponding pit
	dex
	dex
	dex

	lda	v_foemode,x	; Get pit status
	inx
	inx
	inx
	inx

	cmp	#2	; Pit in 2nd frame of opening?
	bcc	.waitloop	; No  - Wait then

	beq	.dostep
.waitloop
	FRAME 11
	bra	LeavesAI

.dostep
	RESET		; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
	STEP		; 7
	STEP		; 8
	STEP		; 9
	STEP		; 10
	STEP		; 11

.exit	
	WAIT
	bra	.exit


*******************************************************************************
*
*
*
*******************************************************************************

ReefAirAI:
	RESET
	STEP
	STEP
	STEP

	bra	ReefAirAI

	.dbreg	$00


*******************************************************************************
*
*
*
*******************************************************************************

LesserApeAI:
	FRAME 14	; Frame 1
	FORCE
.wait4tzan
	WAIT

	jsr	FacingTzan

	bcc	.chkvertical

	jsr	FlipH
.chkvertical
	jsr	DistY

	cmp	#10<<2
	bcs	.wait4tzan

	jsr	DistX

	cmp	#50<<2
	bcs	.wait4tzan

	jsr	FacingTzan

	bcs	.slapimsilly

	jsr	FlipH

.slapimsilly
	WAIT

	jsl	Rndnum

	and	#%00100000
	beq	.slapimsilly

	STEP		; Frame 2
	STEP		; Frame 3
	STEP		; Frame 4
	lda	#1
	sta	v_foepriority,x
	jsr	.doslapsfx
	STEP		; Frame 5
	lda	#1
	sta	v_foepriority,x
	jsr	.doslapsfx
	STEP		; Frame 6
	lda	#1
	sta	v_foepriority,x
	jsr	.doslapsfx
	STEP		; Frame 7
	lda	#1
	sta	v_foepriority,x
	jsr	.doslapsfx
	STEP		; Frame 8
	stz	v_foepriority,x
	STEP		; Frame 9
	STEP		; Frame 10
	STEP		; Frame 11
	STEP		; Frame 12

	jmp	LesserApeAI

.doslapsfx
	lda	v_slapos
	bne	.justret

	stz	v_slapos
	ldy	#punch1_eff

	jsl	SoundEffect
.justret
	rts
