*****************************************************************************
*
*  HDMA.ASM
*
*****************************************************************************

*****************************************************************************
*
* Setup Down River HDMA River Surface Scrolling
*
*****************************************************************************
*

d_Dnrivbg1off	.dw	0	; Offset to 1st original entry
	.dw	3
	.dw	6
	.dw	9
	.dw	12
	.dw	15
	.dw	18

Dnrivtblval1	.dw	$04	; Original table value
Dnrivtblval2	.dw	$04
Dnrivtblval3	.dw	$0B
Dnrivtblval4	.dw	$21
Dnrivtblval5	.dw	$10

****************************************
*
* Enter with reg y containing the offset 
* in the source table
*
****************************************

Dnrivhdmasetup:
	SAI

	ldx	#0
.loop
	lda	d_dnrivbg1a,y	; Copy initial table over to ram
	sta	v_dnrivtabl,x

	inx
	iny

	cpy	#15	; Last address byte + 1?
	bcc	.loop	; No  - continue table transfer

	stz	v_dnrivtabl,x	; Terminate table with a zero

	LAI

	rtl


****************************************
*
* Store scrolled values
*

Dnrivroffset:	
	LAI

	lda	z_hoff1	
	sta	v_dnrivoff	; Store in indirect table loc 1

	lda	v_dnrivhoff
	sta	v_dnrivoff+2	; Store in indirect table loc 2

	lda	v_dnrivhoff+2
	sta	v_dnrivoff+4	; Store in indirect table loc 3

	lda	v_dnrivhoff+4
	sta	v_dnrivoff+6	; Store in indirect table loc 4

	lda	z_hoff1
	sta	v_dnrivoff+8	; Store in indirect table loc 5

	rtl


****************************************
*
* Adjust horizontal position of each
* successive line group for parallax 
* effect.
*
****************************************

Dnrivscroll:
	LAI

	lda	z_h1	; Get 1/4 pixal x
	clc
	adc	v_dnrivincval	; Add an increment value
	tax		; Save intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save thsi intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+2	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+4	; Store

	lda	z_h1	; Get 1/4 pixal x
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+8	; Store

	rtl


****************************************
*
* Programmed once during VBLANK before
* level loop
*

Dnrivrdma:
	SAI

	lda	#0
	sta	z_hdmaenable	; Start with no channels enabled 

	ldy	#$40	; Offset from 1st channel to current channel
.loop      	
	lda	d_dnrivinc,x
	sta	DMA0INC,y	; HDMA parameter (# of bytes, indirect?)

	phx
	txa

	asl	
	tax		; Double x for word index

	lda	d_dnrivdest,x
	sta	DMA0DEST,y

	plx		; Restore byte index x

	lda	#0
	sta	DMA0BANK,y	; Bank of dest address

	lda	#0	;
	sta	DMA0A1B,y

	lda	d_dnrivlow,x	
	sta	DMA0A1L,y	; Pointers to hdma tables

	lda	d_dnrivhigh,x
	sta	DMA0A1H,y

	tya
	clc
	adc	#$10	; Move Y to next channel
	tay

	lda	z_hdmaenable
	ora	d_dnrivmask,x	; Enable current channel
	sta	z_hdmaenable

	inx
	cpx	#1	; Just 1 channel for now

	bcc	.loop

	LAI

	rtl


*****************************************************************************
*

d_dnrivinc
	.db	$42	; Indirect addressing, Write twice
			; ...address - L/H
d_dnrivdest
	.dw	BG1HOFS
d_dnrivlow
	.db	(v_dnrivtabl & $FF)
d_dnrivhigh
	.db	(v_dnrivtabl >> 8)


*
* The table v_dnrivtabl has the following format after  
* initialization
*
* v_dnrivtabl:
*
*	.db	(Num Lines)
*	.dw	(address of data for BGxHOFS register)
*	.db	(Num Lines)
*	.dw	(address of data for BGxHOFS register)
*	.db	(Num Lines)
*	.dw	(address of data for BGxHOFS register)
*	.db	(Num Lines)
*	.dw	(address of data for BGxHOFS register)
*	.db	(Num Lines)
*	.dw	(address of data for BGxHOFS register)
*	.db	0	; End of table

d_dnrivbg1a
	.db	$04
	.dw	(v_dnrivoff)	; Top
d_dnrivbg1b
	.db	$04
	.dw	(v_dnrivoff+2)	; Far Bank
d_dnrivbg1c
	.db	$0B
	.dw	(v_dnrivoff+4)	; Mid Section
d_dnrivbg1d
	.db	$21
	.dw	(v_dnrivoff+6)	; Cut away of wave
d_dnrivbg1e
	.db	$01
	.dw	(v_dnrivoff+8)	; Stop it!
	.db	0
d_dnrivmask
	.db	$10,$20,$40,$80


*****************************************************************************
*****************************************************************************
*****************************************************************************
*
* Beach Level
*
*
****************************************
*
* Enter with reg y containing the offset 
* in the source table
*
****************************************

Beachdmasetup:
	SAI

	ldx	#0
.loop
	lda	d_beachbg2a,y	; Copy initial table over to ram
	sta	v_dnrivtabl,x

	inx
	iny

	cpy	#21	; Last address byte + 1?
	bcc	.loop	; No  - continue table transfer

	stz	v_dnrivtabl,x	; Terminate table with a zero

	LAI

	rtl


****************************************
*
* Store scrolled values
*

Beachoffset:	
	LAI

	lda	z_hoff2	
	sta	v_dnrivoff	; Store in indirect table loc 1

	lda	v_dnrivhoff
	sta	v_dnrivoff+2	; Store in indirect table loc 2
	sta	v_dnrivoff+10	; Store in indirect table loc 4

	lda	v_dnrivhoff+2
	sta	v_dnrivoff+4	; Store in indirect table loc 3

	lda	v_dnrivhoff+4
	sta	v_dnrivoff+6	; Store in indirect table loc 4

	lda	v_dnrivhoff+6
	sta	v_dnrivoff+8	; Store in indirect table loc 4

	lda	v_dnrivhoff+8
	sta	v_dnrivoff+10	; Store in indirect table loc 4

;	lda	z_hoff2
;	sta	v_dnrivoff+10	; Store in indirect table loc 5

	rtl


****************************************
*
* Adjust horizontal position of each
* successive line group for parallax 
* effect.
*
****************************************


Beachscroll1:
	jmp	Beachscroll2

	LAI

	lda	z_h2	; Get 1/4 pixal x
	sec
	sbc	v_dnrivincval	; Add an increment value
	lsr
	tax		; Save intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff	; Store

	txa		; Get intermediate back
	sec
	sbc	v_dnrivincval	; Again add the incremental
	lsr
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+2	; Store


	txa		; Get intermediate back
	sec
	sbc	v_dnrivincval	; Again add the incremental
	lsr
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+4	; Store

	txa		; Get intermediate back
	sec
	sbc	v_dnrivincval	; Again add the incremental
	lsr
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+6	; Store

	txa		; Get intermediate back
	sec
	sbc	v_dnrivincval	; Again add the incremental
	lsr
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+8	; Store

	txa		; Get intermediate back
	sec
	sbc	v_dnrivincval	; Again add the incremental
	lsr
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+10	; Store

	lda	z_h2	; Get 1/4 pixal x
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+10	; Store

	rtl


Beachscroll2:
	LAI

	lda	z_h2	; Get 1/4 pixal x
	clc
	adc	v_dnrivincval	; Add an increment value
	tax		; Save intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+2	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+4	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+6	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+8	; Store

	txa		; Get intermediate back
	clc
	adc	v_dnrivincval	; Again add the incremental
	tax		; Save this intermediate
	lsr		; Scale
	lsr   
	sta	v_dnrivhoff+10	; Store

;	lda	z_h2	; Get 1/4 pixal x
;	lsr		; Scale
;	lsr   
;	sta	v_dnrivhoff+10	; Store

	rtl


****************************************
*
* Programmed once during VBLANK before
* level loop
*

Beachdma:
	SAI

	lda	#0
	sta	z_hdmaenable	; Start with no channels enabled 

	lda	d_beachinc
	sta	DMA4INC	; HDMA parameter (# of bytes, indirect?)

	lda	d_beachdest
	sta	DMA4DEST

	lda	#0
	sta	DMA4BANK	; Bank of dest address

	lda	#0	;
	sta	DMA4A1B

	lda	d_beachlow
	sta	DMA4A1L	; Pointers to hdma tables

	lda	d_beachhigh
	sta	DMA4A1H

	lda	z_hdmaenable
	ora	#$10
	sta	z_hdmaenable

	LAI

	rtl


*****************************************************************************


d_beachinc
	.db	$42	; Indirect addressing, Write twice
			; ...address - L/H
d_beachdest
	.dw	BG2HOFS
d_beachlow
	.db	(v_dnrivtabl & $FF)
d_beachhigh
	.db	(v_dnrivtabl >> 8)


d_beachbg2a
	.db	96
	.dw	(v_dnrivoff)	; Top
d_beachbg2c
	.db	8
	.dw	(v_dnrivoff+2)	; Far Section
d_beachbg2d
	.db	8
	.dw	(v_dnrivoff+4)	; Far Section
d_beachbg2e
	.db	16
	.dw	(v_dnrivoff+6)	; Far Wave
d_beachbg2f
	.db	15
	.dw	(v_dnrivoff+8)	; Near Wave
d_beachbg2g
	.db	15
	.dw	(v_dnrivoff+10)	; Beach
d_beachbg2h
	.db	0
	.dw	(v_dnrivoff+12)	; Stop it!

	.db	0


d_beachmask
	.db	$10,$20,$40,$80


*****************************************************************************

MAPA	.equ	1
MAPB	.equ	2
MAPC	.equ	3

d_maplow
	.db	0
	.db	(d_maptabla & $FF)
	.db	(d_maptablb & $FF)
	.db	(d_maptablc & $FF)
d_maphigh
	.db	0
	.db	(d_maptabla >> 8)
	.db	(d_maptablb >> 8)
	.db	(d_maptablc >> 8)

d_maptabla			; Jungle 1
	.db	42	; Tarzans Tree
	.db	00	; Apes Clearing
	.db	100	; Reef
	.db	255	; Ship
	.db	100
	.db	255
	.db	0

d_maptablb			; Swamp
	.db	24	; Beach
	.db	00
	.db	100
	.db	255
	.db	100
	.db	255
	.db	0

d_maptablc			; Jungle 3
	.db	100	; Downriver
	.db	255	; Jungle 4
	.db	100	; Upriver
	.db	255	; Waterfall
	.db	54	; Caverns
	.db	255	; Opar
	.db	0	; Bonus Level - Exterior



d_win1rdef
	.dw	NONE	; Null Level
	.dw	MAPA	; Jungle 1
	.dw	MAPA	; Tarzans Home
	.dw	MAPA	; Apes Clearing
	.dw	MAPB	; Swamp

	.dw	MAPA	; Beach
	.dw	NONE	; Beach-Bay
	.dw	MAPA	; Reef
	.dw	MAPA	; Ship
	.dw	NONE	; Ship Hold A

	.dw	NONE	; Ship Hold B
	.dw	MAPB	; Jungle 3
	.dw	MAPC	; Downriver
	.dw	MAPC	; Jungle 4
	.dw	MAPC	; UpRiver

	.dw	MAPC	; Waterfall
	.dw	MAPC	; Caverns
	.dw	MAPC	; Opar - Street
	.dw	MAPC	; Opar - Wells
	.dw	MAPC	; Opar - Temple

	.dw	MAPC	; Opar - Shamon

	.dw	MAPC	; Bonus Level - Exterior
	.dw	NONE	; Bonus Level - Interior


*****************************************************************************
*
* Game Over Roll Credits....
*
*****************************************************************************

d_gmovrlow
	.db	(d_gmovrtabla & $FF)
d_gmovrhigh
	.db	(d_gmovrtabla >> 8)

d_gmovrtabla		   
	.db	16 
	.db	0 
	.db	100
	.db	255
	.db	92
	.db	255
	.db	16
	.db	0
	.db	0


*****************************************************************************
*
* Ship Holds - Ceiling mask - left side
*
*****************************************************************************

d_holdsllow
	.db	(d_holdsltabla & $FF)
d_holdslhigh
	.db	(d_holdsltabla >> 8)

d_holdsltabla		   
	.db	16 
	.db	0 
	.db	100
	.db	255
	.db	92
	.db	255
	.db	16
	.db	56
	.db	0



*****************************************************************************
*
* Ship Holds - Ceiling mask - right side
*
*****************************************************************************

d_holdsrlow
	.db	(d_holdsrtabla & $FF)
d_holdsrhigh
	.db	(d_holdsrtabla >> 8)

d_holdsrtabla		   
	.db	16 
	.db	255 
	.db	100
	.db	255
	.db	92
	.db	255
	.db	16
	.db	200
	.db	0

*****************************************************************************
*
* Get Password from Player
*
*****************************************************************************

d_tpasslow
	.db	(d_tpasstabla & $FF)
d_tpasshigh
	.db	(d_tpasstabla >> 8)

d_tpasstabla		   
	.db	16 
	.db	0 
	.db	100
	.db	255
	.db	92
	.db	255
	.db	16
	.db	0
	.db	0



