;                             JUNE 22, 1993
;
; setup of regular music driver:
;
; 1. using SFL download the *.o file that Wolfgang generated. This *.o file
;    contains all music drivers & data. You will not need to download
;    anything else. IMPORTANT: Music data must be within the first 64
;    banks of rom.(DO NOT USE ANY OLD MUSIC DRIVERS!!!)
;
; 2. include the music driver constants,variables & routines that you
;    will need into your code. (SEE variables below)
;
; 3. make sure to include the *.asm file into your code that Wolfgang
;    generated.
;
; 4. clear all ports,set MODULE=$ffff and call Load_Driver
;
; 5. to play effects call load_Effect (load "A" reg with
;     effect id)
;
; 6. to play song call load_Song (load "A" reg with song id)
;
; 7. Remember if you call an effect or a song that is contained in a
;    module that is not yet in APU ram, the the load_Module routine will
;    be called (when using load_Effect & load_Song), so you need to make
;    sure you are not blasting during these cases.
;
; 8. If you experience problems with the Driver Initialisation, try
;    downloading the common module immediately after the driver. For more
;    information on how to do this, see the commented-out code in the
;    Load_Driver routine.
;
;                        Steve Aguirre
;                        Sculptured Software


;************************************************
; NEW SINCE LAST (MAY 20th) version:
; 1.
; A long outstanding bug in the music driver has been
; fixed that previously we worked around(by timing out).  The new
; blasting techniques were less forgiving and so we had to
; fix the bug in the driver.
;
; 2.
; The Pass_Driver has had some new code added.
;
; 3.
; A new variable "BON" has been included.
;************************************************
;************************************************
;
; routine formats:
;  Load_Driver      (long call)    ;initialize driver
;  Pass_DriveLong   (long call)
;  load_Song        (long)         ;start a song
;  load_Module (long)         ;load a module
;  load_Effect (long)         ;start an effect
;  get_module_number(long)         ;get module # for song or effect
;  APU_Upload  (short call)
;  APU_Upload_Ind0  (short call)   ;upload data block to APU
;  Pass_Driver_Command (short)     ;send drive a command
;  set_module_pointer (short)
;
;************************************************
;************************************************
;
; music driver routines
;
; by Steve Aguirre, Dave Ross, & Ricky Loynd
;
;************************************************


*
* zero page variables needed
*
;APU_Ind0      ds   3    ;in direct page
;APU_SN        ds   1    ;can be anywhere in RAM.
;APU_NextLen        ds   2    ;in direct page
;APU_Temp      ds   2    ;in direct page
;ModTabInd          ds   3    ; Indirect location of a module table

*
* regular variables needed
*
;ModTabSize         ds   2    ; Number of entries in the module table
;ModTabPtr          ds   2    ; Save Y index for current module table entry
;MODULE        ds   2    ; (needed only if load_Module is used)
;SONGNUMBER         ds   2    ; (needed only if load_Song is used)
;EFFECTNUMBER  ds   2    ; (needed only if load_Effect is used)
;BON           ds   2    ;blasting on/off (not available with this version)

	;
	; rom locations of music data
	;


	;***NOTE: with each *.o file that WOLFGANG creates, there is 
	;also an *.asm file; include that file here
	;.include 'filename.asm'

		 .include    'tarzan.asm'


BOOT700_CODE        EQU  MusicBootAddress
DRIVER_CODE         EQU  MusicDriverAddress
NUM_LM              EQU  MusicTableAddress
NUM_SONGS           EQU  MusicTableAddress+1
NUM_EFFECTS         EQU  MusicTableAddress+2
MODULE_ADR          EQU  MusicTableAddress+3

*************************************************
*
* MUSIC DRIVER ROUTINES
*
*************************************************

*
* LOADS INITIAL MUSIC DRIVER & DATA
*
Load_Driver:
	.if MUSICOK

	php
	LAI
	sei

	;delay before we download (after we power on)
	; you may not need this delay if enough processing
	; time has elapsed before you call this routine

	ldx  #$1000
.power_up
	dex
	bpl  .power_up

	;download APU init code & music Driver
	ldx  #(BOOT700_CODE & $FFFF)
	lda  #(BOOT700_CODE / $10000)
	jsl  APU_Upload
	ldx  #(DRIVER_CODE & $FFFF)
	lda  #(DRIVER_CODE / $10000)
	jsl  APU_Upload

	; if you are experiencing music/timing problems @
	; startup, try accessing the following code to see if it
	; fixes the problem. (It is safe to add anyway if you want to)
	; send over common

	lda  #DCOM_RELOAD
	jsl  Pass_Driver_Command
	lda  >MODULE_ADR
	tax
	lda  >MODULE_ADR+2
	and  #$ff
	jsl  APU_Upload_Ind
	stz  MODULE

	;start driver
	lda  #DCOM_RUN
	jsl  Pass_Driver_Command

	plp

	.endif

	rtl

*
* This routine uses the new WOLFGANG module loading procedure
* (Using this format will save cart ROM space)
* Use this procedure only on music modules (COMMON_MODULE,
* LOAD_MODULES,etc.) ONLY!!
*
* THIS CODE UPLOADS A SET OF BLOCKS TO THE APU RAM
* X REG=ADDRESS, A=BANK
*
APU_Upload_Ind
	.if MUSICOK

	;
	;new change, turn off nmi's when using this routine
	;

	php
	SA
	LI
	sta  ModTabInd+2
	stz  ModTabInd+1
	stz  ModTabInd
	stx  ModTabPtr
	ldy  ModTabPtr
	LA
	lda  [ModTabInd],Y  ; get number of blocks
	sta  ModTabSize
	jsl  IncY
	jsl  IncY
	sty  ModTabPtr
	stz  APU_Ind0
	stz  APU_Ind0+1
	lda  #$BBAA         ; look for APU ready state
.wait0
	cmp  APUPORT0
	bne  .wait0
	SA
	lda  #$CC      ; load up first ready code

.Newblk
	LA
	dec  ModTabSize
	SA
	bmi  .done
	pha
	LA
	ldy  ModTabPtr
	lda  [ModTabInd],Y  ; get size of block
	bne  .nonz
	jsl  IncY
	jsl  IncY
	jsl  IncY
	jsl  IncY
	jsl  IncY
	jsl  IncY
	jsl  IncY
	sty  ModTabPtr
	SA
	PLA
	jmp  .Newblk

	.LONGA ON
.nonz     
	tax
	jsl  IncY
	jsl  IncY
	lda  [ModTabInd],Y  ; get source address of block
	pha
	jsl  IncY
	jsl  IncY
	SA
	lda  [ModTabInd],Y  ; get bank of block
	sta  APU_Ind0+2
	jsl  IncY
	LA
	lda  [ModTabInd],Y  ; get target address of block
	sta  APUPORT2
	jsl  IncY
	jsl  IncY
	sty  ModTabPtr
	ply
	SA
	lda  #1
	sta  APUPORT1  ; passes 1=more to do
	pla            ; recover ready code
	sta  APUPORT0  ; pass ready code and
.wait3
	cmp  APUPORT0  ; wait for acknowledge
	bne  .wait3

	jsl  PassBlock
	jmp  .Newblk

.done
	ldx  #$700
	stx  APUPORT2  ; jump vector
	pha
	lda  #0
	sta  APUPORT1  ; 0=finished
	pla
	sta  APUPORT0  ; pass ready code and
.wait4
	cmp  APUPORT0  ; wait for acknowledge
	bne  .wait4
	jmp  Done
*
* The following routine is used for BMUS type files
* and older WOLFGANG formats.
* This routine is also used to download BOOT700 & DRIVER_CODE
* which must use this load procedure.
*
* THIS CODE UPLOADS A BLOCK OF DATA TO THE APU RAM
* X REG=ADDRESS, A=BANK
APU_Upload
	php
	SA
	LI
	txy
	stz  APU_Ind0  ; store starting block address
	stz  APU_Ind0+1     ; store starting block address
	sta  APU_Ind0+2
	LA
	lda  #$BBAA         ; look for APU ready state
.wait0
	cmp  APUPORT0
	bne  .wait0
	SA
	lda  #$CC      ; load up next ready code
.Newblk 
	pha
	LA
	lda  [APU_Ind0],Y   ; get length of first block
	iny
	iny
	tax            ; save length in X
	lda  [APU_Ind0],Y   ; get address of first block
	iny
	iny
	sta  APUPORT2  ; store address in port 2
	SA
	cpx  #$0001         ; if more to do, carry will clear
	lda  #$00      ; get carry into accum.
	rol
	sta  APUPORT1  ; passes 1=more to do, 0= finished
	adc  #$7F      ; sets v if more to do
	PLA            ; recover ready code
	sta  APUPORT0  ; pass ready code and
.wait3
	cmp  APUPORT0  ; wait for acknowledge
	bne  .wait3
	bvc  Done      ; more to do?
	jsl  PassBlock
	jmp  .Newblk
Done
	stz  APUPORT0  ; clear ports
	stz  APUPORT1
	stz  APUPORT2  ; last item passed here was jump vector
	stz  APUPORT3
	stz  APU_SN         ; the new Serial Number - this is necessary
	ldx  #$07      ;PAUSE DELAY for APU
.1
	dex
	bpl  .1
	plp
	rtl

	;
	; This loop passes the bytes of the block to the APU
	;
	;      Size    Entry          Exit
	;  A   Short                  Last serial number
	;  X   Long    Block size     0
	;  Y   Long    Block address  Byte after block
	;
	;
	;  APU_Ind0+2  Start of bank
	   ;
PassBlock
	.LONGA OFF
	lda  [APU_Ind0],Y   ; get byte to pass
	iny
	bne  .within1

	;went over a bank boundary, switch banks
	inc  APU_Ind0+2     ;next bank
	ldy  #$8000         ;beginning of next bank
.within1
	xba            ; save it in b
	lda  #$00      ; a = 00  (serial number)
	bra  .stuffbyte     ; b = byte to pass

.anotherbyte
	xba            ; save old serial number
	lda  [APU_Ind0],Y   ; get new byte
	iny
	bne  .within2

	;went over a bank boundary, switch banks
	inc  APU_Ind0+2     ; next bank
	ldy  #$8000         ; beginning of next bank
.within2
	xba            ; b = new byte to pass
.wait1
	cmp  APUPORT0  ; a = old serial number
	bne  .wait1    ; wait for acknowledge of serial #
	inc            ; increment serial number
.stuffbyte
	LA
	sta  APUPORT0  ; Port1 gets the actual data byte
	SA
	dex            ; one less byte to pass
	bne  .anotherbyte

.wait2
	cmp  APUPORT0  ; wait for final acknowledge
	bne  .wait2
.skip0
	adc  #$03      ; carry set: adds 4 to serial number
	beq  .skip0    ; if 0, move on to 4
	rtl

IncY
	iny
	bne  .done
	inc  ModTabInd+2    ;next bank
	ldy  #$8000         ;beginning of next bank
.done
	.endif

	rtl

* THIS ROUTINE PASSES INSTRUCTIONS TO THE MUSIC DRIVER
* (WITH TIMEOUT& PORT RESET)
Pass_Driver_Com
Pass_Driver_Command

	.if MUSICOK

	PHP
	SEI
	SA
	LI
	inc  BON       ;disable any blasting temporarily
	XBA            ; save command in B
	LDX  #$3000
	LDA  APU_SN
.waitack
	CMP  APUPORT2  ; wait for acknowledge of serial
	BEQ  .skip0         ; number
	DEX
	BNE  .waitack
	LDA  APUPORT2
.skip0
	INC            ; INCrement serial number, skipping 0
	BEQ  .skip0
	STA  APU_SN         ;serial number goes into low byte
	LAI
	STY  APUPORT0  ; PLAce parameter on bus first
	STA  APUPORT2  ; then command and serial number
	dec  BON
	PLP

	.endif

	rtl


*************************************************
* END of main driver routines
* the following routines are optional
*************************************************


*************************************************
*
* new module load routines (not always needed)
*
*************************************************
*******************
*
* LOAD and PLAY A SONG
* entry parameters:
*    a=song id #
*
* example:
*    lda  #song_id       ;song_id given by wolfgang
*    jsl  load_Effect         ;normal effect
*
*
*******************
load_Song:
	.if MUSICOK

	LAI

	sta  SONGNUMBER

	;load a module for desired song if needed
	lda  >NUM_LM   ;total number of load modules
	and  #$ff
	inc            ;add 1 for common module
	sta  ModTabInd
	asl
	clc
	adc  ModTabInd ;index past MODULE_ADR table (SONG_MOD table)
	clc
	adc  #MODULE_ADR & $FFFF
	clc
	adc  SONGNUMBER     ;look up module for this song
	sta  ModTabInd
	SA
	lda  #MODULE_ADR >> 16
	sta  ModTabInd+2    ;bank
	LA
	lda  [ModTabInd]    ;module value
	and  #$ff
	jsl  load_Module    ;load module if needed

	;play song
	ldy  SONGNUMBER     ;SONG NUMBER TO PLAY
	lda  #DCOM_START_SONG
	jsl  Pass_Driver_Com     ;PLAY wolfgang

	.endif

	rtl


*******************
*
* LOAD and PLAY AN EFFECT
* entry parameters:
*    a=sound effect #
*
* example:
*    lda  #effect        ;effect given by wolfgang
*    jsl  load_Effect         ;normal effect
*
*******************
load_Effect:
	.if MUSICOK

	LAI

	sta  EFFECTNUMBER        ;save effect number for later use

	lda	v_inoptions	; In options mode?
	bne	.skipchk	; Yes -

	lda	v_contrlmap
	bit	#4	; 1=sound effects turned off
	bne	.xit
.skipchk
	;load a module for desired effect if needed
	lda  >NUM_LM   ;total number of load modules
	and  #$ff
	inc            ;add 1 for common module
	sta  ModTabInd
	asl
	clc
	adc  ModTabInd ;index past MODULE_ADR table (SONG_MOD table)
	clc
	adc  #MODULE_ADR &$FFFF
	sta  ModTabInd
	lda  >NUM_SONGS
	and  #$ff
	clc
	adc  ModTabInd
	clc
	adc  EFFECTNUMBER   ;look up module for this effect
	sta  ModTabInd
	SA
	lda  #MODULE_ADR >> 16
	sta  ModTabInd+2    ;bank
	LA
	lda  [ModTabInd]    ;module value
	and  #$ff
	jsl  load_Module    ;load module if needed

	;play effect
	ldy  EFFECTNUMBER        ;EFFECT NUMBER TO PLAY
	lda  #DCOM_START_EFFECT
	jsl  Pass_Driver_Com     ;PLAY wolfgang

	.endif
.xit
	rtl

*******************
*
* LOAD A NEW SOUND MODULE
* A=MODULE #
*
*******************
load_Module:
	.if MUSICOK

	LAI
	ldx  MODULE         ;is common module in APU yet?
	bmi  .docommon
.newmodule
	tax
	beq  .noload   ;if common, don't reload
	and  #$ff
	cmp  MODULE         ;module already in memory?
	beq  .noload

.common_module
	;load a new module
	sta  MODULE         ;MODULE IN MEMORY
	asl
	clc
	adc  MODULE         ;*3 for index
	tax
	lda  >MODULE_ADR,x
	pha            ;module address
	lda  >MODULE_ADR+2,x
	and  #$ff      
	pha            ;bank location

	;tell driver to load
	lda  #DCOM_RELOAD   ;TELL DRIVER TO LOAD
	jsl  Pass_Driver_Com     ;BEFORE PLAYING !ANY! wolfgang OR EFFECTS

	;transfer module to APU ram
	pla            ;bank
	plx            ;address
	
	jsl  APU_Upload_Ind ;UPLOAD MODULE inderect (NEW)

	;tell driver to run again
	lda  #DCOM_RUN ;TELL DRIVER TO RUN AFTER WE LOAD
	jsl  Pass_Driver_Com
.noload
	.endif

	rtl
	;common was not in memory, send it first
.docommon
	.if MUSICOK

	pha
	lda  #$00      ;common is module #0
	jsl  .common_module
	pla
	bra  .newmodule     ;send over selected module

	.endif

*******************
*
* get address of module table
* and put into ModTabInd also determine which module contains the currently
* selected song (SONGNUMBER) or effect (EFFECTNUMBER).
*
* entry:
* x =          0 get song module
* x !=    0 get effect module
*
* exit:
* a = module # for this song or effect
*
*******************
get_module_number:
	LAI
	;load a module for desired song if needed
	lda  >NUM_LM   ;total number of load modules
	and  #$ff
	inc            ;add 1 for common module
	sta  src_ptr
	asl
	clc
	adc  src_ptr   ;index past MODULE_ADR table (SONG_MOD table)
	clc
	adc  #MODULE_ADR &$FFFF
	txy            ;song or index table
	bne  .do_effect
.do_song
	clc
	adc  SONGNUMBER     ;look up module for this song
	bra  .save_adr
.do_effect
	sta  src_ptr
	lda  >NUM_SONGS
	and  #$ff
	clc
	adc  src_ptr
	clc
	adc  EFFECTNUMBER   ;look up module for this effect
.save_adr
	sta  src_ptr
	lda  #MODULE_ADR >> 16
	sta  src_ptr+2 ;bank
	lda  [src_ptr] ;module value
	and  #$ff
	rtl
*******************
*
* set module pointer to address of selected module table
* entry:
*    a=module #
*******************
set_module_pointer:
	LAI
	sta  ModTabInd
	asl
	clc
	adc  ModTabInd
	tax                 ;find module address
	lda  >MODULE_ADR,x
	sta  ModTabInd      ;module adr
	lda  >MODULE_ADR+2,x
	and  #$ff
	SA
	sta  ModTabInd+2
	LA
	rtl

*************************************************
* END OF MUSIC DRIVER 65816 CODE
*************************************************


