*****************************************************************************
*
* PAL.ASM
*
*****************************************************************************
*
*
* Copy all 8 background palettes
*
* 8 palettes of 16 colors, 1 word per color
*

Copypalall:
	LAI

	ldy	z_level	; Byte level index
	ldx	z_levelword	; Word level index

	lda	_bg1pal,x	; Get palette address offset
	sta	z_temp3

	lda	_bg1palbank,y	; Get palette bank address
	and	#$FF
	sta	z_temp4

	ldy	#$FE	; Number of words to transfer
.loop       
	lda	[z_temp3],y	; Get a color
	sta	PALETTE2,y 	; Store it

	dey
	dey

	bpl	.loop

	rtl

*****************************************************************************
*
* Steps incrementally from one palette to a new palette
* PALETTE2 is new palette, PALETTE is old palette
*
* In other words, fades PALETTE2 in
* Also, this process is non-exclusive from start to finish
*
*****************************************************************************

Colfadeobj0:
	LAI

	lda	z_colfadeip
	beq	.exit

	lda	z_colfadeos
	bne	.loop0

	lda	#31	; 31 steps to do fade
	sta	z_colfade3	; Loop counter

	lda	#1
	sta	z_colfadeos
.loop0
	ldx	#$11E	; Byte index
.loop1
	lda	PALETTE2,x
	sta	z_colfade2	; Old color

	jsl	Colorstep2i	; Step towards new color

	lda	z_colfade1	; Get the new value
	sta	PALETTE,x	; Store new value in old palette

	dex		; Adjust index
	dex

	cpx	#$100
	bcs	.loop1	; Continue till done

	lda	#1
	sta	z_docolfade

	dec	z_colfade3	; Adjust counter
	bpl	.exit

	stz	z_colfadeip
.exit
	rtl

Colfadeobj0v:
	lda	#PALETTE	; Point to newly "stepped" palette
	ldy	#$200	; Length
	ldx	#0	; Bank

	jsl	Sendpalette

	stz	z_docolfade

	rtl


*****************************************************************************
*
* Steps incrementally from one palette to a new palette
* PALETTE2 is new palette, PALETTE is old palette
*
* In other words, fades PALETTE2 in
* Also, this process is non-exclusive from start to finish
*
*****************************************************************************

Colfade2i:
	LAI

	lda	z_colfadeip
	beq	.exit

	lda	z_colfadeos
	bne	.loop0

	lda	#31	; 31 steps to do fade
	sta	z_colfade3	; Loop counter

	lda	#1
	sta	z_colfadeos
.loop0
	ldx	#$1fe	; Byte index
.loop1
	lda	PALETTE2,x
	sta	z_colfade2	; Old color

	jsl	Colorstep2i	; Step towards new color

	lda	z_colfade1	; Get the new value
	sta	PALETTE,x	; Store new value in old palette

	dex		; Adjust index
	dex

	bpl	.loop1	; Continue till done

	lda	#1
	sta	z_docolfade

	dec	z_colfade3	; Adjust counter
	bpl	.exit

	stz	z_colfadeip
.exit
	rtl

Colfade2iv:
	lda	#PALETTE	; Point to newly "stepped" palette
	ldy	#$200	; Length
	ldx	#0	; Bank

	jsl	Sendpalette	; Send the palette

	stz	z_docolfade

	rtl


;used by colfade2i to step one more value towards the goal
;value for a color

Colorstep2i: 
	stz	z_colfade1

	lda	z_colfade2
	and	#$1F
	sec   
	sbc	z_colfade3
	bmi	.next

	sta	z_colfade1
.next
	lda	z_colfade3
	xba
	lsr
	lsr
	lsr
	sta	z_colfade4

	lda	z_colfade2
	and	#$3e0
	sec
	sbc	z_colfade4
	bmi	.next2

	ora	z_colfade1
	sta	z_colfade1
.next2
	lda	z_colfade3
	xba
	asl
	asl
	sta	z_colfade4

	lda	z_colfade2
	and	#$7C00
	sec
	sbc	z_colfade4
	bmi	.end

	ora	z_colfade1
	sta	z_colfade1
.end
	rtl


*****************************************************************************
*
* Steps incrementally from one palette to a new palette
* PALETTE2 is new palette, PALETTE is old palette
*
* In other words, fades PALETTE2 in
* Also, this process is exclusive from start to finish
*
*****************************************************************************

Colfade:    
	lda   #31	; 31 steps to do fade
	sta   z_temp3	; Loop counter
.loop0
	ldx   #$1fe	; Byte index
.loop1
	lda   PALETTE,x
	sta   z_temp1	; Old color

	lda   PALETTE2,x
	sta   z_temp2	; New color

	jsl   Colorstep	; Step towards new color

	lda   z_temp1	; Get the new value
	sta   PALETTE,x	; Store new value in old palette

	dex		; Adjust index
	dex

	bpl   .loop1	; Continue till done

	jsl   Waitfornmi	; Obviously done with BLANKOFF

	lda   #PALETTE	; Point to newly "stepped" palette
	ldy   #$200	; Length
	ldx   #0	; Bank

	jsl   Sendpalette	; Send the palette

	dec   z_temp3	; Adjust counter
	bpl   .loop0	; Continue till fully faded

	rtl


*****************************************************************************
*
* Fade to black, with mosaic effect
*

Fadeoutmosaic:   
	sec

	jsl	Clearpal	; Clear palette

	lda	#31
	sta	z_temp3	; Loop counter
.loop0
	ldx	#$1fe	; Byte index
.loop1
	lda	PALETTE,x
	sta	z_temp1	; Old color

	lda	PALETTE2,x
	sta	z_temp2	; New color

	jsl	Colorstep

	lda	z_temp1
	sta	PALETTE,x

	dex
	dex

	bpl	.loop1

	jsl	Waitfornmi

	lda	#PALETTE
	ldy	#$200
	ldx	#0

	jsl	Sendpalette

	lda	#$33
	SA
	sta	MOSAIC
	LA

	jsl	Waitfornmi

	lda	#$23
	SA
	sta	MOSAIC
	LA

	jsl	Waitfornmi

	lda	#$13
	SA
	sta	MOSAIC
	LA

	dec	z_temp3
	bpl	.loop0

	BLANKON

	stz	MOSAIC

	rtl


*****************************************************************************
*
* Takes old color in z_temp1 and moves it one step closer
* to new color in z_temp2, thrashes on z_temp0
*
*****************************************************************************

Colorstep:
	lda	z_temp2   ;new color
	and	#$1f
	sta	z_temp0

	lda	z_temp1   ;old color
	and	#$1f
	cmp	z_temp0
	beq	.red

	bcc	.yelinc

	dec	z_temp1   ;old color

	bra	.red
.yelinc:
	inc	z_temp1
.red:
	lda	z_temp2
	and	#$3e0
	sta	z_temp0

	lda	z_temp1
	and	#$3e0
	cmp	z_temp0
	beq	.blue

	bcc	.redinc

	lda	z_temp1
	sec
	sbc	#$20
	sta	z_temp1

	bra	.blue
.redinc:
	lda	z_temp1
	clc
	adc	#$20
	sta	z_temp1
.blue:
	lda	z_temp2
	and	#$7c00
	sta	z_temp0

	lda	z_temp1
	and	#$7c00
	cmp	z_temp0
	beq	.end

	bcc	.blueinc

	lda	z_temp1
	sec
	sbc	#$0400
	sta	z_temp1

	bra	.end
.blueinc:
	lda	z_temp1
	clc
	adc	#$0400
	sta	z_temp1
.end:
	rtl


*****************************************************************************
*
* Clears an entire palette to black
* sec for pal2 (new pal), clc for pal (old pal)
*
*****************************************************************************

Clearpal:
	LAI
	ldx	#$1fe
	bcs	.clear2
.clear1
	stz	PALETTE,x

	dex
	dex

	bpl	.clear1

	rtl
.clear2
	stz	PALETTE2,x

	dex
	dex

	bpl	.clear2

	rtl


;call this once a 256 color palette is in PALETTE2 to fade from black to
;the colors in PALETTE2

Fadein:     
	jsl	Colfade2

	LAI

	rtl


;call this to fade to black the entire 256 color palette
Fadeout:   
	sec

	jsl	Clearpal

	jsl	Colfade

	BLANKON

	rtl




;used by fade-in to adjust whole palette incrementally from Palette1
;to palette2
;this routine also downloads the palette at each step (32 steps)
            
Colfade2:
	LAI
	lda	#31
	sta	z_temp3            ;loop counter
.loop0
	ldx	#$1fe
.loop1
	lda	PALETTE2,x
	sta	z_temp2            ;new color

	jsl	Colorstep2

	lda	z_temp1
	sta	PALETTE,x

	dex
	dex

	bpl	.loop1

	jsl	Waitfornmi

	lda	#PALETTE
	ldy	#$200
	ldx	#0

	jsl	Sendpalette

	BLANKOFF

	dec	z_temp3
	bpl	.loop0
.end
	rtl



;used by colfade2 to step one more value towards the goal
;value for a color

Colorstep2: 
	stz	z_temp1

	lda	z_temp2
	and	#$1F
	sec   
	sbc	z_temp3
	bmi	.next

	sta	z_temp1
.next
	lda	z_temp3
	xba
	lsr
	lsr
	lsr
	sta	z_temp4

	lda	z_temp2
	and	#$3e0
	sec
	sbc	z_temp4
	bmi	.next2

	ora	z_temp1
	sta	z_temp1
.next2
	lda	z_temp3
	xba
	asl
	asl
	sta	z_temp4

	lda	z_temp2
	and	#$7C00
	sec
	sbc	z_temp4
	bmi	.end

	ora	z_temp1
	sta	z_temp1
.end
	rtl


d_pallist

; Main Character Sequences

	.dw	TZAN.PALSET

	.dw	TZPUPS1.PALSET
	.dw	TZHBAR.PALSET

	.dw	TZPALWTR.PALSET
	.dw	TZPDRK00.PALSET
	.dw	TZPDRK01.PALSET
	.dw	TZPDRK02.PALSET
	.dw	TZPDRK03.PALSET
	.dw	TZPDRK04.PALSET
	.dw	TZDAMPAL.PALSET

	.dw	DAGGER1.PALSET

	.dw	TZdeadpal

	.dw	TITLE01.PALSET
d_palbank

; Main Character Palettes

	.dw	^TZAN.PALSET

	.dw	^TZPUPS1.PALSET
	.dw	^TZHBAR.PALSET

	.dw	^TZPALWTR.PALSET
	.dw	^TZPDRK00.PALSET
	.dw	^TZPDRK01.PALSET
	.dw	^TZPDRK02.PALSET
	.dw	^TZPDRK03.PALSET
	.dw	^TZPDRK04.PALSET
	.dw	^TZDAMPAL.PALSET

	.dw	^DAGGER1.PALSET

	.dw	^TZdeadpal

	.dw	^TITLE01.PALSET

paladdr:  
	.dw	$11E, $13E, $15E, $17E
	.dw	$19E, $1bE, $1dE, $1fE

Loadsprpal:
	pha                     ; Save palette #

	lda	d_pallist,x	; Get palette pointer offset
	sta	z_temp3

	lda	d_palbank,x	; Get palette pointer bank
	sta	z_temp4
           
	pla		; Get palette number

	asl		; Word
	tay		; Create index
	lda	paladdr,y	; Get CGRAM offset address
	tax		; Create index

	ldy	#$1E	; 1 palette of 16 colors (words)
.loop       
	lda	[z_temp3],y	; Get palette color
	sta	PALETTE2,x	; Store to PALETTE2 for download

	dex		; Adjust index pointer
	dex

	dey		; Adjust index pointer
	dey

	bpl	.loop	; Get another color

	rtl





