*****************************************************************************
*
* Priest AI
*
*****************************************************************************
*
* Flame Downloading Routines
*
*****************************************************************************
*
* Download 1280 bytes of flame data
*
* in:  v_flameaddr = offset in frame data
*      v_flamebank = flame frame bank
*      v_flamedest = destination address in vram
*
*****************************************************************************

	.dbreg $04

FlameData:
	ldy	v_xfercount
	lda	v_flameaddr
	sta	v_xferloc,y

	clc
	adc	#1280
	sta	v_flameaddr

	lda	v_flamebank
	sta	v_xferbank,y

	lda	v_flamedest
	sta	v_xferdest,y

	clc
	adc	#1280/2
	sta	v_flamedest

	lda	#1280	; send 1 1/4K every frame
	sta	v_xferlen,y

	iny
	iny

	sty	v_xfercount

	rts

*****************************************************************************
*
* Download an Entire Flame Frame
*
* in:  v_flamedest = vram slot
*      v_flametype = foe type for flame
*      a = offset in flame data
*      y = flame bank
*
* uses v_foemisc1 and v_foemisc2 of flame foe
*
*****************************************************************************

DownloadFlame:	
	sta	v_flameaddr

	tya
	sta	v_flamebank

	lda	v_flamedest	; save flame vram slot
	sta	v_foemisc1,x

	asl
	asl
	asl
	asl

	adc	#OBJBASE
	sta	v_flamedest

	pla		; save return address
	inc
	sta	v_foemisc2,x

	jsr	FlameData

	WAIT		; tick 1

	jsr	FlameData

	WAIT		; tick 2

	jsr	FlameData    

	WAIT		; tick 3

	jsr	FlameData	

	WAIT		; tick 4
	WAIT		; tick 5
	WAIT		; tick 6

	lda	v_foeoff,x
	and	#$FE00
	ora	v_foemisc1,x
	sta	v_foeactoff,x

	lda	v_flametype
	sta	v_foetype,x

	stz	v_foeactframe,x

	WAIT		; tick 7 - new frame displayed

	jmp	(v_foemisc2,x)


*******************************************************************************
*
* Flame Handler
*
*******************************************************************************

@FireAI:
	lda	#FLAMEON
	sta	v_foetype,x

	stz	v_foeframe,x

	lda	v_foeoff,x
	and	#$F000
	ora	#$0AC0
	sta	v_foeoff,x

	lda	#$25C<<2
	sta	v_foey,x

	lda	#$0505
	sta	v_foedelay,x

	BLAST

	lda	#24	; Nudge down flames so Tzan's feet

	jsr	AdjustY 	; won't show if he is behind them

	STEP		; 1
	STEP		; 2
	STEP		; 3

	stz	v_foedelay,x

	STEP		; 4

	lda	v_foeoff,x	; make flames always blow right
	and	#$3FFF
	sta	v_foeoff,x

	lda	#$0160       	; last start-up frame...
	sta	v_flamedest

	lda	#FLAME1
	sta	v_flametype

	lda	#flame1dat
	ldy	#^flame1dat

	jsr	DownloadFlame

	lda	#4
	sta	v_foeaux1,x

.flicker:
	lda	#$00C0	; flame cycle 1...
	sta	v_flamedest

	lda	#FLAME2
	sta	v_flametype

	lda	#flame2dat
	ldy	#^flame2dat

	jsr	DownloadFlame

	ldy	#firewall_eff

	jsl	SoundEffect

	lda	#$0160	; flame cycle 2...
	sta	v_flamedest

	lda	#FLAME3
	sta	v_flametype

	lda	#flame3dat
	ldy	#^flame3dat

	jsr	DownloadFlame

	lda	#$00C0
	sta	v_flamedest	; flame cycle 3...

	lda	#FLAME4
	sta	v_flametype

	lda	#flame4dat
	ldy	#^flame4dat

	jsr	DownloadFlame
	
	lda	#$0160	; flame cycle 4...
	sta	v_flamedest

	lda	#FLAME5
	sta	v_flametype

	lda	#flame5dat
	ldy	#^flame5dat

	jsr	DownloadFlame

	dec	v_foeaux1,x
	beq	.fade
	jmp	.flicker

.fade:
	lda	#$00C0	; Going...
	sta	v_flamedest

	lda	#FLAME6
	sta	v_flametype

	lda	#flame6dat
	ldy	#^flame6dat

	jsr	DownloadFlame

	lda	#$0160	; Going...
	sta	v_flamedest

	lda	#FLAME7
	sta	v_flametype

	lda	#flame7dat
	ldy	#^flame7dat

	jsr	DownloadFlame

	lda	#$00C0	; Gone!
	sta	v_flamedest

	lda	#FLAME8
	sta	v_flametype

	lda	#flame8dat
	ldy	#^flame8dat	

	jsr	DownloadFlame

	lda	#$0505	; hold last frame for a bit
	sta	v_foedelay,x

	WAIT

	TERM

			
*******************************************************************************
*
* Fireball Handler
*
*******************************************************************************

@FireballAI:
	lda	#FIREBALL
	sta	v_foetype,x

	stz	v_foeframe,x

	lda	v_foeoff,x
	and	#$F000
	ora	#$0AC0
	sta	v_foeoff,x

	lda	#$0303
	sta	v_foedelay,x

	lda	#-24

	jsr	FlipFix

	sta	v_foevelx,x	

	ldy	#firethro_eff

	jsl	SoundEffect

	BLAST
	STEP
	STEP

.inair:
	SAME

	lda	v_foeplot,x
	beq	.inair

	TERM


* ExplodeFB is jumped to by the fireball <-> Tarzan collision routine

ExplodeFB:
	lda	#$0505
	sta	v_foedelay,x

	stz	v_foevelx,x

	FRAME	3

	ldy	#explode3_eff

	jsl	SoundEffect

	STEP
	STEP
	STEP
	TERM


*******************************************************************************
*
* Priest Utility Routines
*
*******************************************************************************

SpawnFire:
	stz	v_foevelx+4
	stz	v_foevely+4
	stz	v_foedelay+4

	lda	#-416

	jsr	FlipFix

	clc
	adc	v_foex,x
	sta	v_foex+4

	stz	v_foeplot+4
	stz	v_foedma+4
	stz	v_foedelay+4

	lda	v_foeoff,x	
	sta	v_foeoff+4

	lda	#FireAI
	sta	v_foeai+4

	rts


SpawnFB:	
	stz	v_foevelx+4
	stz	v_foevely+4
	stz	v_foedelay+4

	lda	#-224

	jsr	FlipFix

	clc	
	adc	v_foex,x
	sta	v_foex+4

	lda	#-96
	clc
	adc	v_foey,x
	sta	v_foey+4

	sta	v_foeframe+4	; no coll detection yet

	stz	v_foeplot+4
	stz	v_foedma+4
	stz	v_foedelay+4

	lda	v_foeoff,x	
	sta	v_foeoff+4

	lda	#FireballAI
	sta	v_foeai+4

	rts


* Choose between fire and fireball

PriestMagic:	
	jsr	FacingTzan

	bcc	.fireball

	jsr	TzanInZone

	bcc	.fireball

	jsr	DistX

	cmp	#$30<<2
	bcc	.fireball

	cmp	#$B0<<2
	bcs	.fireball

	lda	v_foemisc1,x
	cmp	#96
	bcc	.fireball

	lda	v_bosshealth	; flames aren't used until health is
	cmp	#18	; below 18
	bcs	.fireball

	lda	v_foeoff,x	; flames are okay if facing left
	and	#$4000
	beq	.flames

	lda	v_foex,x	; if facing right, must be enough room
	cmp	#$6D3<<2	; for flames
	bcs	.fireball	

.flames:	stz	v_foemisc1,x	; use up fire karma points

	jmp	SpawnFire

.fireball:
	lda	v_foemisc1,x	; get some karma points for throwing
	clc		; a fireball
	adc	#24
	sta	v_foemisc1,x

	jmp	SpawnFB


*******************************************************************************
*
* Oparian Priest Boss
*
* The 3rd foe slot (offset 4) is used for the priest's weapons (fire &
* fireball).  This is the same slot used for the stone before the priest
* is confronted.
*
*******************************************************************************

PriestAI:
	lda	#20
	sta	v_bosshealth
	sta	v_bossoldhealth

	lda	#96
	sta	v_foemisc1,x	; v_foemisc1 = flame karma points
	
	stx	v_priestindex

	; wait for Tarzan to get near the priest

.sleep:
	RESET

	lda	v_playerx
	cmp	#$1A0<<2
	bcc	.sleep

	sta	v_bossactive	; boss is now awake

	; wait until Tarzan is closer before doing anything

.standoff:
	SAME

	lda	v_playerx
	cmp	#$1F0<<2
	bcc	.standoff

	jmp	PriestThrow


PriestStand:
	stz	v_foevelx,x

	RESET

	inc	v_foemisc1,x	; Gain fire karma

	lda	v_foeplot+4	; Don't move while magic is in effect
	bpl	PriestStand

	jsr	DistX

	cmp	#700	; if Tarzan is far away, start walking
	bcs	PriestSxW

	jsr	FacingTzan	; If Tarzan is behind priest, start

	bcc	PriestSxW	; walking

	jmp	PriestThrow


PriestSxW:
	FRAME	1
	STEP
	STEP
	STEP

	lda	#-36

	jsr	AdjustX

PriestWalk:
	lda	#-2

	jsr	FlipFix

	sta	v_foevelx,x

.walk:
	FRAME 5
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP

	lda	v_foemisc1,x	; Gain fire karma
	clc
	adc	#32
	sta	v_foemisc1,x

	STEP
	
	lda	#96	; If there is no room for a walk cycle,

	jsr	NearEdge	; turn around

	bcs	PriestTurn

;	lda	v_playerx	; If Tarzan is on table, keep walking
;	cmp	#$381<<2
;	bcs	.walk

	jsr	TowardTzan	; If Tarzan is in front of priest,

	bcs	PriestWxS	; stop & attack

	lda	#128	; If Tarzan is behind priest and there

	jsr	NearOpEdge	; is room to turn around, go ahead

	bcc	PriestTurn

	bra	.walk

PriestWxS:	stz	v_foevelx,x

	FRAME	16
	STEP
	STEP

	lda	#-16

	jsr	AdjustX

	jmp	PriestStand

PriestTurn:
	stz	v_foevelx,x

	FRAME 13
	STEP
	STEP

	lda	#64

	jsr	AdjustX	

	jsr	FlipH

	lda	v_playerx
	cmp	#$381<<2
	bcs	.walk

	jmp	PriestStand

.walk:
	jmp	PriestWalk


PriestThrow:	
	RESET		; must stand before throwing
	FRAME	19	; 19/0
	STEP		; 20/1
	STEP		; 21/2
	STEP		; 22/3
	SAME
	STEP		; 23/4
	STEP		; 24/5

	jsr	PriestMagic

	STEP		; 25/6
	SAME
	SAME
	STEP		; 26/7
	STEP		; 27/8

	jmp	PriestStand


;
; Priest with Knife - appears after below x hit points
;

PriestKStand:
	lda	#PRIEST1
	sta	v_foetype,x

	RESET

	lda	#PRIEST2
	sta	v_foetype,x
	
	RESET		; 0
	STEP		; 1
	SAME		; 1
	STEP		; 2
	STEP		; 3

.kstand:
	lda	v_playerx
	cmp	#$381<<2
	bcs	.move

	jsr	FacingTzan

	bcc	.move

	jsr	DistX

	cmp	#224
	bcs	.noslash

	jmp	.slash

.noslash:
	cmp	#384
	bcs	.move

	FRAME	3

	bra	.kstand	

.move:
	FRAME	2	; put knife away...
	FRAME 1
	RESET

	lda	#PRIEST1
	sta	v_foetype,x

	lda	#-1	; *** hack alert!! (NearEdge only works

	jsr	FlipFix	; when foe is moving)Q

	sta	v_foevelx,x

	lda	#96

	jsr	NearEdge

	stz	v_foevelx,x	; *** more of the same hack

	bcc	.standxwalk

	jmp	.turn
	
.standxwalk:
	FRAME	1	; stand to walk transition...
	STEP
	STEP
	STEP

	lda	#-36

	jsr	AdjustX

.startwalk:
	lda	#-2

	jsr	FlipFix

	sta	v_foevelx,x

.walk:
	FRAME 5
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP
	STEP

	lda	#96

	jsr	NearEdge

	bcs	.turn

	lda	v_playerx
	cmp	#$381<<2
	bcs	.walk

	jsr	TowardTzan

	bcc	.turn

	jsr	DistX

	cmp	#384
	bcs	.walk

	stz	v_foevelx,x	; walk to stand transition...

	FRAME	16
	STEP
	STEP

	lda	#-16

	jsr	AdjustX

	jmp	PriestKStand	
	
.turn:
	stz	v_foevelx

	FRAME 13
	STEP
	STEP

	lda	#64

	jsr	AdjustX	

	jsr	FlipH

	jmp	.startwalk

.slash:
	FRAME 4	; 0
	STEP		; 1
	STEP		; 2
	STEP		; 3
	STEP		; 4
	STEP		; 5
	STEP		; 6
	STEP		; 7
	STEP		; 8
	STEP		; 9
	STEP		; 10
	STEP		; 11

	jmp	.kstand
		

		


*******************************************************************************
*
* Priest Monitor - checks for knife/dart hits
*
*******************************************************************************

PriestMonitor:
	WAIT

	lda	v_bossinvince
	beq	.notinvince

	dec	v_bossinvince

	bra	PriestMonitor

.notinvince:
	lda	v_bosshealth
	cmp	v_bossoldhealth
	beq	PriestMonitor

	sta	v_bossoldhealth

	lda	#PriestHit
	sta	v_foeai

	lda	#$FFFF
	sta	v_bossinvince

	bra	PriestMonitor
PriestHit:	
	stz	v_foevelx,x

	lda	#$0505
	sta	v_foedelay,x

	lda	#PRIEST2
	sta	v_foetype,x

	FRAME 16	; 16/0
	STEP		; 17/1
	STEP		; 18/2

	ldy	#grrr3_eff

	jsl	SoundEffect

	SAME
	SAME

	lda	v_bosshealth
	beq	.dead

	FRAME	17
	FRAME 16

	lda	#30
	sta	v_bossinvince

	lda	#PRIEST1
	sta	v_foetype,x

	lda	v_bosshealth
	cmp	#6	; when less than 6 hit points left, go
	bcc	.knifestand	; to last ditch knife mode

	jmp	PriestStand

.knifestand:
	jmp	PriestKStand

.dead:
	lda	#$FFFF
	sta	v_bossinvince

	STEP		; 19
	STEP		; 20
	STEP		; 21
	STEP		; 22
	STEP		; 23
	STEP		; 24

	lda	#$FFFF
	sta	v_bossinvince

	lda	#1
	sta	v_priestdead
	sta	v_bossdead

.rot:
	SAME

	bra	.rot

	.dbreg $00

