*****************************************************************************
*
* SPECIALS.ASM
*
*****************************************************************************

ShowMap:
	LAI

	stz	v_cycledly1	; Init color cycling vars
	stz	v_cycledly2
	stz	v_cycledly3
	stz	v_cycledly4
	stz	v_cycledly5
	stz	v_colorcel1
	stz	v_colorcel2
	stz	v_colorcel3
	stz	v_colorcel4
	stz	v_colorcel5

	stz	cloudtime	; Expire cloud movement timer
	stz	v_daggerdone	; Set dagger in motion
	stz	v_daggerflip	; Used only by debug mode

	stz	z_levelx
	stz	z_levely

	lda	z_level	; Get current level
	and	#$FF

	cmp	#BONUSINT	; Inside Bonus Hut?
	bne	.chkholda	; No  - Check for a ship hold

	bra	.dolevel	; Otherwise, skip show map

.chkholda
	cmp	#SHIPHOLDA	; Is this ship hold?
	bne	.chkholdb	; No  - Check other side

	bra	.dolevel	; Otherwise, skip show map

.chkholdb
	cmp	#SHIPHOLDB	; Is this ship hold?
	bne	.chkshamon	; No - Check for Shamon Boss level

	bra	.dolevel	; Otherwise, skip show map

.chkshamon
	cmp	#SHAMONBOSS	; Is this Shamon Boss level?
	bne	.domap	; No - Ok to show map then

.dolevel
	rtl

.domap
	ldy	#$2601	; Done with level - fade music
	lda	#DCOM_FADEOUT
	
	jsl	Pass_Driver_Command

	lda	#SNDSELECT

	jsl	load_Module

	lda	#map_sng

	jsl	load_Song
	
	jsl	.loadmap	; Unpak & Send Bigmap data to VRAM


;	lda	v_DebugMode	; Check master debug switch
;	beq	.automap	; Do normal game mode if not set

;	lda	v_MapGUI	; If Map as GUI level selector, then...
;	beq	.automap

;	jmp	.dbmode	; ...Allow control over dagger & stuf


*****************************************************************************
*
* This is normal processing
*
*****************************************************************************

.automap
	lda	#240  	; Display time for map
	sta	.delay1
	
	jsl	Spotdagger	; Compute BG1 & dagger pos per level

	jsl	.scroll1n3	; Scroll BG1 to position

	jsl	Resetattr	; Reset Objects

	jsl	Updatedggr	; Update dagger position

	jsl	PlotDagger	; Plot dagger

	jsl	Dspleveltitle	; Update & Plot Level title

	jsl	Createoam	; Update dagger in VRAM

	jsl	Sendobj

	jsl	Senddata

	jsl	Fadein	; Fade Map in

	lda	#whoosh1_eff

	jsl	load_Effect

.daggerloop
	jsl	Resetattr	; Reset Objects

	jsl	Updatedggr	; Update dagger position

	jsl	Dspleveltitle	; Update & Plot Level title

	jsl	PlotDagger	; Plot dagger

	jsl	MovClouds	; Scroll Clouds

	jsl	Createoam	; Update dagger & title in VRAM

	jsl	Waitfornmi	; Otherwise, wait for VBLANK

  	jsl	Sendobj	; Send over the OAM

	jsl	Senddata	; Xfer the requested sprite images

	jsl	.scroll1n3	; Scroll Clouds

	jsl	Cyclemap	; Do color cycling of water on map

	lda	v_daggerdone
	bne	.rescan

	bra	.daggerloop

.rescan
	dec	.delay1	; Count down
	bmi	.exitscn	; We're done

	jsl	Scanjoys	; Get user input

	lda	z_joy1	; Any user requests?
	beq	.daggerloop	; No - Just rescan forever then
.exitscn
	jsl	Fadeout	; Fade out & return

	lda	#0
	sta	z_hdmaenable

	SA
	sta	HDMAEN	; Disable HDMA
	LA

	stz	z_joy1
	stz	z_joy2

	rtl

*****************************************************************************
*
* DEBUG Mode
*
*****************************************************************************

.dbmode
	LAI

	lda	z_level	; Get level values
	and	#$FF
	sta	tmplvl

	lda	z_levelword
	sta	tmplvlword

	jsl	Spotdagger	; Compute BG1 & dagger pos per level

	jsl	.scroll1n3	; Scroll BG1 & BG3 to position

	inc	.retrigd3	; Preset dagger scroll offset & flip

	jsl	Detectxy	; Returns level in v_daggerdone

	jsl	Resetattr	; Reset Objects

	jsl	Updatedggr	; Update dagger position

	jsl	PlotDagger	; Plot dagger

	jsl	Dspleveltitle	; Update & Plot Level title

	jsl	Createoam	; Update dagger in VRAM

	jsl	Sendobj

	jsl	Senddata

	jsl	Fadein	; Fade Map in

.rescan1
	jsl	Scanjoys	; Get user input

	lda	z_joy2	; Any user requests?
	ora	z_joy1
	beq	.skipchime	; No - Just rescan forever then

	lda	z_joy2	; The L key permits a break to occur
	bit	v_JOYPADL	; thus enabling SDEBUG to display
	beq	.nobrk	; the x,y positions of BG1 & BG3
.brkpnt 
	nop
	nop
	nop
.nobrk
	lda	z_joy2	
	and	#JOYPADSTART	; If START pressed
	bne	.exitdbug	; We're outa here

	lda	z_joy1	
	and	#JOYPADSTART	; If START pressed
	bne	.exitdbug	; We're outa here

	jsl	.scrolldagger	; Otherwise, wants to scroll dagger

	jsl	Detectxy	; Returns level in x; -1 if no level

	cpx	#-1
	beq	.skipchime

	lda	#getpop3_eff

	jsl	load_Effect

.skipchime
	jsl	Resetattr	; Reset Objects

	jsl	.scrolldagger

	jsl	Dspleveltitle	; Update & Plot Level title

	jsl	PlotDagger	; Plot dagger

	jsl	MovClouds	; Scroll Clouds

	jsl	Createoam	; Update dagger & title in VRAM

	jsl	Waitfornmi	; Otherwise, wait for VBLANK

  	jsl	Sendobj	; Send over the OAM

	jsl	Senddata	; Xfer the requested sprite images

	jsl	.scroll1n3	; Scroll Clouds

	jsl	Cyclemap	; Do color cycling of water on map

	jmp	.rescan1	; Scan again...
.exitdbug
	lda	#tarzroar_eff

	jsl	load_Effect

	jmp	.exitscn


*****************************************************************************
*
* Update BG1 (map) & BG3 (clouds) X,Y positions
*
*****************************************************************************

.scroll1n3
	lda	v_bg1vofs	; Get BG1 Y offset

	SA
	sta	BG1VOFS	; Update PPU
	xba
	sta	BG1VOFS
	LA

	lda	v_bg1hofs	; Get BG1 X offset

	SA
	sta	BG1HOFS	; Update PPU
	xba
	sta	BG1HOFS
	LA

	lda	v_bg3vofs	; Get BG3 Y offset

	SA
	sta	BG3VOFS	; Update PPU
	xba
	sta	BG3VOFS
	LA

	lda	v_bg3hofs	; Get BG3 X offset

	SA
	sta	BG3HOFS	; Update PPU
	xba
	sta	BG3HOFS
	LA

	rtl

*****************************************************************************
*
* Scroll Dagger & Adjust BG1 & BG3 values as required
*
* Dagger points left & down with flip set at - $2000
* Dagger points rite & down with flip set at - $6000
* Dagger points left & up   with flip set at - $A000
* Dagger points rite & up   with flip set at - $E000
*
*   bit 5 of upper nibble in msb is priority
*   bit 3 of lower nibble in msb is palette
*   bit's 0-7 in lsb plus bit 0 in msb are obj name
*       
*                    bit   7654 3210  7654 3210
*                    ---   ---- ----  ---- ----   
*   0 - No    flip  $2xxx  0010 1000  0000 0000
*   1 - X     flip  $6xxx  0110 1000  0000 0000
*   2 - Y     flip  $Axxx  1010 1000  0000 0000
*   3 - X & Y flip  $Exxx  1110 1000  0000 0000
*
*****************************************************************************

.scrolldagger
	lda	z_joy2	; Get user request
	and	#JOYPADLEFT	; Wants to go left?
	beq	.notleft	; No  -

	lda	#$2800
	sta	v_daggerflip

	lda	.retrigl3	; Yes - Are we in dagger scroll mode?
	beq	.centrl3	; No  - Then attempt to scroll map

	lda	v_daggerx	; Get cur dagger position
	dec		; adjust
	dec
	cmp	#.centerdagger	; Hit midpoint yet?
	bcs	.notl1	; Yes - reset trigger & start map scroll

	sta	v_daggerx	; No  - scroll daggerlight some more

	bra	.notleft	; Check right
.notl1
	stz	.retrigl3
.centrl3
	lda	v_bg1hofs	; Get cur map position
	dec		; adjust
	dec
	cmp	#5	; Lower limit yet?
	bcs	.okgoleft	; No  - Ok to scroll map
	
	lda	v_daggerx	; Yes - Limit hit, scroll bg3 now
	dec		; adjust daggerlight position
	dec
	cmp	#.daggermaxright	; Upper limit yet?
	bcs	.notleft	; Yes - Can't move then

	inc	.retrigr3	; No  - Flag dagger scroll mode (right)
	sta	v_daggerx	; Store new dagger position

	bra	.notleft	; Check right...
.okgoleft
	sta	v_bg1hofs

	lda	v_bg3hofs
	dec
	dec
	sta	v_bg3hofs
.notleft
	lda	z_joy2
	and	#JOYPADRIGHT
	beq	.notright

	lda	#$6800
	sta	v_daggerflip

	lda	.retrigr3
	beq	.centrr3

	lda	v_daggerx
	inc
	inc
	cmp	#.centerdagger
	bcc	.notr1

	sta	v_daggerx

	bra	.notright
.notr1
	stz	.retrigr3
.centrr3
	lda	v_bg1hofs	; Get cur map position
	inc		; adjust
	inc
	cmp	#$FA	; Upper limit yet?
	bcc	.okgoright	; No  - Ok to scroll map

	lda	v_daggerx	; Yes - Limit hit, scroll bg3 now
	inc		; adjust daggerlight position
	inc
	cmp	#.daggermaxleft	; Lower limit yet?
	bcc	.notright	; Yes - Can't move then

	inc	.retrigl3	; No  - Flag dagger scroll mode (left)
	sta	v_daggerx	; Store new dagger position

	bra	.notright	; Check right...
.okgoright
	sta	v_bg1hofs

	lda	v_bg3hofs
	inc
	inc
	sta	v_bg3hofs
.notright
	lda	z_joy2
	and	#JOYPADUP
	beq	.notup

	lda	#$8000
	ora	v_daggerflip
	sta	v_daggerflip

	lda	.retrigu3
	beq	.centru3

	lda	v_daggery
	dec
	dec
	cmp	#.centerdagger
	bcs	.notu1

	sta	v_daggery

	bra	.notup
.notu1
	stz	.retrigu3
.centru3
	lda	v_bg1vofs
	dec
	dec
	cmp	#5
	bcs	.okgoup

	lda	v_daggery
	dec
	dec
	cmp	#.daggermaxright
	bcs	.notup

	inc	.retrigd3
	sta	v_daggery

	bra	.notup
.okgoup
	sta	v_bg1vofs

	lda	v_bg3vofs
	dec
	dec
	sta	v_bg3vofs
.notup
	lda	z_joy2
	and	#JOYPADDOWN
	beq	.notdown

	lda	v_daggerflip
	and	#$7FFF
	sta	v_daggerflip

	lda	.retrigd3
	beq	.centrd3

	lda	v_daggery
	inc
	inc
	cmp	#.centerdagger
	bcc	.notd1

	sta	v_daggery

	bra	.notdown
.notd1
	stz	.retrigd3
.centrd3
	lda	v_bg1vofs
	inc
	inc
	cmp	#$120	; 
	bcc	.okgodown

	lda	v_daggery
	inc
	inc
	cmp	#$C8	; 200
	bcc	.notdown

	inc	.retrigu3
	sta	v_daggery

	bra	.notdown
.okgodown 
	sta	v_bg1vofs

	lda	v_bg3vofs
	inc
	inc
	sta	v_bg3vofs
.notdown

	rtl

.centerdagger	.equ	$80
.daggermaxright	.equ	$D0
.daggermaxleft	.equ	$2F

*****************************************

.retrigl3	.equ	v_foetype+0
.retrigr3	.equ	v_foetype+2
.retrigu3	.equ	v_foetype+4
.retrigd3	.equ	v_foetype+6

.delay1	.equ	v_foetype+8

*****************************************
*****************************************************************************
*****************************************************************************

.loadmap
	stz	v_foetype+0	; Init temporary storage
	stz	v_foetype+2
	stz	v_foetype+4
	stz	v_foetype+6
	stz	v_foetype+8
	stz	v_foetype+10
	stz	v_foetype+12

	jsr	.getbigmapchars	; Decompress & Download chars & scrns

	jsr	.dopalettes	; Load palettes

	jsr	.getcloudshadows ; Load Cloud shadow chars & scrns

	jsr	.setmode	; Set hardware mode

	jsr	.dodagger	;

	jsr	.setuphdma

	rtl



.getbigmapchars
	; Unpack Screen Chars

	pea	|bigmapccs	 ; Source
	pea	^bigmapccs
	pea	|SCRATCH	 ; Destination
	pea	^SCRATCH

	jsl	UNPACK

	pla
	pla
	pla
	pla


	; VRAM Xfer for Screen Chars

	lda	#^SCRATCH
	sta	z_sendvrambank

	lda	#SCRATCH
	ldy	#$6000
	ldx	#BG1CHAR

	jsl	Sendvram

	; Unpack Screen
	pea	|bigmapcsm	; push low word of packed file
	pea	^bigmapcsm	; push high word of packed file
	pea	|SCRATCH	; push low word of output buffer
	pea	^SCRATCH	; push high word of output buffer

	jsl	UNPACK	; unpack file to output buffer

	pla
	pla
	pla
	pla

	; 4 1x1 character screens for map

	; Copy Screen to BG1	; 4000-5000
	ldx	#BG1BASE
	lda	#.scrn00
	ldy	#^SCRATCH  ;
	jsl	Sendbg

	; Copy Screen to BG1
	ldx	#BG1BASE1B
	lda	#.scrn01
	ldy	#^SCRATCH  ;
	jsl	Sendbg

	; Copy Screen to BG1
	ldx	#BG1BASE1C
	lda	#.scrn02
	ldy	#^SCRATCH  ;
	jsl	Sendbg

	; Copy Screen to BG1
	ldx	#BG1BASE1D
	lda	#.scrn03
	ldy	#^SCRATCH  ;
	jsl	Sendbg

	rts

***************************************

.dopalettes
	; Copy Screen Palette(s)
	lda	#bigmappal
	sta	z_temp0

	lda	#^bigmappal
	sta	z_temp1

	ldy	#$FE
.palloop    
	lda	[z_temp0],y
	sta	PALETTE2,y

	dey
	dey
	bpl	.palloop

;	stz	PALETTE

	jsl	Loaddagrpal	; Load palette for dagger

	jsl	Loadtitlepal

	rts


****************************************

.getcloudshadows
	; Unpack Screen Chars

	pea	|spyglassccs	 ; Source
	pea	^spyglassccs
	pea	|SCRATCH	 ; Destination
	pea	^SCRATCH

	jsl	UNPACK

	pla
	pla
	pla
	pla


	; VRAM Xfer for Screen Chars

	lda	#^SCRATCH
	sta	z_sendvrambank

	lda	#SCRATCH
	ldy	#$1000	
	ldx	#BG3CHAR

	jsl	Sendvram

	; Unpack Screen
	pea	|spyglasscsm	; push low word of packed file
	pea	^spyglasscsm	; push high word of packed file
	pea	|SCRATCH	; push low word of output buffer
	pea	^SCRATCH	; push high word of output buffer

	jsl	UNPACK	; unpack file to output buffer

	pla
	pla
	pla
	pla

	; Copy Screen to BG3
	ldx	#BG3BASE
	lda	#.scrn00
	ldy	#^SCRATCH
	jsl	Sendbg

	rts


****************************************

.setmode
	; Set Modes

	SA

	lda	#$03
	sta	OBJSEL

	lda	#(BG1BASE/256)+3 ; 2x2 screen
	sta	BG1SC

	lda	#(BG3BASE/256)	; 1x1 screen
	sta	BG3SC

	lda	#$1	; Set mode 1 8x8 char
	sta	BGMODE

	lda	#$11	; Set BG1 & OBJ as main screen
	sta	TM
	sta	TMW

	lda	#$4	; Set BG3 as sub screen
	sta	TS
	sta	TSW

	lda	#$2	; Set add/sub sub screen
	sta	CGSWSEL		

	lda	#$3
	sta	W34SEL	; Enable Window 1 on BG3

	lda	#0	; Set left side to 0
	sta	WH0

	lda	#0	; Set full closed
	sta	WH1
	
	lda	#$81	; Designate BG1 & OBJ as main target &
	sta	CGADSUB	; enable add/sub

	lda	#$E0	; RGB guns
	sta	COLDATA

	stz	BG12NBA

	lda	#BG3CHAR >> 12
	sta	BG34NBA

	LA

	rts

***************************************

.dodagger
	LAI

	ldy	#0	; One and only frame

	lda	#^dagger1	; Point to dagger data structure
	sta	z_dreccount

	lda	#dagger1	; Point to dagger data structure
	sta	z_spridrec

	lda	#OBJBASE	; Set to base

	jsl	Sendframe	; Queue it up

	ldy	#0
	ldx	z_levelword	; One and only frame

	lda	>d_titlebank,x	; Point to dagger data structure
	sta	z_dreccount

	lda	>d_titledrec,x	; Point to dagger data structure
	sta	z_spridrec

	lda	#$20
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; Set to base

	jsl	Sendframe	; Queue it up

	jsl	Senddata	; Send it down to VRAM

	rts


.scrn00	.equ	SCRATCH+$0000	; 
.scrn01	.equ	SCRATCH+$0800	; 
.scrn02	.equ	SCRATCH+$1000	; 
.scrn03	.equ	SCRATCH+$1800	; 


*****************************************************************************
*
* Map tables stored in HDMA.ASM

.setuphdma
	ldx	z_levelword	; Preload normal game mode index

	lda	v_MapGUI	; Is Map GUI level selector?
	beq	.makindex	; No  -

	lda	v_DebugMode	; Is Master Debug enabled?
	beq	.makindex	; No  -

	bra	.makindex	; Bypass !!!!!!!

	lda	#OPARTEMPLE
	asl
	tax
.makindex
	lda	>d_win1rdef,x
	tax

	beq	.exitdmaset	

	SAI

	lda	#0
	sta	z_hdmaenable	; Start with no channels enabled 

	lda	#$00
	sta	DMA3INC	; HDMA parameter

	lda	#$27	; Address $2127 = WH1
	sta	DMA3DEST

	lda	#0	; PPU registers are in bank 0
	sta	DMA3BANK	; Bank of dest address

	lda	#0	; Data Table Address
	sta	DMA3A1B	; Address Bank

	lda	>d_maplow,x	; Data Table Address
	sta	DMA3A1L	; Address Low

	lda	>d_maphigh,x	; Data Table Address
	sta	DMA3A1H	; Address High

	lda	#$8	; Enable HDMA Channel 3
	sta	z_hdmaenable
	sta	HDMAEN

	LAI
.exitdmaset
	rts


*****************************************
*

MovClouds:
	dec	cloudtime
	bpl	.exit

	lda	#10
	sta	cloudtime

	dec	v_bg3hofs	; Get BG3 X offset
	dec	v_bg3hofs	; Get BG3 X offset

	inc	v_bg3vofs	; Get BG3 Y offset
.exit	
	rtl

cloudtime	.equ	v_savpAI


*****************************************

XINC	.equ	10
YINC	.equ	3

Updatedggr:
	lda	z_level
	and	#$FF
	sta	v_daggerdone	; Assume we're done

	lda	v_daggerx	; Get daggers current X position
	cmp	v_foex	; Same at target spot?
	beq	.dodagy	; Yes - Do Y pos then

	stz	v_daggerdone	; No  - Find which direction to go

	sec	      	
	sbc	v_foex
	bmi	.goright

	cmp	#XINC
	bcs	.fullmovxr

	lda	v_foex
	sta	v_daggerx

	bra	.dodagy
.fullmovxr
	lda	v_daggerx
	sec
	sbc	#XINC
	sta	v_daggerx

	bra	.dodagy

.goright
	eor	#$FFFF
	inc
	cmp	#XINC
	bcs	.fullmovxl

	lda	v_foex
	sta	v_daggerx

	bra	.dodagy

.fullmovxl
	lda	v_daggerx
	clc
	adc	#XINC
	sta	v_daggerx

.dodagy
	lda	v_daggery	; Get daggers current Y position
	cmp	v_foey	; Same at target spot?
	beq	.exitupdag	; Yes - Then exit this procedure

	stz	v_daggerdone	; No  - Find which direction to go

	sec	      	
	sbc	v_foey
	bmi	.godown

	cmp	#YINC
	bcs	.fullmovyu

	lda	v_foey
	sta	v_daggery

	bra	.exitupdag
.fullmovyu
	lda	v_daggery
	sec
	sbc	#YINC
	sta	v_daggery

	bra	.exitupdag

.godown
	eor	#$FFFF
	inc
	cmp	#YINC
	bcs	.fullmovyd

	lda	v_foey
	sta	v_daggery

	bra	.exitupdag

.fullmovyd
	lda	v_daggery
	clc
	adc	#YINC
	sta	v_daggery

.exitupdag
	ldy	#0	; One and only frame

	lda	#^dagger1	; Point to dagger data structure
	sta	z_dreccount

	lda	#dagger1	; Point to dagger data structure
	sta	z_spridrec

	lda	#OBJBASE	; Set to base

	jsl	Sendframe	; Queue it up

	ldx	z_levelword
	lda	>d_daggerflip,x
	sta	v_daggerflip	

	rtl



*****************************************************************************

PlotDagger:
	ldy	z_attrcount	; Get attr counter

	lda	v_daggerx	; Get X pos
	sta	ATTRX,y

	lda	v_daggery	; Get Y pos
	sta	ATTRY,y

	lda	#0	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^dagger1	; Point to dagger data structure
	sta	ATTRBANK,y

	lda	#dagger1	; Point to dagger data structure
	sta	ATTRDREC,y

	lda	v_daggerflip	
	sta	ATTROFF,y

	iny
	iny

	sty	z_attrcount

	rtl


*****************************************************************************

Loaddagrpal:
	lda	#4	; Logical Palette number CGRAM
	ldx	#20	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loadtitlepal:
	lda	#5	; Logical Palette number CGRAM
	ldx	#24	; PALSET table index pointer (word)

	jsl	Loadsprpal


	rtl


*****************************************************************************
*
* Get X,Y values for BG1, Dagger Start, Dagger Stop
*
* Pre-Fadein...
* 

Spotdagger:
	ldx	z_levelword         ; Get level

	lda	>d_spotplacesbg1h,x ; Get Map X
	sta	v_bg1hofs

	lda	>d_spotplacesbg1v,x ; Get Map Y
	sta	v_bg1vofs

	lda	>d_spotplacesbg3h,x ; Starting Knife X
	sta	v_daggerx

	lda	>d_spotplacesbg3v,x ; Starting Knife Y
	sta	v_daggery

	lda	>d_daggerstopx,x    ; Dagger stop X
	sta	v_foex

	lda	>d_daggerstopy,x    ; Dagger stop Y
	sta	v_foey

	lda	v_MapGUI	; Is Map GUI level selector?
	beq	.exit	; No  -

	lda	v_DebugMode	; Is Master Debug enabled?
	beq	.exit	; No  -

	bra	.exit	; Bypass !!!!!!!

	lda	>d_daggerstopx,x    ; Dagger stop X
	sta	v_daggerx

	lda	>d_daggerstopy,x    ; Dagger stop Y
	sta	v_daggery
.exit
	rtl


******************************************
*
* Returns level in x; -1 in x if no level
*

Detectxy:
	LAI

	ldx	#42

.loop		           
	cpx	#BEACHBAY*2	; Level doesn't exist
	bne	.chkhlds

	jmp	.exit
.chkhlds
	cpx	#SHIPHOLDB*2
	bne	.chkhlda

	jmp	.exit
.chkhlda
	cpx	#SHIPHOLDA*2
	bne	.validlvl

	jmp	.exit

.validlvl
	lda	>d_spotplacesbg1h,x ; Get selected level BG1 x
	sec
	sbc	#4	; Set left range (push map right)
	
	cmp	v_bg1hofs	; Compare to current setting
	bcc	.okchk1hul	; Chk right if current still >=

	jmp	.exit	; Otherwise, not close enough to spot

.okchk1hul
	lda	>d_spotplacesbg1h,x
	clc
	adc	#4	; Set right range (push map left)

	cmp	v_bg1hofs	; Compare to current setting
	bcs	.okchk1vll	; Chk up if current still <

	jmp	.exit	; Otherwise, not close enough to spot

.okchk1vll
	lda	>d_spotplacesbg1v,x
	sec
	sbc	#4	; Set down range (push map up)
	
	cmp	v_bg1vofs	; Compare to current setting
	bcc	.okchk1vul	; Chk down if current still >=

	jmp	.exit	; Otherwise, not close enough to spot
		   
.okchk1vul
	lda	>d_spotplacesbg1v,x
	clc
	adc	#4	; Set up range (push map down)

	cmp	v_bg1vofs	; Compare to current setting
	bcs	.okchk3hll	; Chk dagger pos if current still <

	jmp	.exit	; Otherwise, not close enough to spot


.okchk3hll
	lda	>d_dbdagstopx,x
	sec
	sbc	#4
	
	cmp	v_daggerx
	bcc	.okchk3hul

	jmp	.exit

.okchk3hul
	lda	>d_dbdagstopx,x
	clc
	adc	#4

	cmp	v_daggerx
	bcs	.okchk3vll

	jmp	.exit

.okchk3vll
	lda	>d_dbdagstopy,x
	sec
	sbc	#4
	
	cmp	v_daggery
	bcc	.okchk3vul

	jmp	.exit

.okchk3vul
	lda	>d_dbdagstopy,x
	clc
	adc	#4

	cmp	v_daggery
	bcs	.locate

	jmp	.exit

.locate
	txa
	sta	z_levelword
	lsr
	sta	z_level
	tax

	rtl

.exit
	dex
	dex
	beq	.fail

	jmp	.loop

.fail
	ldx	#-1
	lda	tmplvl
	sta	z_level

	lda	tmplvlword
	sta	z_levelword

	rtl


tmplvl	.equ	v_foetype+10
tmplvlword	.equ	v_foetype+12


*****************************************************************************
*
* BG1 x,y
*

d_spotplacesbg1h
	.dw	$00F8	; Null Level
	.dw	$00F8	; Jungle 1
	.dw	$00B0	; Tarzans Home
	.dw	$0086	; Apes Clearing
	.dw	$008A	; Swamp

	.dw	$0016	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$0006	; Reef
	.dw	$002A	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0000	; Ship Hold B
	.dw	$0006	; Jungle 3
	.dw	$0012	; Downriver
	.dw	$00F8	; Jungle 4
	.dw	$00F8	; UpRiver

	.dw	$00FC	; Waterfall
	.dw	$00FC	; Caverns
	.dw	$0006	; Opar - Street
	.dw	$0006	; Opar - Wells
	.dw	$0006	; Opar - Temple

	.dw	$0006	; Opar - Shamon
		     
	.dw	$00F8	; Bonus Level - Exterior
	.dw	$00F8	; Bonus Level - Interior



d_spotplacesbg1v
	.dw	$000C	; Null Level
	.dw	$0006	; Jungle 1
	.dw	$0006	; Tarzans Home
	.dw	$0006	; Apes Clearing
	.dw	$0018	; Swamp

	.dw	$0006	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$0006	; Reef
	.dw	$0006	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0000	; Ship Hold B
	.dw	$0014	; Jungle 3
	.dw	$0044	; Downriver
	.dw	$00CA	; Jungle 4
	.dw	$0106	; UpRiver

	.dw	$0120	; Waterfall
	.dw	$010A	; Caverns
	.dw	$011C	; Opar - Street
	.dw	$011E	; Opar - Wells
	.dw	$010C	; Opar - Temple
			
	.dw	$010C	; Opar - Shamon

	.dw	$0064	; Bonus Level - Exterior
	.dw	$00B4	; Bonus Level - Interior

*****************************************************************************
*
* Dagger Start x,y
*

d_spotplacesbg3h
	.dw	$0000	; Null Level
	.dw	$FFD8	; Jungle 1
	.dw	$FFD8	; Tarzans Home
	.dw	$FFD8	; Apes Clearing
	.dw	$0130	; Swamp

	.dw	$FFD8	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$0000	; Reef
	.dw	$0130	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0000	; Ship Hold B
	.dw	$0130	; Jungle 3
	.dw	$0000	; Downriver
	.dw	$0130	; Jungle 4
	.dw	$0000	; UpRiver

	.dw	$0130	; Waterfall
	.dw	$0130	; Caverns
	.dw	$0000	; Opar - Street
	.dw	$0000	; Opar - Wells
	.dw	$0130	; Opar - Temple

	.dw	$0000	; Opar - Shamon
		     
	.dw	$0000	; Bonus Level - Exterior
	.dw	$0000	; Bonus Level - Interior

d_spotplacesbg3v
	.dw	$00F0	; Null Level
	.dw	$0026	; Jungle 1
	.dw	$0072	; Tarzans Home
	.dw	$0038	; Apes Clearing
	.dw	$0044	; Swamp

	.dw	$0020	; Beach
	.dw	$0020	; Beach-Bay
	.dw	$002A	; Reef
	.dw	$0078	; Ship
	.dw	$0020	; Ship Hold A

	.dw	$0020	; Ship Hold B
	.dw	$0030	; Jungle 3
	.dw	$0086	; Downriver
	.dw	$00C0	; Jungle 4
	.dw	$0060	; UpRiver

	.dw	$0070	; Waterfall
	.dw	$0076	; Caverns
	.dw	$006A	; Opar - Street
	.dw	$00D0	; Opar - Wells
	.dw	$006A	; Opar - Temple

	.dw	$00F0	; Opar - Shamon
		
	.dw	$00F0	; Bonus Level - Exterior
	.dw	$00F0	; Bonus Level - Interior


*****************************************************************************
*
* Dagger Stop x,y
*

d_daggerstopx
	.dw	$0000	; Null Level
	.dw	$00AA	; Jungle 1
	.dw	$00B0	; Tarzans Home
	.dw	$0092	; Apes Clearing
	.dw	$006C	; Swamp

	.dw	$00B2	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$007E	; Reef
	.dw	$0064	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0080	; Ship Hold B
	.dw	$0038	; Jungle 3
	.dw	$0080	; Downriver
	.dw	$0080	; Jungle 4
	.dw	$0080	; UpRiver

	.dw	$0047	; Waterfall
	.dw	$0047	; Caverns
	.dw	$008B	; Opar - Street
	.dw	$0092	; Opar - Wells
	.dw	$003E	; Opar - Temple

	.dw	$007E	; Opar - Shamon
		
	.dw	$0090	; Bonus Level - Exterior
	.dw	$0080	; Bonus Level - Interior


d_daggerstopy
	.dw	$0080	; Null Level
	.dw	$0065	; Jungle 1
	.dw	$0032	; Tarzans Home
	.dw	$0070	; Apes Clearing
	.dw	$0080	; Swamp

	.dw	$0062	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$004E	; Reef
	.dw	$003A	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0080	; Ship Hold B
	.dw	$0078	; Jungle 3
	.dw	$00AA	; Downriver
	.dw	$00A0	; Jungle 4
	.dw	$0080	; UpRiver

	.dw	$00AC	; Waterfall
	.dw	$0098	; Caverns
	.dw	$008B	; Opar - Street
	.dw	$00B6	; Opar - Wells
	.dw	$00AA	; Opar - Temple

	.dw	$007E	; Opar - Shamon
		
	.dw	$0060	; Bonus Level - Exterior
	.dw	$0080	; Bonus Level - Interior

	; Dagger flip Top 2 bits OR'd with lower 2 bits
	;
	;	0 - No    flip
	;	1 - X     flip
	;	2 - Y     flip
	;	3 - X & Y flip
d_daggerflip
	.dw	$0000	; Null Level
	.dw	$6800	; Jungle 1
	.dw	$E800	; Tarzans Home
	.dw	$6800	; Apes Clearing
	.dw	$2800	; Swamp

	.dw	$6800	; Beach
	.dw	$6800	; Beach-Bay
	.dw	$6800	; Reef
	.dw	$A800	; Ship
	.dw	$6800	; Ship Hold A

	.dw	$6800	; Ship Hold B
	.dw	$2800	; Jungle 3
	.dw	$6800	; Downriver
	.dw	$A800	; Jungle 4
	.dw	$6800	; UpRiver

	.dw	$2800	; Waterfall
	.dw	$2800	; Caverns
	.dw	$6800	; Opar - Street
	.dw	$E800	; Opar - Wells
	.dw	$2800	; Opar - Temple

	.dw	$6800	; Opar - Shamon
		
	.dw	$E800	; Bonus Level - Exterior
	.dw	$6800	; Bonus Level - Interior


*****************************************************************************
*
* Debug Dagger Stop x,y
*

d_dbdagstopx
	.dw	$0000	; Null Level
	.dw	$00AA	; Jungle 1
	.dw	$00B0	; Tarzans Home
	.dw	$0092	; Apes Clearing
	.dw	$006C	; Swamp

	.dw	$00B2	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$007E	; Reef
	.dw	$0064	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0080	; Ship Hold B
	.dw	$0038	; Jungle 3
	.dw	$0080	; Downriver
	.dw	$0080	; Jungle 4
	.dw	$0080	; UpRiver

	.dw	$0047	; Waterfall
	.dw	$0047	; Caverns
	.dw	$008B	; Opar - Street
	.dw	$0092	; Opar - Wells
	.dw	$003E	; Opar - Temple

	.dw	$007E	; Opar - Shamon
		
	.dw	$0080	; Bonus Level - Exterior
	.dw	$0080	; Bonus Level - Interior


d_dbdagstopy
	.dw	$0080	; Null Level
	.dw	$0065	; Jungle 1
	.dw	$0032	; Tarzans Home
	.dw	$0070	; Apes Clearing
	.dw	$0080	; Swamp

	.dw	$0062	; Beach
	.dw	$0000	; Beach-Bay
	.dw	$004E	; Reef
	.dw	$003A	; Ship
	.dw	$0000	; Ship Hold A

	.dw	$0080	; Ship Hold B
	.dw	$0078	; Jungle 3
	.dw	$00AA	; Downriver
	.dw	$00A0	; Jungle 4
	.dw	$0080	; UpRiver

	.dw	$00AC	; Waterfall
	.dw	$0098	; Caverns
	.dw	$008B	; Opar - Street
	.dw	$00B6	; Opar - Wells
	.dw	$00AA	; Opar - Temple

	.dw	$007E	; Opar - Shamon
		
	.dw	$0080	; Bonus Level - Exterior
	.dw	$0080	; Bonus Level - Interior


*****************************************************************************
*
*
*

Dspleveltitle:
	ldy	#0
	ldx	z_levelword	; One and only frame

	lda	>d_titlebank,x	; Point to dagger data structure
	sta	z_dreccount

	lda	>d_titledrec,x	; Point to dagger data structure
	sta	z_spridrec

	lda	#$20
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; Set to base

	jsl	Sendframe	; Queue it up

	lda	#-40
	sta	z_temp0

	lda	#240
	sta	z_temp1

	lda	v_daggerdone	; Has dagger reached its target?
	beq	.disptitlenot	; No  -

	asl		; Word index
	tax

	lda	>d_titlex,x	; Get X pos
	sta	z_temp0

	lda	>d_titley,x	; Get Y pos
	sta	z_temp1

.disptitlenot
	ldy	z_attrcount	; Get attr counter

	lda	z_temp0
	sta	ATTRX,y

	lda	z_temp1
	sta	ATTRY,y

	lda	#0	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	>d_titlebank,x	; Point to dagger data structure
	sta	ATTRBANK,y

	lda	>d_titledrec,x	; Point to dagger data structure
	sta	ATTRDREC,y

	lda	#$2A20	; Attributes
	sta	ATTROFF,y

	iny
	iny

	sty	z_attrcount

	rtl



levtitlenull	.equ	00	; Null
levtitlejngl1	.equ	02	; Jungle 1
levtitletzhome	.equ	04	; Tarzans Home
levtitleapeclr	.equ	06	; Apes Clearing
levtitleswamp	.equ	08	; Swamp
levtitlebeach	.equ	10	; Beach
levtitlereef	.equ	14	; Reef
levtitleship	.equ	16	; Ship
levtitlejngl3	.equ	22	; Central Jungle
levtitlednrivr	.equ	24	; DownRiver
levtitlejngl4	.equ	26	; Upper Jungle
levtitleuprivr	.equ	28	; UpRiver
levtitlewatrfl	.equ	30	; Waterfall
levtitlecaves	.equ	32	; Caves
levtitlestreet	.equ	34	; Streets
levtitlewells	.equ	36	; Wells
levtitletemple	.equ	38	; Temple
levtitlebonus	.equ	40	; Bonus
levtitleshamon	.equ	42	; Shamon
		  	

* Sequence data

Nullseq	.equ	0

d_titleseq:
	.dw	Nullseq	;	00
	.dw	title01seq00	; Jungle 1	02
	.dw	title02seq00	; Tarzans Home	04
	.dw	title03seq00	; Apes Clearing	06
	.dw	title04seq00	; Swamp	08
	.dw	title05seq00	; Beach	10
	.dw	Nullseq	;	12
	.dw	title06seq00	; Reef	14
	.dw	title07seq00	; Ship	16
	.dw	title07seq00	; Ship	18
	.dw	title07seq00	; Ship	20
	.dw	title08seq00	; Jungle 3	22
	.dw	title09seq00	; Downriver	24
	.dw	title10seq00	; Jungle 4	26
	.dw	title11seq00	; UpRiver	28
	.dw	title12seq00	; Waterfall	30
	.dw	title13seq00	; Caverns	32
	.dw	title14seq00	; Opar - Street	34
	.dw	title15seq00	; Opar - Wells	36
	.dw	title16seq00	; Opar - Temple	38
	.dw	title18seq00	; Shamon Boss	42
	.dw	title17seq00	; Bonus Jungle	40
	.dw	Nullseq	;	44


* Sequence Mode Data Record

d_titledrec
	.dw	Nullseq	;
	.dw	title01	; Jungle 1
	.dw	title02	; Tarzans Home
	.dw	title03	; Apes Clearing
	.dw	title04	; Swamp
	.dw	title05	; Beach
	.dw	Nullseq	;
	.dw	title06	; Reef
	.dw	title07	; Ship
	.dw	title07	; Ship
	.dw	title07	; Ship
	.dw	title08	; Jungle 3
	.dw	title09	; Downriver
	.dw	title10	; Jungle 4
	.dw	title11	; UpRiver
	.dw	title12	; Waterfall
	.dw	title13	; Caverns
	.dw	title14	; Opar - Street
	.dw	title15	; Opar - Wells
	.dw	title16	; Opar - Temple
	.dw	title18	; Shamon Boss
	.dw	title17	; Bonus Jungle
	.dw	Nullseq	;


* Sequence data bank

d_titlebank
	.dw	^Nullseq	;
	.dw	^title01	; Jungle 1
	.dw	^title02	; Tarzans Home
	.dw	^title03	; Apes Clearing
	.dw	^title04	; Swamp
	.dw	^title05	; Beach
	.dw	^Nullseq	;
	.dw	^title06	; Reef
	.dw	^title07	; Ship
	.dw	^title07	; Ship
	.dw	^title07	; Ship
	.dw	^title08	; Jungle 3
	.dw	^title09	; Downriver
	.dw	^title10	; Jungle 4
	.dw	^title11	; UpRiver
	.dw	^title12	; Waterfall
	.dw	^title13	; Caverns
	.dw	^title14	; Opar - Street
	.dw	^title15	; Opar - Wells
	.dw	^title16	; Opar - Temple
	.dw	^title18	; Shamon Boss
	.dw	^title17	; Bonus Jungle
	.dw	^Nullseq	;


d_titlex
	.dw	$0080	; Null Level
	.dw	$0096	; Jungle 1
	.dw	$0090	; Tarzans Home
	.dw	$0078	; Apes Clearing
	.dw	$0040	; Swamp

	.dw	$0070	; Beach
	.dw	$0080	; Beach-Bay
	.dw	$0080	; Reef
	.dw	$0030	; Ship
	.dw	$0080	; Ship Hold A

	.dw	$0080	; Ship Hold B
	.dw	$003C	; Jungle 3
	.dw	$0090	; Downriver
	.dw	$0076	; Jungle 4
	.dw	$0098	; UpRiver

	.dw	$007C	; Waterfall
	.dw	$0086	; Caverns
	.dw	$0077	; Opar - Street
	.dw	$0050	; Opar - Wells
	.dw	$0020	; Opar - Temple

	.dw	$0080	; Opar - Shamon
		
	.dw	$0080	; Bonus Level - Exterior
	.dw	$0080	; Bonus Level - Interior


d_titley
	.dw	$006A	; Null Level
	.dw	$0085	; Jungle 1
	.dw	$0060	; Tarzans Home
	.dw	$009A	; Apes Clearing
	.dw	$00A2	; Swamp

	.dw	$007B	; Beach
	.dw	$006A	; Beach-Bay
	.dw	$0056	; Reef
	.dw	$004A	; Ship
	.dw	$006A	; Ship Hold A

	.dw	$006A	; Ship Hold B
	.dw	$00AD	; Jungle 3
	.dw	$00B7	; Downriver
	.dw	$0080	; Jungle 4
	.dw	$0086	; UpRiver

	.dw	$0078	; Waterfall
	.dw	$00B4	; Caverns
	.dw	$0070	; Opar - Street
	.dw	$007A	; Opar - Wells
	.dw	$0090	; Opar - Temple

	.dw	$006A	; Opar - Shamon
		
	.dw	$006A	; Bonus Level - Exterior
	.dw	$006A	; Bonus Level - Interior


*****************************************************************************
*
* Color Cycling code for Beach Level
*
*****************************************************************************

WP2CYCSTART	.equ	$25
WP4CYCSTART	.equ	$41
WP5CYCSTART	.equ	$51
WP6CYCSTART	.equ	$6B
WP7CYCSTART	.equ	$71

WCYCDLY1	.equ	10
WCYCDLY2	.equ	14
WCYCDLY3	.equ	18
WCYCDLY4	.equ	14
WCYCDLY5	.equ	38



_beachcycp2
	.dw	$7FDA,$7AD2
	.dw	$7FDA,$7AD2

_beachcycp4
	.dw	$635A,$635A,$635A,$7FFF,$635A
	.dw	$635A,$635A,$635A,$7FFF,$635A

_beachcycp5
	.dw	$7FFF,$7FB7,$7F72,$7F2B,$7EE6,$7AA5,$7A81
	.dw	$7FFF,$7FB7,$7F72,$7F2B,$7EE6,$7AA5,$7A81

_beachcycp6
	.dw	$635A,$635A,$635A,$7FFF,$635A
	.dw	$635A,$635A,$635A,$7FFF,$635A

_beachcycp7
	.dw	$7FFF,$7FB7,$7F72,$7F2B,$7EE6,$7AA5,$7A81
	.dw	$7FFF,$7FB7,$7F72,$7F2B,$7EE6,$7AA5,$7A81




;	Cycle Beach

Cyclewaves:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#WCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#8
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#WP4CYCSTART
	sta	CGADD

	lda	>_beachcycp4,x
	sta	CGDATA

	lda	>_beachcycp4+1,x
	sta	CGDATA

	lda	>_beachcycp4+2,x
	sta	CGDATA

	lda	>_beachcycp4+3,x
	sta	CGDATA

	lda	>_beachcycp4+4,x
	sta	CGDATA

	lda	>_beachcycp4+5,x
	sta	CGDATA

	lda	>_beachcycp4+6,x
	sta	CGDATA

	lda	>_beachcycp4+7,x
	sta	CGDATA

	lda	>_beachcycp4+8,x
	sta	CGDATA

	lda	>_beachcycp4+9,x
	sta	CGDATA
	LA

.chk2ndcycle
	dec	v_cycledly2
	bpl	.chk3rdcycle

	lda	#WCYCDLY2
	sta	v_cycledly2

	lda	v_colorcel2
	dec
	dec
	bpl	.okcyc2

	lda	#12
.okcyc2
	sta	v_colorcel2
	tax

	SA
	lda	#WP5CYCSTART
	sta	CGADD

	lda	>_beachcycp5,x
	sta	CGDATA

	lda	>_beachcycp5+1,x
	sta	CGDATA

	lda	>_beachcycp5+2,x
	sta	CGDATA

	lda	>_beachcycp5+3,x
	sta	CGDATA

	lda	>_beachcycp5+4,x
	sta	CGDATA

	lda	>_beachcycp5+5,x
	sta	CGDATA

	lda	>_beachcycp5+6,x
	sta	CGDATA

	lda	>_beachcycp5+7,x
	sta	CGDATA

	lda	>_beachcycp5+8,x
	sta	CGDATA

	lda	>_beachcycp5+9,x
	sta	CGDATA

	lda	>_beachcycp5+10,x
	sta	CGDATA

	lda	>_beachcycp5+11,x
	sta	CGDATA

	lda	>_beachcycp5+12,x
	sta	CGDATA

	lda	>_beachcycp5+13,x
	sta	CGDATA
	LA

.chk3rdcycle
	lda	v_cyclepass	; Password screen?
	beq	.notskip

	jmp	.endcycle
.notskip
	dec	v_cycledly3
	bpl	.chk4thcycle

	lda	#WCYCDLY3
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#8
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#WP6CYCSTART
	sta	CGADD

	lda	>_beachcycp6,x
	sta	CGDATA

	lda	>_beachcycp6+1,x
	sta	CGDATA

	lda	>_beachcycp6+2,x
	sta	CGDATA

	lda	>_beachcycp6+3,x
	sta	CGDATA

	lda	>_beachcycp6+4,x
	sta	CGDATA

	lda	>_beachcycp6+5,x
	sta	CGDATA

	lda	>_beachcycp6+6,x
	sta	CGDATA

	lda	>_beachcycp6+7,x
	sta	CGDATA

	lda	>_beachcycp6+8,x
	sta	CGDATA

	lda	>_beachcycp6+9,x
	sta	CGDATA
	LA

.chk4thcycle
	dec	v_cycledly4
	bpl	.endcycle

	lda	#WCYCDLY4
	sta	v_cycledly4

	lda	v_colorcel4
	dec
	dec
	bpl	.okcyc4

	lda	#12
.okcyc4
	sta	v_colorcel4
	tax

	SA
	lda	#WP7CYCSTART
	sta	CGADD

	lda	>_beachcycp7,x
	sta	CGDATA

	lda	>_beachcycp7+1,x
	sta	CGDATA

	lda	>_beachcycp7+2,x
	sta	CGDATA

	lda	>_beachcycp7+3,x
	sta	CGDATA

	lda	>_beachcycp7+4,x
	sta	CGDATA

	lda	>_beachcycp7+5,x
	sta	CGDATA

	lda	>_beachcycp7+6,x
	sta	CGDATA

	lda	>_beachcycp7+7,x
	sta	CGDATA

	lda	>_beachcycp7+8,x
	sta	CGDATA

	lda	>_beachcycp7+9,x
	sta	CGDATA

	lda	>_beachcycp7+10,x
	sta	CGDATA

	lda	>_beachcycp7+11,x
	sta	CGDATA

	lda	>_beachcycp7+12,x
	sta	CGDATA

	lda	>_beachcycp7+13,x
	sta	CGDATA
	LA

.endcycle
	rtl

;.chk5thcycle
;	dec	v_cycledly5
;	bpl	.endcycle
;
;	lda	#WCYCDLY5
;	sta	v_cycledly5
;
;	lda	v_colorcel5
;	dec
;	dec
;	bpl	.okcyc5
;
;	lda	#2
;.okcyc5
;	sta	v_colorcel5
;	tax
;
;	SA
;	lda	#WP2CYCSTART
;	sta	CGADD
;
;	lda	>_beachcycp2,x
;	sta	CGDATA
;
;	lda	>_beachcycp2+1,x
;	sta	CGDATA
;
;	lda	>_beachcycp2+2,x
;	sta	CGDATA
;
;	lda	>_beachcycp2+3,x
;	sta	CGDATA
;	LA
;
;.endcycle
;	rtl
;
*****************************************************************************
*
* Color Cycling code for Waterfall
*
*****************************************************************************

FP4CYCSTART	.equ	$2A
FP6CYCSTART	.equ	$49
FP7CYCSTART	.equ	$7A

FCYCDLY1	.equ	8
FCYCDLY2	.equ	10
FCYCDLY3	.equ	6


_fallscycp4
	.dw	$7FBB,$7F9A,$7F79,$7F79,$7F58,$7F37
	.dw	$7FBB,$7F9A,$7F79,$7F79,$7F58,$7F37
_fallscycp6
	.dw	$7F33,$418C,$1042,$7F58,$1C84,$7FBC,$7FFF
	.dw	$7F33,$418C,$1042,$7F58,$1C84,$7FBC,$7FFF
_fallscycp7
	.dw	$0000,$0000
	.dw	$0000,$0000


Cyclefalls:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#FCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#10
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#FP4CYCSTART
	sta	CGADD

	lda	>_fallscycp4,x
	sta	CGDATA

	lda	>_fallscycp4+1,x
	sta	CGDATA

	lda	>_fallscycp4+2,x
	sta	CGDATA

	lda	>_fallscycp4+3,x
	sta	CGDATA

	lda	>_fallscycp4+4,x
	sta	CGDATA

	lda	>_fallscycp4+5,x
	sta	CGDATA

	lda	>_fallscycp4+6,x
	sta	CGDATA

	lda	>_fallscycp4+7,x
	sta	CGDATA

	lda	>_fallscycp4+8,x
	sta	CGDATA

	lda	>_fallscycp4+9,x
	sta	CGDATA

	lda	>_fallscycp4+10,x
	sta	CGDATA

	lda	>_fallscycp4+11,x
	sta	CGDATA
	LA

.chk2ndcycle
	dec	v_cycledly3
	bmi	.docycle3

	jmp	.endcycle
.docycle3
	lda	#FCYCDLY2
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#12
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#FP6CYCSTART
	sta	CGADD

	lda	>_fallscycp6,x
	sta	CGDATA

	lda	>_fallscycp6+1,x
	sta	CGDATA

	lda	>_fallscycp6+2,x
	sta	CGDATA

	lda	>_fallscycp6+3,x
	sta	CGDATA

	lda	>_fallscycp6+4,x
	sta	CGDATA

	lda	>_fallscycp6+5,x
	sta	CGDATA

	lda	>_fallscycp6+6,x
	sta	CGDATA

	lda	>_fallscycp6+7,x
	sta	CGDATA

	lda	>_fallscycp6+8,x
	sta	CGDATA

	lda	>_fallscycp6+9,x
	sta	CGDATA

	lda	>_fallscycp6+10,x
	sta	CGDATA

	lda	>_fallscycp6+11,x
	sta	CGDATA

	lda	>_fallscycp6+12,x
	sta	CGDATA

	lda	>_fallscycp6+13,x
	sta	CGDATA
	LA

	jmp	.endcycle

.chk4thcycle
;	dec	v_cycledly4
;	bpl	.endcycle
;
;	lda	#FCYCDLY3
;	sta	v_cycledly4
;
;	lda	v_colorcel4
;	dec
;	dec
;	bpl	.okcyc4
;
;	lda	#10
;.okcyc4
;	sta	v_colorcel4
;	tax
;
;	SA
;	lda	#FP7CYCSTART
;	sta	CGADD
;
;	lda	>_fallscycp7,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+1,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+2,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+3,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+4,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+5,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+6,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+7,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+8,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+9,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+10,x
;	sta	CGDATA
;
;	lda	>_fallscycp7+11,x
;	sta	CGDATA
;	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for Ship Level
*
*****************************************************************************

SP1CYCSTART	.equ	$28
SP2CYCSTART	.equ	$24
SP3CYCSTART	.equ	$21

SCYCDLY1	.equ	12
SCYCDLY2	.equ	16
SCYCDLY3	.equ	30


_shipcycp1
	.dw	$7E25,$7E25,$7E25,$7EA5
	.dw	$7E25,$7E25,$7E25,$7EA5

_shipcycp2
	.dw	$7EA5,$7EA5,$7EA5,$7F0E
	.dw	$7EA5,$7EA5,$7EA5,$7F0E

_shipcycp3
	.dw	$7F0E,$7F0E,$7F73
	.dw	$7F0E,$7F0E,$7F73




Cycleship:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#SCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#6
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#SP1CYCSTART
	sta	CGADD

	lda	>_shipcycp1,x
	sta	CGDATA

	lda	>_shipcycp1+1,x
	sta	CGDATA

	lda	>_shipcycp1+2,x
	sta	CGDATA

	lda	>_shipcycp1+3,x
	sta	CGDATA

	lda	>_shipcycp1+4,x
	sta	CGDATA

	lda	>_shipcycp1+5,x
	sta	CGDATA

	lda	>_shipcycp1+6,x
	sta	CGDATA

	lda	>_shipcycp1+7,x
	sta	CGDATA
	LA

.chk2ndcycle
	dec	v_cycledly3
	bpl	.chk4thcycle

	lda	#SCYCDLY2
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#6
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#SP2CYCSTART
	sta	CGADD

	lda	>_shipcycp2,x
	sta	CGDATA

	lda	>_shipcycp2+1,x
	sta	CGDATA

	lda	>_shipcycp2+2,x
	sta	CGDATA

	lda	>_shipcycp2+3,x
	sta	CGDATA

	lda	>_shipcycp2+4,x
	sta	CGDATA

	lda	>_shipcycp2+5,x
	sta	CGDATA

	lda	>_shipcycp2+6,x
	sta	CGDATA

	lda	>_shipcycp2+7,x
	sta	CGDATA
	LA

.chk4thcycle
	dec	v_cycledly4
	bpl	.endcycle

	lda	#SCYCDLY3
	sta	v_cycledly4

	lda	v_colorcel4
	dec
	dec
	bpl	.okcyc4

	lda	#4
.okcyc4
	sta	v_colorcel4
	tax

	SA
	lda	#SP3CYCSTART
	sta	CGADD

	lda	>_shipcycp3,x
	sta	CGDATA

	lda	>_shipcycp3+1,x
	sta	CGDATA

	lda	>_shipcycp3+2,x
	sta	CGDATA

	lda	>_shipcycp3+3,x
	sta	CGDATA

	lda	>_shipcycp3+4,x
	sta	CGDATA

	lda	>_shipcycp3+5,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for Downriver
*
*****************************************************************************

DR2CYCSTART	.equ	$22
DR3CYCSTART	.equ	$35

DRCYCDLY1	.equ	11
DRCYCDLY2	.equ	7

_rivcycp2
	.dw	$55A7,$72CB,$5E47,$55A7,$4E09,$4560,$55A7
	.dw	$55A7,$72CB,$5E47,$55A7,$4E09,$4560,$55A7

_rivcycp3
	.dw	$7B36,$72D1,$6A6E,$662A
	.dw	$7B36,$72D1,$6A6E,$662A




Cyclednriv:
	dec	v_cycledly1
	bmi	.docycl1

	jmp	.chk2ndcycle
.docycl1
	lda	#DRCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#12
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#DR2CYCSTART
	sta	CGADD

	lda	>_rivcycp2,x
	sta	CGDATA

	lda	>_rivcycp2+1,x
	sta	CGDATA

	lda	>_rivcycp2+2,x
	sta	CGDATA

	lda	>_rivcycp2+3,x
	sta	CGDATA

	lda	>_rivcycp2+4,x
	sta	CGDATA

	lda	>_rivcycp2+5,x
	sta	CGDATA

	lda	>_rivcycp2+6,x
	sta	CGDATA

	lda	>_rivcycp2+7,x
	sta	CGDATA

	lda	>_rivcycp2+8,x
	sta	CGDATA

	lda	>_rivcycp2+9,x
	sta	CGDATA

	lda	>_rivcycp2+10,x
	sta	CGDATA

	lda	>_rivcycp2+11,x
	sta	CGDATA

	lda	>_rivcycp2+11,x
	sta	CGDATA

	lda	>_rivcycp2+12,x
	sta	CGDATA

	lda	>_rivcycp2+13,x
	sta	CGDATA
	LA

.chk2ndcycle
	lda	v_cyclepass	; Password screen?
	bne	.endcycle

	dec	v_cycledly3
	bpl	.endcycle

	lda	#DRCYCDLY2
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#6
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#DR3CYCSTART
	sta	CGADD

	lda	>_rivcycp3,x
	sta	CGDATA

	lda	>_rivcycp3+1,x
	sta	CGDATA

	lda	>_rivcycp3+2,x
	sta	CGDATA

	lda	>_rivcycp3+3,x
	sta	CGDATA

	lda	>_rivcycp3+4,x
	sta	CGDATA

	lda	>_rivcycp3+5,x
	sta	CGDATA

	lda	>_rivcycp3+6,x
	sta	CGDATA

	lda	>_rivcycp3+7,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for Upriver
*
*****************************************************************************

UR1CYCSTART	.equ	$21
UR2CYCSTART	.equ	$2C
UR3CYCSTART	.equ	$45

URCYCDLY1	.equ	14
URCYCDLY2	.equ	14
URCYCDLY3	.equ	10


_urivcycp1
	.dw	$7FDC,$7FB9,$7F96,$7F53,$7F30
	.dw	$7FDC,$7FB9,$7F96,$7F53,$7F30
_urivcycp2
	.dw	$7FFF,$358B,$7FFF
	.dw	$7FFF,$358B,$7FFF
_urivcycp3
	.dw	$7B36,$72D1,$6A6E,$664A
	.dw	$7B36,$72D1,$6A6E,$664A



Cycleupriv:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#URCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#8
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#UR1CYCSTART
	sta	CGADD

	lda	>_urivcycp1,x
	sta	CGDATA

	lda	>_urivcycp1+1,x
	sta	CGDATA

	lda	>_urivcycp1+2,x
	sta	CGDATA

	lda	>_urivcycp1+3,x
	sta	CGDATA

	lda	>_urivcycp1+4,x
	sta	CGDATA

	lda	>_urivcycp1+5,x
	sta	CGDATA

	lda	>_urivcycp1+6,x
	sta	CGDATA

	lda	>_urivcycp1+7,x
	sta	CGDATA

	lda	>_urivcycp1+8,x
	sta	CGDATA

	lda	>_urivcycp1+9,x
	sta	CGDATA
	LA

.chk2ndcycle
	dec	v_cycledly2
	bpl	.chk3ndcycle

	lda	#URCYCDLY2
	sta	v_cycledly2

	lda	v_colorcel2
	dec
	dec
	bpl	.okcyc2

	lda	#4
.okcyc2
	sta	v_colorcel2
	tax

	SA
	lda	#UR2CYCSTART
	sta	CGADD

	lda	>_urivcycp2,x
	sta	CGDATA

	lda	>_urivcycp2+1,x
	sta	CGDATA

	lda	>_urivcycp2+2,x
	sta	CGDATA

	lda	>_urivcycp2+3,x
	sta	CGDATA

	lda	>_urivcycp2+4,x
	sta	CGDATA

	lda	>_urivcycp2+5,x
	sta	CGDATA
	LA

.chk3ndcycle
	dec	v_cycledly3
	bpl	.endcycle

	lda	#URCYCDLY3
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#6
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#UR3CYCSTART
	sta	CGADD

	lda	>_urivcycp3,x
	sta	CGDATA

	lda	>_urivcycp3+1,x
	sta	CGDATA

	lda	>_urivcycp3+2,x
	sta	CGDATA

	lda	>_urivcycp3+3,x
	sta	CGDATA

	lda	>_urivcycp3+4,x
	sta	CGDATA

	lda	>_urivcycp3+5,x
	sta	CGDATA

	lda	>_urivcycp3+6,x
	sta	CGDATA

	lda	>_urivcycp3+7,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for flames in Temple
*
*****************************************************************************

TF4CYCSTART	.equ	$41

TFCYCDLY1	.equ	7


_tmpflmcycp4
	.dw	$077B,$06D9,$0678,$05F6,$0595,$30AC
	.dw	$077B,$06D9,$0678,$05F6,$0595,$30AC


Cycleflames:
	dec	v_cycledly1
	bpl	.endcycle

	lda	#TFCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#10
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#TF4CYCSTART
	sta	CGADD

	lda	>_tmpflmcycp4,x
	sta	CGDATA

	lda	>_tmpflmcycp4+1,x
	sta	CGDATA

	lda	>_tmpflmcycp4+2,x
	sta	CGDATA

	lda	>_tmpflmcycp4+3,x
	sta	CGDATA

	lda	>_tmpflmcycp4+4,x
	sta	CGDATA

	lda	>_tmpflmcycp4+5,x
	sta	CGDATA

	lda	>_tmpflmcycp4+6,x
	sta	CGDATA

	lda	>_tmpflmcycp4+7,x
	sta	CGDATA

	lda	>_tmpflmcycp4+8,x
	sta	CGDATA

	lda	>_tmpflmcycp4+9,x
	sta	CGDATA

	lda	>_tmpflmcycp4+10,x
	sta	CGDATA

	lda	>_tmpflmcycp4+11,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for port hole in shipholds
*
*****************************************************************************

SH1CYCSTART	.equ	$78
SH2CYCSTART	.equ	$78

SHCYCDLY1	.equ	20
SHCYCDLY2	.equ	20


_shphldcycp1
	.dw	$779C,$779C,$779C,$779C,$7E60,$7E60,$7E60,$7E60
	.dw	$779C,$779C,$779C,$779C,$7E60,$7E60,$7E60,$7E60

_shphldcycp2
	.dw	$779C,$779C,$779C,$779C,$7E60,$7E60,$7E60,$7E60
	.dw	$779C,$779C,$779C,$779C,$7E60,$7E60,$7E60,$7E60



Cycleshphld:
	lda	v_waveroll
	beq	.firstcycle

	jmp	.secndcycle

.firstcycle
	dec	v_cycledly1
	bmi	.reloadcycl

	jmp	.endcycle
.reloadcycl
	lda	#SHCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	cmp	#6
	bcs	.rstcel1

	inc
	inc
	sta	v_colorcel1

	bra	.okcyc1
.rstcel1
	inc
	inc
	sta	v_colorcel1

	lda	v_waveroll
	eor	#1
	sta	v_waveroll

	stz	v_colorcel2
.okcyc1
	lda	v_colorcel1
	tax

	SA
	lda	#SH1CYCSTART
	sta	CGADD

	lda	>_shphldcycp1,x
	sta	CGDATA

	lda	>_shphldcycp1+1,x
	sta	CGDATA

	lda	>_shphldcycp1+2,x
	sta	CGDATA

	lda	>_shphldcycp1+3,x
	sta	CGDATA

	lda	>_shphldcycp1+4,x
	sta	CGDATA

	lda	>_shphldcycp1+5,x
	sta	CGDATA

	lda	>_shphldcycp1+6,x
	sta	CGDATA

	lda	>_shphldcycp1+7,x
	sta	CGDATA

	lda	>_shphldcycp1+8,x
	sta	CGDATA

	lda	>_shphldcycp1+9,x
	sta	CGDATA

	lda	>_shphldcycp1+10,x
	sta	CGDATA

	lda	>_shphldcycp1+11,x
	sta	CGDATA

	lda	>_shphldcycp1+12,x
	sta	CGDATA

	lda	>_shphldcycp1+13,x
	sta	CGDATA

	lda	>_shphldcycp1+14,x
	sta	CGDATA

	lda	>_shphldcycp1+15,x
	sta	CGDATA
	LA

.endcycle
	rtl

.secndcycle
	dec	v_cycledly2
	bmi	.reloadcycl2

	jmp	.endcycle2
.reloadcycl2
	lda	#SHCYCDLY2
	sta	v_cycledly2

	lda	v_colorcel2
	cmp	#6
	bcs	.rstcel2

	inc
	inc
	sta	v_colorcel2

	bra	.okcyc2
.rstcel2
	inc
	inc
	sta	v_colorcel2

	lda	v_waveroll
	eor	#1
	sta	v_waveroll

	stz	v_colorcel1
.okcyc2
	lda	v_colorcel2
	tax

	SA
	lda	#SH2CYCSTART
	sta	CGADD

	lda	>_shphldcycp2,x
	sta	CGDATA

	lda	>_shphldcycp2+1,x
	sta	CGDATA

	lda	>_shphldcycp2+2,x
	sta	CGDATA

	lda	>_shphldcycp2+3,x
	sta	CGDATA

	lda	>_shphldcycp2+4,x
	sta	CGDATA

	lda	>_shphldcycp2+5,x
	sta	CGDATA

	lda	>_shphldcycp2+6,x
	sta	CGDATA

	lda	>_shphldcycp2+7,x
	sta	CGDATA

	lda	>_shphldcycp2+8,x
	sta	CGDATA

	lda	>_shphldcycp2+9,x
	sta	CGDATA

	lda	>_shphldcycp2+10,x
	sta	CGDATA

	lda	>_shphldcycp2+11,x
	sta	CGDATA

	lda	>_shphldcycp2+12,x
	sta	CGDATA

	lda	>_shphldcycp2+13,x
	sta	CGDATA

	lda	>_shphldcycp2+14,x
	sta	CGDATA

	lda	>_shphldcycp2+15,x
	sta	CGDATA
	LA

.endcycle2
	rtl



*****************************************************************************
*
* Color Cycling code for surface of Bay
*
*****************************************************************************

BS6CYCSTARTa	.equ	$67	; Index pointer to word entry in pal
BS6CYCSTARTb	.equ	$6A	; Index pointer to word entry in pal
BS6CYCSTARTc	.equ	$6D	; Index pointer to word entry in pal

BSCYCDLY1a	.equ	18	; Cycle Delay
BSCYCDLY1b	.equ	18	; Cycle Delay
BSCYCDLY1c	.equ	18	; Cycle Delay

_baysrfcycp6a
	.dw	$5560,$61C0,$61C0
	.dw	$5560,$61C0,$61C0
_baysrfcycp6b
	.dw	$4D00,$5560,$5560
	.dw	$4D00,$5560,$5560
_baysrfcycp6c
	.dw	$40C0,$4D00,$4D00
	.dw	$40C0,$4D00,$4D00




Cyclebaysurf:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#BSCYCDLY1a
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#4	; Index pointer to word entry in...
			; ...local table
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#BS6CYCSTARTa
	sta	CGADD

	lda	>_baysrfcycp6a,x
	sta	CGDATA

	lda	>_baysrfcycp6a+1,x
	sta	CGDATA

	lda	>_baysrfcycp6a+2,x
	sta	CGDATA

	lda	>_baysrfcycp6a+3,x
	sta	CGDATA

	lda	>_baysrfcycp6a+4,x
	sta	CGDATA

	lda	>_baysrfcycp6a+5,x
	sta	CGDATA
	LA


.chk2ndcycle
	dec	v_cycledly2
	bpl	.chk3rdcycle

	lda	#BSCYCDLY1b
	sta	v_cycledly2

	lda	v_colorcel2
	dec
	dec
	bpl	.okcyc2

	lda	#4
.okcyc2
	sta	v_colorcel2
	tax

	SA
	lda	#BS6CYCSTARTb
	sta	CGADD

	lda	>_baysrfcycp6b,x
	sta	CGDATA

	lda	>_baysrfcycp6b+1,x
	sta	CGDATA

	lda	>_baysrfcycp6b+2,x
	sta	CGDATA

	lda	>_baysrfcycp6b+3,x
	sta	CGDATA

	lda	>_baysrfcycp6b+4,x
	sta	CGDATA

	lda	>_baysrfcycp6b+5,x
	sta	CGDATA
	LA

.chk3rdcycle
	dec	v_cycledly3
	bpl	.endcycle

	lda	#BSCYCDLY1c
	sta	v_cycledly3

	lda	v_colorcel3
	dec
	dec
	bpl	.okcyc3

	lda	#4
.okcyc3
	sta	v_colorcel3
	tax

	SA
	lda	#BS6CYCSTARTc
	sta	CGADD

	lda	>_baysrfcycp6c,x
	sta	CGDATA

	lda	>_baysrfcycp6c+1,x
	sta	CGDATA

	lda	>_baysrfcycp6c+2,x
	sta	CGDATA

	lda	>_baysrfcycp6c+3,x
	sta	CGDATA

	lda	>_baysrfcycp6c+4,x
	sta	CGDATA

	lda	>_baysrfcycp6c+5,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for End of Level - Object
*
*****************************************************************************

EOLCYCSTART	.equ	$31+$80

EOLCYCDLY1	.equ	7


_eolcycp4
	.dw	$7EA7,$7FBA,$7F97,$7F2F,$7EA7,$7E40
	.dw	$7EA7,$7FBA,$7F97,$7F2F,$7EA7,$7E40


Cycleeol:
	lda	v_cyclface
	bne	.docycle

	rtl
.docycle	
	dec	v_cycledly1
	bpl	.endcycle

	lda	#EOLCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#10
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#EOLCYCSTART
	sta	CGADD

	lda	>_eolcycp4,x
	sta	CGDATA

	lda	>_eolcycp4+1,x
	sta	CGDATA

	lda	>_eolcycp4+2,x
	sta	CGDATA

	lda	>_eolcycp4+3,x
	sta	CGDATA

	lda	>_eolcycp4+4,x
	sta	CGDATA

	lda	>_eolcycp4+5,x
	sta	CGDATA

	lda	>_eolcycp4+6,x
	sta	CGDATA

	lda	>_eolcycp4+7,x
	sta	CGDATA

	lda	>_eolcycp4+8,x
	sta	CGDATA

	lda	>_eolcycp4+9,x
	sta	CGDATA

	lda	>_eolcycp4+10,x
	sta	CGDATA

	lda	>_eolcycp4+11,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for Cavern Lamp - Object
*
*****************************************************************************

LMPCYCSTART	.equ	$31+$80

LMPCYCDLY1	.equ	10


_lmpcycp6
	.dw	$0B7C,$0B19,$0679,$05B9,$0518
	.dw	$0B7C,$0B19,$0679,$05B9,$0518

;	.dw	$0518,$05B9,$0679,$0B19,$0B7C
;	.dw	$0518,$05B9,$0679,$0B19,$0B7C

Cyclelamp:
	dec	v_cycledly1
	bpl	.endcycle

	lda	#LMPCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#8
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#LMPCYCSTART
	sta	CGADD

	lda	>_lmpcycp6,x
	sta	CGDATA

	lda	>_lmpcycp6+1,x
	sta	CGDATA

	lda	>_lmpcycp6+2,x
	sta	CGDATA

	lda	>_lmpcycp6+3,x
	sta	CGDATA

	lda	>_lmpcycp6+4,x
	sta	CGDATA

	lda	>_lmpcycp6+5,x
	sta	CGDATA

	lda	>_lmpcycp6+6,x
	sta	CGDATA

	lda	>_lmpcycp6+7,x
	sta	CGDATA

	lda	>_lmpcycp6+8,x
	sta	CGDATA

	lda	>_lmpcycp6+9,x
	sta	CGDATA
	LA

.endcycle
	rtl



*****************************************************************************
*
* Color Cycling code for Level Map
*
*****************************************************************************

MAPCYCSTART1	.equ	$6A
MAPCYCSTART2	.equ	$75

MAPCYCDLY1	.equ	12
MAPCYCDLY2	.equ	16


_mapcycp1
	.dw	$7FBB,$7EAC,$7F11,$7EAC,$7E27,$7EAC
	.dw	$7FBB,$7EAC,$7F11,$7EAC,$7E27,$7EAC
_mapcycp2
	.dw	$7EB0,$7E29,$7DA2,$7F12
	.dw	$7EB0,$7E29,$7DA2,$7F12

Cyclemap:
	dec	v_cycledly1
	bpl	.chk2ndcycle

	lda	#MAPCYCDLY1
	sta	v_cycledly1

	lda	v_colorcel1
	dec
	dec
	bpl	.okcyc1

	lda	#10
.okcyc1
	sta	v_colorcel1
	tax

	SA
	lda	#MAPCYCSTART1
	sta	CGADD

	lda	>_mapcycp1,x
	sta	CGDATA

	lda	>_mapcycp1+1,x
	sta	CGDATA

	lda	>_mapcycp1+2,x
	sta	CGDATA

	lda	>_mapcycp1+3,x
	sta	CGDATA

	lda	>_mapcycp1+4,x
	sta	CGDATA

	lda	>_mapcycp1+5,x
	sta	CGDATA

	lda	>_mapcycp1+6,x
	sta	CGDATA

	lda	>_mapcycp1+7,x
	sta	CGDATA

	lda	>_mapcycp1+8,x
	sta	CGDATA

	lda	>_mapcycp1+9,x
	sta	CGDATA

	lda	>_mapcycp1+10,x
	sta	CGDATA

	lda	>_mapcycp1+11,x
	sta	CGDATA
	LA

.chk2ndcycle
	dec	v_cycledly2
	bpl	.endcycle

	lda	#MAPCYCDLY2
	sta	v_cycledly2

	lda	v_colorcel2
	dec
	dec
	bpl	.okcyc2

	lda	#6
.okcyc2
	sta	v_colorcel2
	tax

	SA
	lda	#MAPCYCSTART2
	sta	CGADD

	lda	>_mapcycp2,x
	sta	CGDATA

	lda	>_mapcycp2+1,x
	sta	CGDATA

	lda	>_mapcycp2+2,x
	sta	CGDATA

	lda	>_mapcycp2+3,x
	sta	CGDATA

	lda	>_mapcycp2+4,x
	sta	CGDATA

	lda	>_mapcycp2+5,x
	sta	CGDATA

	lda	>_mapcycp2+6,x
	sta	CGDATA

	lda	>_mapcycp2+7,x
	sta	CGDATA
	LA

.endcycle
	rtl




*****************************************************************************
*****************************************************************************
*****************************************************************************
*
* Give Password after level
*

Givepasswrd:
	LAI

	lda	v_restart
	beq	.okdopasswrd

	rtl
.okdopasswrd
	lda	#0
	sta	z_hdmaenable

	SAI
	sta	HDMAEN	; Disable HDMA
	sta	MDMAEN	; Disable DMA
	LAI

	stz	z_levelx	; Top left corner for sprites
	stz	z_levely
	stz	v_xfercount   	; Clear number of objects to be sent

	lda	z_level	; Get current level
	and	#$FF

	cmp	#BONUSINT	; Inside Bonus Hut?
	bne	.chkholda	; No  - Check for a ship hold

	bra	.nopasswrd	; Otherwise, skip show map

.chkholda
	cmp	#SHIPHOLDA	; Is this ship hold?
	bne	.chkholdb	; No  - Check other side

	bra	.nopasswrd	; Otherwise, skip show map

.chkholdb
	cmp	#SHIPHOLDB	; Is this ship hold?
	bne	.chkshabos	; No - Check for Shamon Boss level

	bra	.nopasswrd	; Otherwise, skip show map

.chkshabos
	cmp	#SHAMONBOSS	; Is this shamon or out of range?
	bcc	.dogivepass	; No - Give password
	
.nopasswrd
	rtl

.dogivepass
	jsl	GInitobjs	; Do doorway, palettes & stuff

	jsl	GLoadpasscrn	; Load BG chars & screens to VRAM

	jsl	Fadein

.rescan
      	jsl	Scanjoys

	lda	z_joy1
	beq	.rescan

	lda	z_joy1
	cmp	#JOYPADSTART
	bne	.rescan

.rescan1
	jsl	Scanjoys

	lda	z_joy1
	bne	.rescan1

	jsl	Fadeout	; Fade to Target Palette

	rtl

allpassX
	.dw	0	; Null
	.dw	74	; DENWALA
	.dw	74	; MANGANI
	.dw	78	; KAGODA 
	.dw	100	; LUL    

	.dw	82	; SABOR 
	.dw	0	; Nul
	.dw	82	; ABALA 
	.dw	78	; HISTAH
	.dw	0	; Null

	.dw	78	; HISTAH
	.dw	83	; GIMLA 
	.dw	83	; HORTA
	.dw	78	; GOMLUL
	.dw	82	; MEETA

	.dw	82	; KANDO  
	.dw	74	; BUNDOLO
	.dw	76	; TANTOR 
	.dw	72	; KREEGOR	- Shamon Boss

	.dw	100
	.dw	100

	; Password frame by level
allpassfrm
	.dw	0	; No password
	.dw	0	; DENWALA gets TZANHOME
	.dw	1	; MANGANI gets CLEARING
	.dw	2	; KAGODA  gets SWAMP	
	.dw	3	; LUL     gets BEACH

	.dw	4	; LULKOR - null
	.dw	6	; SABOR   gets REEF
	.dw	5	; ABALA   gets SHIP
	.dw	6	; DANO - null
	.dw	6	; NUMA - null

	.dw	6	; HISTAH  gets JUNGLE3
	.dw	7 	; GIMLA   gets DOWNRIVER
	.dw	8 	; HORTA   gets JUNGLE4
	.dw	9 	; GOMLUL  gets UPRIVER
	.dw	10  	; MEETA   gets WATERFALL
	
	.dw	11	; KANDO   gets CAVERN
	.dw	12	; BUNDOLO gets STREETS 
	.dw	13	; TANTOR  gets TEMPLE
	.dw	14
	.dw	14
	.dw	14
			   
GInitobjs:			    
	lda	#$ffff	     
	sta	v_foeframe	      
	stz	v_foedelay	    
			      
	stz	z_levelx
	stz	z_levely
	stz	v_xfercount   	;clear number of objects to be sent

	jsl	GUpdateobjs	; Init & que data for vram

	jsl	Senddata	; Send it

	jsl	Resetattr	; Reset sprite driver

	jsl	GPlotobjs	; Plot objects

	jsl	Createoam	; Create OAM image

	jsl	Sendobj	; Send OAM shadow to vram

	jsl	GPasspals	; Load obj palettes

	rtl

GUpdateobjs:
	dec	v_foedelay
	bpl	.end
	lda	#5
	sta	v_foedelay

	inc	v_foeframe
	lda	v_foeframe
	cmp	#5
	bcc	.plot
	lda	#90
	sta	v_foedelay
	bra	.end
.plot
	asl
	tay
	lda	#^door
	sta	z_dreccount
	lda	#door
	sta	z_spridrec
	lda	#OBJBASE

	jsl	Sendframe	; Set up xfer info
.end
	lda	v_givepass	; This contains completed level
	asl
	tax
	lda	>allpassfrm,x
	sta	v_playerframe
	asl
	tay

	lda	#^allpass	; Point to structure
	sta	z_dreccount
	lda	#allpass	; Point to structure
	sta	z_spridrec
	lda	#(OBJBASE+$800)	; Set to base+

	jsl	Sendframe	; Queue it up

	rtl


GPlotobjs:
	ldy	z_attrcount

	lda	v_givepass
	asl
	tax
	lda	>allpassX,x	; Get X pos
	sta	ATTRX,y
	lda	#48	; Get Y pos
	sta	ATTRY,y
	lda	v_playerframe	; Which frame are we in?
	sta	ATTRFRAME,y
	lda	#^allpass	; Point to dagger data structure
	sta	ATTRBANK,y
	lda	#allpass	; Point to dagger data structure
	sta	ATTRDREC,y
	lda	#$3280
	sta	ATTROFF,y
	iny
	iny

	lda	v_foeframe
	cmp	#5
	bcs	.end

	lda	#128
	sta	ATTRX,y
	lda	#190
	sta	ATTRY,y
	lda	#door
	sta	ATTRDREC,y
	lda	#^door
	sta	ATTRBANK,y	
	lda	v_foeframe
	sta	ATTRFRAME,y
	lda	#$2000
	sta	ATTROFF,y
	iny
	iny

	lda	#192
	sta	ATTRX,y
	lda	#190
	sta	ATTRY,y
	lda	#door
	sta	ATTRDREC,y
	lda	#^door
	sta	ATTRBANK,y	
	lda	v_foeframe
	sta	ATTRFRAME,y
	lda	#$6000
	sta	ATTROFF,y
	iny
	iny

.end
	sty	z_attrcount

	rtl
	

GPasspals:
	lda	#DOOR.PALSET	; Load palette for Doors
	sta	z_temp0
	lda	#^DOOR.PALSET
	sta	z_temp1
	ldy	#$1e
.loop1
	lda	[z_temp0],y
	sta	PALETTE2+$100,y
	dey
	dey
	bpl	.loop1

	lda	#PASS01.PALSET	; Load palette for Password
	sta	z_temp0
	lda	#^PASS01.PALSET
	sta	z_temp1
	ldy	#$1e
.loop2
	lda	[z_temp0],y
	sta	PALETTE2+$120,y
	dey
	dey
	bpl	.loop2

	rtl

	
*****************************************************************************
*
*

GLoadpasscrn:
	; Unpack Screen Chars

	pea	|password.ccs	 ; Source
	pea	^password.ccs
	pea	|SCRATCH	 ; Destination
	pea	^SCRATCH

	jsl	UNPACK

	pla
	pla
	pla
	pla


	; VRAM Xfer for Screen Chars

	lda	#^SCRATCH
	sta	z_sendvrambank

	lda	#SCRATCH
	ldy	#$8000	
	ldx	#BG1CHAR

	jsl	Sendvram

	; Unpack Screen
	pea	|password.clm	; push low word of packed file
	pea	^password.clm	; push high word of packed file
	pea	|SCRATCH	; push low word of output buffer
	pea	^SCRATCH	; push high word of output buffer

	jsl	UNPACK	; unpack file to output buffer

	pla
	pla
	pla
	pla

	ldx	#$7FE	; Set priority
.copyscreen
	lda	>SCRATCH,x
	ora	#$2000
	sta	>SCRATCH,x
	sta	>SCRATCH+$800,x
	dex
	dex
	bpl	.copyscreen


	; Copy "Give Password" Screen to BG1
	ldx	#BG1BASE
	lda	#SCRATCH+$1000
	ldy	#^SCRATCH
	jsl	Sendbg


.dopals
	; Copy Screen Palette(s)
	lda	#password.pal
	sta	z_temp0

	lda	#^password.pal
	sta	z_temp1

	ldy	#$FE
.palloop    
	lda	[z_temp0],y
	sta	PALETTE2,y

	dey
	dey
	bpl	.palloop

	; Set Modes

	SA

	lda	#3
	sta	OBJSEL

	lda	#(BG1BASE/256)
	sta	BG1SC

	lda	#$61
	sta	BGMODE

	lda	#$11
	sta	TM

	stz	TS	

	stz	CGSWSEL		

	stz	CGADSUB

	lda	#DIVIDED
	sta	BG12NBA

	lda	#$ff
	sta	BG1VOFS
	stz	BG1VOFS

	stz	BG1HOFS
	stz	BG1HOFS

	LAI

	NMION

	rtl


*****************************************************************************
*
*

Palettehit:
	LAI

;	lda	z_zerobjpal0	; Want to clear palette?
;	beq	.chkinvince
;
;	jmp	Zeropalobj0

.chkinvince
;	lda	z_colfadeip	; Is palette fade still in progress?
;	bne	.exit	; Yes -

	lda	v_playinvince	; Get players current invincability
	cmp	#1	; Invincible?
	bcc	.skipsdpal	; No -

	lda	v_knifeactive
	bne	.skipsdpal

	dec	v_shimertimer	; Time to switch palettes?
	bmi	.computshimr	; Yes - Adjust shimmer timer

	rtl
.skipsdpal
	lda	v_tzpaltogl
	bne	Loadnormtzpal

	rtl
.computshimr
	lda	#2
	sta	v_shimertimer

	lda	v_tzpaltogl
	eor	#1
	sta	v_tzpaltogl

	beq	Loadnormtzpal

	LAI

	ldx	#^TZdampalset
	lda	#TZdampalset
	ldy	#$20

	sta	DMA0A1L	; set source address
	sty	DMA0NUML	; set dma transfer size

	SAI		; Set 8 bit address mode

	stx	DMA0A1B	; Get source bank
	stz	DMA0INC	; transfer in bytes

	lda	#<CGDATA	; low byte of CGDATA register
	sta	DMA0DEST	; set dma dest (offset from $2100)

	lda	#$80	; start at second half of cgram
	sta	CGADD

	lda	#$01
	sta	MDMAEN

	LAI		; Set 16 bit addr & index mode
.exit
	rtl


Loadnormtzpal:
	LAI

	ldx	v_curtzanpal

	lda	d_pallist,x
	sta	z_temp1

	lda	d_palbank,x
	tax

	lda	z_temp1
	ldy	#$20

	sta	DMA0A1L	; set source address
	sty	DMA0NUML	; set dma transfer size

	SAI		; Set 8 bit address mode

	stx	DMA0A1B	; Get source bank
	stz	DMA0INC	; transfer in bytes

	lda	#<CGDATA	; low byte of CGDATA register
	sta	DMA0DEST	; set dma dest (offset from $2100)

	lda	#$80	; start at second half of cgram
	sta	CGADD

	lda	#$01
	sta	MDMAEN	; start dma

	LAI		; Set 16 bit addr & index mode

	rtl


;Zeropalobj0
;	LAI
;
;	stz	MDMAEN	; start dma
;
;	ldx	#^TZdeadpal
;	lda	#TZdeadpal
;	ldy	#$20
;
;	sta	DMA0A1L	; set source address
;	sty	DMA0NUML	; set dma transfer size
;
;	SAI		; Set 8 bit address mode
;
;	stx	DMA0A1B	; Get source bank
;	stz	DMA0INC	; transfer in bytes
;
;	lda	#<CGDATA	; low byte of CGDATA register
;	sta	DMA0DEST	; set dma dest (offset from $2100)
;
;	lda	#$80	; start at second half of cgram
;	sta	CGADD
;
;	lda	#$01
;	sta	MDMAEN	; start dma
;
;	LAI		; Set 16 bit addr & index mode
;
;	lda	#0	; Logical Palette number CGRAM
;	ldx	#22	; Load "Death" (transparent) palette
;
;	pha                     ; Save palette #
;
;	lda	d_pallist,x	; Get palette pointer offset
;	sta	z_temp3
;
;	lda	d_palbank,x	; Get palette pointer bank
;	sta	z_temp4
;           
;	pla		; Get palette number
;
;	asl		; Word
;	tay		; Create index
;	lda	paladdr,y	; Get CGRAM offset address
;	tax		; Create index
;
;	ldy	#$1E	; 1 palette of 16 colors (words)
;.loop       
;	lda	[z_temp3],y	; Get palette color
;	sta	PALETTE,x	; Store to PALETTE2 for download
;
;	dex		; Adjust index pointer
;	dex
;
;	dey		; Adjust index pointer
;	dey
;
;	bpl	.loop	; Get another color
;
;	stz	z_zerobjpal0	; Reset oneshot
;
;	lda	#1
;	sta	z_colfadeip	; Start fading tarzan in
;
;	rtl
;

TZNormpal:
	.dw	$568A,$2592,$42CC,$2A2C,$2A98,$3A38,$7FFF,$42BC
	.dw	$5F9F,$0420,$7BDE,$7FFF,$04EB,$5DEA,$5ADD,$5F1F


TZdampalset:
	.dw	$3612,$217E,$7FFF,$7FFF,$7FFF,$25BE,$7FFF,$3A9E
	.dw	$437F,$0420,$7BDE,$7FFF,$00D3,$5DF4,$4A7F,$4E7F

TZPaltbl:
	.dw	TZAN.PALSET

	.dw	TZPUPS1.PALSET
	.dw	TZHBAR.PALSET
	.dw	TZPALWTR.PALSET

	.dw	TZPDRK00.PALSET
	.dw	TZPDRK01.PALSET
	.dw	TZPDRK02.PALSET
	.dw	TZPDRK03.PALSET
	.dw	TZPDRK04.PALSET
	.dw	TZDAMPAL.PALSET


TZdeadpal:
	.dw	$568A,$568A,$568A,$568A,$568A,$568A,$568A,$568A
	.dw	$568A,$568A,$568A,$568A,$568A,$568A,$568A,$568A



*****************************************************************************
*	

Ctrlmaptabl:
	.dw	Standardmap
	.dw	Confignum1
	.dw	Confignum2

Ctrlremap:
	LAI

	lda	v_contrlmap	; Get which ctrl key map to use
	and	#3
	beq	Standardmap

	asl		; Create word
	tax		; Index

	jmp	(Ctrlmaptabl,x)	; Remap controller keys to RAM copy

Standardmap:
	lda	#JOYPADY
	sta	v_JOYPADB	; Free
	lda	#JOYPADB	
	sta	v_JUMPBUTTON	; Jump
	lda	#JOYPADSELECT
	sta	v_JOYPADSELECT	; Select Knife or Darts
	lda	#JOYPADSTART	
	sta	v_JOYPADSTART	; Pause
	lda	#JOYPADUP	
	sta	v_JOYPADUP	; Movement
	lda	#JOYPADDOWN	
	sta	v_JOYPADDOWN	; Movement
	lda	#JOYPADLEFT	
	sta	v_JOYPADLEFT	; Movement
	lda	#JOYPADRIGHT	
	sta	v_JOYPADRIGHT	; Movement
			 
	lda	#JOYPADX	
	sta	v_JOYPADA	; Free
	lda	#JOYPADA	
	sta	v_WEAPON	; Attack w/ knife or darts

	lda	#JOYPADR	
	sta	v_JOYPADL	; Free
	lda	#JOYPADL	
	sta	v_RUNBUTTON	; Run

	lda	#JOYPADABXY	
	sta	v_JOYPADABXY	
	lda	#JOYARROWS	
	sta	v_JOYARROWS	
	lda	#JOYLEFTRIGHT
	sta	v_JOYLEFTRIGHT
	lda	#JOYNOARROWS	
	sta	v_JOYNOARROWS	
	lda	#JOYPADLR	
	sta	v_JOYPADLR	

	rtl

Confignum1:
	lda	#JOYPADY
	sta	v_JOYPADB	; Free
	lda	#JOYPADA
	sta	v_JUMPBUTTON	; Jump
	lda	#JOYPADSELECT
	sta	v_JOYPADSELECT	; Select Knife or Darts
	lda	#JOYPADSTART	
	sta	v_JOYPADSTART	; Pause
	lda	#JOYPADUP	
	sta	v_JOYPADUP	; Movement
	lda	#JOYPADDOWN	
	sta	v_JOYPADDOWN	; Movement
	lda	#JOYPADLEFT	
	sta	v_JOYPADLEFT	; Movement
	lda	#JOYPADRIGHT	
	sta	v_JOYPADRIGHT	; Movement
			 
	lda	#JOYPADX	
	sta	v_JOYPADA	; Free
	lda	#JOYPADY	
	sta	v_WEAPON	; Attack w/ knife or darts

	lda	#JOYPADR	
	sta	v_JOYPADL	; Free
	lda	#JOYPADL	
	sta	v_RUNBUTTON	; Run

	lda	#JOYPADABXY	
	sta	v_JOYPADABXY	
	lda	#JOYARROWS	
	sta	v_JOYARROWS	
	lda	#JOYLEFTRIGHT
	sta	v_JOYLEFTRIGHT
	lda	#JOYNOARROWS	
	sta	v_JOYNOARROWS	
	lda	#JOYPADLR	
	sta	v_JOYPADLR	

	rtl

Confignum2:
	lda	#JOYPADB	
	sta	v_JOYPADB	; Free
	lda	#JOYPADY	
	sta	v_JUMPBUTTON	; Jump
	lda	#JOYPADSELECT
	sta	v_JOYPADSELECT	; Select Knife or Darts
	lda	#JOYPADSTART	
	sta	v_JOYPADSTART	; Pause
	lda	#JOYPADUP	
	sta	v_JOYPADUP	; Movement
	lda	#JOYPADDOWN	
	sta	v_JOYPADDOWN	; Movement
	lda	#JOYPADLEFT	
	sta	v_JOYPADLEFT	; Movement
	lda	#JOYPADRIGHT	
	sta	v_JOYPADRIGHT	; Movement

	lda	#JOYPADA	
	sta	v_JOYPADA	; Free
	lda	#JOYPADX	
	sta	v_WEAPON	; Attack w/ knife or darts

	lda	#JOYPADL	
	sta	v_RUNBUTTON	; Free
	lda	#JOYPADR	
	sta	v_JOYPADL	; Run

	lda	#JOYPADABXY	
	sta	v_JOYPADABXY	
	lda	#JOYARROWS	
	sta	v_JOYARROWS	
	lda	#JOYLEFTRIGHT
	sta	v_JOYLEFTRIGHT
	lda	#JOYNOARROWS	
	sta	v_JOYNOARROWS	
	lda	#JOYPADLR	
	sta	v_JOYPADLR	

	rtl



Bogus:
	lda	#1
	sta	z_temp0
	lda	#2
	sta	z_temp1
	lda	#3
	sta	z_temp2
	lda	#4
	sta	z_temp3
	lda	#5
	sta	z_temp4
	lda	#6
	sta	z_temp5

	rtl


