*****************************************************************************
*
* PLAYER.ASM
*
*****************************************************************************
*
* Constants
*
****************************************

MAXVELOCITY	.equ	12	; Max speed obtained when falling


*****************************************************************************
*
* Ancillary Support Routines
*
*****************************************************************************


****************************************

	.MACRO	GETPXY

	lda	v_playerx	; Stored as 1/4 pixel scaling
	sta	v_playeroldx

	lda	v_playery	; Stored as 1/4 pixel scaling
	sta	v_playeroldy

	.ENDM


****************************************

	.MACRO	VERIFYMODE

	jsr	Validatetype

	dec	v_playerdelay	; Adjust frame count
	bpl	.noframe	; Process if still going...

	jmp	Newframe	; Do a new frame
.noframe
	.ENDM


****************************************

	.MACRO	CANSTAND

	lda	z_level
	and	#$FF
	cmp	#SWAMP
	bne	.nosand

	jsl	Handlesand

	lda	v_insand
	bne	.oktostand
.nosand
	jsl	Chek4floor

	bne	.oktostand

	jmp	TZStartfall	; not on solid ground, begin fall

.oktostand 
	.ENDM



****************************************

TZOffset:
	lda	v_playerdir	; Yes - Get players current direction
	and	#MOVRT	; Is player moving right?
	bne	.right	; Yes -

;	lda	#MOVLT
;	and	_walls
;	bne	.exit

	lda	#8
	clc
	adc	v_stride
	sta	v_stride

	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx

	rtl
	
.right 
;	lda	#MOVRT
;	and	_walls
;	bne	.exit

	lda	v_playerx	; Stride length
	clc
	adc	v_stride
	sta	v_playerx
.exit
	rtl

*****************************************************************************
*
*

Relocatetz:
	ldx	z_levelword	; Get word level index

	lda	_dystart,x	; Starting y position for player
	asl
	asl
	sta	v_playery	; Stored as 1/4 pixel scaling

	lda	_dxstart,x	; Starting x position for player
	asl
	asl
	sta	v_playerx	; Stored as 1/4 pixel scaling

	rtl


*****************************************************************************
*
* Validatetype compares the type in v_playertype to the type
*              supplied in the a register.
*              If they are equal, the routine returns normally...
*
*              If not equal, the delay timer is adjusted, the interbank
*              (word size) return is removed from the stack, and the
*              long return is executed.
*
*              If not equal, the delay timer is adjusted. If the timer
*              has expired, then the interbank (word size) return is removed
*              from the stack, the new type (supplied by the caller in
*              register a) is submitted to the Newframe procedure.

Validatetype:
	cmp	v_playertype	; Already in this mode?
	bne	.chkdelay	; No  - Check delay from ending anim

	rts
.chkdelay
	plx		; Remove interbank (local) return
          	
	dec	v_playerdelay	; Then finish last sequence
	bmi	.changemode	; Change modes if done

	rtl	
.changemode 
	sta	v_playernewtype	; Store new type to restart

	jmp	Newframe	; Do a new frame


*****************************************************************************
*
*

Chek4sand:
       	lda	z_level
	and	#$FF
	cmp	#SWAMP
	bne	.chkstat

	jsl	Chek4number

	lda	v_tilenum

	cmp	#17
	bne	.not17

	inc	v_playery

	bra	.insand
.not17
	cmp	#33
	bne	.chkstat

	inc	v_playery
	inc	v_playery
.insand
	jsr	Setidlecount

	lda	#1
	sta	v_insand

	sec		; Set carry indicates sand

	rtl
.chkstat
	stz	v_insand
	clc		; Clear carry - no quiksand

        	rtl


*****************************************************************************
*
*

Handlesand:
	jsl	Chek4sand	; Returns carry set if in quiksand

	bcc	.nosand
	
	lda	v_playery	; Is player below screen?
	cmp	#$3CA<<2
	bcc	.nosand	; No  - Keep processing player control

	stz	v_playerdelay
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag
.nosand
	rtl

*****************************************************************************
*
Setidlecount:
;	lda	z_joy1
;	beq	.exit

	jsl	Rndnum	; Generate time "bwtween" idles
	clc
	adc	#100

	sta	v_idlecount	; Goes here
.exit
	rts


****************************************

Prechktile:
	clc
	rts

	ldx	v_playerx	; Save original X

	lda	v_playerdir
	and	#MOVRT
	beq	.applylft

	lda	v_playerx
	clc
	adc	#17<<2
	sta	v_playerx

	phx

	jsl	Chek4wall

	plx

	stx	v_playerx	; Restore original X

	lda	#MOVRT
	and	_walls
	bne	.nowalkr

	clc
	rts
.nowalkr
	sec
	rts

.applylft
	lda	v_playerx
	sec
	sbc	#17<<2
	sta	v_playerx

	phx

	jsl	Chek4wall

	plx

	stx	v_playerx	; Restore original X

	lda	#MOVRT
	and	_walls
	bne	.nowalkl

	clc
	rts
.nowalkl
	sec
	rts


*****************************************************************************
*
* Control Routines for Tarzans various moves
*
* Player directions are:
*		  1 - Down
*                         2 - Up
*		  4 - Right
*		  8 - Left
*
* Controller "1" keys perform the following functions:
* (depending on the mode selction.)
*
* The following represents option "A"
*
* Select - Weapons Select
* Start  - Pause Game
* L      - Run (w/directional)
* R      - 
* Y      - 
* B      - Jump (up, or long jump when preceeded by directional, down w/down)
* X      - Roar
* A      - Use Weapon
* left   - movement
* right  - movement
* up     - movement
* down   - movement
*
* Combinations :
*
*
*
* Controller "2" keys perform the following functions:  "Debug"
*
* Select - Toggle Tarzan's collision box display
* Start  - Enter "Debug Mode" - able to place Tarzan - coordinates display... 
* L      - Slow frame rate
* R      - Slow frame rate even more
* Y      - Toggle between x,y display & tile number & status display
* B      - Go to previous level
* X      - Toggle between Coin/Dart count display & frame/mode display
* A      - Restart level
* left   - movement
* right  - movement
* up     - movement
* down   - movement
*
* 
*
*
*
*
*
*
*
*
* Procedures:
*
*	Relocatetz
*	Validatetype
*	Chek4sand
*	TZIdle1		Idle #1 Sequence - Yawn
*	TZIdle2		Idle #2 Sequence - Haunch down
*	TZIdle3		Idle #3 Sequence - Flip knife
*	TZIdle4		Idle #4 Sequence - Nervous
*	TZLive		Fadein from death sequence
*	TZWait		Wait Sequence
*	TZWaittowalk	Wait to Walk Sequence
*	TZWalktowait	Walk to Wait Sequence
*	TZWalktorun	Walk to Run Sequence
*	TZWalk		Walk Sequence
*	TZWaittorun	Wait to Run Transition Sequence
*	TZRuntowait	Run to Wait Sequence
*	TZRuntowalk	Run to Walk Sequence
*	TZRun		Run Sequence
*	TZStartfall
*	TZFall
*	TZStartfall1
*	TZFall1
*	TZStartjump
*	TZJmpup		Jump UP
*	TZJmpnstb	Stab while Jump-Up Sequence
*	TZLand0
*	TZLand1
*	TZStdjmp	Long jump
*	TZStab1		Stab from Stand
*	TZDuck00	Duck down
*	TZDuck01	Stand up from Duck
*	TZBighit	Take a big hit
*	TZDart1		Do Blowpipe while standing
*	@TZDart2	Shoot dart while ducking
*	Blowdart
*	TZDart3		Put Blowpipe away
*	TZFloat		Tarzan floats back & down.
*	TZFloatknife	Do Float Knife attack
*	TZSwimturn
*	TZSwim
*	TZSwimtoFloat
*	TZFloaturn
*	TZFloat2swim
*	TZWalkaway
*	TZStartSwing1	Setup Swing Stuff
*	TZSwing1	Do Canned Swing Sequence 1
*	TZSwing2	Do Canned Swing Sequence 2
*	TZOffswing1	Dismount from vine
*	TZWalknclimb
*	TZClimbtowalk
*	TZClimbdn
*	TZClimbup
*	TZJmptoladr
*	TZOffladder
*
*
*
*
*****************************************************************************

Notdone
	rtl


*****************************************************************************
*
* Main Animation Control Routines
*
*****************************************************************************
*
* Idle #1 Sequence - Yawn
*
*****************************************************************************

TZIdle1:
	lda	#tzidle1mode00	; Animation type ID

	jsr	Validatetype	; If not valid, continue processing elsewhere

	lda	v_fastidle
	beq	.relax

	lda	v_playerseqlog
	cmp	#19
	bcs	.continue

	stz	v_playerdelay
.relax
	dec	v_playerdelay
	bpl	.tzidle1a

	jmp	Newframe
.tzidle1a
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	lda	z_joy1
	beq	.continue

	cmp	#JOYPADSTART
	beq	.continue

	cmp	v_JUMPBUTTON
	bne	.notjmp

	lda	#TZStartjump
	sta	v_savpAI

	jmp	TZStartjump

.notjmp
	lda	#1
	sta	v_fastidle

	stz	v_playerdelay
.continue
	lda	v_playerseqlog
	cmp	#19
	bcc	.exit

	stz	v_fastidle

	lda	#TZWait
	sta	v_playerAI

	cmp	#TZStartjump
	bne	.exit

	jmp	TZStartjump

.exit
	rtl



*****************************************************************************
*
* Idle #2 Sequence - Haunch down
*
*****************************************************************************

TZIdle2:
	lda	#tzidle2mode00

	jsr	Validatetype

	lda	v_fastidle
	beq	.relax

	lda	v_playerseqlog
	cmp	#19
	bcs	.keepchking

	stz	v_playerdelay
.relax
	dec	v_playerdelay
	bpl	.tzidle2a

	jmp	Newframe
.tzidle2a
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	lda	z_joy1
	beq	.continue

	cmp	#JOYPADSTART
	beq	.continue

	cmp	v_JUMPBUTTON
	bne	.notjmp

	lda	#TZStartjump
	sta	v_savpAI

	jmp	TZStartjump

.notjmp
	lda	#1
	sta	v_fastidle

	stz	v_playerdelay
.continue
	lda	v_playerseqlog
	cmp	#10
	bne	.keepchking

	dec	.downtime
	bpl	.exit

	lda	#10
	clc
	adc	v_playerdelay
	sta	v_playerdelay
	
	bra	.exit
.keepchking
	lda	v_playerseqlog
	cmp	#19
	bcc	.exit

	stz	v_fastidle

	lda	#TZWait
	sta	v_playerAI

	cmp	#TZStartjump
	bne	.exit

	jmp	TZStartjump

.exit
	rtl

.downtime	.equ	v_scratch

*****************************************************************************
*
* Idle #3 Sequence - Flip knife
*
*****************************************************************************

TZIdle3:
	lda	#tzidle3mode00

	jsr	Validatetype

	lda	v_fastidle
	beq	.relax

	lda	v_playerseqlog
	cmp	#24
	bcc	.nowait

	jmp	.backtowait
.nowait
	stz	v_playerdelay
.relax
	dec	v_playerdelay
	bpl	.tzidle3a

	jmp	Newframe
.tzidle3a
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	lda	z_joy1
	beq	.continue

	cmp	#JOYPADSTART
	beq	.continue

	and	v_JUMPBUTTON	; No  - Wants to jump?
	beq	.chkdir4	;   No  - Check directions

	lda	#TZStartjump	; Yes - Start jump
	sta	v_savpAI

	jmp	TZStartjump
.chkdir4
	lda	z_joy1
	and	#JOYARROWS
	beq	.notjmp

	jmp	Noidleyet
.notjmp
	lda	#1
	sta	v_fastidle

	stz	v_playerdelay
.continue
	lda	.maxflip
	cmp	.flipcount
	bcc	.chkforend

.okmumbly
	lda	z_joy1
	beq	.nrmalproc

	lda	v_playerseqlog
	cmp	#14
	bcs	.chkupper

	lda	#20; Preload frame
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzidle3mode00
	sta	v_playernewtype

	stz	v_playerdelay	; Time it out so frame 12 loads next

	jmp	Newframe

.chkupper
	lda	v_playerseqlog
	cmp	#18
	bcs	.exit


.nrmalproc
	lda	v_playerseqlog
	cmp	#22
	bcc	.exit

	cmp	#23
	bcs	.chkforend

	jsl	Rndnum

	cmp	#128
	bcc	.exit

	inc	.flipcount

	lda	#11	; Preload frame
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzidle3mode00
	sta	v_playernewtype

	stz	v_playerdelay	; Time it out so frame 12 loads next

	jmp	Newframe

.chkforend
	lda	v_playerseqlog
	cmp	#24
	bcc	.exit
.backtowait
	lda	#TZWait
	sta	v_playerAI

	stz	v_fastidle
.exit
	rtl

.flipcount	.equ	v_scratch+2
.maxflip	.equ	v_scratch+6


*****************************************************************************
*
* Idle #4 Sequence - Nervous
*
*****************************************************************************

TZIdle4:
	lda	#tzidle4mode00	; Animation type ID

	jsr	Validatetype	; If not valid, continue processing elsewhere

	lda	z_joy1	; Controller active?
	beq	.skiphands	; No  - Just continue on...

	cmp	#JOYPADSTART	; Pause game?
	beq	.skiphands	; Yes - Just continue out then

	and	v_JUMPBUTTON	; No  - Wants to jump?
	beq	.chkdir4jmp	;   No  - Check directions

	lda	#$fff0	; Initial velocity
	sta	v_velocity

	lda	#2	; Gravity constant
	sta	v_gravity

	jmp	TZStartjump
.chkdir4jmp
	lda	v_fastidle
	beq	.relax
.skiphands
	lda	v_playerseqlog
	cmp	#10
	bcc	.speedup

	jmp	.gowait
.speedup
	stz	v_playerdelay
;	inc	v_playerdelay
.relax
	dec	v_playerdelay
	bpl	.tzidle4a

	lda	v_playerseqlog
	cmp	#3
	bne	.chkf7

	stz	v_playerdelay

	dec	.holdframe
	bmi	.regen

	bra	.tzidle4a
.regen
	jsl	Rndnum

	and	#$1F
	clc
	adc	#7
	sta	.holdframe

	jmp	Newframe
.chkf7
	lda	v_playerseqlog
	cmp	#7
	beq	.dofrm7

	stz	v_playerdelay
	dec	v_playerdelay

	jmp	Newframe
.dofrm7
	lda	#TZIdle4c
	sta	v_playerAI

	rtl

	dec	.holdframe
	bmi	.regen
.tzidle4a
	lda	z_joy1	; Controller active?
	beq	.continue	; No  - Just continue on...

	cmp	#JOYPADSTART	; Pause game?
	beq	.continue	; Yes - Just continue out then

	and	v_JUMPBUTTON	; No  - Wants to jump?
	beq	.chkdir4	;   No  - Check directions

	lda	#TZStartjump	; Yes - Start jump
	sta	v_savpAI

	jmp	TZStartjump
.chkdir4
	lda	z_joy1
	and	#JOYARROWS
	beq	.notjmp

	jmp	Noidleyet
.notjmp
	lda	#1
	sta	v_fastidle

	stz	v_playerdelay
.continue
	lda	v_playerseqlog
	cmp	#10
	bcc	.exit
.gowait
	stz	v_fastidle

	lda	#TZWait
	sta	v_playerAI
.exit
	rtl


.holdframe	.equ	v_scratch+4


*****************************************************************************
*
* Idle #4a Sequence - Nervous: moves other direction
*
*****************************************************************************

TZIdle4a:
	lda	#tzidle4mode01	; Animation type ID

	jsr	Validatetype	; If not valid, continue processing elsewhere

	lda	v_fastidle
	beq	.relax

	lda	v_playerseqlog
	cmp	#10
	bcc	.speedup

	jmp	.gowait
.speedup
	stz	v_playerdelay
;	inc	v_playerdelay
.relax
	dec	v_playerdelay
	bpl	.tzidle4a

	lda	v_playerseqlog
	cmp	#3
	bne	.chkf7

	stz	v_playerdelay

	dec	.holdframe
	bmi	.regen

	bra	.tzidle4a
.regen
	jsl	Rndnum

	and	#$1F
	clc
	adc	#7
	sta	.holdframe

	jmp	Newframe
.chkf7
	lda	v_playerseqlog
	cmp	#7
	beq	.dofrm7

	stz	v_playerdelay
	dec	v_playerdelay

	jmp	Newframe
.dofrm7
	dec	v_idle4pkdly
	bmi	.setpk

	lda	#4
	sta	v_playerdelay

	rtl
.setpk
	lda	#TZIdle4b
	sta	v_playerAI

	rtl

	dec	.holdframe
	bmi	.regen
.tzidle4a
;	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	lda	z_joy1	; Controller active?
	beq	.continue	; No  - Just continue on...

	cmp	#JOYPADSTART	; Pause game?
	beq	.continue	; Yes - Just continue out then

	and	v_JUMPBUTTON	; No  - Wants to jump?
	beq	.chkdir4	;   No  - Check directions

	lda	#TZStartjump	; Yes - Start jump
	sta	v_savpAI

	jmp	TZStartjump
.chkdir4
	lda	z_joy1
	and	#JOYARROWS
	beq	.notjmp

	jmp	Noidleyet
.notjmp
	lda	#1
	sta	v_fastidle

	stz	v_playerdelay
.continue
	lda	v_playerseqlog
	cmp	#10
	bcc	.exit
.gowait
	stz	v_fastidle

	lda	#TZWait
	sta	v_playerAI
.exit
	rtl


.holdframe	.equ	v_scratch+4



*****************************************************************************
*
* Idle #4b Sequence - Nervous: 5 frame move back from Idle #4
*
*****************************************************************************

TZIdle4b:
	lda	#tzidle4mode02	; Animation type ID

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#7	
	bcc	.stay

	lda	#TZWait
	sta	v_playerAI
.stay
	rtl

*****************************************************************************
*
* Idle #4c Sequence - Nervous: 5 frame move back from Idle #4a
*
*****************************************************************************

TZIdle4c:
	lda	#tzidle4mode03	; Animation type ID

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#7	
	bcc	.stay

	lda	#TZWait
	sta	v_playerAI
.stay
	rtl


;	lda	v_playerseqlog
;	cmp	#4
;	bcc	.stay
;
;	lda	#6	; Preload frame
;	asl
;	asl
;	sta	v_playerseq
;	sta	v_playeroveride
;
;	lda	#tzidle4mode00
;	sta	v_playernewtype
;
;	jsl	Newframe
;
;	lda	#tzidle4mode00
;	sta	v_playernewtype
;
;	stz	v_playerdelay	; Time it out so next frame loads
;
;	lda	#TZIdle4
;	sta	v_playerAI
;
;	jmp	Newframe
;
;.stay
;	rtl
;

*****************************************************************************
*
* Fadein from death sequence
*
*****************************************************************************

TZLive:
	lda	#tzregentolife

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	z_sup	; Does scroller want to go?
	bne	.exit

	lda	v_playerseqlog	; Get frame in sequence...
     	cmp	#7	; Last frame?
	bcc	.exit	; No

	lda	#TZWait
	sta	v_playerAI

	stz	v_playerdelay
.exit
	rtl


*****************************************************************************
*
* Wait Sequence
*
*****************************************************************************

TZWait:
	lda	#tzwait1mode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	stz	v_stride	; Reset stride value
	stz	v_playchngdir	; Reset direction change flag

	dec	v_idlecount
	bpl	Noidleyet

	jmp	.processidle	; Process Idle Animations
@Noidleyet
	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.oktostand1	; No  - Then no vine or climb

.look4vine
	jsl	Lookforvine

	beq	.chkforclmbstrt

	jmp	TZStartSwing1

.chkforclmbstrt
	lda	v_tilestat	; Yep - Get tile type
	cmp	#CLIMBTYPE
	bne	.oktostand1

	lda	#TZWalknclimb
	sta	v_playerAI

	rtl

.oktostand1 
	lda	z_joy1	; Check for attack request
	and	v_WEAPON	; Knife 'em?
	beq	.skipattk	; No  -

	lda	v_playerAI	; Yes - Save current vector
	sta	v_savpAI

	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	jmp	Newframe

	rtl
.dodarts 
	lda	v_numdart
	beq	.skipattk

         	lda	#TZDart1	; Dart attack
	sta	v_playerAI

	lda	#tzshtstdmode00
	sta	v_playernewtype

	jmp	Newframe

	rtl
.skipattk 
	lda	z_joy1	; Get controller status
	bit	v_JOYPADDOWN	; Does Tarzan want to go down?
	beq	.chkjmp	; No  - Process others then

	bit	v_JUMPBUTTON	; Have to push jump for jump down too
	beq	.duck	; Do duck then

	lda	v_tilestat	; Yep - Get tile type
	bit	#PASSTHRUTYPE
	bne	.okfall
	jmp	.stand	; Don't do anything
.duck 
	lda	#TZDuck00
	sta	v_playerAI

	stz	v_playerdelay

	rtl
.okfall 
	jmp	TZStartfall	; Start fall sequence
.chkjmp 
	bit	v_JUMPBUTTON	; Does Tarzan want to jump?
	beq	.norm	; No  - Process others then

	jmp	TZStartjump	; Yes - Do jump up

.norm 
	lda	z_joy1	; Yes -
	and	v_RUNBUTTON	; Does Tarzan want to run?
	beq	.chkwalks	; No  - Just check for walk

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	bne	.waittorun	; Yes -

	jmp	.standr	; No  - Just stand here then

.waittorun 
	lda	v_playchngdir	; Did player change direction?
	beq	.ok3	; No  -

	jmp	.handledirchng	; Yes - Then flip next frame
.ok3
	jsr	Prechktile	; Wall tile sets carry

	bcc	.ok2run

	jmp	.standr
.ok2run
	lda	z_joy1
	and	v_JOYPADLEFT	; Did user press left?
	bne	.runleft	; Yes - Process left then


	; controller pressed to the right
.runright
	lda	v_playerdir
	and	#MOVRT	; Player already going right?
	bne	.setrun	; Yes - Walkies!

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.setrun


	; controller pressed to the left
.runleft
	lda	v_playerdir
	and	#MOVLT	; Player already going left?
	bne	.setrun	; Yes - 

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going left

.setrun    
	lda	#MOVRT|MOVLT
	and	v_playerdir
	and	_walls
	beq	.setrun2
.standr      
	rtl

.setrun2
	lda	#TZWalktorun	; Yes - Load new player AI vector
	sta	v_playerAI

	lda	#tzwkrnxumode00	; Load new anim type immediately
	sta	v_playernewtype

	jmp	Newframe	; Do a new sequence


.chkwalks 
	lda	v_playchngdir	; Did player change direction?
	beq	.ok2	; No  -

	jmp	.handledirchng	; Yes - Then flip next frame
.ok2
	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.stand	; No  - Just stand here then

	jsr	Prechktile	; Wall tile sets carry

	bcc	.nowchk4walk

	jmp	.xstand	
.nowchk4walk
	lda	z_joy1
	and	v_JOYPADLEFT	; Did user press left?
	bne	.walkleft	; Yes - Process left then


	; controller pressed to the right
.walkright  
	lda	v_playerdir
	and	#MOVRT	; Player already going right?
	bne	.setwalk	; Yes - Walkies!

	lda	#4	; No  - Setup a debounce timer
	sta	.turndly	;

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.setwalk


	; controller pressed to the left
.walkleft   
	lda	v_playerdir
	and	#MOVLT	; Player already going left?
	bne	.setwalk	; Yes - Walkies!

	lda	#4	; No  - Setup a debounce timer
	sta	.turndly	;

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going left

.setwalk    
	lda	.turndly
	bne	.stand

	lda	#MOVRT|MOVLT
	and	v_playerdir
	and	_walls
	beq	.setwalk2

	jsl	Chek4stat

	cmp	#WALKSOLID	
	beq	.setwalk2

.stand      
	dec	.turndly
	bpl	.xstand

	stz	.turndly
	stz	.wasinwalk

.chkbonusdoor
	rtl

	lda	z_level	; Get level
	cmp	#BONUSEXT	; Is Trazan at Bonus Room door?
	bne	.chkwatrfall	; No  -

	lda	z_joy1	; Yes - Get controller key
	and	v_JOYPADUP	; User request to enter cave?
	beq	.xstand	; No  - 

	lda	v_playerx	; Yes - Validate location
	lsr
	lsr
	cmp	#$5B0	; X - window
	bcc	.xstand

	cmp	#$5C4
	bcs	.xstand

	lda	v_playery
	lsr
	lsr
	cmp	#$A9	; Y - window
	bcs	.xstand

	lda	#TZWalkaway	; If in window, then allow a walk
	sta	v_playerAI
.xstand
	rtl

.chkwatrfall
	lda	z_level	; Get level
	cmp	#WATERFALL	; Is Trazan at waterfall?
	bne	.xstand	; No  -

	lda	z_joy1	; Yes - Get controller key
	and	v_JOYPADUP	; User request to enter cave?
	beq	.xstand	; No  - 

	lda	v_playerx	; Yes - Validate location
	lsr
	lsr
	cmp	#$2E0	; X - window
	bcc	.xstand

	cmp	#$330
	bcs	.xstand

	lda	v_playery
	lsr
	lsr
	cmp	#$E9	; Y - window
	bcs	.xstand

	lda	#TZWalkaway	; If in window, then allow a walk
	sta	v_playerAI

	rtl

.setwalk2
	lda	#TZWaittowalk	; Load new player AI vector
	sta	v_playerAI

	lda	#tzwalk3mode01	; Load new anim type immediately
	sta	v_playernewtype

	jmp	Newframe


.handledirchng
	stz	v_playchngdir	; Reset change

	lda	v_playerdir
	and	#MOVLT
	bne	.chng2rgt

	lda	#MOVUP|MOVDN
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going left

	rtl
.chng2rgt

	lda	#MOVUP|MOVDN
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	rtl

*****************************************

.processidle
	jsr	Setidlecount

	stz	.downtime	; Clear "squat" time

	jsl	Rndnum	; Generate number of knife flips

	and	#3	; Keep it to a conserative number
	inc
	sta	.maxflip	; Goes here


	; Randomly select an Idle Animation (1-4)

	jsl	Rndnum

	and	#$7F	; 0 thru 127
	lsr		; 0 thru  63
	lsr		; 0 thru  31
	tax
	lda	>.idletbl,x	; Pick a card, any card...
	and	#$0F	; This is a byte size table
	sta	z_temp0	; Our selection

*****************************************************************************
*
* TZIdle1	Idle #1 Sequence - Yawn
* TZIdle2	Idle #2 Sequence - Haunch down
* TZIdle3	Idle #3 Sequence - Flip knife
* TZIdle4	Idle #4 Sequence - Nervous
* TZIdle4a	Idle #4 Sequence - Nervous - Opposite
* TZIdle4b	Idle #4 Sequence - Nervous - cont. 4a
* TZIdle4c	Idle #4 Sequence - Nervous - cont. 4
*
*****************************************************************************

	lda	v_bosslevel
	beq	.skpchk

	lda	v_bossdead
	bpl	.notthree

	; Compute for Idle Animation #1
.skpchk
	lda	z_temp0

	cmp	#1
	bne	.notone

         	lda	#TZIdle1
	sta	v_playerAI

	lda	#tzidle1mode00
	bra	.settype


	; Compute for Idle Animation #2
.notone
	cmp	#2
	bne	.nottwo

.set2
	lda	v_tilestat
	bit	#SOLID
	beq	.notthree
	
         	lda	#TZIdle2
	sta	v_playerAI

	jsl	Rndnum

	and	#$F
	sta	.flipcount

	lda	#tzidle2mode00
	bra	.settype


	; Compute for Idle Animation #3
.nottwo
	cmp	#3
	bne	.notthree
.set3
         	lda	#TZIdle3
	sta	v_playerAI

	lda	#tzidle3mode00
	bra	.settype


.nervtbl
	.dw	TZIdle4
	.dw	TZIdle4a
	.dw	TZIdle4b
	.dw	TZIdle4c

.nervtbla
	.dw	tzidle4mode00
	.dw	tzidle4mode01
	.dw	tzidle4mode02
	.dw	tzidle4mode03


	; Compute for Idle Animation #4
.notthree

	jsl	Rndnum

	and	#$1F
	clc
	adc	#7
	sta	.holdframe

	jsl	Rndnum

	and	#$F
	sta	v_idle4pkdly

	and	#1
	asl
	tax

         	lda	>.nervtbl,x
	sta	v_playerAI

	lda	>.nervtbla,x
.settype
	sta	v_playernewtype

	lda	#TZWait
	sta	v_savpAI

	jmp	Newframe


.downtime	.equ	v_scratch
.flipcount	.equ	v_scratch+2
.holdframe	.equ	v_scratch+4
.maxflip	.equ	v_scratch+6

; .offsetx	.equ	v_scratch+8	; Not used

.wasinwalk	.equ	v_scratch+10
.turndly 	.equ	v_scratch+12

; 20 entries

.idletbl
	.db	1,4,3,1,2,1,3,4,2,3,1,2,4,2,3,1,4,2,3,4,1
	.db	1,4,3,1,2,1,3,4,2,3,1,2,4,2,3,1,4,2,3,4,1


*****************************************************************************
*
* Wait to Walk Sequence
*
*****************************************************************************

TZWaittowalk:
	lda	#tzwalk3mode01

	jsr	Validatetype

	lda	v_playchngdir
	beq	.normdec

	lda	v_playerdelay
	lsr
	sta	v_playerdelay
.normdec
	lda	v_insand
	beq	.normdelay

	lda	v_sandlytog
	eor	#1
	sta	v_sandlytog

	beq	.tzwaittw
.normdelay
	dec	v_playerdelay	; Adjust frame count
	bpl	.tzwaittw	; Process if still going...

	jmp	Newframe	; Do a new frame
.tzwaittw 
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	z_joy1
	bit	v_JUMPBUTTON	; Does Tarzan want to long jump?
	beq	.chkduck	; No  - Check for duck

	bra	.chkduck

	lda	#TZStdjmp	; Yes
	sta	v_playerAI

	lda	#$fff0	; Initial velocity
	sta	v_velocity

	lda	#2	; Gravity constant
	sta	v_gravity

	dec
	sta	v_whichjmp	; Indicate jump from stand

	rtl
.chkduck 
	bit	v_JOYPADDOWN	; Wants to duck?
	beq	.chkeos	; No  - Check for end of sequence
.duck 
	lda	#TZDuck00	; Yes - Vector to duck sequence then
	sta	v_playerAI

	stz	v_playerdelay

	rtl
.chkeos
	lda	v_playerseqlog	; Get frame
     	cmp	#3	; Are we there yet?
	bcc	.contseq	; No  - Continue this sequence then

	lda	#TZWalk	; Yes - Go to Walk transition
	sta	v_playerAI

	rtl
.contseq
	lda	z_joy1	; Get controller keys
	and	v_JOYLEFTRIGHT	; Want's to go left or right?
	beq	.exit	; No  - 

	lda	v_playerdir	; Yes - Get player's current direction
	and	#MOVLT	; Moving left?
	bne	.goinleft	; Yes -

	lda	z_joy1	; No  - moving right
	and	v_JOYPADLEFT	; Wants to move left now?
	bne	.saychange	; Yes -

	rtl
.goinleft
	lda	z_joy1	; Moving left
	and	v_JOYPADRIGHT	; Wants to move right now?
	bne	.saychange

	rtl
.saychange
	lda	#1	; Yes -
	sta	v_playchngdir
.exit
	rtl

*****************************************************************************
*
* Walk to Wait Sequence
*
*****************************************************************************

TZWalktowait:
	lda	#tzwalk3mode02

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	v_playchngdir
	beq	.normdec

	lda	v_playerdelay
	lsr
	sta	v_playerdelay
.normdec
	lda	v_playerseqlog	; Get frame in sequence...
     	cmp	#4	; Last frame?
	bcc	.chkup	; Yes - Check for duck before Wait

	lda	z_joy1	; 
	and	v_JOYPADDOWN	; Wants to duck?
	beq	.preframe 	; No  - Process transition
.duck 
	lda	#TZDuck00
	sta	v_playerAI

	stz	v_playerdelay
.exit	
	rtl

.chkup
	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.chkjmp	; No  -
.look4vine
	jsl	Lookforvine	; Yes - Look for vine

	beq	.chkforclmbstrt	; Check for climb if no vine

	jmp	TZStartSwing1	; No - Load new player AI vector

	rtl
.chkforclmbstrt
	lda	z_joy1	; No - 
	and	v_JOYPADUP	; Wants to climb?
	beq	.chkjmp	; No  - Cont. transistion

	jsl	Chek4stat

	lda	v_tilestat	; Yep - Get tile type
	cmp	#CLIMBTYPE	; Is this a climbable tile?
	bne	.chkjmp	; No  - Cont. transistion

	lda	#TZWalknclimb	; Yes - Go climb a tree...
	sta	v_playerAI

	stz	v_playchngdir

	rtl
.chkjmp
	lda	z_joy1
	and	v_JUMPBUTTON	; Does Tarzan want to jump?
	beq	.gowait	; No  - Process others then

	jmp	TZStartjump	; Yes - Do jump up
.gowait 
	lda	v_playchngdir
	bne	.exit

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Want's to go left or right?
	beq	.exit	; No  - 

	lda	v_playerdir	; Yes - Get player's current direction
	and	#MOVLT	; Moving left?
	bne	.goinleft	; Yes -

	lda	z_joy1	; No  - moving right
	and	v_JOYPADLEFT	; Wants to move left now?
	beq	.exit	; No - 

	lda	#1	; Yes -
	sta	v_playchngdir

	rtl

.goinleft
	lda	z_joy1	; Moving left
	and	v_JOYPADRIGHT	; Wants to move right now?
	beq	.exit	; No - 

	lda	#1	; Yes -
	sta	v_playchngdir

	rtl

.preframe
	lda	#TZWait
	sta	v_playerAI

	rtl

*****************************************************************************
*
* Walk to Run Sequence
*
*****************************************************************************

TZWalktorun:
	lda	#tzwkrnxumode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
.ok2run
	lda	v_playerseqlog	; Get frame in sequence...
     	cmp	#3	; Last frame?
	bcc	.chkup	; Yes - Check for vine before Wait

	lda	#TZRun
	sta	v_playerAI

	rtl
.chkup
	dec	v_playerdelay

	rtl


*****************************************************************************
*
* Walk Sequence
*
*****************************************************************************

TZWalk:
	lda	#tzwalk3mode00

	jsr	Validatetype

	dec	v_playerdelay	; Adjust frame count
	bpl	.tzwalk	; Process if still going...

	stz	v_playchngdir

	jmp	Newframe	; Do a new frame
.tzwalk 
	lda	#4	; Walking Stride
	sta	v_stride

	jsl	Chek4sand
	bcc	.chk4stand

	lda	v_stride
	lsr
	sta	v_stride
.chk4stand
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	jsl	Dopitstuff

	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.oktostand1	; No  - Then no vine or climb
.look4vine
	jsl	Lookforvine	; Yes - Look for a vine

	beq	.chkforclmbstrt	; Check for climb if no vine found

	lda	#TZWalktowait	; Load new player AI vector
	sta	v_playerAI

	rtl

.chkforclmbstrt
	lda	v_tileindex	; Yep - Get tile type
	cmp	#CLIMBTYPE
	bne	.oktostand1

	lda	#TZWalknclimb
	sta	v_playerAI

	rtl

.oktostand1 
	lda	z_joy1	; Check for attack request
	and	v_WEAPON	; Knife 'em?
	beq	.skipattk	; No  -

	lda	v_playerAI	; Yes - Save current vector
	sta	v_savpAI

	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	rtl
.dodarts 
	lda	v_numdart
	beq	.skipattk

         	lda	#TZDart1	; Dart attack
	sta	v_playerAI

	lda	#tzshtstdmode00
	sta	v_playernewtype

	rtl
.skipattk 
	lda	z_joy1
	and	v_RUNBUTTON	; Run Requested?
	beq	.chkwalks	; No  -

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Yes - But is direction pressed?
	bne	.wktorn	;     Yes -

	jmp	.stand	; No  - Just stand around then
.wktorn 
	lda	#TZWalktorun
	sta	v_playerAI

	rtl
.chkwalks 
	lda	z_joy1	; Get pad stats
	and	v_JOYLEFTRIGHT	; Left or right pressed?
	bne	.lftorrght	; Yes -

	jmp	.stand	; No  - Just stand then

.lftorrght 
	lda	z_joy1
	and	v_JUMPBUTTON	; Does Tarzan want to long jump?
	beq	.norm	; No  - Continue the sequence then

	lda	v_ycount	; Yes - debounce button
	bne	.norm	; No jump if not clear
	
	lda	#TZStdjmp	; Load Long-Jump AI vector
	sta	v_playerAI

	lda	#$fff0	; Initial velocity
	sta	v_velocity

	lda	#2	; Gravity component
	sta	v_gravity

	sta	v_whichjmp	; Let em know where we came from

	rtl

.norm 
	lda	v_playerdir	; Yes - Get players current direction
	and	#MOVRT	; Is player moving right?
	bne	.right	; Yes -

	lda	z_joy1	; No  - Get controller bits again
	and	v_JOYPADLEFT	; Is player wanting to go left?
	bne	.kglft	; Yes - Make it so...

	lda	#TZWalktowait	; No - Load new player AI vector
	sta	v_playerAI

	rtl		; All clear - move right

.kglft 
	lda	#MOVLT
	and	_walls
	beq	.movhimleft

	jmp	.stand
.movhimleft
	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx

	bra	.chkbonusdoor

	rtl
	
.right 
	lda	z_joy1
	and	v_JOYPADRIGHT	; Is player pushing right?
	bne	.kgrgt	; Yes -

	lda	#TZWalktowait	; No - Load new player AI vector
	sta	v_playerAI

	rtl
.kgrgt 
	lda	#MOVRT
	and	_walls
	bne	.stand

	lda	v_playerx	; Stride length
	clc
	adc	v_stride
	sta	v_playerx

.chkbonusdoor
	rtl

	lda	z_level	; Get level
	and	#$FF
	cmp	#BONUSEXT	; Is Trazan at Bonus Room door?
	bne	.chkwatrfall	; No  -

	lda	z_joy1	; Yes - Get controller key
	and	v_JOYPADUP	; User request to enter cave?
	beq	.xstand	; No  - 

	lda	v_playerx	; Yes - Validate location
	lsr
	lsr
	cmp	#$5B0	; X - window
	bcc	.xstand

	cmp	#$5C4
	bcs	.xstand

	lda	v_playery
	lsr
	lsr
	cmp	#$A9	; Y - window
	bcs	.xstand

	lda	#TZWalkaway	; If in window, then allow a walk
	sta	v_playerAI
.xstand
	rtl

.chkwatrfall
	cmp	#WATERFALL	; Is Trazan at waterfall?
	bne	.xstand	; No  -

	lda	z_joy1	; Yes - Get controller key
	and	v_JOYPADUP	; User request to enter cave?
	beq	.xstand	; No  - 

	lda	v_playerx	; Yes - Validate location
	lsr
	lsr
	cmp	#$2B0	; X - window
	bcc	.xstand

	cmp	#$340
	bcs	.xstand

	lda	v_playery
	lsr
	lsr
	cmp	#$E9	; Y - window
	bcs	.xstand

	lda	#TZWalkaway	; If in window, then allow a walk
	sta	v_playerAI

	rtl
.stand      	
	lda	#TZWalktowait	; Otherwise initiate transition
	sta	v_playerAI

	rtl

*****************************************************************************
*
* Wait to Run Transition Sequence
*
*****************************************************************************

TZWaittorun:
	lda	#tzrunxumode00

	jsr	Validatetype

	lda	v_playchngdir
	beq	.normdec

	dec	v_playerdelay
.normdec
	dec	v_playerdelay	; Adjust frame count
	bpl	.tzwtr1	; Process if still going...

	jmp	Newframe	; Do a new frame
.tzwtr1 
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	jsl	Dopitstuff

	lda	z_joy1
	and	v_RUNBUTTON	; Run Requested?
	beq	.stand	; No  -

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Yes - But is direction pressed?
	beq	.stand	; No  - Just stand around then

	lda	v_playerseqlog
	cmp	#7
	bcc	.walk

	stz	v_playchngdir

	lda	#14	; Running Stride
	sta	v_stride

	lda	#TZRun	; Load new player AI vector
	sta	v_playerAI
.walk 
	lda	v_playchngdir
	bne	.swfrmrun

	lda	v_playerdir
	and	#MOVRT
	bne	.right

	lda	#MOVLT
	and	_walls
	bne	.stand

	rtl
	
.right 
	lda	#MOVRT
	and	_walls
	bne	.stand

	rtl

.stand      
	lda	#TZRuntowait	; Load new player AI vector
	sta	v_playerAI

	stz	v_playchngdir
	stz	v_velocity

	rtl

.swfrmrun
	lda	#4	; Move that player into a slide...
	sta	v_stride

	lda	v_playerdir
	and	#MOVRT
	bne	.movlft

	lda	v_playerx
	clc
	adc	v_stride	; Stride length
	sta	v_playerx
		
	rtl

.movlft
	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx
		
	rtl


*****************************************************************************
*
* Run to Wait Sequence
*
*****************************************************************************

TZRuntowait:
	lda	#tzrunxdmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	jsl	Dopitstuff

	lda	#8	; Move that player into a slide...
	sta	v_stride

	lda	v_playerseqlog
	cmp	#6	; At end of sequence?
	bcc	.notdone	; Yes - Then go into wait sequence

	lda	#TZWait
	sta	v_playerAI
.exit
	rtl
.notdone
	cmp	#2	; No  - Need to check direction change
	bcc	.movhim	; Otherwise, just keep sliding
.chngdir
	stz	v_stride

	bne	.figure
.slidem
	lda	#8
	sta	v_stride

	dec	v_velocity
	bmi	.figure

	lda	v_playerdelay
	clc
	adc	#16
	sta	v_playerdelay
.figure
	stz	v_velocity

	lda	z_joy1
	and	v_RUNBUTTON	; Is run button pressed?
	beq	.movhim

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Want's to go left or right?
	beq	.movhim

	lda	v_playerdir	; Yes - Get player's current direction
	and	#MOVLT	; Moving left?
	bne	.goinleft	; Yes -

	lda	z_joy1	; No  - moving right
	and	v_JOYPADLEFT	; Wants to move left now?
	bne	.setleft1

	jmp	.preframe	; No - But wants to run anyway

.setleft1
	lda	#MOVUP|MOVDN	; No  - Then reset direction flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir

	bra	.preframe

	lda	v_playerdir	; Yes - Get player's current direction
	and	#MOVLT	; Moving left?
	bne	.goinleft	; Yes -
.goinleft
	lda	z_joy1	; Moving left
	and	v_JOYPADRIGHT	; Wants to move right now?
	bne	.movrgt

	rtl
.movrgt
	lda	#MOVUP|MOVDN	; No  - Then reset direction flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir

	bra	.preframe

.movhim 
	lda	z_joy1	; Get controller status
	bit	v_JOYPADDOWN	; Does Tarzan want to go down?
	beq	.chkjmp1	; No  - Process others then

	bit	v_JUMPBUTTON	; Have to push jump for jump down too
	beq	.duck1	; Do duck then

	lda	v_tilestat	; Yep - Get tile type
	bit	#PASSTHRUTYPE
	bne	.okfall1

	jmp	.norm1
.duck1
	lda	#TZDuck00
	sta	v_playerAI

	stz	v_playerdelay

	rtl
.okfall1 
	jmp	TZStartfall	; Start fall sequence
.chkjmp1 
	bit	v_JUMPBUTTON	; Does Tarzan want to jump?
	beq	.norm1	; No  - Process others then

	jmp	TZStartjump	; Yes - Do jump up

.norm1 
	lda	v_playerdir
	and	#MOVLT
	bne	.movlft

	lda	#MOVRT
	and	_walls
	bne	.stand

	lda	v_playerx
	clc
	adc	v_stride	; Stride length
	sta	v_playerx
		
	rtl

.movlft
	lda	#MOVLT
	and	_walls
	bne	.stand

	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx
		
	rtl

.preframe
	lda	#1
	sta	v_playchngdir

	lda	#2	; Preload frame
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzrunxumode00
	sta	v_playernewtype

	jsl	Newframe

	lda	#tzrunxumode00
	sta	v_playernewtype

	lda	#TZWaittorun
	sta	v_playerAI

	stz	v_playerdelay	; Time it out so next frame loads

	jmp	Newframe

.stand      	
	lda	#TZWalktowait	; Otherwise initiate transition
	sta	v_playerAI

	rtl

	
*****************************************************************************
*
* Run to Walk Sequence
*
*****************************************************************************

TZRuntowalk:

	lda	#tzrnwkxdmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	jsl	Dopitstuff

	stz	v_stride

	lda	v_playerseqlog
     	cmp	#3
	bcc	.whoooa

	lda	#9	; Preload frame 9 of walk sequence
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzwalk3mode00
	sta	v_playernewtype

	stz	v_playerdelay	; Time it out so frame 10 loads next

	jsl	Newframe

	lda	#tzwalk3mode00
	sta	v_playernewtype

	lda	#TZWalk
	sta	v_playerAI

	jmp	Newframe

.whoooa 
	lda	#4
	sta	v_stride

	lda	v_playerdir
	and	#MOVLT
	bne	.movlft

	lda	v_playerx
	clc
	adc	v_stride	; Stride length
	sta	v_playerx
		
	rtl
.movlft
	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx
.end		
	rtl


*****************************************************************************
*
* Run Sequence
*
*****************************************************************************

TZRun:
	lda	#tzrunmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	#14
	sta	v_stride

	jsl	Chek4sand
	bcc	.nosand

	lda	v_stride
	lsr
	sta	v_stride
.nosand
	inc	v_velocity

	lda	#$F
	and	v_velocity
	sta	v_velocity

	jsl	Chek4floor

	bne	.ok2run

	jmp	TZStartfall	; not on solid ground, begin fall

.ok2run
	jsl	Dopitstuff

	lda	z_joy1
	and	v_RUNBUTTON	; Is run button pressed?
	bne	.chkruns	; Yes - Then check direction

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; No  - Going left or right?
	beq	.gowait	; No  -

	lda	#TZRuntowalk	; No  - Load new player AI vector
	sta	v_playerAI

	rtl
.chkruns 
	lda	z_joy1	; Get controller bits
	and	v_JOYLEFTRIGHT	; Going left or right?
	bne	.which	; Yes - Which one

	jmp	.stand	; No  - Just stand around then

.which 
	lda	z_joy1
	bit	v_JUMPBUTTON	; Does Tarzan want to long jump?
	beq	.norm	; No  - Continue the sequence then

	lda	#TZStdjmp	; Load new player AI vector
	sta	v_playerAI

	lda	#$fff0
	sta	v_velocity

	lda	#2
	sta	v_gravity

	inc
	sta	v_whichjmp

	rtl

.norm 
	lda	v_playerdir	; Yes - Get players current direction
	and	#MOVRT	; Is player moving right?
	bne	.right	; Yes -

	lda	z_joy1	; No  - Get controller bits again
	and	v_JOYPADLEFT	; Is player wanting to go left?
	bne	.kglft	; Yes - Make it so...

	lda	#TZRuntowait	; No - Load new player AI vector
	sta	v_playerAI

	rtl		; All clear - move right

.kglft 
	lda	#MOVLT
	and	_walls
	bne	.stand

	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx

	rtl
	
.right 
	lda	z_joy1
	and	v_JOYPADRIGHT	; Is player pushing right?
	bne	.kgrgt	; Yes -
.gowait
	lda	#TZRuntowait	; No - Load new player AI vector
	sta	v_playerAI

	rtl
.kgrgt 
	lda	#MOVRT
	and	_walls
	bne	.stand

	lda	v_playerx	; Stride length
	clc
	adc	v_stride
	sta	v_playerx

	rtl

.stand 
	lda	#TZRuntowait	; Load new player AI vector
	sta	v_playerAI

	stz	v_playerdelay

	rtl

	jmp	Newframe	; Do a new sequence


*****************************************************************************
*
* Falling Sequence
*
*****************************************************************************

TZStartfall:
	lda	#2
	sta	v_velocity
	lda	#2
	sta	v_gravity

	lda	v_playery
	sta	v_markfall

	lda	#TZFall
	sta	v_playerAI

	lda	#tzfallonly
	sta	v_playernewtype	; Store new type to restart

	stz	v_playerdelay

	jmp	Newframe

	rtl


*****************************************************************************
*
* Fall Sequence
*
*****************************************************************************

TZFall:
	lda	#tzfallonly

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	lda	#10
	sta	v_playerdelay

	lda	v_falling
	bne	.skipdetect

	lda	v_pitopen
	bne	.opit

	bra	.skipdetect
.opit
	jsl	Chek4number	; Gets tile number	

	cmp	#$38	; Over pit area?
	beq	.fallpit	; Yes -

	bra	.cont
.fallpit
	stz	v_falling
	inc	v_falling

.skipdetect
	lda	v_playery
	lsr
	lsr
	sec
	sbc	z_levely
	bmi	.cont

	cmp	#$130
      	bcs	.killem
.cont
	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir

	lda	v_playery
	sec
	sbc	v_markfall

	cmp	v_fallmax
         	bcc	.chkforflrnow

	lda	#1
	sta	v_disbgscroll
	
	lda	z_levely
	asl
	asl
	clc
	adc	#330<<2
	cmp	v_playery
	bcs	.chkforflrnow
.killem
	stz	v_playerdelay
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag
	sta	v_felldie

.xitfall
	rtl

.chkforflrnow
;	lda	v_hitmonkey
;	beq	.nomonkey
;	
;	dec	v_hitmonkey
;	bpl	.falldown
;
;	stz	v_hitmonkey
;
;	bra	.falldown
;.nomonkey
	jsl	Chek4floorfall	; Have we detected a floor?

	bne	.landitnow	; Yes - switch to finish jump
.chk4ladder
	lda	z_joy1	; Get keys
	bit	v_JOYPADUP	; Pressing up?
	beq	.falldown	; No  - 

	lda	v_tilestat	; Yep - Get tile stat
	cmp	#CLIMBTYPE
	bne	.falldown

	lda	#TZJmptoladr
	sta	v_playerAI

	rtl
.falldown
	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	v_velocity	; Get velocity
	clc
	adc	v_gravity	; Subtract gravity
	sta	v_velocity

	cmp	#(MAXVELOCITY*4)
	bcc	.oktomovdn	; If still some left, then move

	lda	#(MAXVELOCITY*4)	; Terminal velocity
	sta	v_velocity
.exithold 
;	rtl

.oktomovdn 
;	dec	v_lrmovdly
;	bpl	.xitlrchk
;
;	stz	v_lrmovdly
;
	jsl	Chklrmov	; Check for left/right movement
.xitlrchk
	rtl

.landitnow 
	lda	v_falling
	beq	.justlandit

	bra	.falldown
.justlandit
	stz	v_playerdelay

	lda	#TZLand0
	sta	v_playerAI

	stz	v_lrmovdly

	rtl


*****************************************************************************
*
* Falling Sequence with transition
*
*****************************************************************************

TZStartfall1:
	lda	#TZStartfall1
	sta	v_playerAI

	lda	#2
	sta	v_velocity

	lda	#2
	sta	v_gravity

	lda	v_playery
	sta	v_markfall

	dec	v_playerdelay	; Then finish last sequence
	bmi	.changemode	; Change modes if done

	rtl	
.changemode 
	lda	#tzfalltrans
	sta	v_playernewtype	; Store new type to restart

	lda	#TZFall1
	sta	v_playerAI

	lda	#24
	clc
	adc	v_playery
	sta	v_playery

	lda	v_playerdir
	and	#MOVRT
	bne	.adjright

	lda	v_playerx
	sec
	sbc	#32
	sta	v_playerx

	jmp	Newframe
.adjright
	lda	v_playerx
	clc
	adc	#32
	sta	v_playerx
.donewframe
	jmp	Newframe	; Do a new frame


*****************************************************************************
*
* Fall Sequence with transition
*
*****************************************************************************

TZFall1:
	lda	#tzfalltrans

	jsr	Validatetype

	lda	#10
	sta	v_playerdelay

	lda	v_pitopen
	and	#$FF00
	bne	.fallpit

	bra	.cont
.fallpit
	lda	v_playery
	lsr
	lsr
	cmp	#$330
	bcs	.killem
.cont
	ldx	z_levelword
	lda	v_playery
	lsr
	lsr
	cmp	d_maxy,x
	bcs	.killem

	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir

	lda	v_playery
	sec
	sbc	v_markfall

	cmp	v_fallmax
         	bcc	.chkforflrnow
	
	lda	#1
	sta	v_disbgscroll
	
	lda	z_levely
	asl
	asl
	clc
	adc	#330<<2
	cmp	v_playery
	bcs	.chkforflrnow
.killem
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag
.xitfall
	rtl

.chkforflrnow
	jsl	Chek4floorfall	; Have we detected a floor?

	bne	.landitnow	; Yes - switch to finish jump

	lda	z_joy1	; Get keys
	bit	v_JOYPADUP	; Pressing up?
	beq	.falldown	; No  - 

	lda	v_tilestat	; Yep - Get tile stat
	cmp	#CLIMBTYPE
	bne	.falldown

	lda	#TZJmptoladr
	sta	v_playerAI

	rtl
.falldown

	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	v_velocity	; Get velocity
	clc
	adc	v_gravity	; Subtract gravity
	sta	v_velocity

	cmp	#(MAXVELOCITY*4)
	bcc	.oktomovdn	; If still some left, then move

	lda	#(MAXVELOCITY*4)	; Terminal velocity
	sta	v_velocity
.exithold 
;	rtl

.oktomovdn 
	jsl	Chklrmov	; Check for left/right movement

	rtl

.landitnow 
	stz	v_playerdelay

	lda	#TZLand0
	sta	v_playerAI

	rtl


*****************************************************************************

TZStrtfallx1:
	lda	#TZFalltrans1
	sta	v_playerAI

	lda	#tzjmpfal00
	sta	v_playernewtype	; Store new type to restart

	lda	#2
	sta	v_velocity
	lda	#3
	sta	v_gravity

	lda	v_playery
	sta	v_markfall

	lda	#1
	sta	v_playerdelay


*****************************************************************************
*
* Fall Sequence
*
*****************************************************************************

TZFalltrans1:
	lda	v_playertype	; Get current mode type
	cmp	#tzjmpfal00	; Already in this mode?
	beq	.tzjmpfal1	; Yes - Just do timer thing then
          	
	dec	v_playerdelay	; No - Then finish last sequence
	bmi	.changemode	; Change modes if done

	rtl

.changemode 
	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir

	and	#MOVRT
	beq	.movlft

	lda	v_playerx
	sec
	sbc	#14<<2
	sta	v_playerx

	bra	.fixmov
.movlft
	lda	v_playerx
	clc
	adc	#14<<2
	sta	v_playerx

.fixmov
	lda	v_playery
	clc
	adc	#16<<2
	sta	v_playery
.nxtfrm
	lda	#tzjmpfal00
	sta	v_playernewtype	; Store as new type to restart

	jmp	Newframe	; Do a new frame
.tzjmpfal1
	dec	v_playerdelay
	bpl	.chk4pit

	lda	v_playerseqlog
	cmp	#3
	bcc	.rotate

	jmp	TZStartfall
.rotate
	cmp	#2
	bne	.nxtfrm

	lda	v_playerdir

	and	#MOVRT
	bne	.movlft1

	lda	v_playerx
	sec
	sbc	#14<<2
	sta	v_playerx

	bra	.fixmov1
.movlft1
	lda	v_playerx
	clc
	adc	#14<<2
	sta	v_playerx

.fixmov1
;	lda	v_playery
;	clc
;	adc	#16<<2
;	sta	v_playery

	jmp	TZStartfall

.chk4pit
	lda	v_falling
	bne	.skipdetect

	lda	v_pitopen	; Detect a pit
	bne	.opit

	bra	.skipdetect
.opit
	jsl	Chek4number	; Gets tile number	

	cmp	#$38	; Over pit area?
	beq	.fallpit	; Yes -

	bra	.cont
.fallpit
	stz	v_falling
	inc	v_falling

.skipdetect
	lda	v_playery
	lsr
	lsr
	sec
	sbc	z_levely
	bmi	.cont

	cmp	#$130
      	bcs	.killem
.cont
	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir

	lda	v_playery
	sec
	sbc	v_markfall

	cmp	v_fallmax
         	bcc	.chkforflrnow

	lda	#1
	sta	v_disbgscroll
	
	lda	z_levely
	asl
	asl
	clc
	adc	#330<<2
	cmp	v_playery
	bcs	.chkforflrnow
.killem
	stz	v_playerdelay
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag
.xitfall
	rtl

.chkforflrnow
	lda	v_hitmonkey
	beq	.nomonkey
	
	dec	v_hitmonkey
	bpl	.falldown

	stz	v_hitmonkey

	bra	.falldown
.nomonkey
	jsl	Chek4floorfall	; Have we detected a floor?

	bne	.landitnow	; Yes - switch to finish jump
.chk4ladder
	lda	z_joy1	; Get keys
	bit	v_JOYPADUP	; Pressing up?
	beq	.falldown	; No  - 

	lda	v_tilestat	; Yep - Get tile stat
	cmp	#CLIMBTYPE
	bne	.falldown

	lda	#TZJmptoladr
	sta	v_playerAI

	rtl
.falldown

	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	v_velocity	; Get velocity
	clc
	adc	v_gravity	; Subtract gravity
	sta	v_velocity

	cmp	#(MAXVELOCITY*4)
	bcc	.oktomovdn	; If still some left, then move

	lda	#(MAXVELOCITY*4)	; Terminal velocity
	sta	v_velocity
.exithold 
;	rtl

.oktomovdn 
	jsl	Chklrmov	; Check for left/right movement

	rtl

.landitnow 
	lda	v_falling
	beq	.justlandit

	bra	.falldown
.justlandit
	stz	v_playerdelay

	lda	#TZLand0
	sta	v_playerAI

	rtl


*****************************************************************************
*
* Jump Sequence
*
*****************************************************************************

TZStartjump:
	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	lda	v_stride
	lsr   
	sta	v_stride	; shorten stride length

	lda	#TZJmpup
	sta	v_playerAI

	lda	#$ffe0	; Normal jump velocity
	sta	v_velocity	; Store initial velocity

	lda	#2	; Gravity constant
	sta	v_gravity

	stz	v_playerdelay


	rtl

*****************************************************************************
*
* Jump Sequence
*
*****************************************************************************

TZJmpup:
	lda	v_playertype	; Get current mode type
	cmp	#tzjumpupmode00	; Already in this mode?
	beq	.tzjmpup1	; Yes - Just do timer thing then
          	
	dec	v_playerdelay	; No - Then finish last sequence
	bmi	.changemode	; Change modes if done

	rtl

.changemode 
	lda	#MOVRT|MOVLT	;
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	lda	#tzjumpupmode00	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart

	jmp	Newframe	; Do a new frame
.tzjmpup1 
	lda	v_playerseqlog	; Get Tarzans step into current seqlog
	cmp	#6	; Last frame of start sequence?
	bcs	.chngtohold	; No  -

	lda	v_playerseqlog	; Get Tarzans step into current seqlog
	cmp	#5	; Last frame of start sequence?
	bcc	.exitstart	; No  -

	lda	#MOVRT|MOVLT	;
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir
.chngtohold 
	lda	#TZDohold	; Branch to hold frames
	sta	v_playerAI
.exitstart 
	dec	v_playerdelay	; Adjust frame ^count
	bpl	.tzjmpup1	; Process if still going...

	jmp	Newframe	; Do a new frame

	rtl


; ---------------------------------------

TZDohold:
	lda	v_playerseqlog
	cmp	#10	; At top of jump?
	bcs	.exithold	; Yes -

.chk4attck
;	lda	v_playerseqlog
;	cmp	#9
;	bcs	.skipattk
;
;	lda	z_joy1
;	and	v_WEAPON
;	beq	.skipattk
;
;	stz	v_donenifsfx
;	stz	v_playerdelay
;
;	lda	#TZJmpnstb
;	sta	v_playerAI
;
;	rtl

.skipattk
	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.chkceil	; No  - Then no vine or climb

	lda	v_tilestat	; Yep - Get tile type
	cmp	#4
	bne	.chkceil

	lda	#TZClimbup
	sta	v_playerAI

	rtl

.chkceil
	lda	#CEILING
	and	_walls
	beq	.adjvel

	lda	#TZStartfall	; Yes - Branch to hold frame
	sta	v_playerAI

	rtl
.adjvel 
	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	z_joy1
	bit	v_JUMPBUTTON
	beq	.normalgrav

	lda	v_gravity
	lsr
	bra	.addgravity
.normalgrav
  	lda	v_gravity	; Get velocity
.addgravity
	clc
	adc	v_velocity	; Subtract gravity
	sta	v_velocity	; New velocity
	bmi	.oktomovup

.exithold 
;	lda	z_joy1	; Get keys
;	and	v_JOYPADUP	; Pressing up?
;	beq	.novinefound	; No  - Then no vine or climb
;
;.look4vine
;	jsl	Lookforvine
;
;	beq	.novinefound
;
;	lda	#TZSwing2
;	sta	v_playerAI
;
;	stz	v_playerdelay
;
;	rtl
;.novinefound
	stz	v_playerdelay
	
	jmp	TZStartfall

.oktomovup 
	jsl	Chklrmov	; Check for left/right movement


	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	bra	.xithold

.floordet 
	lda	#TZFinijmp
	sta	v_playerAI

	inc	v_ycount

	stz	v_playerdelay
.xithold 
	dec	v_playerdelay	; Adjust frame count
	bpl	.xithold1	; Process if still going...

	lda	#tzjumpupmode00	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart

	jmp	Newframe	; Do a new frame
.xithold1 
	rtl



; ---------------------------------------

TZFinijmp:
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#17	; Last frame of sequence?
	bcc	.chkjmp	; No  -

	lda	#TZWait	; Yes -
	sta	v_playerAI

	bra	.exitfinish

.chkjmp 
	lda	z_joy1	; Get keys
	bit	v_JUMPBUTTON	; Jump again?
	beq	.chkdir	; No  -

	bit	v_JOYPADDOWN	; Wants to jump down?
	beq	.jmpupthen	; No  -

	lda	v_tilestat	; Get tile type
	bit	#PASSTHRUTYPE
	beq	.exitfinish	; don't allow drop
.okfall 
	jmp	TZStartfall1
.jmpupthen 
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#12	; Last frame of sequence?
	bcc	.chkdir

	lda	v_ycount
	bne	.exitfinish

;	inc	v_ycount

	jmp	TZStartjump

.chkdir 
	lda	z_joy1	; 
	and	v_JOYLEFTRIGHT	; Is directional still pressed?
	beq	.exitfinish	; No - Finish stand-up then

	lda	v_playerdelay

	lsr
	lsr

	sta	v_playerdelay

	lda	v_playerseqlog	; Get Tarzans step into current seqlog
	cmp	#17	; Last frame of start sequence?
	bcc	.exitfinish	; No  -

	lda	#TZWalk	; ???????????????? check this ?????
	sta	v_playerAI
.exitfinish 
	dec	v_playerdelay	; Adjust frame count
	bpl	.xit	; Process if still going...

	jmp	Newframe	; Do a new frame

.xit
	rtl



; =======================================

Chklrmov:
	lda	#8	; Preload normal left/right movement
	sta	v_stride

	lda	z_level
	and	#$FF
	cmp	#BEACH
	bne	.skipbeachk

	lda	v_playerx
	lsr
	lsr
	cmp	#$890
	bcc	.skipbeachk

	cmp	#$895
	bcs	.skipbeachk

	rtl
.skipbeachk
	lda	z_joy1	; Get controller pad bits
	and	v_JOYLEFTRIGHT	; Was left or right pushed?
	bne	.chkdir	; Yes - Check which direction
.nomov 
	rtl		; Else we're done
.chkdir 
	lda	v_playerdir	; Get players direction
	and	#MOVRT	; Is he moving right?
	bne	.right	; Yes - Process right movement

	lda	z_joy1	; Get pad stats
	and	v_JOYPADLEFT	; User pressing left?
	beq	.turnright	; No  - Pressing "right" so change dir

	lda	#MOVLT	; Yes - Check for walls
	and	_walls
	bne	.nomov	; There are walls, so no move allowed

	lda	v_playerx	; Otherwise, move player 4 pix
	sec
	sbc	v_stride
	sta	v_playerx

	rtl
.turnright 
	lda	#MOVUP|MOVDN	; No  - Then reset left flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir

	rtl
	
.right 
	lda	z_joy1	; Get pad stats
	and	v_JOYPADRIGHT	; User pressing right?
	beq	.turnleft	; No  - Pressing "left" so change dir

	lda	#MOVRT	; Yes - Check for walls
	and	_walls
	bne	.nomov	; There are walls, so no move allowed

	lda	v_playerx	; Otherwise, move player 4 pix
	clc
	adc	v_stride
	sta	v_playerx

	rtl
.turnleft 
	lda	#MOVUP|MOVDN	; No  - Then reset left flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir

	rtl


*****************************************************************************
*
* Stab while Jump-Up Sequence
*
*****************************************************************************

TZJmpnstb:
	lda	#tzjmpnstb

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#5
	bcc	.chk4hit

	stz	v_foekillfrm
	stz	v_knifeactive

	jmp	TZStartfall
.chk4hit
	lda	v_playerseqlog
	cmp	#3	; Identify "knifing" frames
	beq	.notfyfoe	; Set killer frame flag for foes

	cmp	#4
	beq	.killframe

	stz	v_foekillfrm
	stz	v_knifeactive

	bra	.chkceil

.killframe
	lda	v_donenifsfx
	bne	.notfyfoe

	lda	#whoosh1_eff

	jsl	load_Effect

	lda	#1
	sta	v_donenifsfx

.notfyfoe 
	lda	#1
	sta	v_foekillfrm
	sta	v_knifeactive

	lda	#4
	sta	v_playinvince

	stz	v_bosshit

.chkceil
	lda	#CEILING
	and	_walls
	beq	.adjvel

	lda	#TZStartfall	; Yes - Branch to hold frame
	sta	v_playerAI

	rtl
.adjvel 
	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery


	lda	v_gravity
	lsr
	clc
	adc	v_velocity	; Subtract gravity
	sta	v_velocity	; New velocity
	bmi	.oktomovup

.exithold 
	jsl	Chek4floor	; Check for a floor

	bne	.mustland	; Found one

	stz	v_foekillfrm
	stz	v_knifeactive

	jmp	TZStartfall	; Not on solid ground, begin fall

.mustland
	lda	#TZLand0
	sta	v_playerAI
	stz	v_playerdelay
	
	rtl

.oktomovup 
	jsl	Chklrmov	; Check for left/right movement

	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

.xithold 
	dec	v_playerdelay	; Adjust frame count
	bpl	.xithold1	; Process if still going...

	lda	#tzjmpnstb	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart

	jmp	Newframe	; Do a new frame
.xithold1 
	rtl


*****************************************************************************
*
* Land Sequence - Land to Stand / Walk
*
*****************************************************************************

TZLand0:
	lda	#tzlandmode00	; Already in Land to Stand?

	VERIFYMODE

	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	jsl	Dopitstuff

	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#3	; Last frame to allow mode change?
	bcs	.procland	; Yes -

	lda	z_joy1	; No  - Wants to go Down?
	bit	v_JOYPADDOWN	
	beq	.ckhnewjmp	; No  -

	bit	v_JUMPBUTTON	; Jump pressed
	beq	.duck	; No  - Then Tarzan wants to duck

	lda	v_tilestat	; Get tile type
	bit	#PASSTHRUTYPE
	beq	.duck

	jmp	TZStartfall
.duck 
	lda	#tzducksmode00
	sta	v_playernewtype
	sta	v_playeroveride

	jsl	Newframe	; Crank the machine, but no start

	lda	#1
	asl
	asl
	sta	v_playerseq

	lda	#TZDuck00	; Need to specify 2nd frame
	sta	v_playerAI

	stz	v_playerdelay

	jmp	Newframe

.ckhnewjmp 
	bit	v_JUMPBUTTON	; Jump pressed?
	beq	.procland	; No  - Then Tarzan wants to duck

	jmp	TZStartjump
.exitm
	rtl

.procland 
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#4	; Last frame to allow mode change?
	bcc	.chkwlk	; No  -

	lda	#TZWait
	sta	v_playerAI

	rtl

.chkwlk 
	lda	z_joy1
	bit	v_WEAPON	; Wants to stab something?
	beq	.chk4run

	jmp	.setstab	; Yes -
.chk4run
	bit	v_RUNBUTTON	; Wants to run now?
	beq	.chkwlk1	; No  - Check for request walk

	bit	v_JOYLEFTRIGHT	; Yes - But is a direction pressed as well?
	beq	.exit	; No -

	lda	#MOVRT|MOVLT
	and	_walls
	beq	.ok2run

	lda	v_playerdir
	and	#MOVRT
	beq	.applylft

	lda	v_playerx
	tax
	clc
	adc	#12<<2
	sta	v_playerx

	jsl	Chek4stat

	stx	v_playerx	; Restore original X

	lda	v_tilestat
	cmp	#WALKSOLID	
	beq	.ok2run


.applylft
	lda	v_playerx
	tax
	sec
	sbc	#12<<2
	sta	v_playerx

	stx	v_playerx	; Restore original X

	lda	v_tilestat
	cmp	#WALKSOLID	
	beq	.ok2run

	jmp	.exit
.ok2run
	lda	v_playerseqlog	;
	cmp	#2
	bcc	.xitwdelay

	lda	#tzlandmode01	; Yes - Load Land to Run sequence
	sta	v_playernewtype
	sta	v_playeroveride

	jsl	Newframe	; Crank the machine, but no start

	lda	#2
	asl
	asl
	sta	v_playerseq

	lda	#TZLand1	; Need to specify 2nd frame
	sta	v_playerAI

	stz	v_playerdelay

	jmp	Newframe

.chkwlk1 
	lda	z_joy1	;
	and	v_JOYLEFTRIGHT	; Direction?
	beq	.exit	; None
.xitwdelay
	lda	v_playerdelay
	lsr
	sta	v_playerdelay
.exit 
	rtl

.setstab 
	lda	#TZWait
	sta	v_savpAI

	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	jmp	Newframe

	rtl
.dodarts 
	lda	v_numdart
	beq	.skipattk

         	lda	#TZDart1	; Dart attack
	sta	v_playerAI

	lda	#tzshtstdmode00
	sta	v_playernewtype

	jmp	Newframe

.skipattk
	rtl

*****************************************************************************
*
* Land Sequence - Land to Run
*
*****************************************************************************

TZLand1:
	lda	#tzlandmode01	; Already in Land to Run?

	VERIFYMODE

	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	jsl	Dopitstuff

	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#3	; Last frame to allow mode change?
	bcs	.procland	; Yes -

	lda	z_joy1	; Down?
	bit	v_JOYPADDOWN	
	beq	.ckhnewjmp	; No  -

	bit	v_JUMPBUTTON	; Jump pressed
	beq	.duck	; No  - Then Tarzan wants to duck

	lda	v_tilestat	; Get tile type
	bit	#PASSTHRUTYPE
	beq	.duck

	jmp	TZStartfall
.duck 
	lda	#tzducksmode00
	sta	v_playernewtype
	sta	v_playeroveride

	jsl	Newframe	; Crank the machine, but no start

	lda	#1
	asl
	asl
	sta	v_playerseq

	lda	#TZDuck00	; Need to specify 2nd frame
	sta	v_playerAI

	stz	v_playerdelay

	jmp	Newframe

.ckhnewjmp 
	bit	v_JUMPBUTTON	; Jump pressed
	beq	.procland	; No  - Then Tarzan wants to duck

	jmp	TZStartjump
.exitm
	rtl

.procland 
	lda	z_joy1
	bit	v_WEAPON	; Wants to stab something?
	bne	.setstab	; Yes -

	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.exitlandmode	; No  -

	stz	v_stride

	lda	v_playerseqlog	;
	cmp	#4	;
	bne	.set5

	lda	#4
	sta	v_stride

	bra	.chkdirld1
.set5
	lda	v_playerseqlog	;
	cmp	#5	;
	bne	.chkdirld1

	lda	#12
	sta	v_stride


.chkdirld1
	lda	z_joy1
	and	v_JOYPADLEFT
	bne	.setlft

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	lda	v_playerx
	clc
	adc	v_stride
	sta	v_playerx

	bra	.exitltr
.setlft 
	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going right

	lda	v_playerx
	sec
	sbc	v_stride
	sta	v_playerx

.exitltr 
	lda	v_playerseqlog	;
	cmp	#5	;
	bcc	.exitm

	lda	#TZRun
	sta	v_playerAI

	rtl

.exitlandmode
	lda	#TZWait
	sta	v_playerAI

	rtl

.setstab 
	lda	#TZWait
	sta	v_savpAI

	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	jmp	Newframe

	rtl
.dodarts 
	lda	v_numdart
	beq	.skipattk

         	lda	#TZDart1	; Dart attack
	sta	v_playerAI

	lda	#tzshtstdmode00
	sta	v_playernewtype

	jmp	Newframe

.skipattk
	rtl

*****************************************************************************
*
* Walk/Run/Stand to Long Jump Sequence
*
* The variable v_whichjmp is set to 1,2 or 3 depending on which type
* of sequence is required.
*
* The variable v_stride is set for the appropiate amount of "hang-time"
* required by the type of sequence.
*
*****************************************************************************

TZStdjmp:

	lda	v_playertype	; Get current mode type
	cmp	#tzstdjmpmode00	; Already in this mode?
	bne	.chkjpmode1

	jmp	.ckfloor	; Yes - 

.chkjpmode1 
	cmp	#tzstdjmpmode01	; Already in this mode?
	bne	.chkjpmode2

	jmp	.ckfloor	; Yes - 

.chkjpmode2 

	cmp	#tzstdjmpmode02	; Already in this mode?
	bne	.chkjpmode3

	jmp	.ckfloor	; Yes - 

.chkjpmode3 

	dec	v_playerdelay	; No - Then finish last sequence
	bmi	.sortitout	; Change modes if done

	rtl


****************************************

.sortitout 

	lda	v_whichjmp	; This is set in previous mode
	beq	.exitout	; No sequence transition set

	cmp	#2		; Stand to jump?
	bcs	.maybe2	; Yes - 


	lda	#tzstdjmpmode00	; Set to starting long jump sequence
	sta	v_playernewtype	; Store as new type to restart

	bra	.changemode
.maybe2 
	bne	.mustbe3	; No  -

	lda	#32
	sta	v_stride	; Store new stride length

	lda	#tzstdjmpmode01	; Yes - Set Walk to Jump sequence
	sta	v_playernewtype	; Store as new type to restart

	bra	.changemode

.mustbe3 
	lda	#48
	sta	v_stride	; Store new stride length

	lda	#tzstdjmpmode02	; Set Run to Jump sequence
	sta	v_playernewtype	; Store as new type to restart

.changemode 
	lda	v_playerdir	; Get direction
	and	#MOVRT	; Moving right?
	beq	.chklft	; No  - Must be left then

	lda	v_stride
	clc
	adc	v_playerx	; Move to the right
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.setsail
.chklft 
	lda	v_playerx
	sec
	sbc	v_stride	; Move to the left
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir
.setsail 
	lda	_walls
	and	v_playerdir	; Did we hit a wall?
	beq	.exitout	; No - Keep going

	jmp	TZStartfall	; Begin falling down wall

.exitout 
	jmp	Newframe	; Do a new frame

	rtl



****************************************

.ckfloor 
	jsl	Dopitstuff

	lda	v_playertype	; Get current mode type
	cmp	#tzstdjmpmode00	; Already in this mode?
	bne	.chkwlkjmp	; No  -

	jmp	Standingjmp	; Yes -
.chkwlkjmp 
	cmp	#tzstdjmpmode01	; Already in this mode?
	bne	Dorunjmp	; No  - 

	jmp	Walkingjmp	; Yes -


****************************************

RJFRM2_5x	.equ	88	; #22<<2
RJFRM2_5y	.equ	40	; #10<<2
RJFRM6x	.equ	64	; #16<<2
RJFRM6y	.equ	16	; #04<<2


Dorunjmp:
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#6	; Transition frame hit yet
	bcc	.chklow

	jmp	.doending
.chklow
	cmp	#2	; Transition frame hit yet
	bcc	.fixit

	ldx	v_playerx	; Save original X
	ldy	v_playery

	cmp	#6
	bcc	.dofrm2_5

	lda	#RJFRM6x
	sta	z_temp0

	lda	#RJFRM6y
	sta	z_temp1

	bra	.setnewxy
.dofrm2_5
	lda	#RJFRM2_5x
	sta	z_temp0

	lda	#RJFRM2_5y
	sta	z_temp1
.setnewxy
	lda	v_playerdir
	and	#MOVRT
	beq	.applylftn

	lda	v_playerx
	clc
	adc	z_temp0
	sta	v_playerx

	bra	.nowchkflrn

.applylftn
	lda	v_playerx
	sec
	sbc	#z_temp0
	sta	v_playerx
.nowchkflrn
	tya
	sec
	sbc	#z_temp1
	sta	v_playery

	phx
	phy

	jsl	Chek4wall

	ply
	plx

	lda	#MOVLT|MOVRT
	and	_walls
	bne	.fallnow

	jsl	Chek4floor

	bne	.fallnow

	stx	v_playerx	; Restore original X
	sty	v_playery

	jmp	Fixuppos

.fallnow
	stx	v_playerx	; Restore original X
	sty	v_playery

	lda	v_playerseqlog
	cmp	#7
	bcs	.ok2fall

	stz	v_playerdelay

	jmp	Fixuppos
.ok2fall
	jmp	TZStartfall

.fixit
	jmp	Fixuppos

.doending 
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#7	; Transition frame hit yet
	bcs	.chfnewfrm

	jsl	Chek4floor

	bne	.chfnewfrm

	stz	v_whichjmp

	jmp	TZStrtfallx1
           	
.chfnewfrm 
;	lda	v_playerseqlog
;	cmp	#7
;	bne	.nrml

	lda	v_playerdelay
	lsr
	sta	v_playerdelay
.nrml 
	lda	#6
	sta	v_stride	; Store new stride length

	lda	v_playerdir	; Get direction
	and	#MOVRT	; Moving right?
	beq	.chklft	; No  - Must be left then

	lda	v_stride
	clc
	adc	v_playerx	; Move to the right
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.setsail
.chklft 
	lda	v_playerx
	sec
	sbc	v_stride	; Move to the left
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir
.setsail 
	lda	_walls
	and	v_playerdir	; Did we hit a wall?
	beq	.exitout	; No - Keep going

	jmp	TZStartfall	; Begin falling down wall

.exitout
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#8	; Transition frame hit yet?
	bcs	.run

	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...
	
	jmp	Newframe	; Do a new frame

	rtl

.run 
	lda	#TZRun
	sta	v_playerAI

	lda	#12
	sta	v_stride

	stz	v_whichjmp
.exit
	rtl


****************************************

Walkingjmp:
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#5	; Transition frame hit yet?
	bcc	.chklow

	jmp	.doending
.chklow
	cmp	#2
	bcc	.fixit

	ldx	v_playerx	; Save original X
	ldy	v_playery

	cmp	#6
	bcc	.dofrm2_5

	lda	#RJFRM6x
	sta	z_temp0

	lda	#RJFRM6y
	sta	z_temp1

	bra	.setnewxy
.dofrm2_5
	lda	#RJFRM2_5x
	sta	z_temp0

	lda	#RJFRM2_5y
	sta	z_temp1
.setnewxy
	lda	v_playerdir
	and	#MOVRT
	beq	.applylftn

	lda	v_playerx
	clc
	adc	z_temp0
	sta	v_playerx

	bra	.nowchkflrn

.applylftn
	lda	v_playerx
	sec
	sbc	#z_temp0
	sta	v_playerx
.nowchkflrn
	tya
	sec
	sbc	#z_temp1
	sta	v_playery

	phx
	phy

	jsl	Chek4wall

	ply
	plx

	lda	#MOVLT|MOVRT
	and	_walls
	bne	.fallnow

	jsl	Chek4floor

	bne	.fallnow

	stx	v_playerx	; Restore original X
	sty	v_playery

	jmp	Fixuppos

.fallnow
	stx	v_playerx	; Restore original X
	sty	v_playery

	lda	v_playerseqlog
	cmp	#7
	bcs	.ok2fall

	stz	v_playerdelay

	jmp	Fixuppos
.ok2fall
	jmp	TZStartfall

.fixit
	jmp	Fixuppos

.chfnewfrm 
	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	jmp	Newframe	; Do a new frame

.doending 
	cmp	#5
	beq	.look4flr

	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#6	; Transition frame hit yet?
	bcs	.walk

	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	jmp	Newframe	; Do a new frame
.exit
	rtl

.walk 
	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	lda	#tzwalk3mode00	; Yes - Load Walk sequence
	sta	v_playernewtype
	sta	v_playeroveride

	jsl	Newframe	; Crank the machine, but no start

	lda	#2
	asl
	asl
	sta	v_playerseq

	lda	#TZWalk	; Need to specify 2nd frame
	sta	v_playerAI

;	stz	v_playerdelay

	lda	#50
	sta	v_stride

	jsl	TZOffset

	jmp	Newframe

.look4flr 
	jsl	Chek4floor

	bne	.chfnewfrm

	stz	v_playerdelay

	jmp	TZStartfall
           	

****************************************

Standingjmp:	
	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#8	; Transition frame hit yet?
	bcs	.doending

	cmp	#4
	bcc	.chfnewfrm

	jmp	Fixuppos

.chfnewfrm 
	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	jmp	Newframe	; Do a new frame

.doending 
	cmp	#8
	beq	.look4flr

	lda	v_playerseqlog	; Get Tarzans step into current seq
	cmp	#11	; Transition frame hit yet?
	bcc	.chfnewfrm

.standup 
	lda	#TZWait
	sta	v_playerAI

	stz	v_whichjmp
.exit 
	rtl

.look4flr 
	jsl	Chek4floor

	bne	.chfnewfrm

	stz	v_playerdelay

	jmp	TZStartfall
           	

****************************************

Fixuppos:
	lda	#15
	sta	v_stride	; Store new stride length

	lda	v_playerdir	; Get direction
	and	#MOVRT	; Moving right?
	beq	.chklft	; No  - Must be left then

	lda	v_stride
	clc
	adc	v_playerx	; Move to the right
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.setsail
.chklft 
	lda	v_playerx
	sec
	sbc	v_stride	; Move to the left
	sta	v_playerx

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir
.setsail 
	lda	_walls
	and	v_playerdir	; Did we hit a wall?
	beq	.oktomovert	; No - Keep going

	jmp	TZStartfall	; Begin falling down wall

.oktomovert 
	lda	v_playertype	; Get current mode type
	cmp	#tzstdjmpmode02
	beq	.do3rd

	lda	v_playertype	; Get current mode type
	cmp	#tzstdjmpmode00	; Already in this mode?
	bne	.maybe2	; Yes - 

	lda	v_playerseqlog
	sec
	sbc	#4
	bmi	.exitctrl

	bra	.getoffset
.maybe2 
	lda	v_playerseqlog
	sec
	sbc	#1
	bmi	.exitctrl
.getoffset 
	asl
	tax
	lda	>.d_offsetaby,x
	clc
	adc	v_playery
	sta	v_playery
.exitctrl 
	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	jmp	Newframe	; Do a new frame
.exit 
	inc	v_ycount

	rtl

.do3rd 
;	lda	v_playerseqlog
;	cmp	#2
;	bcc	.noadj
;
;	cmp	#6
;	bcs	.noadj
;
;	lda	v_playerx
;	pha
;
;	lda	v_playery
;	pha
;
	


.noadj
	lda	v_playerseqlog
	asl
	tax
	lda	>.d_offsetaby3,x
	clc
	adc	v_playery
	sta	v_playery

	dec	v_playerdelay	; Adjust frame count
	bpl	.exit	; Process if still going...

	jmp	Newframe	; Do a new frame

; --------------------------------------------------------

.d_offsetaby 
	.dw	-16,-9,9,16,0,0,0,0,0,0,0

.d_offsetaby3 
	.dw	-16,-8,-2,2,8,16,0,0,0,0,0,0,0


*****************************************************************************
*
* Stab from Stand
*
*****************************************************************************

TZStab1:
	lda	#tzstab1mode00

	jsr	Validatetype

	lda	v_playerseqlog	; Get current frame
     	cmp	#10	; Last frame?
	bcc	.stayhere	; No - Keep processing sequence

	lda	z_joy1	; Yes - Then see if we go again
	and	v_WEAPON
	bne	.doagain	; Yes - Do another stab

	lda	v_savpAI
	sta	v_playerAI

	stz	v_donenifsfx

	bra	.exittzstab1
.stayhere 
	lda	z_joy1
	and	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.looknife	; No  -

	lda	z_joy1
	and	v_JOYPADLEFT
	bne	.setlft

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	bra	.looknife
.setlft 
	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going right

.looknife 
	lda	v_playerseqlog
	cmp	#4	; Identify "knifing" frames
	beq	.notfyfoe	; Set killer frame flag for foes

	cmp	#5
	beq	.killframe

	cmp	#6
	beq	.notfyfoe

	stz	v_foekillfrm
	stz	v_knifeactive

	bra	.exittzstab1

.killframe
	lda	v_donenifsfx
	bne	.notfyfoe

	lda	#whoosh1_eff

	jsl	load_Effect

	lda	#1
	sta	v_donenifsfx

.notfyfoe 
	lda	#1
	sta	v_foekillfrm
	sta	v_knifeactive

	lda	#4
	sta	v_playinvince

	stz	v_bosshit

	bra	.exittzstab1

.doagain 
	lda	#1	; Request frame number
	asl
	asl
	sta	v_playerseq	; Get sequence pointer

	stz	v_donenifsfx

.exittzstab1 
	dec	v_playerdelay	; Adjust frame count
	bpl	.xit	; Process if still going...

	jmp	Newframe	; Do a new frame
.xit
	rtl


*****************************************************************************
*
* Duck down
*
*****************************************************************************

TZDuck00:
	lda	#tzducksmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if floor was found	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	z_joy1	; Check for attack request
	and	v_WEAPON	; Knife 'em?
	beq	.skipattk	; No  -

	lda	v_playerAI	; Yes - Save current vector
	sta	v_savpAI

	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	rtl
.dodarts 
	lda	v_numdart
	beq	.skipattk

         	lda	#TZDart2	; Dart attack
	sta	v_playerAI

	lda	#tzducksmode02
	sta	v_playernewtype

	jmp	Newframe

	rtl
.skipattk 
	lda	v_playerseqlog
     	cmp	#3
	bcc	.stayhere

	lda	z_joy1	; Stay down?
	bit	v_JOYPADDOWN	
	beq	.standup	; No  -

	bit	v_JUMPBUTTON	; Jump pressed
	beq	.stayduck	; No  -	

	lda	v_tilestat	; Get tile type
	bit	#PASSTHRUTYPE
	beq	.stayduck
.okfall 
	jmp	TZStartfall

.stayduck 
	lda	#10
	sta	v_playerdelay

	lda	z_joy1
	bit	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.exitdir	; No  -

	bit	v_JOYPADLEFT
	bne	.setlft

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right
.exitdir 
	rtl

.setlft 
	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going right

	rtl

.standup 
	lda	#TZDuck01
	sta	v_playerAI

	lda	#tzducksmode01	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart

	jmp	Newframe
.stayhere 
	rtl


*****************************************************************************
*
* Stand up from Duck
*
*****************************************************************************

TZDuck01:
	lda	#tzducksmode01

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	v_playerframe
     	cmp	#0
	bne	.stayhere

	lda	#TZWait
	sta	v_playerAI
.stayhere 
	rtl




*****************************************************************************
*
* Take a big hit
*
*****************************************************************************

TZBighit:
	lda	#tzhitfalmode00	; Otherwise, init sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	stz	v_noplayxmov

	lda	v_playerseqlog
	cmp	#3
	bcc	.chkwalls

	jmp	.chkend
.chkwalls

	lda	v_playerx
	tax
	tay

	lda	v_playerdir
	and	#MOVRT
	beq	.kickrt2a

	tya
	sec
	sbc	#12

	bra	.chkwallnow
.kickrt2a
	tya
	clc
	adc	#12
.chkwallnow
	sta	v_playerx

	lda	_walls
	and	#(MOVUP|MOVDN)
	sta	_walls

	phx

	jsl	Chek4wall

	plx

	lda	#(MOVRT|MOVLT)
	and	_walls
	bne	.noxmove

	lda	v_playerseqlog
	cmp	#2
	beq	.startdwn

	bcc	.procfly
.startdwn
	stz	v_bighityadj
	bra	.procfly

	jmp	.chkend
.noxmove
	stx	v_playerx

	lda	#1
	sta	v_noplayxmov

	lda	#8
	sta	v_velocity

	stz	v_bighityadj
.procfly 
	lda	v_playery
	sec
	sbc	v_bighityadj
	sta	v_playery

	lda	v_noplayxmov
	bne	.stayh

	lda	v_playerdir
	and	#MOVRT
	beq	.kickrt

	lda	v_playerx
	sec
	sbc	v_bighitxadj
	sta	v_playerx

	bra	.stayh
.kickrt 
	lda	v_playerx
	clc
	adc	v_bighitxadj
	sta	v_playerx
.stayh
	lda	v_playerseqlog
	cmp	#2
	bcc	.stayhere

.chkend 
	jsl	Chek4floorfall	; Have we detected a floor?

	beq	.dofall	; No  - Do fall

	lda	v_bighitos	; Yes - Is this 1st floor detect?
	bne	.exithold1	; No  -

	lda	#1	; Yes -
	sta	v_bighitos	; Set oneshot
	stz	v_playerdelay	; Quickly change to landing

	bra	.stayhere	;
.dofall
	lda	#10
	sta	v_playerdelay

	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	v_velocity	; Get velocity
	clc
	adc	v_gravity	; Add gravity
	sta	v_velocity

	cmp	#(MAXVELOCITY*4)
	bcc	.exithold1

	lda	#(MAXVELOCITY*4)	; Terminal velocity
	sta	v_velocity
.exithold1
	lda	v_playerseqlog
     	cmp	#16
	bcc	.stayhere

	stz	v_bighitos
	stz	v_bosshit

	lda	#TZWait
	sta	v_playerAI
.stayhere 
	rtl


*****************************************************************************
*
* Take a Big Boss Hit from behind
*
*****************************************************************************

TZBighitback:
	lda	#tzhitfalmode01	; Otherwise, init sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#1
	bcs	.dobhit

	rtl

.dobhit
	stz	v_playerdelay
	inc	v_playerdelay

	lda	#TZBighit
	sta	v_playerAI

	rtl


*****************************************************************************
*
* Do Blowpipe while standing
*
*****************************************************************************

TZDart1:
	lda	#tzshtstdmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	z_joy1
	and	v_WEAPON	; Is player shooting?
	beq	.chknxt	; No  - Just do sequence then

	lda	v_playerseqlog	; Yes - Get current frame number
	cmp	#2	; Is Blowpipe extended?
	bne	.chknxt	; No  -

	lda	v_numdart	; Yes - Get number of darts left
	beq	.chknxt

       	lda	#6	; Maintain frame display
	sta	v_playerdelay

	jsl	Blowdart	; Shoot dart if able

	rtl
.chknxt
	lda	v_playerseqlog
	cmp	#4
	bcc	.exit

	lda	#TZWait
	sta	v_playerAI

.exit
	rtl



*****************************************************************************
*
* Shoot dart while ducking
*
*****************************************************************************

TZDart2:
	lda	#tzducksmode02

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.

	lda	v_playerseqlog	; Yes - Get current frame number
	cmp	#6	; Is Blowpipe extended?
	bne	.chknxt2	; No  -

	lda	z_joy1
	and	v_WEAPON	; Is player shooting?
	beq	.chknxt2	; No  - Just do sequence then

	lda	v_numdart	; Yes - Get number of darts left
	beq	.chknxt2

       	lda	#6
	sta	v_playerdelay

	jsl	Blowdart

	lda	z_joy1
	bit	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.exitdir	; No  -

	bit	v_JOYPADLEFT
	bne	.setlft

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right
.exitdir 
	rtl

.setlft 
	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going right

	rtl

.chknxt2
	lda	v_playerseqlog	; Get current frame
	cmp	#8	; Last frame of sequence?
	bcc	.exit2	; No  -

	lda	z_joy1	; Yes -
	and	v_JOYPADDOWN	; Is still ducking?
	beq	.noshoot2	; No  - Then put pipe away

	lda	z_joy1	; Yes -
	and	v_WEAPON	; Is still wanting to shoot?
	bne	.extend2	; Yes - Then extend frame
.noshoot2 
	lda	#TZDart3
	sta	v_playerAI

	lda	#tzducksmode03	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart

	stz	v_playerdelay


	jmp	Newframe	; Do a new frame
.exit2
	rtl
.extend2
       	lda	#10
	sta	v_playerdelay

	rtl


*****************************************************************************
*
* Put Blowpipe away
*
*****************************************************************************

TZDart3:
	lda	#tzducksmode03

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	CANSTAND	; Returns if standable tile, otherwise...
			; ...vectors to Startfall.
	lda	v_playerseqlog	
	cmp	#6
	bcc	.exit

	lda	z_joy1	; Yes -
	and	v_JOYPADDOWN	; Is still ducking?
	beq	.endduck	; No  - Then end sequence
.extend
	lda	#10	; Yes - Extend the frame display time
	sta	v_playerdelay

	lda	z_joy1	; Check for attack request
	bit	v_WEAPON	; Knife 'em?
	bne	.chkweapons	; Yes -

	bit	v_JOYLEFTRIGHT	; Was left or right dir pressed?
	beq	.exitdir	; No  -

	bit	v_JOYPADLEFT
	bne	.setlft

	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right
.exitdir 
	rtl

.setlft 
	lda	#MOVUP|MOVDN	; No  - Then reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going right
.exit 
	rtl		; No  - Return and wait
.chkweapons
	lda	v_weaponsel	; Which weapon?
	bne	.dodarts	; Do darts

         	lda	#TZStab1	; Otherwise do knife attack
	sta	v_playerAI

	lda	#tzstab1mode00
	sta	v_playernewtype

	rtl
.dodarts 
	lda	v_numdart
	beq	.endduck

         	lda	#TZDart2	; Dart attack
	sta	v_playerAI

	lda	#tzducksmode02
	sta	v_playernewtype

	jmp	Newframe

	rtl
.endduck
	lda	#TZDuck01
	sta	v_playerAI

	rtl


*****************************************************************************
*
* Blowdart
*
*****************************************************************************

Blowdart:
	lda	v_dartactive	; Is dart flying?
	bne	.exitblow	; Yes -

	dec	v_numdart	; No  - Take one away
	
	lda	v_numdart	; Get number of darts left
	cmp	#1	; Does Tarzan still have some?
	bcs	.stillhavesome	; Yes -

	stz	v_playerdelay	; No  - 

	lda	v_weaponsel	; Toggle weapon select - knife
	eor	#1	
	sta	v_weaponsel

.stillhavesome
	lda	#60
	sta	v_dartactive	; Set dart active

	lda	v_playerdir
	and	#MOVRT
	beq	.setleft

	stz	v_dartdir

	bra	.cheksfxos
.setleft
	lda	#$4000
	sta	v_dartdir
.cheksfxos
	lda	v_donedrtsfx	; No  - Has SFX been initiated?
	bne	.exitblow	; Yes - The skip

	lda	#blopipe_eff	; No  - Then Shoot dart

	jsl	load_Effect

	lda	#1
	sta	v_donedrtsfx	; Set SFX oneshot

	lda	#4	; Yes - Update attitude
	sta	v_dartstep

	jmp	Computidartbox	; Compute collision box
.exitblow
	rtl


***************************************
*
* Computes collision box for dart
* and determines initial x,y position
* based on player stance and direction
*

Computidartbox:
	lda	v_playerAI
	cmp	#TZDart1	; Stand & Shoot?
	bne	.doduckadj	; No  - adjust for duck

	lda	v_dartdir	; Yes -
	bne	.dartleft1

	lda	v_playerx
	lsr
	lsr
	clc
	adc	#16
	sta	v_dartleft
	sta	v_dartx

	bra	.computey1

.dartleft1
	lda	v_playerx
	lsr
	lsr
	sec
	sbc	#16
	sta	v_dartleft
	sta	v_dartx
.computey1
	clc
	adc	#8
	sta	v_dartrite

	lda	v_playery
	lsr
	lsr
	sec	
	sbc	#54
	sta	v_darty
	dec
	dec
	sta	v_darttop
	clc
	adc	#5
	sta	v_dartbot

	jmp	.exit

.doduckadj
	lda	v_dartdir
	bne	.dartleft2

	lda	v_playerx
	lsr
	lsr
	clc
	adc	#22
	sta	v_dartleft
	sta	v_dartx

	bra	.computey2

.dartleft2
	lda	v_playerx
	lsr
	lsr
	sec
	sbc	#22
	sta	v_dartleft
	sta	v_dartx
.computey2
	clc
	adc	#8
	sta	v_dartrite

	lda	v_playery
	lsr
	lsr
	sec	
	sbc	#33
	sta	v_darty
	dec
	dec
	sta	v_darttop
	clc
	adc	#5
	sta	v_dartbot
.exit
	lda	#0
	sta	v_dartgrav	; Starting gravity
	stz	v_dartdy	; Starting Y (results in vertical drop)

	rtl


***************************************
*
* Computes collision box for dart
* and determines initial x,y position
* based on player stance and direction
*

Computdrtbx:
	lda	v_dartactive
	beq	.exit

	lda	v_dartx
	sta	v_dartleft

	clc
	adc	#8
	sta	v_dartrite

	lda	v_darty
	dec
	dec
	sta	v_darttop
	clc
	adc	#5
	sta	v_dartbot
.exit

	rtl


*****************************************************************************
*
* Do Float - If no swim button action, then Tarzan floats back & down.
*
*****************************************************************************

TZFloat:
	lda	#tzswimmode00

	VERIFYMODE

	lda	v_playery
	lsr
	lsr
	sec
	sbc	z_levely
	bmi	.cont

	cmp	#$130
      	bcc	.cont

	stz	v_playerdelay	          	
	stz	v_playinvince	          	
	stz	v_health	          	
			          
	lda	#1
	sta	v_damageflag

	rtl
.cont
	lda	z_joy1	; Is player swimming?
	and	#(JOYPADUP|JOYPADDOWN|JOYPADLEFT|JOYPADRIGHT)
	beq	.tryknife	; No  -

	lda	#TZSwim	; Yes -
	sta	v_playerAI

	stz	v_playerdelay
	stz	v_repeat

	lda	#44
	sta	v_playerseq

	rtl
.tryknife
	lda	z_joy1
	bit	v_WEAPON	; Tarzan Knifing?
	beq	.sinkim	; No - Then sink

	lda	#TZFloatknife
	sta	v_playerAI

	stz	v_playerdelay
.sinkim
	jsl	Chek4stat

	lda	v_tilestat
	cmp	#DEATHTYPE
	beq	.takeahit

	lda	v_playery
	clc
	adc	v_gravity
	sta	v_playery

	lda	#MOVRT|MOVLT
	and	v_playerdir
	and	_walls
	bne	.exit

	lda	v_playerx
	sec
	sbc	v_current
	sta	v_playerx
.exit
	rtl
	
.takeahit
	lda	#1
	sta	v_damageflag

	rtl


*****************************************************************************
*
* Do Float Knife attack - If knifing, tarzan sinks as though in float mode 
*
*****************************************************************************

TZFloatknife:
	lda	#tzswimmode02	; Otherwise, init sequence

	jsr	Validatetype

	lda	v_playery
	lsr
	lsr
	sec
	sbc	z_levely
	bmi	.cont

	cmp	#$130
      	bcc	.cont

	stz	v_playerdelay
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag

	rtl
.cont
	jsl	Chek4stat

	lda	v_tilestat
	cmp	#(CEILINGTYPE|WALLTYPE)
	beq	.decfrmcnt

	lda	v_playery
	clc
	adc	v_gravity
	sta	v_playery

	lda	v_playerx
	sec
	sbc	v_current
	sta	v_playerx
.decfrmcnt
	dec	v_playerdelay	; Adjust frame count
	bmi	.nextframe	; Process if still going...

	rtl
.nextframe
	lda	v_playerseqlog
	cmp	#10
	bcc	.looknife 

	bne	.ending

	lda	z_joy1
	bit	v_WEAPON
	beq	.looknife 

	lda	#12
	sta	v_playerseq
.ending
	ldy	v_playertype	; Get Type
	lda	d_playerbank,y	; Get Bank
	sta	z_temp1

	lda	d_playerseq,y	; Get sequence offset
	sta	z_temp0

	lda	v_playerseq	; Get sequence pointer
	clc
	adc	#4	; Point to next frame data entry

	tay		; Index

	lda	[z_temp0],y	; Get frame data
	bpl	.newframe	; If not end of seq, then frame...

	lda	#TZFloat
	sta	v_playerAI
	stz	v_playerdelay

	bra	.exittzstab1

.looknife 
	lda	v_playerseqlog
	cmp	#5	; Identify "knifing" frames
	beq	.notfyfoe	; Set killer frame flag for foes

	cmp	#6
	beq	.notfyfoe

	cmp	#7
	beq	.notfyfoe

	stz	v_foekillfrm
	stz	v_knifeactive

	bra	.exittzstab1

.notfyfoe 
	lda	#1
	sta	v_foekillfrm
	sta	v_knifeactive

	lda	#6
	sta	v_playinvince

	stz	v_bosshit

	bra	.exittzstab1

.doagain 
	lda	#1	; Request frame number
	asl
	asl
	sta	v_playerseq	; Get sequence pointer

.exittzstab1 
	dec	v_playerdelay	; Adjust frame count
	bpl	.xit	; Process if still going...

	jmp	Newframe	; Do a new frame
.xit
	rtl

.newframe		

	jmp	Newframe	; Do a new frame

	rtl

*****************************************************************************
*
* Do Swim turn 
*
*****************************************************************************

TZSwimturn:
	lda	#tzswimmode01	; Otherwise, init sequence

	jsr	Validatetype

	dec	v_playerdelay	; Adjust frame count
	bmi	.nextframe	; Process if still going...

	rtl
.nextframe
	ldy	v_playertype	; Get Type
	lda	d_playerbank,y	; Get Bank
	sta	z_temp1

	lda	d_playerseq,y	; Get sequence offset
	sta	z_temp0

	lda	v_playerseq	; Get sequence pointer
	clc
	adc	#4		; Point to next frame data entry

	tay		; Index

	lda	[z_temp0],y	; Get frame data
	bpl	.newframe	; If not end of seq, then frame...

	lda	v_playerdir
	eor	#MOVRT|MOVLT
	sta	v_playerdir

	lda	#TZFloat
	sta	v_playerAI

	lda	#tzswimmode00
	sta	v_playernewtype
.newframe		
	jmp	Newframe	; Do a new frame

	rtl

*****************************************************************************
*
* Do Swim - 
*
*****************************************************************************

TZSwim:
	lda	#tzswimmode00

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	jsl	Chek4reef

	beq	.cutem
.chkbottom
	lda	v_playery
	lsr
	lsr
	sec
	sbc	z_levely
	bmi	.cont

	cmp	#$130
      	bcc	.cont
.killem
	stz	v_playerdelay
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag

	rtl
.cutem
	lda	#1
	sta	v_damageflag
.cont
	lda	#6	; Swim speed
	sta	v_stride
	stz	v_stroke

	lda	z_joy1	; Check for attack request
	bit	v_WEAPON	; Knife 'em?
	beq	.skipattk	; No  -

	lda	#TZFloatknife	; Yes - Attack
	sta	v_playerAI

	stz	v_playerdelay

	rtl
.skipattk 
.chk4swim
	bit	v_JOYLEFTRIGHT	; Is player swimming?
	beq	.noletup	; No  - Check timer
.rstrep
	stz	v_repeat	; Yes - Reset timer

	bit	v_RUNBUTTON	; Extra speed?
	beq	.stroke	; No  -

	lda	#3	; Yes - Add a little extra speed
	sta	v_stroke

	bra	.stroke	;

.noletup 
	bit	v_JOYPADUP	; Pushing up?
	beq	.rstrep

	bit	v_JOYPADDOWN
	beq	.rstrep

	inc	v_repeat	; Adjust float timer
	lda	v_repeat
	cmp	#30	; More than allowable?
	bcc	.stroke	; No  - Keep moving

	lda	#TZFloat	; Yes - Hasn't stroked, so sink
	sta	v_playerAI

	stz	v_repeat

	rtl

.stroke 
	lda	v_tilestat
	and	#DEATHTYPE
	beq	.gostrokeok

	lda	#-3
	sta	v_stroke

.gostrokeok
	lda	z_joy1	; Get pad stats
	and	v_JOYLEFTRIGHT
	bne	.norm	; Yes -

	stz	v_stride

	and	v_JOYPADUP|v_JOYPADDOWN
	bne	.juststroke

	lda	v_playerdir	; No  - Get players direction
	and	#MOVRT	; Currently facing right?
	bne	.swimleft2	; Yes - Then start swiming

	bra	.swimright2

.norm 
	lda	v_stride
	clc
	adc	v_stroke
	sta	v_stride
.juststroke
	lda	z_joy1
	and	v_JOYPADLEFT	; Did user press left?
	bne	.swimleft	; Yes - Process left then


	; controller pressed to the right
.swimright 
	lda	z_joy1
	and	v_JOYPADRIGHT
	beq	.chkupdnmov

	lda	v_playerdir	; No  - Get players direction
	and	#MOVRT	; Currently facing right?
	bne	.swimright2	; Yes - Then start swiming

	lda	#TZSwimturn
	sta	v_playerAI

	rtl

.swimright2  
	lda	#MOVRT
	and	_walls
	bne	.chkupdnmov

	lda	v_stride	; Stride length
	clc
	adc	v_playerx
	sec
	sbc	v_current
	sta	v_playerx

	bra	.chkupdnmov


	; controller pressed to the left
.swimleft   
	lda	v_playerdir	; Get players direction
	and	#MOVLT
	bne	.swimleft2	; then swim

	lda	#TZSwimturn
	sta	v_playerAI

	rtl

.swimleft2  
	lda	#MOVLT
	and	_walls
	bne	.chkupdnmov

	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	clc
	adc	v_current
	sta	v_playerx


; **************************************

.chkupdnmov 
	stz	v_xspeed

	lda	z_joy1
	bit	v_RUNBUTTON	; Extra speed?
	beq	.udstroke	; No  -

	lda	#3
	sta	v_xspeed

.udstroke
	lda	z_joy1
	and	v_JOYPADUP	; Did user press UP?
	bne	.swimup	; Yes - Process UP then

.swimdown 
	bit	v_JOYPADDOWN
	beq	.swimdwn

	jmp	.setswim

.swimdwn
	lda	#MOVRT|MOVLT	; No  - Then affect dir flag
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir

.swimdown2  
	jsl	Chek4stat

	lda	v_tilestat
	cmp	#DEATHTYPE
	beq	.nosetdwn

	lda	#4	; Stride length
	clc
	adc	v_playery
	adc	v_xspeed
	sta	v_playery
.nosetdwn
	jmp	.setswim


.swimup
	lda	#MOVRT|MOVLT	; No  - 
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

.swimup2  
	jsl	Chek4stat

	lda	v_tilestat
	cmp	#(CEILINGTYPE|WALLTYPE)
	beq	.nosetdwn

	lda	v_playery
	sec
	sbc	#4	; Stride length
	sbc	v_xspeed
	lsr
	lsr
	sta	z_temp5

	lda	z_level
	cmp	#DOWNRIVER
	bne	.notdwnriv

	lda	z_temp5
	cmp	#$2E6
	bcc	.exitswim

	bra	.finish

.notdwnriv 
	lda	z_level
	cmp	#UPRIVER
	bne	.notupriv

	lda	z_temp5
	cmp	#$2E6
	bcc	.exitswim

	bra	.finish
.notupriv

	lda	z_level
	cmp	#REEF
	bne	.finish

	lda	z_temp5
	cmp	#$86
	bcc	.exitswim
.finish
	lda	_walls
	and	v_playerdir
	bne	.float

	lda	z_temp5
	asl
	asl
	sta	v_playery

	rtl
.setswim 
	lda	_walls
	and	v_playerdir
	bne	.takeahit
.exitswim	
	rtl
.float
	lda	#TZFloat
	sta	v_playerAI

	rtl
.takeahit
	lda	#1
	sta	v_damageflag

	rtl


*****************************************************************************
*
* Do Swim to Float
*
*****************************************************************************

TZSwimtoFloat:
	lda	#tzfloating	; Animation type ID

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog	
	cmp	#5
	beq	.float

	rtl
.float
	lda	z_joy1	; Get pad stats
	and	v_JOYLEFTRIGHT	; Left or right pressed?
	beq	.chkforair	; No  -

	jmp	.dofloaturn	; Yes

.chkforair
	lda	z_joy1
	and	v_JOYPADUP	; Still going up?
	beq	.donebob	; No - Transition to swim

	jsl	Chek4waterstat	; Yes - Check for water

	lda	v_tilestat	; Get tile type
	and	#WATERTYPE	; Is head in water?
	beq	.bob	; No  - 

	lda	v_playery	; Yes - Move up then
	sec
	sbc	v_stride
	sta	v_playery

	lda	#MOVLT|MOVRT	; No  - Then reset up/dwn flag
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir	; Set player going up

	lda	#30	; Extend frame display
	sta	v_playerdelay

	rtl

.bob
	dec	v_bobdelay
	bpl	.bobxit

	lda	#30	; Extend frame display
	sta	v_playerdelay
	sec
	sbc	#10
	sta	v_bobdelay

	lda	v_playerdir
	and	#MOVUP	; Currently going up?
	beq	.bobup	; No - Then bob up

	lda	v_playery
	clc
	adc	v_stride
	sta	v_playery

	lda	#MOVLT|MOVRT	; No  - Then reset up/dwn flag
	and	v_playerdir
	ora	#MOVDN
	sta	v_playerdir	; Set player going down
.bobxit
	rtl
.bobup
	lda	v_playery
	sec
	sbc	v_stride
	sta	v_playery

	lda	#MOVLT|MOVRT	; No  - Then reset up/dwn flag
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir	; Set player going up

	rtl
.donebob
	lda	#TZFloat2swim
	sta	v_playerAI

	stz	v_playerdelay

	rtl

.dofloaturn
	lda	z_joy1	; Is user pressing left?
	and	v_JOYPADLEFT 
	bne	.maskleft	; Yes -

	lda	v_playerdir	; No  - Pressing right
	and	#MOVRT	; Is player already moving right?
	bne	.nochnge	; Yes -

	lda	#TZSwimtoFloat
	sta	v_savpAI

	lda	#TZFloaturn
	sta	v_playerAI

	stz	v_playerdelay

	rtl
.maskleft
	lda	v_playerdir	; User is pressing left
	and	#MOVLT	; Is player already going left?
	bne	.nochnge	; Yes -

	lda	#TZFloaturn
	sta	v_playerAI

	stz	v_playerdelay
.nochnge
	rtl

.chksurf
	jmp	.chkforair

*****************************************************************************
*
* Do Floaturn
*
*****************************************************************************

TZFloaturn:
	lda	#tzfloaturn	; Animation type ID

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	z_joy1
	and	v_JOYPADLEFT
	bne	.turnleft	; Yes - Process left then

.turnright 
	lda	#MOVUP|MOVDN	; No  - Then reset up/dwn flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir

.preframe
	lda	#4	; Preload frame 6
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzfloating
	sta	v_playernewtype

	jsl	Newframe

	lda	#tzfloating
	sta	v_playernewtype

	lda	#TZSwimtoFloat
	sta	v_playerAI

	lda	#6
	sta	v_stride

	stz	v_playerdelay	; Time it out so next frame loads

	jmp	Newframe

.turnleft   
	lda	#MOVUP|MOVDN	; No  - Then reset up/dwn flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir

        	bra	.preframe


*****************************************************************************

TZFloat2swim:
	lda	#tzfloat2swim	; Animation type ID

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog	; Otherwise, get logical sequence pointer
	cmp	#3
	bcc	.exit

	lda	#TZSwim
	sta	v_playerAI
.exit
	rtl



*****************************************************************************
*
* Do Walkaway - Player must press up to initiate this function
*
*****************************************************************************

TZWalkaway:
	lda	#tzwkawaymode00	; Otherwise, init sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames
	lda	v_playerseqlog
	cmp	#5
	bcc	.exitnoinc

	lda	v_playerseqlog
	cmp	#13
	bcc	.exit

	lda	#1
	sta	v_intcompletion

.exit 
	dec	v_playery
;	dec	v_playery

.exitnoinc 
	rtl


*****************************************************************************
*
* Do Canned Swing Sequence 1
*
*****************************************************************************

TZStartSwing1:
	lda	#24<<2
	sta	v_playeroffy1

	lda	#40<<2
	sta	v_playeroffx1

	lda	#TZSwing1	; No - Load new player AI vector
	sta	v_playerAI

	stz	v_playerdelay

*****************************************************************************

TZSwing1:
	lda	v_playertype	; Get current mode type
	cmp	#tzswingmodeb1	; Already in this mode?
	beq	.adjsttmr	; Yes - Just do timer thing then
          	
	dec	v_playerdelay	; No - Then finish last sequence
	bmi	.changemode	; Change modes if done

	rtl

.changemode 
	lda	#tzswingmodeb1	; Otherwise, init sequence
	sta	v_playernewtype	; Store as new type to restart
	sta	v_playertype	; Store as new type to restart

	lda	#8
	sta	v_playerseq

	ldx	v_curvine
	lda	#$ffff
	sta	v_vinetype,x

	lda	v_playerdir
	and	#MOVLT
	beq	.vadjrgt

	lda	v_playerx
	sec
	sbc	v_playeroffx1
	sta	v_playerx

	bra	.exitvadj
.vadjrgt
	lda	v_playeroffx1
	clc
	adc	v_playerx
	sta	v_playerx

.exitvadj
	stz	v_playeroffx1

	lda	v_playery
	sec
	sbc	v_playeroffy1
	sta	v_playery

	stz	v_playeroffy1

	lda	_dxmid
	sta	v_savxmid

	ldx	v_playerdir
	cpx	#MOVLT
	bne	.dxadjrgt

	sec
	sbc	v_playeroffx1
	sta	_dxmid

	bra	.xxadjrgt
.dxadjrgt
	clc
	adc	v_playeroffx1
	sta	_dxmid
.xxadjrgt
	lda	_dymid
	sta	v_savymid

	sec
	sbc	#32
	sta	_dymid

	jmp	Newframe	; Do a new frame
.adjsttmr 
	dec	v_playerdelay	; Adjust frame count
	bpl	.tzswing1	; Process if still going...

	jmp	Newframe	; Do a new frame

.tzswing1 
	lda	v_playerseqlog
	cmp	#9	;13
	bcc	.exit

	lda	#TZSwing2
	sta	v_playerAI

.exit
	rtl


*****************************************************************************
*
* Do Canned Swing Sequence 2
*
*****************************************************************************

TZSwing2:
	lda	#tzswingmodeb2	; Already in this mode?

	VERIFYMODE

	lda	v_playerseqlog
	cmp	#4
	bne	.skipwoosh

	lda	v_vin1sot
	bne	.skipwoosh

	lda	#1
	sta	v_vin1sot

	lda	#whoosh2_eff

	jsl	load_Effect
.skipwoosh
	lda	v_playerseqlog
	asl
	tax

	lda	v_playerdir
	bit	#MOVRT
	beq	.left

	lda	v_playerx
	clc
	adc	>.swingx,x
	bra	.y
.left
	lda	v_playerx
	sec
	sbc	>.swingx,x
.y
	sta	v_playerx

	lda	v_playery
	clc
	adc	>.swingy,x
	sta	v_playery
.timer
	lda	v_playerseqlog
	cmp	#8
	bcc	.exit

	stz	v_vin1sot

	lda	#TZOffswing1
	sta	v_playerAI

.exit
	rtl

.swingx
	.dw	4,6,8,16,24,36,36,24,16
.swingy
	.dw	2,4,8,12,8,4,4,-8,-12



*****************************************************************************
*
* Dismount from vine
*
*****************************************************************************

TZOffswing1:
	lda	v_savxmid
	sta	_dxmid

	lda	v_savymid
	sta	_dymid

	ldx	v_curvine

	lda	v_vinerst,x
	cmp	#VINERST
	beq	.rstinital

	lda	v_vinedir,x
	eor	#$4000
	sta	v_vinedir,x
.tmp
	lda	v_vinedir,x
	bit	#$4000
	beq	.vineleft

	lda	#$110

	bra	.vinex
.vineleft
	lda	#$fef0
.vinex
	clc
	adc	v_vinex,x
	sta	v_vinex,x

.rstinital
	stz	v_vinetype,x
.exit
	jsl	Sendvineonly


*****************************************************************************

TZStartfallvine:

	lda	#TZFallvine
	sta	v_playerAI

	lda	#tzfall2land
	sta	v_playernewtype	; Store new type to restart

	lda	#2
	sta	v_velocity

	lda	#-20
	sta	v_gravity

	lda	#4
	sta	v_stride

	lda	v_playery
	sta	v_markfall

	lda	#MOVRT|MOVLT
	and	v_playerdir
	ora	#MOVUP
	sta	v_playerdir

	lda	#$FFFF	; Preload frame
	asl
	asl
	sta	v_playerseq
	sta	v_playeroveride

	lda	#tzfall2land
	sta	v_playernewtype

	jsl	Newframe	; Crank the machine, but no start

	lda	#1
	asl
	asl
	sta	v_playerseq

	stz	v_playerdelay

	jmp	Newframe


*****************************************************************************
*
* Fall Sequence
*
*****************************************************************************

TZFallvine:
	lda	#tzfall2land

	VERIFYMODE

	lda	v_playerseqlog
	cmp	#3
	bcc	.chkfall
	
	cmp	#8
	bcs	.finish

	rtl

.finish
	lda	#TZWait
	sta	v_playerAI

	rtl

.chkfall
	lda	#10
	sta	v_playerdelay

	lda	v_playery
	sec
	sbc	v_markfall

	cmp	v_fallmax	; Have we fallen too far?
         	bcc	.chkforflrnow	; No  -

	lda	#1	; Yes -
	sta	v_disbgscroll
	
	lda	z_levely
	asl
	asl
	clc
	adc	#330<<2
	cmp	v_playery
	bcs	.chkforflrnow
.killem
	stz	v_playinvince
	stz	v_health

	lda	#1
	sta	v_damageflag
.xitfall
	rtl

.chkforflrnow
	jsl	Chek4floorfall	; Have we detected a floor?

	bne	.landitnow	; Yes - switch to finish jump
.falldown
	lda	v_playerdir	; Yes - Get players current direction
	and	#MOVRT	; Is player moving right?
	bne	.right	; Yes -

	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx

	bra	.applivert
	
.right 
	lda	v_playerx	; Stride length
	clc
	adc	v_stride
	sta	v_playerx
.applivert
	lda	v_playery
	clc
	adc	v_velocity
	sta	v_playery

	lda	v_velocity	; Get velocity
	clc
	adc	v_gravity	; Add gravity
	sta	v_velocity

	inc	v_gravity

	cmp	#(MAXVELOCITY*4)
	bcc	.oktomovdn	; If still some left, then move

	lda	#(MAXVELOCITY*4)	; Terminal velocity
	sta	v_velocity
.exithold 
	rtl
.oktomovdn 
	jsl	Chklrmov	; Check for left/right movement

	rtl
.landitnow 
	stz	v_playerdelay

	rtl



*****************************************************************************
*
* Do Walk/Stand to Climb Sequence
*
*****************************************************************************

TZWalknclimb:
	lda	#tzwalktoclimb	; Otherwise, reinit sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#3
	bcc	.exit

	lda	#TZClimbup
	sta	v_playerAI

	dec	v_playery
	dec	v_playery
	dec	v_playery
	dec	v_playery
.exit
	rtl


*****************************************************************************
*
* Do Climb to Walk/Stand Sequence
*
*****************************************************************************

TZClimbtowalk:
	lda	#tzclimbtowalk	; Otherwise, reinit sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#3
	bcc	.exit

	lda	#4
	clc
	adc	v_playery
	sta	v_playery

	lda	#TZWait
	sta	v_playerAI
.exit
	inc	v_playery
	inc	v_playery

	rtl


*****************************************************************************
*
* Just Climbdn
*
*****************************************************************************

TZClimbdn:
	lda	#tzclimbingdn	; Otherwise, reinit sequence

	jsr	Validatetype

	lda	v_playerdir
	and	#MOVUP|MOVDN	; Clear Left & Right
	sta	v_playerdir
	ora	#MOVDN
	sta	v_playerdir	; Set player going right

	lda	z_joy1	; Get keys
	and	v_JOYPADDOWN	; Pressing down?
	beq	.chkup	; No  - Then noclimb

	bra	.adjdelay
.chkup
	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.chkdir	; No  - Then check for jmp off

	lda	#TZClimbup
	sta	v_playerAI

	rtl
.adjdelay
	dec	v_playerdelay	; Adjust frame count
	bpl	.tzwclmb	; Process if still going...

	jmp	Newframe	; Do a new frame
.tzwclmb
	jsl	Chek4number

	lda	z_level
	cmp	#SHIP
	bne	.chkcave

	lda	v_tilenum
	cmp	#$93
	bne	.oktgo

	lda	#TZClimbtowalk
	sta	v_playerAI

	rtl
.chkcave
	lda	z_level	; Has to be cavern!!!
	cmp	#CAVERNS
	bne	.oktgo

	cmp	#$1B
	bne	.oktgo

	lda	#TZClimbtowalk
	sta	v_playerAI

	rtl

.oktgo
	jsl	Chek4floor

	lda	v_tilestat
	cmp	#CLIMBTYPE
	beq	.tileok

	jmp	TZStartfall

.tileok

	inc	v_playery
	inc	v_playery
	inc	v_playery
	inc	v_playery

.exitclmb
	rtl

.chkdir
	lda	z_joy1	; Get keys
	tax
	and	v_JUMPBUTTON	; Wants to jmp?
	beq	.exitclmb	; No  -  

	txa		; Yes -
	and	v_JOYLEFTRIGHT	; Directional pressed?
	beq	.exitclmb	; No  -  

	lda	#12
	sta	v_stride
	sta	v_stridey

          	lda	#TZOffladder	; Yes -
	sta	v_playerAI

	rtl

*****************************************************************************
*
* Just Climbup
*
*****************************************************************************

TZClimbup:
	lda	#tzclimbingup

	jsr	Validatetype

	lda	#MOVUP
	sta	v_playerdir	; Set player going 

	lda	v_playerx
	lsr
	lsr
	and	#$FFF0
	clc
	adc	#10
	asl
	asl
	sta	v_playerx

	lda	z_joy1	; Get keys
	and	v_JOYPADUP	; Pressing up?
	beq	.chkdn	; No  - Then no climb

	bra	.adjdelay
.chkdn
	lda	z_joy1	; Get keys
	and	v_JOYPADDOWN	; Pressing down?
	beq	.chkdir	; No  - Then check for jump off

	lda	#TZClimbdn	; Yes - Start climb down
	sta	v_playerAI

	rtl
.adjdelay
	dec	v_playerdelay	; Adjust frame count
	bpl	.tzwclmb	; Process if still going...

	jmp	Newframe	; Do a new frame
.tzwclmb
	ldx	v_playery
	txa
	sec
	sbc	#56<<2
	bmi	.newseq

	sta	v_playery

	jsl	Chek4floor

	stx	v_playery

	lda	v_tilestat
	cmp	#CLIMBTYPE
	beq	.tileok
.newseq
	lda	#6
	asl
	asl
	sta	v_playerseq

	jmp	Newframe

.tileok
	dec	v_playery
	dec	v_playery
	dec	v_playery
	dec	v_playery

	lda	z_level
	cmp	#SHIPHOLDA
	bne	.chkhb

	jsr	Computeholdno

.chkhb
	lda	z_level
	cmp	#SHIPHOLDB
	bne	.exitclmb

	jsr	Computeholdno

.exitclmb
	rtl

.chkdir
	lda	z_joy1	; Get keys
	tax
	and	v_JUMPBUTTON	; Wants to jmp?
	beq	.exitclmb	; No  -  

	txa		; Yes -
	and	v_JOYLEFTRIGHT	; Directional pressed?
	beq	.exitclmb	; No  -  

	lda	#12
	sta	v_stride
	sta	v_stridey
	
          	lda	#TZOffladder	; Yes -
	sta	v_playerAI

	rtl

****************************************
*

Computeholdno

	rts
*****************************************************************************
*
* Jump to Ladder
*
*****************************************************************************

TZJmptoladr:
	lda	#tzljmptoladr	; Otherwise, reinit sequence

	jsr	Validatetype

	lda	v_playerdir
	and	#MOVUP|MOVDN	; Clear Left & Right
	sta	v_playerdir
	ora	#MOVUP
	sta	v_playerdir	; Set player going right

	lda	v_playerx
	lsr
	lsr
	and	#$FFF0
	clc
	adc	#10
	asl
	asl
	sta	v_playerx

	dec	v_playerdelay	; Adjust frame count
	bpl	.tztolad	; Process if still going...

	jmp	Newframe	; Do a new frame
.tztolad
	lda	v_playerseqlog
	cmp	#3
	bcc	.exit

	lda	#4
	clc
	adc	v_playery
	sta	v_playery

	lda	#TZClimbup
	sta	v_playerAI

.exit
	rtl


*****************************************************************************
*
* Jump off Ladder to Platform
*
*****************************************************************************

TZOffladder:
	lda	#tzladrtoplat	; Otherwise, reinit sequence

	VERIFYMODE	; Returns if in mode & not changing...
			; ...frames

	lda	v_playerseqlog
	cmp	#3
	bcc	.exit

	lda	#4
	clc
	adc	v_playery
	sta	v_playery

	jmp	TZStartfall	; not on solid ground, begin fall

.exit
	lda	v_playerseqlog
	cmp	#2
	bcs	.dwnnow

	lda	v_playery
	sec
	sbc	v_stridey
	sta	v_playery

	bra	.chkcntrlr
.dwnnow
	lda	v_playery
	clc
	adc	v_stridey
	sta	v_playery
.chkcntrlr
	lda	z_joy1
	and	v_JOYPADLEFT	; Did user press left?
	bne	.jmpleft	; Yes - Process left then


	; controller pressed to the right
.jmpright  
	lda	v_playerdir	; No  - Get players direction
	and	#MOVRT	; Currently facing right?
	bne	.jmpright2	; Yes - Then start walking

	lda	#MOVRT
	sta	v_playerdir	; Set player going right
.jmpright2
	lda	_walls
	and	v_playerdir
	bne	.setjmp
	
	lda	v_stride	; Stride length
	clc
	adc	v_playerx
	sta	v_playerx

	bra	.setjmp


	; controller pressed to the left
.jmpleft
	lda	v_playerdir	; No  - Get players direction
	and	#MOVLT	; Currently facing right?
	bne	.jmpleft2	; Yes - Then start walking

	lda	#MOVLT
	sta	v_playerdir	; Set player going left
.jmpleft2  
	lda	_walls
	and	v_playerdir
	bne	.setjmp
	
	lda	v_playerx
	sec
	sbc	v_stride	; Stride length
	sta	v_playerx

.setjmp
	rtl


*****************************************************************************
*****************************************************************************
*
*============================================================================
*
*****************************************************************************
*****************************************************************************

*****************************************************************************
*
* Load Selected Tarzan Animation Frame
*
* z_attrcount : Comes from higher priority plotted anims
*
*****************************************************************************

PlotPlayer:
	ldy	z_attrcount	; Get attr counter

	lda	v_playery	; Players Y 1/4 pixal
	lsr
	lsr
	sta	ATTRY,y	; Players Y Global (level)

	lda	v_playerx	; Players X 1/4 pixal
	lsr
	lsr
	sta	ATTRX,y	; Players X Global (level)

	lda	v_playerframe
	sta	ATTRFRAME,y

	lda	v_playerbank	; Get anim data bank
	sta	ATTRBANK,y

	lda	v_playerdrec	; Get animation record
	sta	ATTRDREC,y

	lda	v_playeroff	; Get slot
	and	#$CFFF	; Mask out priority bits
	ora	v_objpriority	; Add in priority for this level
	ora	v_playerflip	; Apply flip if required
	sta	ATTROFF,y	; Store attributes

	iny
	iny

	sty	z_attrcount

	jsl	Plotvines

	rtl

*****************************************************************************
*
*
*****************************************************************************

Loadtzanpal:
	lda	#0	; Logical Palette number CGRAM
	ldx	#0	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaditemspal:
	lda	#1	; Logical Palette number CGRAM
	ldx	#2	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl


Loadhealthpal:
	lda	#2	; Logical Palette number CGRAM
	ldx	#4	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loadundrwtr:
	lda	#0	; Logical Palette number CGRAM
	ldx	#6	; PALSET table index pointer (word)

	jsl	Loadsprpal


Loaddamage:
	lda	#3	; Logical Palette number CGRAM
	ldx	#18	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaddrkpal00:
	lda	#0	; Logical Palette number CGRAM
	ldx	#8	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaddrkpal01:
	lda	#0	; Logical Palette number CGRAM
	ldx	#10	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaddrkpal02:
	lda	#0	; Logical Palette number CGRAM
	ldx	#12	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaddrkpal03:
	lda	#0	; Logical Palette number CGRAM
	ldx	#14	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl

Loaddrkpal04:
	lda	#0	; Logical Palette number CGRAM
	ldx	#16	; PALSET table index pointer (word)

	jsl	Loadsprpal

	rtl


*****************************************************************************
*
* Only called prior to level fade-in
*
*****************************************************************************

InitPlayer:
	stz	v_playerflip	; Init flip

	ldx	z_levelword	; Get word level index
	lda	d_phyics,x
	xba
	and	#$FF	; Get river current
	sta	v_current
	sta	v_gravity

	ldx	z_levelword	; Get word level index
	lda	d_initdir,x	; Get initial conditions for level
	tax		; Save mode type & direction
	and	#$FF	; Just direction
	sta	v_playerdir	; Store initial direction

	txa		; Get mode type index
	xba		
	and	#$FF	; Animation routine
	tax
	lda	d_aiproc,x	; word index

	sta	v_playerAI
	txa
	lsr
	stz	v_playertype	; Animation type number
	sta	v_playernewtype

	stz	v_playerdelay

	lda	d_playerdrec,x	; Get drec (structure el pointer)
	sta	v_playerdrec	; Store
	sta	z_temp0

	lda	d_playerbank,x	; Get bank of anim
	sta	v_playerbank	; Store
	sta	z_temp1

	lda	[z_temp0]	; Get word at sequence data
	xba		; Switch to frame time
	and	#$1F	; Limit frm/sec to 47
	asl
	tay

;	lda	d_delconv,y	; Convert from frm/sec to 1/60
;	sta	v_playerdelay	; Store delay
	stz	v_playerdelay	; Store delay

	lda	#0	; Stand and wait for further orders...
	sta	v_playerframe
	tay		; (for Sendframe)

	lda	v_playerbank	;
	sta	z_dreccount	; Also for vram init

	lda	v_playerdrec	;
	sta	z_spridrec

	lda	v_tzanslot	; Slot x
	sta	v_playeroff
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; Compute VRAM address

	jsl	Sendframe	; Queue it up...

	jsl	Senddata

	jsl	InitVines

	lda	v_playerframe	; Get frame data back
	and	#$FF	; Just the frame number
	asl
	asl
	tax
  	ldy	v_playertype	; Get Type

	lda	d_playercol,y	; Relative offset of collide data
	sta	z_temp0

	lda	d_playerbank,y	; Get data bank
	sta	z_temp1

	jsl	GetColBox	; Get Collision Box Data

	stz	v_playerflip	; Preload player going right

	lda	v_playerdir	; Is player going right?
	and	#MOVRT	
	bne	.exit	; Yes - Ok then...

	lda	#$4000	; No - Flip player
	sta	v_playerflip	; Player going left
.exit
	rtl


*****************************************************************************
*
*
*****************************************************************************

Newframe:
	lda	v_playernewtype	; Want to do different sequence?
	cmp	v_playertype	; Yes - Same as current sequence?
	beq	.nextframe	;   Yes - Just do next next frame then

	sta	v_playertype	;   No  - Store new type
	tay		; Create index

	stz	v_playerseq	; Clear sequence data index
	stz	v_playerseqlog	; Clear sequence data index

	lda	d_playerbank,y	; Get data bank
	sta	v_playerbank	; Store
	sta	z_temp1

	lda	d_playerseq,y	; Get relative offset
	sta	z_temp0

	ldy	#0	; Point to 1st frame in sequence
	lda	[z_temp0],y	; Get the data

	bra	.frame	; Continue processing frame info

.nextframe
	ldy	v_playertype	; Get Type

	lda	d_playerbank,y	; Get Bank
	sta	v_playerbank	; Store
	sta	z_temp1

	lda	d_playerseq,y	; Get sequence offset
	sta	z_temp0

	lda	v_playerseq	; Get sequence pointer
	clc
	adc	#4	; Point to next frame data entry
	sta	v_playerseq	; Store it

	tay		; Index
	lsr
	lsr
	sta	v_playerseqlog	; Clear sequence data index

	lda	[z_temp0],y	; Get frame data
	bpl	.frame	; If not end of seq, then frame...

	stz	v_playerseq	; Otherwise, zero out frame pointer
	stz	v_playerseqlog	; Clear sequence data index

	lda	[z_temp0]	; Get 1st frame info

.frame
	sta	v_playerframe	; Store both frm# & time for now

	iny
	iny

	lda	[z_temp0],y
	sta	v_playergloboff

	lda	v_playerframe

	xba		; Switch frm# & frame time
	and	#$3f	; Limit value to 32
	asl
	tay		; Create index

	lda	d_delconv,y	; Convert from frames/sec to 1/60th
	sta	v_playerdelay	; Delay for this frame

	lda	z_level
	and	#$FF
	cmp	#SWAMP
	bne	.nosand

	lda	v_playertype
	cmp	#tzwalk3mode00
	beq	.chk4sand

	cmp	#tzrunmode00
	beq	.chk4sand

	cmp	#tzwkrnxumode00
	beq	.chk4sand

	bra	.nosand
.chk4sand
	jsl	Chek4sand
	bcc	.nosand

	lda	v_playerdelay
	asl
	sta	v_playerdelay

.nosand
	lda	v_playerframe	; Get frame data back
	and	#$FF	; Just the frame number
	sta	v_playerframe	; Store it

	asl
	asl
	tax		; Playercol(x+4)

	ldy	v_playertype	; Get Type

	lda	d_playercol,y	; Relative offset of collide data
	sta	z_temp0	; Bank is still in z_temp1

	jsl	GetColBox	; Get Collision Box Data

	lda	v_playerframe	; Get frame data back
	asl		; Word entrys
	tay		; (Store for Sendframe)

	ldx	v_playertype	; Get Type

	lda	v_playerbank	; Get Bank
	sta	z_dreccount

	lda	d_playerdrec,x	; Get sprite table loc of anim
	sta	v_playerdrec	; Store
	sta	z_spridrec

	lda	v_playeroveride
	bne	.end

	lda	v_playeroff	; Get slot for this player
	and	#$1FF	; Just the "name"
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; Compute VRAM Address (for Sendframe)

	jsl	Sendframe	; Queue it up
.end
	stz	v_playeroveride

	rtl


****************************************
*
* x = Collision table entry data pointer
*     (Frame * 4)
*
* Loads global values for collision box.
*
* [z_levelx,z_levely] offset
*
* Roxy format in .ssm file is:
*
*	X1,X2,Y1,Y2
*
*               z_temp0 = Long d_playercol 
*                         pointer
*

GetColBox:
	txy

	lda	v_playerx	; Get Scaled X
	lsr
	lsr
	sta	z_temp3

	lda	v_playery	; Get Scaled Y
	lsr
	lsr
	sta	z_temp4

	lda	[z_temp0],y	; Get collide X's
	sta	z_temp2

	lda	v_playerdir	; Is player going right?
	bit	#MOVRT	
	bne	.right	; Yes - Ok then...

	lda	z_temp2
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext1a

	ora	#$FF00	; sign extend the number
.noext1a 
	eor	#$FFFF
	inc
	clc
	adc	z_temp3
	sta	v_playerrite

	lda	z_temp2	; Get collide X's
	xba
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext2a

	ora	#$FF00	; sign extend the number
.noext2a 
	eor	#$FFFF
	inc
	clc
	adc	z_temp3
	sta	v_playerleft

	bra	.dowhys

.right 
	lda	z_temp2
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext1

	ora	#$FF00	; sign extend the number
.noext1 
	clc
	adc	z_temp3
	sta	v_playerleft

	lda	z_temp2	; Get collide X's
	xba
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext2

	ora	#$FF00	; sign extend the number
.noext2 
	clc
	adc	z_temp3
	sta	v_playerrite
.dowhys 
	iny
	iny

	lda	[z_temp0],y	; Get collide Y's
	sta	z_temp2
	xba
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext3

	ora	#$FF00	; sign extend the number
.noext3 
	clc
	adc	z_temp4
	sta	v_playerbot

	lda	z_temp2	; Get collide Y's
	and	#$FF
	bit	#$0080	; check sign bit
	beq	.noext4

	ora	#$FF00	; sign extend the number
.noext4 
	clc
	adc	z_temp4
	sta	v_playertop

	clc
	adc	#32
	sta	v_playercentry

	lda	v_playerrite
	sec
	sbc	v_playerleft
	lsr
	clc
	adc	v_playerleft
	sta	v_playercentrx

	rtl

****************************************
*
*

Cmputplaygbloff:
	lda	v_playergloboff
	and	#$FF	; Get X component
	bit	#$0080	; check sign bit
	beq	.noext1

	ora	#$FF00	; sign extend the number
	inc
.noext1
	sta	z_temp4

	lda	v_playerdir
	bit	#MOVLT
	beq	.okasis

	lda	#0
	sec
	sbc	z_temp4
	sta	z_temp4

.okasis
	lda	v_playergloboff
	xba
	and	#$FF	; Get Y component
	bit	#$0080	; check sign bit
	beq	.noext2

	ora	#$FF00	; sign extend the number
	inc
.noext2
	sta	z_temp6

	rtl

*****************************************************************************
*****************************************************************************
*
*============================================================================
*
*****************************************************************************
*****************************************************************************

*****************************************************************************
*
* Doplayer main routine
*
*****************************************************************************

Doplayer:   
	LAI

	jsl	Detector	; Item detect and results

	lda	v_playinvince	; Is player invincible?
	bne	.nobosshit	; Yes -

	lda	v_bosshit	; No  - Has a boss hit Tarzan
	beq	.nobosshit	;     No  -
	     
	lda	v_priestdead	; Is Priest dead? -then walk over him
	bne	.nobosshit	; 

	jsr	Qualbosshit	;     Yes - Qualify Big Hit

	lda	v_bosshit	; OK to do a big hit?
	beq	.nobosshit	; No  -

	lda	#40	; Yes - Renew invincible counter
	sta	v_addbossdamage	;       Also, vector has been loaded

.nobosshit 
	lda	v_pause2	; Debug scroller?
	beq	.donormplay	; No  -

	jsl	Movebydebug	; Yes - Sub in "Easy-Scroll"

	jsl	Objxy	; Construct player x,y in objects

	bra	.skipnormply
.donormplay
	lda	v_pause
	beq	.doplayr

	lda	z_joy1
	bit	#JOYPADSTART
	bne	.doplayr

	stz	v_startcount

	rtl
.doplayr
	jsl	Player	; Handle player control

.skipnormply
	lda	v_playerframe	; Get frame data back
	and	#$FF	; Just the frame number
	asl
	asl
	tax
  	ldy	v_playertype	; Get Type

	lda	d_playercol,y	; Relative offset of collide data
	sta	z_temp0

	lda	d_playerbank,y	; Get data bank
	sta	z_temp1

	jsl	GetColBox	; Get Collision Box Data

	stz	v_playerflip	; Preload player going right

	lda	v_playerdir	; Is player going right?
	and	#MOVRT	
	bne	.chkdart	; Yes - Ok then...

	lda	#$4000	; No - Flip player
	sta	v_playerflip	; Player going left
.chkdart
	lda	v_dartactive	; Is there a dart in the air
	beq	.noactvdrt	; No  -
.step 
	dec	v_dartactive
	dec	v_dartstep	; Yes - Time to update?
	bpl	.noactvdrt	; No  -

	lda	#2	; Yes - Update attitude
	sta	v_dartstep

	jsl	Updatedartstuf

.noactvdrt
	lda	z_joy1
	beq	.chkresets

	jsr	Setidlecount

.chkresets
	jsr	Debouncekeys

****************************************
*
* Player Health
*
****************************************

.playerhealth 
	lda	v_playinvince	; Is Tarzan invincible?
	bne	.adjinvince	; Yes - Skip check for damage

	bra	.chk4damge

.adjinvince
	dec	v_playinvince	; Adjust invincible counter
	bmi	.rstiflg	; No action if we're invincible

	stz	v_damageflag	; Clear damage flag
	stz	v_bosshit

	jmp	.exitplayer	; No action if we're invincible

.rstiflg
	stz	v_playinvince	; Otherwise reset invince counter
	stz	v_damageflag	; Clear damage flag
	stz	v_bosshit
	stz	v_slapos
.chk4damge
	lda	v_priestdead
	bne	.exitplayer	;     No  -

	lda	v_damageflag	; Is a foe trying to do damage?
	ora	v_bosshit	; No  - Has a boss hit Tarzan
	beq	.exitplayer	; No  - Then skip process

	lda	v_health	; Get current health value
	sec
	sbc	v_damageflag	; Minus the magnitude of the hit
	sta	v_health	; New health value

	lda	#newuhh_eff

	jsl	load_Effect

	stz	v_damageflag	; Clear damage flag
	stz	v_bosshit

	lda	#60	; Renew invincible counter
	clc
	adc	v_addbossdamage
	sta	v_playinvince

	stz	v_addbossdamage

	lda	v_health	; Get a copy of Tarzan's health
	bpl	.exitplayer	; Still hangin' in there
	
	lda	#MAXHEALTH	; Otherwise, regen current life
	sta	v_health

	lda	v_demorecord
	beq	.adjlives

	lda	#1
	sta	v_killdemorec

.adjlives
	dec	v_lives	; Knock a life off the scale
	bpl	.keepgoing	; Lets keep on rolling if any left

	lda	#1
	sta	v_levelfail

	rtl
.keepgoing 
	lda	v_lives
	cmp	v_lastlives
	bcs	.exitplayer

	lda	v_demoplay
	beq	.just1life

	lda	#1
	sta	v_levelfail

	rtl

.just1life
	lda	#1
	sta	v_lostlife
	sta	v_return
.exitplayer
	rtl


********************************************
*
*

Updatedartstuf:
	lda	v_darthit	; Has dart hit target?
	bne	.killdart	; Yes - Then kill dart

	dec	v_dartactive
	bmi	.killdart

	lda	v_dartgrav	; No  - Get gravity
	clc
	adc	v_darty
	sta	v_darty	; New vetical position

	inc	v_dartgrav	; Increase gravity
	
	lda	v_dartdir	; Get direction
	beq	.addx	; 

	lda	v_dartx	; No  - 
	sec
	sbc	#4<<2
	sta	v_dartx

	bra	.chkdartime

.addx
	lda	v_dartx
	clc
	adc	#4<<2
	sta	v_dartx
.chkdartime
	jsl	Computdrtbx

	dec	v_dartactive
	bpl	.updatexy
.killdart
	stz	v_dartactive
	stz	v_donedrtsfx	; Reset SFX oneshot
	stz	v_darthit
.updatexy
	rtl


*****************************************************************************

Debouncekeys:
	lda	z_joy1	; Get controller bits
	bit	v_JUMPBUTTON	; Isolate Y button
	bne	.xcount	; Check X if set

	stz	v_ycount	; Otherwise clear repeat flag
.xcount 
	bit	v_WEAPON	; Isolate X button
	bne	.lcount	; Check B if set

	stz	v_xcount	; Otherwise clear repeat flag
.lcount 
	bit	v_JOYPADL	; Isolate X button
	bne	.bcount	; Check B if set

	stz	v_lcount	; Otherwise clear repeat flag
.bcount 
	bit	v_JOYPADB	; Isolate B button
	bne	.acount	; Check A if set

	stz	v_bcount	; Otherwise clear repeat flag
.acount 
	bit	v_JOYPADA	; Isolate A button
	bne	.selectcnt

	stz	v_acount	; Otherwise clear repeat flag
.selectcnt
	bit	#JOYPADSELECT
	bne	.startcnt

	stz	v_selectcount

.startcnt
	bit	#JOYPADSTART
	bne	.ycnt2

	stz	v_startcount

.ycnt2
	lda	v_ycount2	
	beq	.selectcnt2

	lda	z_joy2
	and	#JOYPADY
	bne	.selectcnt2

	stz	v_ycount2

.selectcnt2
	lda	z_joy2
	and	#JOYPADSELECT
	bne	.okchkpause

	stz	v_selectcount2

.okchkpause
	lda	v_pause2	; Debug scroller?
	beq	.exit	; No  -

	lda	z_joy2
	and	#JOYPADSTART
	bne	.exit

	dec	v_startcount2
	bpl	.exit

	stz	v_startcount2
.exit
	rts

*****************************************************************************
*
* Qualify a boss hit...
*

Qualbosshit:
	lda	v_playerAI
	cmp	#TZJmpup
	beq	.staycurrent

	cmp	#TZStdjmp
	beq	.staycurrent

	cmp	#TZFall
	beq	.staycurrent

	cmp	#TZLand0
	beq	.staycurrent

	lda	v_bosshit
	sec
	sbc	v_playerx
	bpl	.ontheright

	lda	v_playerdir
	and	#MOVRT
	beq	.justhit

	lda	#MOVUP|MOVDN	; No  - Then reset direction flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir

	bra	.hitinback

.ontheright
	lda	v_playerdir
	and	#MOVLT
	beq	.justhit

	lda	#MOVUP|MOVDN	; No  - Then reset direction flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir

	bra	.hitinback

.justhit
	stz	v_playerdelay

	lda	#4
	sta	v_velocity
	lda	#2
	sta	v_gravity

	lda	#16
	sta	v_bighitxadj
	sta	v_bighityadj

	lda	#TZBighit	
	sta	v_playerAI

	bra	.sethit
.staycurrent
	stz	v_bosshit
.sethit
	lda	#1
	sta	v_damageflag	; Clear damage flag

	rts

.hitinback

	lda	#tzhitfalmode01	; Otherwise, init sequence
	sta	v_playernewtype	; Store new type to restart

	stz	v_playerdelay

	lda	#4
	sta	v_velocity
	lda	#2
	sta	v_gravity

	lda	#16
	sta	v_bighitxadj
	sta	v_bighityadj

	lda	#TZBighitback
	sta	v_playerAI

	bra	.sethit


*****************************************************************************
*
* Debug Movement
*

Movebydebug:
	LAI

.ctrl
	jsl	Scanjoys

	lda	#12
	sta	z_temp0

	lda	z_joy2
	bit	#JOYPADX
	bne	.fast

	lda	#4
	sta	z_temp0
.fast
	lda	z_joy2
	and	#JOYPADSTART
	beq	.lookforkey

	lda	v_startcount2	; Start button debounce
	bne	.stillinpm

	lda	v_pause2
	eor	#1	; Toggle Debug mode
	sta	v_pause2

	lda	#12
	sta	v_startcount2
.stillinpm
	stz	v_lasthealth

	rtl

.lookforkey
	lda	z_joy2
	bit	#JOYPADLEFT
	bne	.left

	bit	#JOYPADRIGHT
	beq	.tryupdown

	lda	z_temp0	; Moving right
	clc   
	adc	v_playerx
	lsr
	lsr
	sta	z_temp2

	lda	v_hoff1max
	clc
	adc	#240
	cmp	z_temp2
	bcc	.tryupdown

	lda	z_temp2
	asl
	asl
	sta	v_playerx

	bra	.tryupdown
.left
	lda	v_playerx	; Moving left
	sec
	sbc	z_temp0
	lsr
	lsr
	sta	z_temp2

	lda	v_hoff1min
	clc
	adc	#16
	cmp	z_temp2
	bcs	.tryupdown

	lda	z_temp2
	asl
	asl
	sta	v_playerx
.tryupdown
	lda	z_joy2
	bit	#JOYPADUP
	bne	.up

	bit	#JOYPADDOWN
	bne	.down

	jmp	.proclevel

.down
	lda	z_temp0
	clc
	adc	v_playery
	lsr
	lsr
	sta	z_temp2

	lda	v_voff1max
	clc
	adc	#196
	cmp	z_temp2
	bcc	.proclevel

	lda	z_temp2
	asl
	asl
	sta	v_playery

	bra	.proclevel
.up
	lda	v_playery
	sec
	sbc	z_temp0
	lsr
	lsr
	sta	z_temp2

	lda	v_voff1min
	clc
	adc	#64
	cmp	z_temp2
	bcs	.proclevel

	lda	z_temp2
	asl
	asl
	sta	v_playery

	rtl

.proclevel:
	rtl


*****************************************************************************

Dopitstuff:
	lda	v_pitopen
	bne	.opit

	rtl
.opit
	jsl	Chek4number	; Gets tile number	

	cmp	#$38	; Over pit area?
	beq	.fallpit	; Yes -

	rtl
.fallpit
	lda	#2
	sta	v_velocity
	lda	#2
	sta	v_gravity

	stz	v_falling
	inc	v_falling

	lda	#TZFall
	sta	v_playerAI

	stz	v_playerdelay

	pla		; Pull the return for this call off

	SA
	pla
	LA
	
	rtl		; Return "from" Player

*****************************************************************************
*
* Player Control Routines
*

Player:
	LAI

	jsl	Chek4wall	; Get new wall tile direction stats

	lda	v_playerAI	; Get vector for player mode
	sta	z_temp0

	jmp	(z_temp0)	; Jump to it



*****************************************************************************
*****************************************************************************
*****************************************************************************
*
* Vine Handlers / Initialization
*
*****************************************************************************

InitVines:
	lda	v_vinedat	; Init'd in scroll.asm
	bne	.dovines

	rtl

.dovines 
	ldx	z_levelword

	lda	>d_vinedats,x
	sta	z_temp0

	lda	#^d_vinedats
	sta	z_temp1

	ldy	#0
	tyx		; Index into vine table
.loadvine 
	lda	[z_temp0],y	; Get vine type
	cmp	#$FFFF	; End of vines?
	bne	.nextvine	; No  - 

	beq	.staticvine

	rtl
.nextvine 
	sta	v_vinetype,x	; Store Vine Type
	
	iny
	iny
	lda	[z_temp0],y	; Vine x coord
	sta	v_vinex,x
	
	iny
	iny
	lda	[z_temp0],y	; Vine y coord
	sta	v_viney,x          

	iny
	iny
	lda	[z_temp0],y	; Initial vine direction
	sta	v_vinedir,x

	iny
	iny
	lda	[z_temp0],y	; vine direction to reset after swing
	sta	v_vinerst,x

	iny
	iny

	inx
	inx

	jmp	.loadvine
.staticvine
	stx	v_vinedat

	jsl	Sendvineonly

	jsl	Senddata

	rtl

Sendvineonly:
	ldx	#tzvineonly00
	lda	d_playerdrec,x
	sta	z_spridrec

	lda	d_playerbank,x
	sta	z_dreccount	; Also for vram init

	ldy	#0

	lda	v_tzanslot	; Slot 4
	clc	
	adc	#$40
	asl
	asl
	asl
	asl
	adc	#OBJBASE	; Compute VRAM address

	jsl	Sendframe	; Queue it up...

	rtl

*****************************************************************************
*
* Plot Selected Vine Animation Frame
*
*****************************************************************************

Plotvines:
	ldy	z_attrcount	; Get attr counter
			  
	ldx	v_vinedat	; count of vines
.loop
	dex
	dex
	bmi	.done

	stx	z_temp0

	lda	v_vinetype,x
	bmi	.loop	;don't plot

	sta	ATTRFRAME,y

	lda	v_viney,x	; Players Y 1/4 pixal
	sta	ATTRY,y	; Players Y Global (level)

	lda	v_vinex,x	; Players X 1/4 pixal
	sta	ATTRX,y	; Players X Global (level)

	lda	v_playeroff
	and	#$0fff
	clc
	adc	#$40
	ora	v_vinedir,x
	ora	v_objpriority	; Add in priority for this level
;	ora	#$3000
	sta	ATTROFF,y	; Store attributes

	ldx	#tzvineonly00

	lda	d_playerbank,x	; Get anim data bank
	sta	ATTRBANK,y

	lda	d_playerdrec,x	; Get animation record
	sta	ATTRDREC,y

	ldx	z_temp0

	iny
	iny

	bra	.loop
.done
	sty	z_attrcount

	rtl


*****************************************************************************
*
* Plot Selected Vine Animation Frame
*
*****************************************************************************

Lookforvine:
	ldx	v_vinedat	;count of vines
	bne	.check

	jmp	.done
.check

	lda	>tzvine1acol	; Point to vine collision box
	and	#$ff	; Store left,right,top,down in temps
	bit	#$80
	beq	.x1
	ora	#$ff00
.x1
	sta	z_temp1	; Store X1

	lda	>tzvine1acol+1
	and	#$ff
	bit	#$80
	beq	.x2
	ora	#$ff00
.x2
	sta	z_temp2	; Store X2

	lda	>tzvine1acol+2
	and	#$ff
	bit	#$80
	beq	.y1
	ora	#$ff00
.y1
	sta	z_temp3	; Store Y1

	lda	>tzvine1acol+3
	and	#$ff
	bit	#$80
	beq	.y2
	ora	#$ff00
.y2
	sta	z_temp4	; Store Y2

	lda	v_playerx
	bpl	.px
	lsr
	lsr
;	ora	#$c000
	bra	.px2
.px
	lsr
	lsr
.px2
	sta	z_temp5	; Store player X

	lda	v_playery
	bpl	.py
	lsr
	lsr
;	ora	#$c000
	bra	.py2
.py
	lsr
	lsr
.py2
	sec
	sbc	#$30
	sta	z_temp6	; Store player Y-48 (top)

.loop
	dex
	dex

	bmi	.done	; Found no vine (in database)

	stx	z_temp0

	lda	v_vinedir,x	; Get the vines swing direction
	bit	#$4000	; Is it right?
	bne	.right	; Yes -

	lda	v_vinex,x
	clc
	adc	z_temp1
	cmp	z_temp5	; Players X
	bcs	.loop

	lda	v_vinex,x
	adc	z_temp2
	cmp	z_temp5
	bcc	.loop

	lda	v_viney,x
	clc
	adc	z_temp3
	cmp	z_temp6
	bcs	.loop

	lda	v_viney,x
	adc	z_temp4
	cmp	z_temp6
	bcc	.loop
;foundvine
	stx	v_curvine

.left
	lda	#MOVUP|MOVDN	; Reset left/right flag
	and	v_playerdir
	ora	#MOVRT
	sta	v_playerdir	; Set player going right

	lda	#1

	rtl
.done
	lda	#0

	rtl
.right
	lda	#MOVUP|MOVDN	; Reset left/right flag
	and	v_playerdir
	ora	#MOVLT
	sta	v_playerdir	; Set player going left

;	lda	z_temp2
;	sec
;	sbc	v_vinex,x
;	cmp	z_temp5
;	bcs	.loop

	lda	v_vinex,x
	sec
	sbc	z_temp2
	cmp	z_temp5
	bcs	.loop

;	lda	z_temp1
;	sec
;	sbc	v_vinex,x
;	cmp	z_temp5
;	bcc	.loop

	lda	v_vinex,x
	sec
	sbc	z_temp1
	cmp	z_temp5
	bcc	.loop

	lda	v_viney,x
	clc
	adc	z_temp3
	cmp	z_temp6
	bcs	.loop

	lda	v_viney,x
	adc	z_temp4
	cmp	z_temp6
	bcc	.loop
;foundvine
	stx	v_curvine

	lda	v_vinedir,x
	sta	v_vinedirparm

	lda	#1

	rtl


