*****************************************************************************
*
* SCROLL.ASM
*
*****************************************************************************

	.MACRO	ASR

	  pha		; Save it

	  asl		; Shift negative bit into proc status

	  php		; Save processor status flags

	  ror		; Rotate neg bit back into offset

	  plp		; Restore flags
	  php

	  ror		; Rotate neg bit into word again

	  plp

	  ror

	.ENDM


*****************************************************************************
*
* Initialize Scroller
*
* Get data from WORLDDAT.ASM & Put 1st 32 columns on screen
*
*****************************************************************************

Scrollinit:
	LAI

	jsr	Initspritvars

	stz	z_colfade1	
	stz	z_colfade2	
	stz	z_colfade3	
	stz	z_colfade4	
	stz	z_colfadeip
	stz	z_docolfade
	stz	z_colfadeos

	stz	z_sdir	; No scroll direction yet
	stz	z_bg1size	; Clear BG sizes
	stz	z_bg2size
	stz	z_bg3size
	
	ldy	z_level	; Get byte level index

	SAI

	lda	_xfertypes,y	; Get tranfer type for this level
	sta	z_xfertype	; Store Vertical / Horizontal

	lda	_bg1sizes,y	; Get BG1 size (0,1,2,3)
	sta	z_bg1size

	lda	_bg2sizes,y	; Get BG2 size (0,1,2,3)
	sta	z_bg2size

	lda	_bg3sizes,y	; Get BG3 size (0,1,2,3)
	sta	z_bg3size

	lda	_skipsups,y	; Get BG update skip flags
	sta	z_skipsup
            
	LAI
	
	ldy	z_levelword	; Get word level index

	lda	v_return
	bne	.skipcoininit

	lda	d_numcoins,y	; Get number of bonus coins this level
	sta	v_coincur
	dec
	dec
	sta	v_coinlast

.skipcoininit
	lda	d_fallmax,y
	sta	v_fallmax

	lda	d_tzanslot,y	; Get Tarzan VRAM Slot
	sta	v_tzanslot

	lda	d_vinedats,y	; Do we do vines?
	sta	v_vinedat	; Get indicator

	lda	d_priority,y	; Get mainly Tarzans BG | OBJ priority
	sta	v_objpriority

	lda	_bg1widths,y	; Get BG1 Width (in screens)
	sta	z_bg1width

	lda	_bg1heights,y	; Get BG1 height (in screens)
	sta	z_bg1height

	lda	d_maxxs,y	; Get right edge stop
	sta	v_maxx

	lda	d_minx,y	; Get right edge stop
	sta	v_minx

	lda	_hoff1maxs,y	; Get right edge scroll stop
	sta	v_hoff1max

	lda	_hoff1mins,y	; Get left edge scroll stop
	sta	v_hoff1min

	lda	_voff1maxs,y	; Get bottom edge scroll stop
	sta	v_voff1max

	lda	_voff1mins,y	; Get top edge scroll stop
	sta	v_voff1min

	lda	_bg2widths,y	; Get BG2 width
	sta	z_bg2width

	lda	_bg3widths,y	; Get BG3 width
	sta	z_bg3width

	lda	_bg2heights,y	; Get BG2 height
	sta	z_bg2height

	lda	_bg3heights,y	; Get BG3 height
	sta	z_bg3height

	lda	v_leavehold	; Are we leaving hold?
	bne	.fixshipret

	jmp	.normshipstart

.fixshipret
	lda	v_holdnum
	cmp	#1
	bne	.notholda

	lda	.bg1hshipoffa
	sta	z_hoff1	; Store copy

	asl		; Generate 1/4 pixal virtual offset
	asl
	sta	z_h1

	lda	.bg2hshipoffa
	sta	z_hoff2

	asl
	asl
	sta	z_h2

	lda	.bg3hshipoffa
	sta	z_hoff3

	asl
	asl
	sta	z_h3

	bra	.doys
.notholda
	lda	.bg1hshipoffb
	sta	z_hoff1	; Store copy

	asl		; Generate 1/4 pixal virtual offset
	asl
	sta	z_h1

	lda	.bg2hshipoffb
	sta	z_hoff2

	asl
	asl
	sta	z_h2

	lda	.bg3hshipoffb
	sta	z_hoff3

	asl
	asl
	sta	z_h3

	bra	.doys

.normshipstart

	lda	_bg1hoffs,y	; Get starting Horizontal Offsets BG1
	sta	z_hoff1	; Store copy

	asl		; Generate 1/4 pixal virtual offset
	asl
	sta	z_h1

	lda	_bg2hoffs,y	; Same for BG2
	sta	z_hoff2

	asl
	asl
	sta	z_h2

	lda	_bg3hoffs,y	; Same for BG3
	sta	z_hoff3

	asl
	asl
	sta	z_h3

.doys
	lda	_bg1voffs,y	; Get starting Vertical Offsets BG1
	sta	z_voff1	; Store copy

	asl		; Generate 1/4 pixal virtual offset
	asl
	sta	z_v1

	lda	_bg2voffs,y	; Same for BG2
	sta	z_voff2

	asl
	asl
	sta	z_v2

	lda	_bg3voffs,y	; Same for BG3
	sta	z_voff3

	asl
	asl
	sta	z_v3

.shipfixd
	lda	z_xfertype	; Get current transfer type
	and	#1	; Horiz. or Vert. ?
	beq	.copyrows	; Do row updates

	lda	z_hoff1	; Else do column updates
	ldy	z_hoff2
	ldx	z_hoff3

	jsl	Offsetconv	; Convert offsets

	bra	.copy	; Copy screen

.copyrows   
	lda	z_voff1	; Do Row updates
	ldy	z_voff2
	ldx	z_voff3

	jsl	Offsetconv	; Convert offsets
.copy
	jsl	Copyscr	; Copy screen

	lda	z_newcol1
	dec
	and	#$1f
	sta	z_newcol1	; Next column BG1

	lda	z_newcol2
	dec
	and	#$1f
	sta	z_newcol2	; Next column BG2

	lda	z_newcol3
	dec
	and	#$1f
	sta	z_newcol3	; Next column BG3

	lda	_bgenable
	bit	#1
	beq	.nobg1
      
	jsl	Offsetset	; Put first 32 columns on screen

	jsl	Initrowlocs

	jsl	Inittiletypes
.nobg1
	rtl


.bg1hshipoffa	.dw	1459	; Hold A
.bg1hshipoffb	.dw	16	; Hold B
.bg2hshipoffa	.dw	16
.bg2hshipoffb	.dw	16
.bg3hshipoffa	.dw	16
.bg3hshipoffb	.dw	16

.bg1vshipoff	.dw	728,728
.bg2vshipoff	.dw	752,752
.bg3vshipoff	.dw	288,288



Initspritvars:
	stz	z_attrcount
	stz	z_oamcount	
	stz	z_levelx	
	stz	z_levely	
	stz	z_sprix	
	stz	z_spriy	
	stz	z_drecframe
	stz	z_dreccount
	stz	z_spridrec	
	stz	z_drecsize	
	stz	z_drecnum	
	stz	z_drecx	
	stz	z_drecy	
	stz	z_drecfpc	
	stz	z_drecdim	
	stz	z_sprisize	
	
	rts



*****************************************************************************
*
* converts offsets to column or row number/32 and stores in col0 col31
* used as a prelim to copyscr
*  a contains offset 1
*  y contains offset 2
*  x contains offset 3
*
*****************************************************************************

Offsetconv:
	LAI

	stx	z_temp8	; Save bg3 val

	lsr		; Divide bg1 offset by 16
	lsr
	lsr
	lsr

	dec		; Move back to hidden column
	tax		; Save it
	and	#$1f	; Only 32 of em'
	sta	z_newcol1	; Store new column

	txa
	sta	z_bg1col0	; first row/column

	clc
	adc	#31   

	ldx	z_xfertype
	cpx	#$80	; check for vertical addressing
	beq	.vertical1

	cmp	z_bg1width	; Compare with level width
	bcc	.end1	; If < then done with bg1

	sec		; Otherwise, adjust
	sbc	z_bg1width

	bra	.end1
.vertical1
	cmp	z_bg1height
	bcc	.end1

	sec   
	sbc	z_bg1height
.end1
	sta	z_bg1col31	; Store column/row 31
.chkbg2
;	lda	z_skipsup
;	bit	#2
;	bne	.chkbg3

	tya

	lsr		; Divide bg1 offset by 16
	lsr
	lsr
	lsr

	dec		; Move back to hidden column
	tax		; Save it
	and	#$1f	; Only 32 of em'
	sta	z_newcol2	; Store new column

	txa
	sta	z_bg2col0	; first row/column
			
	clc
	adc	#31

	ldx	z_xfertype
	cpx	#$80	; check for vertical addressing
	beq	.vertical2

	cmp	z_bg2width
	bcc	.end2

	sec
	sbc	z_bg2width

	bra	.end2
.vertical2
	cmp	z_bg2height
	bcc	.end2

	sec   
	sbc	z_bg2height
.end2
	sta	z_bg2col31
.chkbg3
	lda	z_skipsup
	bit	#4
	bne	.exitconv

	lda	z_temp8

	lsr		
	lsr
	lsr
	lsr

	dec
	tax
	and	#$1f
	sta	z_newcol3

	txa
	sta	z_bg3col0

	clc
	adc	#31

	ldx	z_xfertype
	cpx	#$80
	beq	.vertical3

	cmp	z_bg3width
	bcc	.end3

	sec
	sbc	z_bg3width

	bra	.end3
.vertical3
	cmp	z_bg3height
	bcc	.end3

	sec   
	sbc	z_bg3height
.end3
	sta	z_bg3col31

.exitconv
	rtl



*****************************************************************************
*
*
* Setbuff1, Setbuff2, Setbuff3
*
* will transfer data from levels to column buffers
* for backgrounds 1, 2 & 3.  Transfers 32 words to BGxBUFF
* where x is BG#.  This routine can be used during any period
* Runs in 16-bit mode
*
*  Inputs:  A = level width in columns
*           X = offset column in level
*  Outputs: A = ?
*           X = ?
*           Y = #$40

Setbuff1:
	LAI

	asl		;level width in words
	sta	z_temp1	;save level width

	txa		;offset in columns
	asl
	tax		;offset in words

	ldy	#0
.setloop
	lda	>BG1MAP,x	;fetch from ram
	sta	BG1BUFF,y	;store in column buffer

	txa
	clc
	adc	z_temp1	;level width
	tax

	iny
	iny

	cpy	z_bg1size	;length of buffer
	bcc	.setloop

	rtl


Setbuff2:
	LAI

	asl		;level width in words
	sta	z_temp1	;save level width

	txa		;offset in columns
	asl
	tax		;offset in words

	ldy	#0
.setloop
	lda	>BG2MAP,x	;fetch from ram
	sta	BG2BUFF,y	;store in column buffer

	txa
	clc
	adc	z_temp1	;level width
	tax

	iny
	iny

	cpy	z_bg2size	;length of buffer
	bcc	.setloop

	rtl


Setbuff3:
	LAI

	asl		;level width in words
	sta	z_temp1   	;save level width

	txa               	;offset in columns
	asl
	tax               	;offset in words

	ldy	#0
.setloop
	lda	>BG3MAP,x 	;fetch from ram
	sta	BG3BUFF,y 	;store in column buffer

	txa
	clc
	adc	z_temp1   	;level width
	tax

	iny
	iny

	cpy	z_bg3size 	;length of buffer
	bcc	.setloop

	rtl


*****************************************************************************
*
* Bgdma
*
* transfers  Bgbuff's to vram addresses
*
*  Inputs:  A = vram addr
*           X = Buffer addr (#BG1BUFF, #BG2BUFF, #BG3BUFF)
*           Y = length of transfer buffer
*  Outputs: A = ?
*
*****************************************************************************

Bgdma:     
	LAI

	stx	DMA0A1L        ;buffer address in x (word)
	sty	DMA0NUML       ;size of transfer
	sta	VMADDL         ;vram address (word)

	SAI
	lda	z_xfertype      ;row or screen increments
	sta	VMAINC         ;so column is altered (not row)

	stz	DMA0A1B        ;buffers are in bank zero

	lda	#<VMDATAL      ;vram data buffer
	sta	DMA0DEST

	lda	#1             ;increment by one from buffer
	sta	DMA0INC

	sta	MDMAEN         ;enable transfer
	LAI

	rtl

*****************************************************************************
*
*  prescroll is called by scroll to adjust background offsets 
*            will skip backgrounds based on z_skipsup bits
*            uses _sdir to determine scroll direction
*
* updates h/v offsets based on sdir and sud, slr values
*
*****************************************************************************

Prescroll:
	LAI

	lda	z_sdir	; Sscroll direction
	bit	#1	; Check for down scroll (low bit)
	beq	.scrollup	; If not branch over section

	lda	z_skipsup	;get skip scroll update flags (per bg)
	bit	#1	;check to skip bg1 (low bit)
	bne	.psd2	;skip to bg2

	; bg1 

	lda	z_sud1	;scroll up/down increment bg1
	clc
	adc	z_v1	;vertical offset bg1
	sta	z_v1
	lsr		;divide by two for actual offset
	lsr		;actually by four
	sta	z_voff1	;actual vertical offset bg1

	; bg2 
.psd2
	lda	z_skipsup	;skip scroll flags
	bit	#2	;check to skip bg2
	bne	.psd3	;branch to bg3 if bit was set

	lda	z_sud2
	clc
	adc	z_v2
	sta	z_v2
	lsr
	lsr
	sta	z_voff2

	; bg3 
.psd3
	lda	z_skipsup	;skip scroll flags
	bit	#4	;check to skip bg2
	bne	.scrollup

	lda	z_sud3
	clc
	adc	z_v3
	sta	z_v3
	lsr
	lsr
	sta	z_voff3

.scrollup   
	lda	z_sdir	;scroll direction
	bit	#2	; Check for up scroll (low bit) 
	beq	.scrollrite	; If not branch over section

	lda	z_skipsup	;check each background for update
	bit	#1	;check to skip bg1 (low bit)
	bne	.psu2	;skip to bg2

	; bg1

	lda	z_v1
	sec
	sbc	z_sud1
	sta	z_v1
	lsr
	lsr
	sta	z_voff1

	; bg2
.psu2       
	lda	z_skipsup	;skip scroll flags
	bit	#2	;check to skip bg2
	bne	.psu3	;branch to bg3 if bit was set

	lda	z_v2
	sec
	sbc	z_sud2
	sta	z_v2
	lsr
	lsr
	sta	z_voff2

	; bg3
.psu3       
	lda	z_skipsup	;skip scroll flags
	bit	#4	;check to skip bg2
	bne	.scrollrite

	lda	z_v3
	sec
	sbc	z_sud3
	sta	z_v3
	lsr
	lsr
	sta	z_voff3

.scrollrite 
	lda	z_sdir	;scroll direction
	bit	#4	; Check for right scroll (low bit)
	beq	.scrollleft	; If not branch over section

	lda	z_skipsup	;check each background for update
	bit	#1	;check to skip bg1 (low bit)
	bne	.psr2	;skip to bg2

	; bg1

	lda	z_slr1	;scroll left/right increment bg1
	clc
	adc	z_h1	;add to horizontal offset
	ASR
	sta	z_hoff1	;net effect is ASR instead of LSR in hoff
	pla		;restore h1
	and	#OFFSETMASK2	;wrap around mask
	sta	z_h1	;finally store it

	; bg2
.psr2       
	lda	z_skipsup	;skip scroll flags
	bit	#2	;check to skip bg2
	bne	.psr3	;branch to bg3 if bit was set

	lda	z_slr2
	clc
	adc	z_h2
	ASR
	sta	z_hoff2
	pla		;restore h2
	and	#OFFSETMASK2
	sta	z_h2

	; bg3
.psr3       
	lda	z_skipsup	;skip scroll flags
	bit	#4	;check to skip bg2
	bne	.scrollleft	;branch to bg3 if bit was set

	lda	z_slr3
	clc
	adc	z_h3
	ASR
	sta	z_hoff3
	pla		;restore h3
	and	#OFFSETMASK2
	sta	z_h3

.scrollleft
	lda	z_sdir	;scroll direction
	bit	#8	; Check for left scroll (low bit)
	beq	.scrollend	; If not branch over section

	lda	z_skipsup	;check each background for update
	bit	#1	;check to skip bg1 (low bit)
	bne	.psl2	;skip to bg2

	; bg1

	lda	z_h1
	sec		;left so subtract instead of add
	sbc	z_slr1
	ASR
	sta	z_hoff1
	pla		;pull h1
	and	#OFFSETMASK2
	sta	z_h1

	; bg2
.psl2
	lda	z_skipsup	;skip scroll flags
	bit	#2	;check to skip bg2
	bne	.psl3	;branch to bg3 if bit was set

	lda	z_h2            
	sec
	sbc	z_slr2
	ASR
	sta	z_hoff2
	pla		;restore h2
	and	#OFFSETMASK2
	sta	z_h2

	; bg3
.psl3
	lda	z_skipsup	;skip scroll flags
	bit	#4	;check to skip bg2
	bne	.scrollend	;branch to bg3 if bit was set

	lda	z_h3            
	sec
	sbc	z_slr3
	ASR
	sta	z_hoff3
	pla		;restore h3
	and	#OFFSETMASK2
	sta	z_h3

.scrollend
	rtl


*****************************************************************************
*
* to use scroll, set scroll dir bits (z_sdir: 0000lrud)
* and set skip scroll update bits (z_skipsup: 00000321) to skip
* certain backgrounds (or make all slr, sud values 0)
* call this routine, which will call prescroll, figure
* new column pointers, and set column buffers.  Then use colupdate
* during vblank to put the data in vram
*
*****************************************************************************

Scroll:
	LAI

	lda	v_disbgscroll
	beq	.nodisable

	jmp	.mainexitscroll	;no scroll this time

.nodisable
	stz	z_sdir

	jsl	Bounds	; Check scroll bounds

	stz	z_sup	; Clear update flags

	lda	z_skipsup	;check user update flags
	cmp	#7	;skip all bgs?
	bne	.scroll

	jmp	.mainexitscroll	;no scroll this time
.scroll
	lda	z_xfertype
	and	#$3	;check for 1 byte increment mode
	bne	.scrollhorz	;set means column updates

	jmp	Scrollvert	;not set means vertical updates
.scrollhorz
	lda	z_hoff1	;actual bg1 horizontal offset
	and	#CHARMASK16	;character width mask
	sta	z_temp0	;save for later

	lda	z_hoff2	;actual bg2 horizontal offset
	and	#CHARMASK16	;character width mask
	sta	z_temp2	;save for later

	lda	z_hoff3	;actual bg3 horizontal offset
	and	#CHARMASK16	;character width mask
	sta	z_temp8	;save for later

	jsl	Prescroll	;update other offsets


	;start bg1 column calcs and buffer loads

	lda	z_skipsup	;check if scroll updates needed
	bit	#1
	bne	.bg2	;skip bg1

	lda	z_hoff1	;new offset as per prescroll
	bpl	.bg1	;negative requires special compare

	and	#OFFSETMASK	;screen wrap mask
	sta	z_hoff1	;store it back

	bra	.bg1dec	;was negative, scrolled left, skip section
.bg1
	tax		;check for positive values if new column
	and	#CHARMASK16	;needed, save high bits (lose charwidth)
	cmp	z_temp0	;is it same column as old value
	php		;push answer
	txa		;transfer and save offset
	and	#OFFSETMASK
	sta	z_hoff1

	plp		;restore answer
	beq	.bg2	;same value then skip new column update

	bcc	.bg1dec	;lower column, then decrement columns

	lda	#8	;otherwise we do increment
	sta	z_sup	;bg1 is being updated right flag

	inc	z_newcol1	;inc new col bg1 pointer

	lda	z_newcol1
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap1	;if not at right edge then skip ahead

	stz	z_newcol1	;is now at left edge of screen
.nowrap1
	inc	z_bg1col0	;increment left edge pointer

	lda	z_bg1col0
	cmp	z_bg1width	;check screen bounds
	bne	.bg12

	stz	z_bg1col0
.bg12
	inc	z_bg1col31	;increment right edge pointer

	lda	z_bg1col31
	cmp	z_bg1width	;check screen bounds
	bne	.set131

	stz	z_bg1col31
.set131
	lda	z_bg1width	;now transfer far right column to buffer
	ldx	z_bg1col31

	jsl	Setbuff1

	bra	.bg2	;branch to background 2
.bg1dec
	inc	z_sup	;bg1 updating left

	dec	z_newcol1	;decrement new column
	bpl	.nowrap2

	lda	#SCREENEDGE-1	;if wrap then set to right edge
	sta	z_newcol1
.nowrap2
	dec	z_bg1col0	;dec left column
	bpl	.bg13	;check bounds

	ldy	z_bg1width
	dey
	sty	z_bg1col0
.bg13
	dec	z_bg1col31	;dec right column
	bpl	.set10	;check bounds

	ldy	z_bg1width
	dey
	sty	z_bg1col31
.set10
	lda	z_bg1width	;transfer left column to buffer
	ldx	z_bg1col0

	jsl	Setbuff1
.bg2
	lda	z_skipsup	;check if scroll updates needed
	bit	#2
	bne	.bg3	;skip bg2

	lda	z_hoff2
	bpl	.bg222

	and	#OFFSETMASK
	sta	z_hoff2

	bra	.bg2dec
.bg222
	tax
	and	#CHARMASK16
	cmp	z_temp2

	php
	txa
	and	#OFFSETMASK
	sta	z_hoff2

	plp
	beq	.bg3

	bcc	.bg2dec

	lda	#16
	ora	z_sup
	sta	z_sup	;bg2 is being updated right

	inc	z_newcol2

	lda	z_newcol2
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap21

	stz	z_newcol2
.nowrap21
	inc	z_bg2col0

	lda	z_bg2col0
	cmp	z_bg2width
	bne	.bg22

	stz	z_bg2col0
.bg22
	inc	z_bg2col31

	lda	z_bg2col31
	cmp	z_bg2width
	bne	.set231

	stz	z_bg2col31
.set231
	lda	z_bg2width	;transfer right column to buffer
	ldx	z_bg2col31

	jsl	Setbuff2

	bra	.bg3	;skip to bg3
.bg2dec
	lda	#2
	ora	z_sup
	sta	z_sup	;bg2 updating left

	dec	z_newcol2
	bpl	.nowrap22

	lda	#SCREENEDGE-1
	sta	z_newcol2
.nowrap22
	dec	z_bg2col0
	bpl	.bg23

	ldy	z_bg2width
	dey
	sty	z_bg2col0
.bg23
	dec	z_bg2col31
	bpl	.set20

	ldy	z_bg2width
	dey
	sty	z_bg2col31
.set20
	lda	z_bg2width	;transfer left column to buffer
	ldx	z_bg2col0

	jsl	Setbuff2

.bg3
	lda	z_skipsup	;check if scroll updates needed
	bit	#4
	bne	.scrollend	;skip bg3
@Scrollmist:
	lda	z_hoff3
	bpl	.bg333

	and	#OFFSETMASK
	sta	z_hoff3

	bra	.bg3dec
.bg333
	tax
	and	#CHARMASK16
	cmp	z_temp8
	php
	txa
	and	#OFFSETMASK
	sta	z_hoff3

	plp
	beq	.scrollend

	bcc	.bg3dec

	lda	#32
	ora	z_sup
	sta	z_sup	;bg3 is being updated right

	inc	z_newcol3

	lda	z_newcol3
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap31

	stz	z_newcol3
.nowrap31
	inc	z_bg3col0

	lda	z_bg3col0
	cmp	z_bg3width
	bne	.bg32

	stz	z_bg3col0
.bg32
	inc	z_bg3col31

	lda	z_bg3col31
	cmp	z_bg3width
	bne	.set331

	stz	z_bg3col31
.set331
	lda	z_bg3width	;transfer right column to buffer
	ldx	z_bg3col31

	jsl	Setbuff3

	bra	.scrollend	;skip to end
.bg3dec
	lda	#4
	ora	z_sup
	sta	z_sup	;bg3 updating left

	dec	z_newcol3
	bpl	.nowrap33

	lda	#SCREENEDGE-1
	sta	z_newcol3
.nowrap33
	dec	z_bg3col0
	bpl	.bg33

	ldy	z_bg3width
	dey
	sty	z_bg3col0
.bg33
	dec	z_bg3col31
	bpl	.set30

	ldy	z_bg3width
	dey
	sty	z_bg3col31
.set30
	lda	z_bg3width	;transfer left column to buffer
	ldx	z_bg3col0

	jsl	Setbuff3
.scrollend
	jsl	Colupdate	;figure out background vram pointers

.mainexitscroll
	lda	z_hoff1
	sta	z_levelx

	lda	z_voff1
	sta	z_levely

	rtl


*****************************************************************************
* 
* figures out bg pointers in vram for the Colxfer sub
* no inputs needed except z_sup flags from Scroll
*

Colupdate:  
	LAI

	lda	z_sup	;flags set by scroll
	and	#9	;bg1 mask
	beq	.bg2	;branch to bg2 if no bits set

	lsr		;left pan?

	lda	#0	;carry is set if left pan!
	adc	z_newcol1
	cmp	#SCREENEDGE
	bne	.nowrap1

	lda	#0
.nowrap1    
	clc		;add column to screen base
	adc	#BG1BASE
	sta	v_storeaddr1	;word address is produced
.bg2        
	lda	z_sup
	and	#$12	;bg2 mask
	beq	.bg3

	lsr
	lsr		;left pan?

	lda	#0
	adc	z_newcol2
	cmp	#SCREENEDGE
	bne	.nowrap2

	lda	#0
.nowrap2    
	clc
	adc	#BG2BASE
	sta	v_storeaddr2
.bg3        
	lda	z_sup
	and	#$24	;bg3 mask
	beq	.cupdatend

	lsr
	lsr
	lsr		;left pan?

	lda	#0
	adc	z_newcol3
	cmp	#SCREENEDGE
	bne	.nowrap3

	lda	#0
.nowrap3    
	clc
	adc	#BG3BASE
	sta	v_storeaddr3
.cupdatend  
	rtl


*****************************************************************************
* 
* Transfers Background Columns 
*
* (Use During VBLANK)
*
*****************************************************************************

Colxfer:    
	LAI

	lda	z_sup	; no need to do bg1 if
	and	#9	; masked bits are zero
	beq	.bg2

	lda	z_xfertype	; check for case where
	and	#1	; row updates and
	bne	.doit	; full size happen at same time

	lda	z_bg1size    
	cmp	#128
	bne	.doit

	lda	v_storeaddr1
	ldx	#BG1BUFF
	ldy	#64

	jsl	Bgdma

	lda	v_storeaddr1
	clc
	adc	#$400

	ldx	#BG1BUFF+64
	ldy	#64

	jsl	Bgdma

	bra	.bg2
.doit
	lda	v_storeaddr1	;address produced by colupdate
	ldx	#BG1BUFF	;address of column buffer bg1
	ldy	z_bg1size

	jsl	Bgdma	;transfer to vram
.bg2
	lda	z_sup
	and	#$12	;bg2 mask
	beq	.bg3

	lda	z_xfertype	; check for case where
	and	#1	; row updates and
	bne	.doit2	; full size happen at same time

	lda	z_bg2size    
	cmp	#128
	bne	.doit2

	lda	v_storeaddr2
	ldx	#BG2BUFF
	ldy	#64

	jsl	Bgdma

	lda	v_storeaddr2
	clc
	adc	#$400

	ldx	#BG2BUFF+64
	ldy	#64

	jsl	Bgdma

	bra	.bg3

.doit2
	lda	v_storeaddr2
	ldx	#BG2BUFF
	ldy	z_bg2size

	jsl	Bgdma
.bg3
	lda	z_sup
	and	#$24	;bg3 mask
	beq	.xferend

	lda	v_storeaddr3
	ldx	#BG3BUFF
	ldy	z_bg3size

	jsl	Bgdma

.xferend
	rtl


*****************************************************************************
* 
* Stores new BG offset values to create scrolling effect
*
* (Use During VBLANK)
*
*****************************************************************************

Offsetset:
	LAI

	dec   z_voff1
	dec   z_voff2
	dec   z_voff3

	SAI

	lda   z_voff1
	sta   BG1VOFS
	lda   z_voff1+1
	sta   BG1VOFS
	lda   z_hoff1
	sta   BG1HOFS
	lda   z_hoff1+1
	sta   BG1HOFS

	lda   z_voff2     
	sta   BG2VOFS
	lda   z_voff2+1
	sta   BG2VOFS
	lda   z_hoff2
	sta   BG2HOFS
	lda   z_hoff2+1
	sta   BG2HOFS

	lda   z_voff3     
	sta   BG3VOFS
	lda   z_voff3+1
	sta   BG3VOFS
	lda   z_hoff3
	sta   BG3HOFS
	lda   z_hoff3+1
	sta   BG3HOFS

	LAI

	inc   z_voff1
	inc   z_voff2
	inc   z_voff3

	rtl


*****************************************************************************
*
* Bounds Checker	  
*
* Make horizontal / vertical changes based on player's position
*
*****************************************************************************

MAXSCROLL	.equ	16
MAXSCROLL1	.equ	32
MAXSCROLLR	.equ	32

Bounds:
	LAI

	lda	#MAXSCROLL
	clc
	adc	v_addscroll
	sta	v_maxscroll

	lda	v_playertype
	cmp	#tzswingmodeb1
	bne	.chkmode2

	lda	#MAXSCROLLR
	sta	v_maxscroll

	bra	.chkbounds

.chkmode2
	cmp	#tzswingmodeb2
	bne	.chkbounds

	lda	#MAXSCROLLR
	sta	v_maxscroll
		 
.chkbounds
	lda	z_level
	cmp	#SHIPHOLDA
	beq	.horizontal

	cmp	#SHIPHOLDB
	beq	.horizontal

	lda	v_playery	; Get scaled y
	lsr
	lsr
	sec
	sbc	z_levely	; Normalize to screen coordinates
	bmi	.maxup	; Above top of screen

	sec
	sbc	_dymid	; Apply Midpoint
	beq	.horizontal	; No work if equal

	bpl	.down	; Move down if positive

	eor	#$FFFF	; Otherwise abs(x)
	inc
	cmp	v_maxscroll	; Need to move up at max rate?
	bcc	.up2	; No  - lesser rate
.maxup
	sta	z_temp0

	lda	v_donedark
	bne	.docalcd

	lda	v_maxscroll	; Yes - Set max scroll rate
	sta	z_temp0
.docalcd
	lda	z_temp0
	sta	z_sud1

	bra	.setdirup
.up2
	cmp	#8
	bcc	.up3

	lda	#8
	sta	z_sud1

	bra	.setdirup
.up3
	lda	#4
	sta	z_sud1

.setdirup
	lda	z_sdir
	ora	#SCRUP

	bra	.sdirud

.down
	cmp	v_maxscroll
	bcc	.dn2

	sta	z_sud1

	bra	.setdirdn
.dn2
	cmp	#8
	bcc	.dn3

	lda	#8
	sta	z_sud1

	bra	.setdirdn
.dn3
	lda	#4
	sta	z_sud1

.setdirdn
	lda	z_sdir
	ora	#SCRDN
.sdirud
	sta	z_sdir

.horizontal
	lda	v_playerx
	lsr
	lsr
	sec
	sbc	z_levelx
	bmi	.maxleft

	sec
	sbc	_dxmid
	beq	.bounds

	bpl	.right

	eor	#$FFFF
	inc
	cmp	v_maxscroll
	bcc	.left2
.maxleft
	lda	v_maxscroll
	sta	z_slr1

	bra	.setdirlft
.left2
	cmp	#8
	bcc	.left3

	lda	#8
	sta	z_slr1

	bra	.setdirlft
.left3
	lda	#4
	sta	z_slr1

.setdirlft
	lda	z_sdir
	ora	#SCRLT

	bra	.sdirlrt

.right      
	cmp	v_maxscroll
	bcc	.right2

	sta	z_slr1

	bra	.setdirrgt
.right2
	cmp	#8
	bcc	.rght3

	lda	#8
	sta	z_slr1

	bra	.setdirrgt
.rght3
	lda	#4
	sta	z_slr1
.setdirrgt
	lda	z_sdir
	ora	#SCRRT
.sdirlrt
	sta	z_sdir

.bounds
	jsl	Boundcheck

	lda	z_sdir
	bne	.fixratios

	rtl


.jmptbl
	.dw	.stndrdratio	; Null Level
	.dw	.stndrdratio	; Jungle 1
	.dw	.chkTZANHOME	; Tarzans Home
	.dw	.stndrdratio	; Apes Clearing
	.dw	.chkSWAMP	; Swamp

	.dw	.chkBEACH	; Beach
	.dw	.stndrdratio	; Beach-Bay
	.dw	.chkREEF	; Reef
	.dw	.chkSHIP	; Ship
	.dw	.stndrdratio	; Ship Hold A

	.dw	.stndrdratio	; Ship Hold B
	.dw	.stndrdratio	; Jungle 3
	.dw	.chkDOWNRIVER	; Downriver
	.dw	.stndrdratio	; Jungle 4
	.dw	.chkUPRIVER	; UpRiver

	.dw	.chkWATERFALL	; Waterfall
	.dw	.chkCAVERN	; Caverns
	.dw	.stndrdratio	; Opar - Street
	.dw	.chkOPARWELLS	; Opar - Wells
	.dw	.stndrdratio	; Opar - Temple

	.dw	.stndrdratio	; Opar - Shamon Boss

	.dw	.stndrdratio	; Bonus Level - Exterior
	.dw	.stndrdratio	; Bonus Level - Interior

.fixratios
	ldx	z_levelword

	jmp	(.jmptbl,x)

****************************************
*
* Set standard scroll ratios
*


.stndrdratio
	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast
	lsr
	sta	z_slr3	; BG3 is 1/4 as fast

	lda	z_sud1
	lsr
	sta	z_sud2

	lda	z_sud1
	cmp	#17
	bcc	.set3

	lda	#0
.set3
	lsr
	lsr
	lsr
	lsr
	lsr
	sta	z_sud3


	rtl


****************************************
*
* Set standard scroll ratios
*


.chkCAVERN
	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast

	stz	z_slr3	; BG3 stays still

	lda	z_sud1	; Get BG1 u/d scroll speed
	lsr      
	sta	z_sud2	; BG2 is half as fast

	stz	z_sud3	; BG3 stays still

	rtl


****************************************
*
* Handle Swamp
*
.chkSWAMP
	lda	z_slr1
	sta	z_slr3
	lsr
	sta	z_slr2

	inc	z_slr3
	inc	z_slr3


	lda	z_sud1	; Get BG1 u/d scroll speed
	sta	z_sud3
	lsr
	sta	z_sud2	; BG2 is half as fast
	
;	inc	z_sud3
;	inc	z_sud3

	rtl


****************************************
*
* Handle Beach
*

.chkBEACH
	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2
	lsr
	sta	z_slr3

	lda	z_sud1	; Get BG1 u/d scroll speed
	lsr      
	sta	z_sud2	; BG2 is half as fast
	lsr
	sta	z_sud3

	rtl


****************************************
*
* Handle Reef
*

.chkREEF
	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2
	lsr
	sta	z_slr3

	lda	z_sud1	; Get BG1 u/d scroll speed
	lsr      
	sta	z_sud2	; BG2 is half as fast
	lsr
	sta	z_sud3

	rtl


****************************************
*
* Handle Ship
*

.chkSHIP
	lda	z_sud1
	lsr
	sta	z_sud2
	lsr
	sta	z_sud3

	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast
	lsr
	sta	z_slr3	; BG3 is 1/4 as fast

	rtl


****************************************
*
* Handle Downriver
*

.chkDOWNRIVER
	lda	z_sud1
	sta	z_sud3
	lsr
	sta	z_sud2

	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast
	sta	z_slr3	; BG3 is 1/4 as fast

	rtl


****************************************
*
* Handle Tarzan Tree Fort
*

.chkTZANHOME
	lda	z_sud1
	lsr
	sta	z_sud2

	lda	z_sud1
	cmp	#17
	bcc	.sethome3

	lda	#0
.sethome3
	lsr
	lsr
	lsr
	lsr
	lsr
	sta	z_sud3

	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast
	lsr
	sta	z_slr3	; BG3 is 1/4 as fast

	rtl


****************************************
*
* Handle Upriver
*

.chkUPRIVER
	lda	z_sud1
	sta	z_sud2
	sta	z_sud3

	lda	z_slr1	; Get BG1 l/r scroll speed
	lsr    
	sta	z_slr2	; BG2 is half as fast
	lsr
	sta	z_slr3	; BG3 is 1/4 as fast

	rtl


****************************************
*
* Handle Waterfall
*

.chkWATERFALL
	lda	z_sud1
	sta	z_sud2

	lda	z_slr1
	sta	z_slr2

	rtl




****************************************
*
* Handle Wells
*

.chkOPARWELLS
	lda	z_sud1
	sta	z_sud2

	lda	z_slr1
	sta	z_slr2

	rtl





****************************************

.exithndlrs
	rtl


*****************************************************************************
*
* Entry  :
*         z_sdir - Contains requested scroll directions
*
* Returns:
*         z_sdir - Modified based on Min / Max level boundaries
*
*****************************************************************************
 
Boundcheck:
	LAI

	lda	z_sdir	; Get scroll direction
	bit	#SCRDN	; Scrolling down?
	beq	.scup	; No  - Then goto up

	lda	z_v1	; Yes - Add scroll speed to vertical
	clc
	adc	z_sud1
	lsr
	lsr
	cmp	v_voff1max	; Compare to bottom of level distance
	bcc	.scup	; If < allow scroll

	beq	.scup	; If = allow scroll

	dec	z_sdir	; Otherwise kill down scroll bit
.scup
	lda	z_sdir	; Get scroll direction
	bit	#SCRUP	; Scrolling up?
	beq	.scright	; If not scroll up, then check right

	lda	z_v1	; Otherwise check upper limit
	sec
	sbc	z_sud1
	lsr
	lsr
	cmp	v_voff1min
	bcs	.scright	; Allow scroll if >= to top

	dec	z_sdir	; Otherwise kill up scroll bit
	dec	z_sdir
.scright
	lda	z_sdir	; Get scroll direction
	bit	#SCRRT	; Scrolling right?
	beq	.scleft	; if not scroll right, then check left

	lda	z_h1	; Otherwise, Check rightmost limit
	clc
	adc	z_slr1
	lsr
	lsr
	cmp	v_hoff1max
	bcc	.scleft	; If < allow scroll

	beq	.scleft	; If = allow scroll

	lda	z_sdir	; Otherwise kill right scroll bit
	and	#$B
	sta	z_sdir
.scleft
	lda	z_sdir	; Get scroll direction
	bit	#SCRLT	; Scrolling left?
	beq	.scend	; No  - We're done

	lda	z_h1	; Yes - Check leftmost limit
	sec
	sbc	z_slr1
	lsr
	lsr
	cmp	v_hoff1min
	bpl	.scend	; If >= left limit then allow scroll

	lda	z_sdir	; Otherwise kill left scroll bit
	and	#7
	sta	z_sdir	
.scend
	rtl		; Return Modified z_sdir scroll flags


*****************************************************************************
*
* Prescrollv is called by scrollvert to adjust background offsets 
*            will skip backgrounds based on z_skipsup bits
*            uses z_sdir to determine scroll direction
*
*****************************************************************************

Prescrollv:
	LAI

	lda	z_sdir	;scroll direction
	bit	#1	;check for down scroll (low bit)
	beq	.scrollup	;if not branch over section

	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#1	;check to skip bg1 (low bit)
	bne	.psd2	;skip to bg2

	; bg1 

	lda	z_v1
	clc		;down so add
	adc	z_sud1
	ASR
	sta	z_voff1
	pla		;pull v1
	and	#OFFSETMASK2
	sta	z_v1

	; bg2
.psd2       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#2	;check to skip bg1 (low bit)
	bne	.psd3	;branch to bg3 if bit was set

	lda	z_v2
	clc		;down so add
	adc	z_sud2
	ASR
	sta	z_voff2
	pla		;pull v2
	and	#OFFSETMASK2
	sta	z_v2

	; bg3
.psd3       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#4	;check to skip bg1 (low bit)
	bne	.scrollup	;branch to bg3 if bit was set

	lda	z_v3
	clc		;down so add
	adc	z_sud3
	ASR
	sta	z_voff3
	pla		;pull v3
	and	#OFFSETMASK2
	sta	z_v3

.scrollup   
	lda	z_sdir	;scroll direction
	bit	#2	;check for down scroll (low bit)
	beq	.scrollrite	;if not branch over section

	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#1	;check to skip bg1 (low bit)
	bne	.psu2	;skip to bg2

	; bg1

	lda	z_v1
	sec                  ;up so subtract instead of add
	sbc	z_sud1
	ASR
	sta	z_voff1
	pla                  ;pull v1
	and	#OFFSETMASK2
	sta	z_v1

	; bg2
.psu2       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#2	;check to skip bg1 (low bit)
	bne	.psu3	;skip to bg2

	lda	z_v2
	sec                  ;up so subtract instead of add
	sbc	z_sud2
	ASR
	sta	z_voff2
	pla                  ;pull v2
	and	#OFFSETMASK2
	sta	z_v2

	; bg3
.psu3       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#4	;check to skip bg1 (low bit)
	bne	.scrollrite	;skip to bg2

	lda	z_v3
	sec                  ;up so subtract instead of add
	sbc	z_sud3
	ASR
	sta	z_voff3
	pla                  ;pull v3
	and	#OFFSETMASK2
	sta	z_v3

.scrollrite 
	lda	z_sdir	;scroll direction
	bit	#4	;check for down scroll (low bit)
	beq	.scrollleft	;if not branch over section

	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#1	;check to skip bg1 (low bit)
	bne	.psr2	;skip to bg2

	;bg1

	lda	z_slr1          ;scroll left/right increment bg1
	clc
	adc	z_h1            ;add to horizontal offset
	sta	z_h1                     
	lsr
	lsr
	sta	z_hoff1         

	; bg2
.psr2       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#2	;check to skip bg1 (low bit)
	bne	.psr3	;skip to bg2

	lda	z_slr2
	clc
	adc	z_h2
	sta	z_h2
	lsr
	lsr
	sta	z_hoff2

	; bg3
.psr3       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#4	;check to skip bg1 (low bit)
	bne	.scrollleft	;skip to bg2

	lda	z_slr3
	clc
	adc	z_h3
	sta	z_h3
	lsr
	lsr
	sta	z_hoff3

.scrollleft 
	lda	z_sdir	;scroll direction
	bit	#8	;check for down scroll (low bit)
	beq	.scrollend	;if not branch over section

	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#1	;check to skip bg1 (low bit)
	bne	.psl2	;skip to bg2

	;bg1

	lda	z_h1
	sec                  ;left so subtract instead of add
	sbc	z_slr1
	sta	z_h1
	lsr
	lsr
	sta	z_hoff1

	; bg2
.psl2       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#2	;check to skip bg1 (low bit)
	bne	.psl3	;skip to bg2

	lda	z_h2            
	sec
	sbc	z_slr2
	sta	z_h2
	lsr
	lsr
	sta	z_hoff2

	; bg3
.psl3       
	lda	z_skipsup	;skip scroll update flags (per bg)
	bit	#4	;check to skip bg1 (low bit)
	bne	.scrollend	;skip to bg2

	lda	z_h3            
	sec
	sbc	z_slr3
	sta	z_h3
	lsr
	lsr
	sta	z_hoff3

.scrollend  
	rtl


*****************************************************************************
*
* vertical scroll routine is based on horizontal routine, uses
* same variables (newcol means newrow, col31 means row31, etc)
*
*****************************************************************************

Scrollvert:
	LAI

	lda	z_voff1         ;actual bg1 horizontal offset
	and	#CHARMASK16    ;character width mask
	sta	z_temp0         ;save for later

	lda	z_voff2         ;bg2 horiz offset
	and	#CHARMASK16
	sta	z_temp2         ;save

	lda	z_voff3
	and	#CHARMASK16
	sta	z_temp8

	jsl	Prescrollv      ;update other offsets


	;start bg1 column calcs and buffer loads

	lda	z_skipsup       ;check if scroll updates needed
	bit	#1
	beq	.bg1beg

	jmp	.bg2	;skip bg1
.bg1beg
	lda	z_voff1	;new offset as per prescroll
	bpl	.bg1	;negative requires special compare

	and	#OFFSETMASK	;screen wrap mask
	sta	z_voff1	;store it back

	bra	.bg1dec	;was negative, scrolled up, skip section
.bg1
	tax		;check for positive values if new row
	and	#CHARMASK16	;needed, save high bits (lose charwidth)
	cmp	z_temp0	;is it same column as old value
	php		;push answer
	txa		;transfer and save offset
	and	#OFFSETMASK
	sta	z_voff1

	plp		;restore answer
	beq	.bg2	;same value then skip new row update

	bcc	.bg1dec	;lower row, then decrement rows

	lda	#8	;otherwise we do increment
	sta	z_sup	;bg1 is being updated flag

	inc	z_newcol1	;inc new row bg1 pointer

	lda	z_newcol1
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap1	;if not at edge then skip ahead

	stz	z_newcol1	;is now at top edge of screen
.nowrap1
	inc	z_bg1col0	;increment top edge pointer

	lda	z_bg1col0
	cmp	z_bg1height	;check screen bounds
	bne	.bg12

	stz	z_bg1col0
.bg12
	inc	z_bg1col31	;increment bottom edge pointer

	lda	z_bg1col31
	cmp	z_bg1height	;check screen bounds
	bne	.set131

	stz	z_bg1col31
.set131
	lda	z_bg1width	;now transfer bottom row to buffer
	ldx	z_bg1col31	;row by row width

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	nop
	nop
	nop

	lda	#1	;set buff increment value in words
	ldx	RDMPYL	;read multiplier result

	jsl	Setbuff1

	bra	.bg2	;branch to background 2
.bg1dec
	inc	z_sup           ;bg1 updating down/up?

	dec	z_newcol1       ;decrement new column
	bpl	.nowrap2

	lda	#SCREENEDGE-1	;if wrap then set to bottom edge
	sta	z_newcol1
.nowrap2
	dec	z_bg1col0	;dec top row
	bpl	.bg13	;check bounds

	ldy	z_bg1height
	dey
	sty	z_bg1col0
.bg13
	dec	z_bg1col31	;dec bottom column
	bpl	.set10	;check bounds

	ldy	z_bg1height
	dey
	sty	z_bg1col31
.set10
	lda	z_bg1width      
	ldx	z_bg1col0

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	nop
	nop
	nop

	lda	#1	;increment value in words
	ldx	RDMPYL

	jsl	Setbuff1
.bg2
	lda	z_skipsup	;check if scroll updates needed
	bit	#2
	beq	.bg2beg

	jmp	.bg3	;skip bg2
.bg2beg
	lda	z_voff2
	bpl	.bg222

	and	#OFFSETMASK
	sta	z_voff2

	bra	.bg2dec
.bg222
	tax
	and	#CHARMASK16
	cmp	z_temp2

	php
	txa
	and	#OFFSETMASK
	sta	z_voff2

	plp
	beq	.bg3a

	bcc	.bg2dec

	lda	#16
	ora	z_sup
	sta	z_sup	;bg2 is being updated up/down?

	inc	z_newcol2

	lda	z_newcol2
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap21

	stz	z_newcol2
.nowrap21
	inc	z_bg2col0

	lda	z_bg2col0
	cmp	z_bg2height
	bne	.bg22

	stz	z_bg2col0
.bg22
	inc	z_bg2col31

	lda	z_bg2col31
	cmp	z_bg2height
	bne	.set231

	stz	z_bg2col31
.set231
	lda	z_bg2width	;transfer right column to buffer
	ldx	z_bg2col31

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	nop
	nop
	nop

	lda	#1
	ldx	RDMPYL

	jsl	Setbuff2
.bg3a
	bra	.bg3	;skip to bg3
.bg2dec
	lda	#2
	ora	z_sup
	sta	z_sup	;bg2 updating left

	dec	z_newcol2
	bpl	.nowrap22

	lda	#SCREENEDGE-1
	sta	z_newcol2
.nowrap22
	dec	z_bg2col0
	bpl	.bg23

	ldy	z_bg2height
	dey
	sty	z_bg2col0
.bg23
	dec	z_bg2col31
	bpl	.set20

	ldy	z_bg2height
	dey
	sty	z_bg2col31
.set20
	lda	z_bg2width	;transfer left column to buffer
	ldx	z_bg2col0

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	nop
	nop
	nop

	lda	#1
	ldx	RDMPYL

	jsl	Setbuff2
.bg3
	lda	z_skipsup	;check if scroll updates needed
	bit	#4
	beq	.bg3beg

	jmp	.scrollend	;skip bg3
.bg3beg
	lda	z_voff3
	bpl	.bg322

	and	#OFFSETMASK
	sta	z_voff3

	bra	.bg3dec
.bg322
	tax
	and	#CHARMASK16
	cmp	z_temp8
	php
	txa
	and	#OFFSETMASK
	sta	z_voff3

	plp
	beq	.bg3enda

	bcc	.bg3dec

	lda	#32
	ora	z_sup
	sta	z_sup	;bg3 is being updated up/down?

	inc	z_newcol3

	lda	z_newcol3
	cmp	#SCREENEDGE	;wrap around
	bne	.nowrap31

	stz	z_newcol3
.nowrap31
	inc	z_bg3col0

	lda	z_bg3col0
	cmp	z_bg3height
	bne	.bg32

	stz	z_bg3col0
.bg32
	inc	z_bg3col31

	lda	z_bg3col31
	cmp	z_bg3height
	bne	.set331

	stz	z_bg3col31
.set331
	lda	z_bg3width	;transfer right column to buffer
	ldx	z_bg3col31

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	lda	#1
	nop
	nop
	nop

	ldx	RDMPYL

	jsl	Setbuff3
.bg3enda
	bra	.scrollend	;skip to bg3
.bg3dec
	lda	#4
	ora	z_sup
	sta	z_sup	;bg3 updating left

	dec	z_newcol3
	bpl	.nowrap32

	lda	#SCREENEDGE-1
	sta	z_newcol3
.nowrap32
	dec	z_bg3col0
	bpl	.bg33

	ldy	z_bg3height
	dey
	sty	z_bg3col0
.bg33
	dec	z_bg3col31
	bpl	.set30

	ldy	z_bg3height
	dey
	sty	z_bg3col31
.set30
	lda	z_bg3width	;transfer left column to buffer
	ldx	z_bg3col0

	SAI
	sta	WRMPYA
	stx	WRMPYB
	LAI

	lda	#1
	nop
	nop
	nop

	ldx	RDMPYL

	jsl	Setbuff3
.scrollend
	jsl	Colupdatev	;figure out background vram pointers

.mainexitscroll
	lda	z_hoff1
	sta	z_levelx

	lda	z_voff1
	sta	z_levely

	rtl


*****************************************************************************
*
* figures out bg pointers in vram for the Colxfer sub
* no inputs needed except z_sup flags from Scroll
*

Colupdatev: 
	LAI

	lda	z_sup           ;flags set by scroll
	and	#9             ;bg1 mask
	beq	.bg2           ;branch to bg2 if no bits set

	lsr                  ;down? pan?

	lda	#0             ;carry is set if down? pan!
	adc	z_newcol1
	cmp	#SCREENEDGE
	bne	.nowrap1

	lda	#0
.nowrap1
	SAI             
	sta	WRMPYA
	lda	#32            ;_bg1width
	sta	WRMPYB
	LAI

	nop
	nop
	nop
	nop

	lda	RDMPYL
	clc
	adc	#BG1BASE
	sta	v_storeaddr1    ;word address is produced


***************************************

.bg2
	lda	z_sup
	and	#$12           ;bg2 mask
	beq	.bg3

	lsr
	lsr                  ;left pan?

	lda	#0
	adc	z_newcol2
	cmp	#SCREENEDGE
	bne	.nowrap2

	lda	#0
.nowrap2
	SAI
	sta	WRMPYA
	lda	#32
	sta	WRMPYB
	LAI

	nop
	nop
	nop
	nop

	lda	RDMPYL
	clc
	adc	#BG2BASE
	sta	v_storeaddr2


***************************************

.bg3
	lda	z_sup
	and	#$24           ;bg3 mask
	beq	.cupdatend

	lsr
	lsr
	lsr                  ;left pan?

	lda	#0
	adc	z_newcol3
	cmp	#SCREENEDGE
	bne	.nowrap3

	lda	#0
.nowrap3
	SAI
	sta	WRMPYA
	lda	z_bg3width
	sta	WRMPYB
	LAI

	nop
	nop
	nop
	nop

	lda	RDMPYL
	clc
	adc	#BG3BASE
	sta	v_storeaddr3
.cupdatend
	rtl

