*****************************************************************************
*
* SNDPROCS.ASM
*
* Main Music & Sound Effects Modules File
*
*****************************************************************************

NOSOUND	.equ	1	; remove sound routines from game

MUSMODULES	.equ	1	; Number of modules or 1 if exists


	
; Relative offsets for sound modules
;
; The common module is listed 1st.
	
_modaddress
	.dw	PFCOMMON_ADDRESS
	.dw	JUNGLE_ADDRESS
	.dw	SAVANNA_ADDRESS

	.dw	$FFFF	; End of List


; Bank address for sound modules
;
; The common module is listed 1st.

_modbank

	.dw	PFCOMMON_BANK 
	.dw	JUNGLE_BANK   
	.dw	SAVANNA_BANK  


*****************************************************************************

Initsounds:
	.if	NOSOUND

	rtl

	.endif


	LAI
	stz	_curmusmodule	; Init Current Music Module

	stz	_totalsongs  	; Init Total Number of Songs
	stz	_totaleffects	; Init Total Number of Effects

	stz	_cursong	; Init Current Song
	stz	_cureffect	; Init Current Sound Effect

	stz	APUPORT0	; Clear APU Ports for Transfer
	stz	APUPORT2

	jsl	Load_Driver	; Reset APU, Load & Run Driver

	rtl

*****************************************************************************
*
* Loadmmodule
*
* Inputs:
*	_curmusmodule	; Module to load
*
*
*****************************************************************************


Loadmmodule:
	.if	NOSOUND

	rtl

	.endif


	LAI
.reload
	lda	_curmusmodule	; Get requested module number to load
	beq	.noload	; Don't load the common module

	lda	#DCOM_RELOAD	; Mute driver & rst APU upload cmd
	ldy	#0

	jsl	Pass_Driver_Com	; Send command

	lda	_curmusmodule
	asl
	tax
	ldy	|_modaddress,x
	cpy	#$FFFF
	beq	.reset

	lda	|_modbank,x
	tyx

	jsl	APU_Upload


.noload
	lda	#DCOM_RUN	; Run after loading
	ldy   #0

	jsl	Pass_Driver_Com

	lda   #60000
.loop1      
	dec
	bne   .loop1

	rtl

.reset
	lda	#1
	sta	_curmusmodule

	bra	.reload

*****************************************************************************
*
* Loadmeffect
*	Load and Play a Sound Effect
*
* Inputs:
*	_cureffect	; Sound Effect to load
*
*
*****************************************************************************
LSoundeffect
           jsl   Soundeffect
           rtl

Soundeffect                        ;Doug's sound effect entry point
           sta   _cureffect        

Loadmeffect:
	.if	NOSOUND

	rtl

	.endif


	LAI

	lda	_sound
	beq	.exitlef
   
	lda	#DCOM_START_EFFECT
	ldy	_cureffect	; Effect number to play

	jsl	Pass_Driver_Com
.exitlef
	rtl

*****************************************************************************
*
* Loadmsong
*
* Inputs:
*	_cursong	; Song to load
*
*
*****************************************************************************


Loadmsong:
	.if	NOSOUND

	rtl

	.endif


	lda	#DCOM_STOP_SONG
	ldy	#$0

	jsl	Pass_Driver_Com

	lda	#DCOM_STOP_SOUND
	ldy	#$8080

	jsl	Pass_Driver_Com

	
	; Play Song
		
	LAI

	lda	#DCOM_START_SONG
	ldy   _cursong

	jsl	Pass_Driver_Com

	lda   #DCOM_FADEOUT
	ldy   #$7f01

	jsl   Pass_Driver_Com

	lda   #20000
.loop
	dec
	bne   .loop
.exitlsng
	rtl

*****************************************************************************
*
*
*****************************************************************************

	; Loads & fade-in the title song

Titlefi:
	.if	NOSOUND

	rtl

	.endif


	LAI

	lda	_music
	beq	.exitm

	stz	_cursong

	jsl	Loadmsong

	lda	#DCOM_FADEIN
	ldy	#$0101

	jsl	Pass_Driver_Com

.exitm
	rtl

*****************************************************************************
*
*
*****************************************************************************

	; Loads & fade-in level song

SMfadein:
	.if	NOSOUND
	rtl
	.endif

	LAI
	lda	_music
	beq	.exitm

	lda	_cursong	; Song number to play
	and	#$ff00
	xba
	cmp	_curmusmodule
	sta	_curmusmodule

;	beq	.startsong

;	lda	_curmusmodule
;	beq	.startsong

	lda	#DCOM_STOP_SOUND
	ldy	#$8080

	jsl	Pass_Driver_Com

	jsl	Loadmmodule

	lda	_cursong
	and	#$ff
	sta	_cursong

	sta	_songplaying
	bra	.startsong2

.startsong
	lda	_cursong
	and	#$ff
	sta	_cursong
	cmp	_songplaying
	beq	.exitm
	sta	_songplaying
.startsong2
	jsl	Loadmsong
	lda	#DCOM_FADEIN
	ldy	#$0101
           jsl	Pass_Driver_Com
.exitm
	LAI

	rtl

	; Loads & fade-in level song

*****************************************************************************
; switch to different tune (module is ASSUMED to be loaded)
*****************************************************************************
LSMswitch
	.if	NOSOUND
	rtl
	.endif

	LAI
	lda	_music
	beq	.exitm

	lda	_cursong	; Song number to play
	and	#$ff00
	xba
	cmp	_curmusmodule
	sta	_curmusmodule

	lda	#DCOM_STOP_SOUND
	ldy	#$8080

	jsl	Pass_Driver_Com

;	jsl	Loadmmodule

	lda	_cursong
	and	#$ff
	sta	_cursong

	sta	_songplaying

	jsl	Loadmsong
	lda	#DCOM_FADEIN
	ldy	#$0101
           jsl	Pass_Driver_Com
.exitm
	LAI

	rtl


*****************************************************************************
*
*
*****************************************************************************

	; Fade-out a song

SMfadeout:
	.if	NOSOUND

	rtl

	.endif


	LAI

	lda	_music
	beq	.exitm

	lda	#DCOM_FADEOUT
	ldy	#$0201

	jsl	Pass_Driver_Com

	lda	#$ffff
	sta	_songplaying
.exitm
	rtl


*****************************************************************************
*
*
*****************************************************************************

Loadminterlude:
	.if	NOSOUND

	rtl

	.endif


	; Play Song
		
	LAI

	ldy	_cursong	; Song number to play

	lda	#DCOM_INTERLUDE

	jsl	Pass_Driver_Com

	rtl

*****************************************************************************
*
*
*****************************************************************************

; Faster Fadein

SMfadein2:
	.if	NOSOUND

	rtl

	.endif


	LAI
	lda	_music
	beq   .exitm

	lda	_cursong	; Song number to play
	and	#$ff00
	xba
	cmp	_curmusmodule
	sta	_curmusmodule

	beq	.startsong

	lda	_curmusmodule
	beq	.startsong

	lda	#DCOM_STOP_SOUND
	ldy	#$8080

	jsl	Pass_Driver_Com

	jsl	Loadmmodule

	lda	_cursong
	and	#$ff
	sta	_cursong

	sta	_songplaying

	bra	.startsong2


.startsong
	lda	_cursong
	and	#$ff
	sta	_cursong

	cmp	_songplaying
	beq	.exitm

	sta	_songplaying

.startsong2

	jsl	Loadmsong

	lda	#DCOM_FADEIN
	ldy	#$0201

	jsl	Pass_Driver_Com

.exitm
	LAI

	rtl
