************************************************************************
*                                                 
*              Copyright 1992 Manley & Associates           
*                   All Rights Reserved                
*                                                 
************************************************************************
*
* File Name : 
*         SNESMAC.ASM
*
*
* Version   :
*         1.3                 22-FEB-94
*
*
* Authors   : 
*         James R. Grundell
*         Timothy C. Benham
*
*
* Audit Trail
*
*
* 22-FEB-94    TCB  Added XOR to list
* 17-DEC-93    TCB  transfer macros added, relocated Z-page reg to file
*                   called ZPGMAC.ASM (must be included in assemble),
*                   change LDC to L6A and added LDC macro
* 21-OCT-93    TCB  16-bit C,E,Z registers added, push, pull, load, &
*                   exchange macros added
* 24-FEB-92    JRG  Module Created.
*                                            
************************************************************************
*
*
* Abstract:
*
*    SNESMAC.ASM contains the following macros.
*
*    ----------------------------------------------------------------------
*
*     BLANKON  - Turns Video Blanking ON by writing to PPU register
*                and shadow variable. Preserves register bits.
*
*     BLANKOFF - Turns Video Blanking OFF by writing to PPU register
*                and shadow variable. Preserves register bits.
*
*     NMION         - Enable Vertical Blanking NMI
*
*     NMIOFF        - Disable Vertical Blanking NMI
*
*     SHORTA        - Set  8 bit Addressing mode (same as SA)
*
*     SHORTI        - Set  8 bit Indexing mode (same as SI)
*
*     LONGAI        - Set 16 bit Address & Indexing mode (same as LAI)
*
*     SHORTAI  - Set  8 bit Address & Indexing mode (same as SAI)
*
*     LA       - Set 16 bit Addressing mode
*
*     SA       - Set  8 bit Addressing mode (same as SHORTA)
*
*     LI       - Set 16 bit Indexing mode
*
*     SI       - Set  8 bit Indexing mode (same as SHORTI)
*
*     LAI      - Set 16 bit Address & Indexing mode (same as LONGAI)
*
*     SAI      - Set  8 bit Address & Indexing mode (same as SHORTAI)
*
*     BLT label     - Branch [<]  Less Than (label)
*
*     BGE label     - Branch [] Greater Than or Equal to (label)
*
*     LDSA arg - Load 8 bit accumulator with arg
*
*     LDB arg  - Load high byte of accumulator (same as LDSB)
*
*     LDSB arg - Load high byte of accumulator (same as LDB)
*
*     L6A arg  - Load 16 bit accumulator (assumes 8 bit mode active)
*
*     STSA arg - Store 8 bit low  byte of accumulator
*
*     STSB arg - Store 8 bit high byte of accumulator
*
*     STB arg  - Store high byte of accumulator
*
*     STC arg  - Store 16 bit accumulator (assumes 8 bit mode active)
*
*     INS arg  - 8 bit increment
*
*     DES arg  - 8 bit decrement
*
*     STSZ arg - 8 bit store zero
*
*     PLSA     - 8 bit pull into low  byte of accumulator
*
*     PLSB     - 8 bit pull into high byte of accumulator
*
*     PHSA     - 8 bit push of low  byte of accumulator
*
*     PHSB     - 8 bit push of high byte of accumulator
*
*     PLSX     - 8 bit pull of X
*
*     PLSY     - 8 bit pull of Y
*
*     PHSX     - 8 bit push of X
*
*     PHSY     - 8 bit push of Y
*
*     XCD      - Exchange of C & D zero page registers
*
*     XEF      - Exchange of E & F zero page registers
*
*     XAC      - Exchange of A & C 16-bit registers
*
*     XAE      - Exchange of A & E 16-bit registers
*
*     XAZ      - Exchange of A & Z 16-bit registers
*
*     XXC      - Exchange of X & C 16-bit registers
*
*     XXE      - Exchange of X & E 16-bit registers
*
*     XXZ      - Exchange of X & Z 16-bit registers
*
*     XYC      - Exchange of Y & C 16-bit registers
*
*     XYE      - Exchange of Y & E 16-bit registers
*
*     XYZ      - Exchange of Y & Z 16-bit registers
*
*     XCE      - Exchange of C & E 16-bit registers
*
*     XCZ      - Exchange of C & Z 16-bit registers
*
*     XEZ      - Exchange of E & Z 16-bit registers
*
*     LDZ arg  - Load Z register with arg
*
*     LDE arg  - Load E register with arg
*
*     LDC arg  - Load C register with arg
*
*     LDD arg  - Load D register with arg (8 bit)
*
*     LDF arg  - Load F register with arg (8 bit)
*
*     PHC      - Push 16-bit C register onto stack
*
*     PHE      - Push 16-bit E register onto stack
*
*     PHZ      - Push 16-bit Z register onto stack
*
*     TAC      - Transfer A-reg to C-reg
*
*     TAE      - Transfer A-reg to E-reg
*
*     TAZ      - Transfer A-reg to Z-reg
*
*     TXC      - Transfer X-reg to C-reg
*
*     TXE      - Transfer X-reg to E-reg
*
*     TXZ      - Transfer X-reg to Z-reg
*
*     TYC      - Transfer Y-reg to C-reg
*
*     TYE      - Transfer Y-reg to E-reg
*
*     TYZ      - Transfer Y-reg to Z-reg
*
*     TCA      - Transfer C-reg to A-reg
*
*     TEA      - Transfer E-reg to A-reg
*
*     TZA      - Transfer Z-reg to A-reg
*
*     TCX      - Transfer C-reg to X-reg
*
*     TEX      - Transfer E-reg to X-reg
*
*     TZX      - Transfer Z-reg to X-reg
*
*     TCY      - Transfer C-reg to Y-reg
*
*     TEY      - Transfer E-reg to Y-reg
*                                   
*     TZY      - Transfer Z-reg to Y-reg
*
*     XOR      - Exclusive OR
*
*     xor      - Exclusive OR
*
************************************************************************


; BLANKON
	.macro    BLANKON
		SA
		lda   #$80
		sta  INIDISP
		sta  z_inidispshd
		LA
	.endm

; BLANKOFF
	.macro    BLANKOFF
		SA
		lda   #$0f
		sta  INIDISP
		sta  z_inidispshd
		LA
	.endm

; NMION
	.macro    NMION
		SA
		lda  z_nmitimensh
		ora  #$80
		sta  z_nmitimensh
		sta  NMITIMEN
		LA
	 .endm


; NMIOFF
	.macro    NMIOFF
		SA
		lda  z_nmitimensh
		and  #$7f
		sta  z_nmitimensh
		sta  NMITIMEN
		LA
	.endm


; SHORTA
	; 8-bit mode (same as SA)

	.macro    SHORTA

		sep  #$20	; set memory select status bit
		LONGA OFF	; assembler directive

	.endm


; SHORTI
	; 8-bit indexing mode (same as SI)

	.macro    SHORTI

		sep  #$10	; set index register select status bit
		LONGI OFF	; assembler directive

	.endm


; LONGAI
	; 16-bit mode and 16-bit indexing mode (same as LAI)

	.macro    LONGAI

		rep  #$30	; clear memory and index register select bits
		LONGI ON	; assembler directive
		LONGA ON	; assembler directive

	.endm


; SHORTAI
	; 8-bit mode and 8-bit indexing mode (same as SAI)

	.macro    SHORTAI

		sep  #$30      ; set memory and index register select bits
		LONGI     OFF       ; assembler directive
		LONGA     OFF       ; assembler directive

	.endm


; LA
	; 16-bit mode

	.macro    LA

		rep  #$20      ; clear memory select status bit
		LONGA     ON        ; assembler directive

	.endm


; SA
	; 8-bit mode (same as SHORTA)

	.macro    SA

		sep  #$20      ; set memory select status bit
		LONGA     OFF       ; assembler directive

	.endm


; LI
	; 16-bit indexing mode

	.macro    LI

		rep  #$10      ; clear index register select status bit
		LONGI     ON        ; assembler directive

	.endm


; SI
	; 8-bit indexing mode (same as SHORTI)

	.macro    SI

		sep  #$10      ; set index register select status bit
		LONGI     OFF       ; assembler directive

	.endm


; LAI
	; 16-bit mode, 16-bit indexing mode (same as LONGAI)

	.macro    LAI

		rep  #$30      ; clear memory and index register select bits
		LONGI     ON        ; assembler directive
		LONGA     ON        ; assembler directive

	.endm


; SAI
	; 8-bit mode, 8-bit indexing mode (same as SHORTAI)

	.macro    SAI

		sep  #$30      ; set memory and index register select bits
		LONGI     OFF       ; assembler directive
		LONGA     OFF       ; assembler directive

	.endm


; BLT
	; branch less than <label>

	.macro    BLT  arg1

		bcc       \arg1

	.endm


; BGE
	; branch greater or equal <label>

	.macro    BGE  arg1

		bcs       \arg1

	.endm


; LDSA
	; load 8-bit accumulator

	.macro    LDSA arg1

		SA
		lda  \arg1
		LA

	.endm


; LDB
	; load high byte of accumulator (same as LDSB)

	.macro    LDB  arg1

		SA
		xba
		lda  \arg1
		xba
		LA

	.endm


; LDSB
	; load high byte of accumulator (same as LDB)

	.macro    LDSB arg1

		SA
		xba
		lda  \arg1
		xba
		LA

	.endm


; L6D
	; load 16-bit accumulator (assumes starting in 8-bit mode)

	.macro    L6D  arg1

		LA
		lda  \arg1
		SA

	.endm


; STSA
	; store 8-bit accumulator

	.macro    STSA arg1

		SA
		sta  \arg1
		LA

	.endm


; STSB
	.macro    STSB arg1

		SA
		xba
		sta  \arg1
		xba
		LA

	.endm


; STB
	; store high byte of accumulator

	.macro    STB  arg1

		SA
		xba
		sta  \arg1
		xba
		LA

	.endm


; STC
	; store 16-bit accumulator (assumes starting in 8-bit mode)

	.macro    STC  arg1

		LA
		sta  \arg1
		SA

	.endm


; INS
	; 8-bit increment

	.macro    INS  arg1

		SA
		inc  \arg1
		LA

	.endm


; DES
	; 8-bit decrement

	.macro    DES  arg1

		SA
		dec  \arg1
		LA

	.endm


; STSZ
	; 8-bit store zero

	.macro    STSZ arg1

		SA
		stz  \arg1
		LA

	.endm
	

; PLSA
	; 8-bit pull into accumulator

	.macro    PLSA

		SA
		pla
		LA

	.endm


; PLSB
	; 8-bit pull into high byte of accumulator

	.macro    PLSB

		SA
		xba
		pla
		xba
		LA

	.endm


; PHSA
	; 8-bit push of accumulator

	.macro    PHSA

		SA
		pha
		LA

	.endm


; PHSB
	; 8-bit push of high byte of accumulator

	.macro    PHSB

		SA
		xba
		pha
		xba
		LA

	.endm


; PLSX
	; 8-bit pull of x

	.macro    PLSX

		SI
		plx
		LI

	.endm


; PLSY
	; 8 bit pull of y

	.macro    PLSY

		SI
		ply  
		LI

	.endm
	

; PHSX
	; 8-bit push of x

	.macro    PHSX

		SI
		phx
		LI

	.endm


; PHSY
	; 8-bit push of y

	.macro    PHSY

		SI
		phy
		LI

	.endm

      
; XCD		
	; exchange of C & D registers

	.macro    XCD

		pha
		lda  C
		xba
		sta  C
		pla

	.endm
		

; XEF
	; exchange of E & F registers

	.macro    XEF

		pha
		lda  E
		xba
		sta  E
		pla

	.endm


; XAC
	; exchange of A & C registers

	.macro    XAC

		phx
		ldx  C
		sta  C
		txa
		plx

	.endm

; XAE
	; exchange of A & E registers

	.macro    XAE

		phx
		ldx  E
		sta  E
		txa
		plx

	.endm

; XAZ
	; exchange of A & Z registers

	.macro    XAZ

		phx
		ldx  Z
		sta  Z
		txa
		plx

	.endm


; XXC
	; exchange of X & C registers

	.macro    XXC

		pha
		lda  C
		stx  C
		tax
		pla

	.endm


; XXE
	; exchange of X & E registers

	.macro    XXE

		pha
		lda  E
		stx  E
		tax
		pla

	.endm


; XXZ
	; exchange of X & Z registers

	.macro    XXZ

		pha
		lda  Z
		stx  Z
		tax
		pla

	.endm


; XYC
	; exchange of Y & C registers

	.macro    XYC

		pha
		lda  C
		sty  C
		tay
		pla

	.endm


	; exchange of Y & E registers

	.macro    XYE

		pha
		lda  E
		sty  E
		tay
		pla

	.endm


	; exchange of Y & Z registers

	.macro    XYZ

		pha
		lda  Z
		sty  Z
		tay
		pla

	.endm


	; exchange of C & E registers

	.macro    XCE

		pha
		phx
		lda  C
		ldx  E
		sta  E
		stx  C
		plx
		pla

	.endm


	; exchange of C & Z registers

	.macro    XCZ

		pha
		phx
		lda  C
		ldx  Z
		sta  Z
		stx  C
		plx
		pla

	.endm


	; exchange of E & Z registers

	.macro    XEZ

		pha
		phx
		lda  E
		ldx  Z
		sta  Z
		stx  E
		plx
		pla

	.endm


	; load Z register with arg

	.macro    LDZ  arg

		pha
		lda  \arg
		sta  Z
		pla

	.endm


	; load E register with arg

	.macro    LDE  arg

		pha
		lda  \arg
		sta  E
		pla

	.endm


	; load C register with arg

	.macro    LDC  arg

		pha
		lda  \arg
		sta  C
		pla

	.endm


	; load D register with arg (8-bit)

	.macro    LDD  arg

		pha
		SA
		lda  \arg
		sta  C+1
		LA
		pla

	.endm


	; load F register with arg (8-bit)

	.macro    LDF  arg

		pha
		SA
		lda  \arg
		sta  E+1
		LA
		pla

	.endm


	; push C register onto stack

	.macro    PHC

		pha
		lda  C
		stx  C
		plx
		pha
		txa
		ldx  C

	.endm


	; push E register onto stack

	.macro    PHE

		pha
		lda  E
		stx  E
		plx
		pha
		txa
		ldx  E

	.endm


	; push Z register onto stack

	.macro    PHZ

		pha
		lda  Z
		stx  Z
		plx
		pha
		txa
		ldx  Z

	.endm
		
		
	; pull stack into C register 

	.macro    PLC

		sta  C
		pla
		phx
		ldx  C
		sta  C
		txa
		plx

	.endm


	; pull stack into E register

	.macro    PLE

		sta  E
		pla
		phx
		ldx  E
		sta  E
		txa
		plx

	.endm


	; pull stack into Z register

	.macro    PLZ

		sta  E
		pla
		phx
		ldx  E
		sta  E
		txa
		plx

	.endm


; TAC
	; xfer A-reg to C-reg

	.macro    TAC

		sta  C

	.endm


; TAE
	; xfer A-reg to E-reg

	.macro    TAE

		sta  E

	.endm


; TAZ
	; xfer A-reg to Z-reg

	.macro    TAZ

		sta  Z

	.endm


; TXC
	; xfer X-reg to C-reg

	.macro    TXC

		stx  C

	.endm


; TXE
	; xfer X-reg to E-reg

	.macro    TXE

		stx  E

	.endm


; TXZ
	; xfer X-reg to Z-reg

	.macro    TXZ

		stx  Z

	.endm


; TYC
	; xfer Y-reg to C-reg

	.macro    TYC

		sty  C

	.endm


; TYE
	; xfer Y-reg to E-reg

	.macro    TYE

		sty  E

	.endm


; TYZ
	; xfer Y-reg to Z-reg

	.macro    TYZ

		sty  Z

	.endm


; TCA
	; xfer C-reg to A-reg

	.macro    TCA

		lda  C

	.endm


; TEA
	; xfer E-reg to A-reg

	.macro    TEA

		lda  E

	.endm


; TZA
	; xfer Z-reg to A-reg

	.macro    TZA

		lda  Z

	.endm


; TCX
	; xfer C-reg to X-reg

	.macro    TCX

		ldx  C

	.endm


; TEX
	; xfer E-reg to X-reg

	.macro    TEX

		ldx  E

	.endm


; TZX
	; xfer Z-reg to X-reg

	.macro    TZX

		ldx  Z

	.endm


; TCY
	; xfer C-reg to Y-reg

	.macro    TCY

		ldy  C

	.endm


; TEY
	; xfer E-reg to Y-reg

	.macro    TEY

		ldy  E

	.endm


; TZY
	; xfer Z-reg to Y-reg

	.macro    TZY

		ldy  Z

	.endm


; XOR
	; exclusive OR

	.macro    XOR  arg

		eor  \arg

	.endm


; xor
	; exclusive OR

	.macro    xor  arg

		eor  \arg

	.endm


*************** End - SNESMAC.ASM

