*****************************************************************************
*
* TZAN2.ASM
*
* Main Event Loop
*
*****************************************************************************

	lda	#1
	sta	MEMSEL	; Select Hi Speed ROMs

	jsl	Topmainev

Domain:
	lda	#SNDTZCALL

	jsl	load_Module

	lda	#drumpat1_sng

	jsl	load_Song
.logos
	.if	LOGOSON

	jsl	Logos	; Yes - Lets do Logos then...

	.endif

@MainOptions
	stz	z_level	; Start at null level

	jsl	Options	; Do options page

	lda	z_level	; Return with a level?
             	bne	.startgame	; Yes - Start game then

	bra	MainOptions	; No - Do title screen again

.startgame
	lda	z_level	; Get returned level
	asl	
	sta	z_levelword	; Create word index

	lsr
	cmp	#BONUSEXT
	bne	.chk1st

	jsl	RunCredits

	lda	#PLAYDEMO	; Enable demo
	sta	v_demoplay

	jml	Domain	; Start at Title screen again

.chk1st
	lda	z_levelword
	cmp	#2	; Is this the 1st level?
	bne	.nointro	; No  - No intro stuff then

	jsl	Introscreens	; Yes - 1st level gets the intro
.nointro
	lda	z_level	; Get level
	bmi	MainOptions	; No good if negative flag
        
	jsl	Newgame         ; New Game Initialization

*****************************************************************************
*
* Loop for each level played
*

.levelloop
;	lda	#CLEARING
;	sta	z_level
;	asl	
;	sta	z_levelword

	lda	v_holdnum
	bne	.dolevel

	jsl	ShowMap	; Show location on map
@dolevel
	jsl	GlobalLevInit	; Run Designated Level

	jsl	Fadeout	; Fade to black

	lda	v_endgameyes
	bne	.noxlife
.chkhealth
	lda	#MAXHEALTH
	sta	v_health	

	lda	v_levelfail	; Check for level failure
	bne	.failevel	; Failed to complete level successfully

	lda	v_lostlife
	bne	dolevel

	lda	v_coincur
	bne	.donxtlevel

	jsl	Itemoneup

	bra	.donxtlevel
.noxlife
	ldy	#$2601	; Done with game - fade music
	lda	#DCOM_FADEOUT
	
	jsl	Pass_Driver_Command
.tmpbra
	jsl	Gameover	; Success - Game Over!!!!

	jml	Domain	; Start at Title screen again

.failevel
	ldy	#$2601	; Done with level - fade music
	lda	#DCOM_FADEOUT
	
	jsl	Pass_Driver_Command

	jsl	GameFail

	lda	#PLAYDEMO	; Enable demo
	sta	v_demoplay

	jml	Domain	; Start at Title screen again

.donxtlevel
	lda	v_holdnum
	bne	.skippasswrd

	jsl	Givepasswrd	; Give out password if successful
.skippasswrd	
	jmp	.levelloop	; loop back to next level


****************************************

Topmainev:
	LAI

	lda	#JOYPAD2ENABLE
	sta	v_JOYPAD2ENABLE

	lda	#DEBUGMODE
	sta	v_DebugMode

	lda	#MAPGUI
	sta	v_MapGUI

	lda	#PLOTCBOX
	sta	v_plotcbox

	lda	#RECDEMO
	sta	v_demorecord

	lda	#PLAYDEMO
	sta	v_demoplay

	jsl	Rndnum

	sta	z_temp0
	clc
	adc	#DEMODELAY
	sta	v_demodelay

	lda	#$6000
	sta	v_demoptr

	lda	#DEMODELAY
	sta	v_demodelay

	stz	v_debugswitch1

	lda	#16	; Largest object dimension is 16x16
	sta	v_spritedim

	stz	v_caller
	stz	v_pause2   	; Deactivate debug mode
	stz	v_contrlmap	; Set control option 'A'
	stz	v_demoreturn	; Prevents Tarzan's Yell from becoming
			; totally annoying...

	stz	v_curmusmodule	; Init Current Music Module
	stz	v_totalsongs	; Init Total Number of Songs
	stz	v_totaleffects	; Init Total Number of Effects
	stz	v_cursong	; Init Current Song
	stz	v_cureffect	; Init Current Sound Effect

	stz	APUPORT0	; Clear APU Ports for Transfer
	stz	APUPORT2

	lda	#$FFFF
	sta	MODULE

	.if MUSICOK

	jsl	Load_Driver	; Reset APU, Load & Run Driver

	.endif

	rtl


****************************************

HandleBonus:
	rtl

	ldx	z_levelword	; Get current level
	dex
	dex
	lda	d_numcoinsget,x	; Get num coins required for bonus
	cmp	v_coincur	; Get actual num coins collected
	bcc	.nobonus	; Sorry, not enough for bonus trip

	lda	v_lives
	cmp	#9
	bcs	.nolife

	inc
	sta	v_lives
.nolife:
	rtl

.nobonus

	rtl


*****************************************************************************
*
*  Initialization for starting a new game
*
*****************************************************************************

Newgame:            
	lda	#3
	sta	v_lives

	lda	#MAXHEALTH
	sta	v_health	

	lda	#6
	sta	v_numdart
	dec
	dec 
	sta	v_lastnumdart

	stz	v_totalpoints	
	stz	v_extrapoints
	stz	v_return	; Reset "return to level" flag

	stz	v_holdnum
	stz	v_leavehold

	stz	v_apebossdead

	rtl

*****************************************************************************
*
* GlobalLevInit - Common to All Levels
*
*****************************************************************************

GlobalLevInit:
	LAI

       	jsl	Preinit	; Init variables

	lda	z_level	; Get level
	asl
	sta	z_levelword

	tax
	lda	_jumpvectors,x	; And the address to it's control
	sta	z_temp0	; Stored

	ldx	z_level	; Create byte wide level index
.deltaset
	lda	_bg12nba,x	; Get BG 1&2 Char data base address
	and	#$FF

	SA
	sta	BG12NBA	; Stored
	LA

.leveljmp
	jmp	(z_temp0)	; Go to selected level driver routine


Preinit:
	jsl	Ctrlremap	; Remap controller keys

	lda	#0
	sta	z_hdmaenable

	SAI
	sta	HDMAEN	; Disable HDMA
	sta	MDMAEN	; Disable DMA
	LAI

	lda	#DXMID	; Assign Tarzans midpoint location
	sta	_dxmid

	lda	#DYMID
	sta	_dymid

	jsl	Initjoys	; Clear key debounce

	lda	v_return
	bne	.skipweaprst

	stz	v_weaponsel
	stz	v_lastweap
	inc	v_lastweap	; Select Knife
.skipweaprst
	stz	v_tileindex
	stz	v_tilenum	
	stz	v_sigtiles	

	stz	v_storeaddr1	
	stz	v_storeaddr2	
	stz	v_storeaddr3	

	stz	v_hoff1max  	
	stz	v_hoff1min  	
	stz	v_voff1max  	
	stz	v_voff1min  	

            	stz	v_togstatupdate
	stz	v_lrmovdly
	stz	v_xspeed

	stz	v_maxx	
	stz	v_minx	

	stz	v_donedark
	stz	v_felldie

	stz	v_fallmax	
	stz	v_falling
	stz	v_mistdly

	stz	v_dartactive
	stz	v_dartstep
	stz	v_bossbaron

	stz	v_bosshit	; Reset Boss "hit me" flag
	stz	v_hitmonkey
	stz	v_darthit

	stz	v_cyclface	; 
	stz	v_eolhit	; Tarzan intersects EOL icon
	stz	v_plyrxsmpl

	stz	v_frmdsp
	stz	v_frmdsposhot

	stz	v_repeat	; Key repeat counter
	stz	v_restart	; Restart level ind. - music solution
	stz	v_bakup	;

	stz	v_codedelay
	stz	v_addscroll
	stz	v_insand
	stz	v_sandlytog

	stz	v_bighitxadj
	stz	v_bighityadj
	stz	v_bighitos
	stz	v_cyclepass	; Allows partial cycling in Password if 1

	stz	v_cycledly1	; Init color cycling vars
	stz	v_cycledly2
	stz	v_cycledly3
	stz	v_cycledly4
	stz	v_cycledly5
	stz	v_colorcel1
	stz	v_colorcel2
	stz	v_colorcel3
	stz	v_colorcel4
	stz	v_colorcel5

	stz	v_1upcount	
	stz	v_1upseq
	stz	v_1upflip

	stz	v_coinstep
	stz	v_coinseq
	sta	v_coinflip

	stz	v_xyoneshot
	stz	v_whichjmp
	stz	v_disbgscroll	; Used to disable scroller

	stz	v_invinceff
	stz	v_playinvince	; No invicibility yet
	stz	v_damageflag	; No damage yet

	stz	v_vin1sot
	stz	v_fastidle
	stz	v_foekillfrm
	stz	v_knifeactive

	stz	v_lasthealth	; Force a new health plot
	stz	v_healthframe	; Start with 1st frame
	stz	v_riveron
	stz	v_levelpoints	; No points for this level yet

	stz	v_startcount	; Init start button debounce
	stz	v_startcount2

	stz	v_lastlives	; Force a new lives plot
	stz	v_intcompletion

	stz	z_sdir	; No scrolling yet
	stz	v_endlevel
	stz	v_addbossdamage

	stz	v_pause
	stz	v_pauseos
	stz	v_resettbos

	stz	v_xfercount
	stz	v_animcount
	stz	v_idle4pkdly

	stz	v_donenifsfx
	stz	v_donedrtsfx
	stz	v_doneroar

	stz	v_tzpaltogl
	stz	v_bobwater

	stz	v_pitopen
	stz	v_levelfail

	stz	v_playeroffx1
	stz	v_playeroffy1

	stz	v_tilestat	; Clear tile status
	stz	v_killdemorec

	stz	v_endgameyes

	lda	v_numdart
	cmp	#6
	bcs	.skipfill

	lda	#6
	sta	v_numdart
.skipfill
	dec
	dec 
	sta	v_lastnumdart

	lda	#1
	sta	v_waveroll

	lda	#4
	sta	v_sudebounce	; scroller debounce for fall death

	ldx	#30	; Init scratch RAM
.loop1
	stz	v_scratch,x
	dex
	dex
	bpl	.loop1

	ldx	#72	; Init player variables
.loop2
	stz	v_playertype,x
	dex
	dex
	bpl	.loop2

	ldx	#768
.loop3
	stz	ATTRFRAME+$80,x
	dex
	dex
	bpl	.loop3

	lda	#2	; Tarzan takes a hit
	sta	v_shimertimer

	lda	#MAXSCROLL
	sta	v_maxscroll

	lda	#200
	sta	v_idlecount	; 

	rtl


*****************************************************************************
*
* Init Specific Level
* 
*****************************************************************************

InitLevel:
	lda	v_restart
	bne	.nomusrst

	jsl	MusicInit	; load music & sound effects for level

.nomusrst
	SAI

	lda	#0	; Disable HDMA
	sta	z_hdmaenable
	sta	HDMAEN
	sta	MDMAEN

	LAI

	; ----------------------------------------------------------
	; This code fixes the problem of trashed BG2 screens during
	; level starts only in the sense that the screens are now
	; clean as opposed to full of garbage. The problem probably
	; stems from the scroll initializer.

	lda	#BG2BASE

	jsl	Clrbg

	lda	#BG2BASE2B

	jsl	Clrbg

	; ----------------------------------------------------------

	jsl	Copyvram	; download BG chars, palettes, etc.

	lda	z_level	; Get byte level index
	and	#$FF
	tay

	lda	_bgenables,y	; Get BG enable flags
	and	#$FF	; mask
	sta	_bgenable

	jsl	Copylevel	; Copy level bg data to ram

	jsl	Scrollinit	; Init scroll data pointers, etc.

	jsl	Modeinit	; Init Display Mode
 
         
	LAI

	ldx	z_levelword	; Get word level index

	lda	z_hoff1	; Store copy
	sta	z_levelx

	lda	z_voff1	; Store copy
	sta	z_levely

	lda	_dxstart,x	; Starting x position for player
	asl
	asl
	sta	v_playerx	; Stored as 1/4 pixel scaling

	lda	_dystart,x	; Starting y position for player
	asl
	asl
	sta	v_playery	; Stored as 1/4 pixel scaling

	stz	v_lostlife
	stz	v_restart

	NMION

	jsl	Preload	; Load & Init Everything, Then Fadein

	lda	#1	; Enable Dynamic Fade in
	sta	z_colfadeip
	stz	z_colfadeos

	stz	v_selectcount2
	stz	v_priestdead

	stz	v_bosslevel

	lda	z_level
	and	#$FF
	cmp	#BEACH
	beq	.setboss

	cmp	#JUNGLE4
	beq	.setboss

	cmp	#CLEARING
	beq	.setboss

	cmp	#SHAMONBOSS
	beq	.setboss

	lda	#0
.setboss
	sta	v_bosslevel
	
	rtl

*****************************************************************************
*
* Level Call Stub
* 
*****************************************************************************

NoLevelYet:
	jml	Endit


*****************************************************************************
*
* Standard Level Call
* 
*****************************************************************************

Tzanlevel:
	jsl	InitLevel	; Level init

.levelloop
	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	ora	v_levelfail	; Failed level alltogether?
	bne	.endit	; Yes - Either one

	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* Beach Level Call - Boss
* 
*****************************************************************************

Beachlevel:
	jsl	InitLevel	; Level init

.levelloop
	jsl	ProcessLevel

	lda	v_bossbaron
	eor	#1
	and	v_lostlife	; Lost life in level?
	ora	v_levelfail	; Failed level alltogether?
	bne	.endit	; Yes - Either one

	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	lda	v_playerx
	lsr
	lsr
	cmp	#$980
	bcc	.levelloop

	lda	#1
	sta	v_bossbaron

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* Ape Level Call - Boss
* 
*****************************************************************************

Apelevel:
	jsl	InitLevel	; Level init

	lda	#1
	sta	v_bossbaron

.levelloop
	jsl	ProcessLevel

	lda	v_felldie
	bne	.endit
	
	lda	v_levelfail	; Failed level alltogether?
	bne	.endit	; Yes - 

	lda	v_bossdead	; Is Boss dead?
	bpl	.skiplifeck	; No  - Don't exit-die until boss dead.

	lda	#1
	sta	v_apebossdead

	lda	v_lostlife	; Yes -
	bne	.endit
.skiplifeck
	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* Swamp Level Call
* 
*****************************************************************************

Swamplevel:
	jsl	InitLevel	; Level init

.levelloop
	jsl	ProcessLevel

	stz	z_slr3

	lda	z_slr1
	beq	.setbg3rate

	sta	z_slr3	; Set scroll speed
.setbg3rate
	inc	z_slr3
	inc	z_slr3

	lda	#SCRRT
	ora	z_sdir
	sta	z_sdir	; Set direction

	lda	v_lostlife	; Lost life in level?
	bne	.endit

	lda	v_levelfail
	bne	.endit
	
	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* A Day on the Beach - OLD!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
* 
*****************************************************************************

Beachlevel1:
	jsl	InitLevel	; Level init

	ldx	eq_rivdmalen+1
	lda	#0

	SA
.initloop
	sta	v_riveron,x
	dex
	bpl	.initloop

	LA

	stz	v_hdmatblptr	; 0th entry in ROM table
	ldy	d_Dnrivbg1off	; Point to HDMA table entry

	jsl	Beachdmasetup	; Set up HDMA table in RAM

	jsl	Waitfornmi	; Guns off

	ldx	#0	; Index into tables

	jsl	Beachdma
            
	stz	v_dnrivincval	; Init river scroll increment

	SAI

	lda	z_hdmaenable	; Enable HDMA
	sta	HDMAEN

	LAI

	jsl	Fadein

.levelloop
	jsl	ProcessLevel

	lda	v_bossbaron
	eor	#1
	and	v_lostlife	; Lost life in level?
	ora	v_levelfail	; Failed level alltogether?
	bne	.endit	; Yes - Either one

	jsl	Beachscroll1

	jsl	Corecalls	; Do Major work....

	dec	v_wavcount
	bpl	.skipcnt

	lda	#4
	sta	v_wavcount

	inc	v_dnrivincval	; Init river flow var

.skipcnt
	jsl	Beachoffset	; Move computed values for HDMA to see

	lda	v_eolhit
	bne	.endit

	lda	v_playerx
	lsr
	lsr
	cmp	#$980
	bcc	.levelloop

	lda	#1
	sta	v_bossbaron

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* A Sailing Ship
* 
*****************************************************************************

Shiplevel:
	jsl	InitLevel	; Level init

	stz	v_sky3dly

	lda	v_holdnum	; Returning from hold A?
	beq	.skipold	; No  - neither hold B...

	lda	#$36C	; Otherwise, preload y coordinate
	asl
	asl
	sta	v_playery	; Stored as 1/4 pixel scaling

	lda	v_holdnum	; Get hold number
	cmp	#1	; Is it hold A?
	bne	.notholda	; No  - it's hold B

	lda	#$633	; Otherwise, load hold A ret x,y
	asl
	asl
	sta	v_playerx	; Stored as 1/4 pixel scaling

	bra	.newinit
.notholda
	lda	#$60	; Load hold B ret x,y
	asl
	asl
	sta	v_playerx	; Stored as 1/4 pixel scaling
.newinit
            	stz	v_leavehold
	stz	v_xfercount

;	jsl	Statupdate	; Initial Status
;
;	jsl	Doplayer	; Initial Tarzan
;
;	jsl	Dofoes	; Update Foe AI Logic
;
;	jsl	Resetattr	; Reset Objects
;
;	jsl	PlotStats	; Plot the current health & lives
;
;	jsl	PlotItems	; Plot powerups & coins
;	
;	jsl	PlotFrontFoes	; As it says
;
;	jsl	PlotColBox	; Plot collision box around Tarzan
;
;	jsl	PlotPlayer	; Plot Tarzan
;
;	jsl	PlotBackFoes	; As it says
;
;	jsl	PlotHoles	; Plot Holes
;
;	jsl	Createoam	; Create OAM from ATTR
;
;  	jsl	Sendobj	; Send over the OAM
;
;	jsl	Senddata	; Xfer the requested sprite images
;
;	jsl	Scroll
;

.skipold
	jsl	Fadein
.levelloop
	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	beq	.chkfail

	jmp	.endit
.chkfail
	lda	v_levelfail
	beq	.cont

	jmp	.endit
.cont
	jsl	Corecalls	; Do Major work....

;	jsl	Scrollsky

	lda	v_eolhit
	bne	.endit1

	lda	v_playery
	lsr
	lsr
	sta	z_temp2	; Y

	lda	v_playerx
	lsr
	lsr
	sta	z_temp0	; X

	cmp	#$0024	; Jump off stern?
	bcc	.nothold	; Yes -

	cmp	#.holdaleftsd	; No  - Possible he in hold B?
	bcc	.levelloop	; No  - On deck

	cmp	#.holdarigtsd	; In hold B?
	bcs	.notholdb	; No  - On deck or in hold A
			; Yes - In holdb horizontal
	lda	z_temp2	; Get Y then
	cmp	#.holdbottom	; Fallen as far as can go?
	bcc	.levelloop	; No  - Keep processing


	; **********************

	lda	#2
	sta	v_holdnum

	lda	#SHIPHOLDB	; Do shiphold B
	sta	z_level
	asl
	sta	z_levelword

	jsl	Fadeout	; Fade to black

	jml	dolevel

	; **********************

.nothold

.endit1
	jml	Endit


.notholdb
	lda	z_temp0	; Get X
	cmp	#.holdbleftsd	; Off to left?
	bcc	.lloop	; Yes -
	
	cmp	#.holdbrigtsd	; No  - Within the right side?	
	bcs	.lloop	; No  - On deck

	lda	z_temp2	; Get Y then
	cmp	#.holdbottom	; Past the bottom?
	bcs	.doholda

.lloop
	jmp	.levelloop	; Yes - So do it

.doholda

	; **********************

	lda	#1
	sta	v_holdnum

	lda	#SHIPHOLDA
	sta	z_level
	asl
	sta	z_levelword

	jsl	Fadeout	; Fade to black

	jml	dolevel

	; **********************

	lda	v_eolhit
	bne	.endit

	jmp	.levelloop
.endit
	jml	Endit



.holdaleftsd	.equ	$07E
.holdarigtsd	.equ	$111

.holdbleftsd	.equ	$5AD
.holdbrigtsd	.equ	$601

.holdbottom	.equ	$3C0

; ---------------------------------------------------------------------------

Scrollsky:
	dec	v_sky3dly
	bpl	.skiptime

	lda	#2
	sta	v_sky3dly

	inc	z_hoff3
.skiptime
	rtl


*****************************************************************************
*
* Shiphold Level Call
* 
*****************************************************************************

Holdslevel:
	jsl	InitLevel	; Level init

	lda	#100
	sta	v_timefall


	; Init left side of mask hdma

	SAI

	lda	#$0
	sta	DMA2INC	; HDMA parameter

	lda	#$26	; Address $2126 = WH0
	sta	DMA2DEST

	lda	#0	; PPU registers are in bank 0
	sta	DMA2BANK	; Bank of dest address

	lda	#0	; Data Table Address
	sta	DMA2A1B	; Address Bank

	lda	>d_holdsllow	; Data Table Address
	sta	DMA2A1L	; Address Low

	lda	>d_holdslhigh	; Data Table Address
	sta	DMA2A1H	; Address High

	lda	#$4	; Enable HDMA Channel 2
	sta	z_hdmaenable

	LAI

	; Init right side of mask hdma

	SAI

	lda	#$0
	sta	DMA3INC	; HDMA parameter

	lda	#$27	; Address $2127 = WH1
	sta	DMA3DEST

	lda	#0	; PPU registers are in bank 0
	sta	DMA3BANK	; Bank of dest address

	lda	#0	; Data Table Address
	sta	DMA3A1B	; Address Bank

	lda	>d_holdsrlow	; Data Table Address
	sta	DMA3A1L	; Address Low

	lda	>d_holdsrhigh	; Data Table Address
	sta	DMA3A1H	; Address High

	lda	#$8	; Enable HDMA Channel 3
	ora	z_hdmaenable
	sta	z_hdmaenable

	LAI

	jsl	Fadein

	jsl	Waitfornmi

	lda	z_hdmaenable

	SA
	sta	HDMAEN
	LA

.levelloop

	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	bne	.endit

	lda	v_levelfail
	bne	.endit
	
	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	dec	v_timefall
	bpl	.levelloop

	stz	v_timefall

	lda	v_playery
	cmp	#$21D<<2
	bcc	.chkexit

	bra	.levelloop
.endit
	lda	#1
	sta	v_leavehold
	sta	v_return

	stz	z_hdmaenable

	SA
	stz	HDMAEN
	LA

	jml	Endit

.chkexit
	lda	v_playerx
	lsr
	lsr
	sta	z_temp0	; X

	cmp	#.holdaleftsd	; Possible he jump out hold B?
	bcc	.levelloop	; No  - Didn't make the window

	cmp	#.holdarigtsd	; Maybe he's going out A
	bcs	.notholdb	; Yes - Maybe, go check

	lda	#2
	sta	v_holdnum

	bra	.endit

.notholdb
	lda	z_temp0	; Get X
	cmp	#.holdbleftsd	; Off to left?
	bcc	.levelloop	; Yes -
	
	cmp	#.holdbrigtsd	; No  - Within the right side?	
	bcs	.levelloop	; No  - On deck

	lda	#1
	sta	v_holdnum

	jmp	.endit



.holdaleftsd	.equ	$050
.holdarigtsd	.equ	$0E3

.holdbleftsd	.equ	$518
.holdbrigtsd	.equ	$5A0


*****************************************************************************
*
* Swimming up stream
* 
*****************************************************************************

Downriver:
	jsl	InitLevel	; Level init
 
	lda	z_voff1	; Get current vertical position
	sta	v_dnrivpos	; Sample it

	stz	v_hdmatblptr	; 0th entry in ROM table
	ldy	d_Dnrivbg1off	; Point to HDMA table entry

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	jsl	Waitfornmi	; Guns off

	ldx	#0	; Index into tables

	jsl	Dnrivrdma	; Program HDMA
            
	stz	v_dnrivincval	; Init river scroll increment

	lda	#1
	sta	v_riveron	; Enable HDMA type I table processing

	SAI

	lda	z_hdmaenable	; Enable HDMA
	sta	HDMAEN

	LAI

	jsl	Fadein

.levelloop
	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	bne	.endit

	lda	v_levelfail
	bne	.endit

	jsl	Dnrivscroll	; Scroll river

	jsl	Corecalls	; Do Major work....

	jsr	Domovriver	; Process HDMA type I table entries

	inc	v_dnrivincval	; Init river flow var
	inc	v_dnrivincval	; Init river flow var

	jsl	Dnrivroffset	; Move computed values for HDMA to see

	lda	v_riveron	; 
	bne	.levelloop

	lda	z_voff1	; Get current vertical position
	cmp	v_dnrivpos	; Test where river disappeared
	bcc	.reactivate	; Start doing river hdma again

	bra	.levelloop	; River still above screen

.reactivate
	inc	v_riveron

	lda	#9
	tay

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	lda	#1	; Just 1 line on expansion
	ldx	#9

	SA
	sta	v_dnrivtabl,x	; Store new NOL in top entry
	LA

	lda	v_eolhit
	bne	.endit

	bra	.levelloop
.endit
	jml	Endit


****************************************

Domovriver:
	lda	v_riveron	; Is HDMA on?
	bne	.rivon	; Yes - Then do process
.rivonx
	rts

.rivon
	lda	z_voff1	; Get current vertical position
	cmp	v_dnrivpos	; Compare with latest sample
	beq	.rivonx	; Start processing if river visable

	bcs	.movindn	; Moving down if voff1 increased

	lda	#$99	; Big value to fail compare
	sta	z_temp4	; Preload
   
	lda	v_hdmatblptr	; Get current index to table head
	beq	.nolimitchk	; If top of table, then no check

	asl		; Otherwise, create index
	tax
	lda	Dnrivtblval1,x	; Get original table entry
	sta	z_temp4
	
.nolimitchk
	lda	v_dnrivpos	; Get latest sample
	sec		; Want to add lines cause going up
	sbc	z_voff1	; Get deviation

	SA 		; NOL = Num of Lines entry
	clc
	adc	v_dnrivtabl	; Add deviation to top entry NOL
	cmp	z_temp4	; Greater than original NOL value?
	bcc	.noxpand	; No  - Less than

	beq	.noxpand	; No  - Equal to

	jmp	.expandtbl	; Yes - Need to expand table
.noxpand
	sta	v_dnrivtabl	; Store new NOL in top entry
	LA

	lda	z_voff1	; Get new vertical position
	sta	v_dnrivpos	; Sample it

	bra	.endmovriver	; Now go scroll river

.movindn			; Tarzan going down
	lda	z_voff1	; Compute diff between sample...
	sec		; ...and current position
	sbc	v_dnrivpos
	sta	z_temp4

	SA
	lda	v_dnrivtabl	; Get num lines value from table entry
	sec
	sbc	z_temp4	; Apply difference
	beq	.compresstbl	; Compress table if zero
	bmi	.compresstbl	; Compress table if negative

	sta	v_dnrivtabl	; Store new number in top entry
	LA

	lda	z_voff1	; Get new vertical position
	sta	v_dnrivpos	; Sample it

	bra	.endmovriver	; Now go scroll river

.compresstbl
	LA

	lda	v_hdmatblptr	; Get current index to table head
	inc		; Next entry
	sta	v_hdmatblptr	; Store it
	cmp	#5	; River off top of screen?
	bcs	.disablehdma	; Yes - disable HDMA

	asl		; Create word index
	tax
	lda	d_Dnrivbg1off,x	; Get fixed table entry index
	tay

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	lda	z_voff1	; Get new vertical position
	sta	v_dnrivpos	; Sample it

.endmovriver
	rts

.expandtbl
	lda	v_hdmatblptr	; Get current index to table head
	beq	.endmovriver	; We're at top now

	dec		; Prev entry
	sta	v_hdmatblptr	; Store it
	asl
	tax
	lda	d_Dnrivbg1off,x	; Get fixed table entry index
	tay

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	lda	#1	; Just 1 line on expansion

	SA
	sta	v_dnrivtabl	; Store new NOL in top entry
	LA

	rts

.disablehdma
	lda	z_voff1	; Get new vertical position
	sta	v_dnrivpos	; Sample it

	stz	v_riveron	; Disable this procedure

	SA
	stz	v_dnrivtabl	; Disable table with a 0 NOL
	LA

	rts


*****************************************************************************
*
* Swimming up stream
* 
*****************************************************************************

Upriver:
	jsl	InitLevel	; Level init
 
	lda	z_voff1	; Get current vertical position
	sta	v_dnrivpos	; Sample it

	stz	v_hdmatblptr	; 0th entry in ROM table
	ldy	d_Dnrivbg1off	; Point to HDMA table entry

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	jsl	Waitfornmi	; Guns off

	ldx	#0	; Index into tables

	jsl	Dnrivrdma	; Program HDMA
            
	stz	v_dnrivincval	; Init river scroll increment

	lda	#1
	sta	v_riveron	; Enable HDMA type I table processing

	SAI

	lda	z_hdmaenable	; Enable HDMA
	sta	HDMAEN

	LAI

	jsl	Fadein

.levelloop
	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	bne	.endit

	lda	v_levelfail
	bne	.endit

	lda	v_eolhit
	bne	.endit

	jsl	Dnrivscroll	; Scroll river

	jsl	Corecalls	; Do Major work....

	inc	v_dnrivincval	; Init river flow var
	inc	v_dnrivincval	; Init river flow var

	jsl	Dnrivroffset	; Move computed values for HDMA to see

	lda	v_riveron	; 
	bne	.levelloop

	lda	z_voff1	; Get current vertical position
	cmp	v_dnrivpos	; Test where river disappeared
	bcc	.reactivate	; Start doing river hdma again

	bra	.levelloop	; River still above screen

.reactivate
	inc	v_riveron

	lda	#9
	tay

	jsl	Dnrivhdmasetup	; Set up HDMA table in RAM

	lda	#1	; Just 1 line on expansion
	ldx	#9

	SA
	sta	v_dnrivtabl,x	; Store new NOL in top entry
	LA

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* ]Waterfall
* 
*****************************************************************************

Watfallevel:
	lda	#$FFFF
	sta	v_playertype

	jsl	InitLevel	; Level init

;	jsl	Fadein

	lda	#1
	sta	v_donedark

	lda	v_playery
	pha
	pha

.loop1
	pla
	clc
	adc	#80
	sta	v_playery

	pla
	pha
	sta	z_temp0

	lda	v_playery
	pha
	sec
	sbc	z_temp0

	cmp	#2600
	php

	jsl	ProcessLevel

	jsl	Corecalls1

	plp
	bcc	.loop1

.loop2
	pla	          	; current
	sec
	sbc	#80
	sta	v_playery

	jsl	ProcessLevel

	jsl	Corecalls1

	pla		; original
	pha
	ldx	v_playery	; current
	phx
	cmp	v_playery
	bcc	.loop2
	
	pla		; current
	pla		; original

	lda	#21
	sta	v_donedark
.loop3
	jsl	ProcessLevel

	jsl	Corecalls1

	dec	v_donedark
	bne	.loop3

	stz	v_donedark

	jsl	Fadein


.levelloop

	jsl	ProcessLevel

	lda	v_lostlife	; Lost life in level?
	bne	.endit

	lda	v_levelfail
	bne	.endit
	
	jsl	Corecalls	; Do Major work....

	lda	v_eolhit
	bne	.endit

	lda	v_intcompletion
	bne	.endit

	bra	.levelloop
.endit
	jml	Endit


*****************************************************************************
*
* Kill the Preist!
* 
*****************************************************************************

DoShamonBoss:
	lda	#DYMID+47
	sta	_dymid

	jsl	InitLevel	; Level init

	stz	v_xpostate

.levelloop
	jsl	ProcessLevel

	lda	v_levelfail
	bne	.endit1

	lda	v_bossdead
	bpl	.skiplifeck

	lda	v_lostlife
	bne	.endit
.skiplifeck
	jsr	.updatexpos	; Adjust X scroll point

	jsl	Corecalls	; Do Major work....

	lda	v_priestdead
	bne	.endit

	bra	.levelloop
.endit
	lda	v_playerx
	cmp	#$360<<2
	bcs	.endit1

	lda	#JOYPADRIGHT
	sta	z_joy1

	sta	v_endgameyes

	bra	.levelloop

.xposjmptbl
	.dw	.cmidtoleft	; 0
	.dw	.cmidtorght	; 1
	.dw	.clefttomid	; 2
	.dw	.crghttomid	; 3
	.dw	.cleft2rght	; 4
	.dw	.crght2left	; 5
	.dw	.monitor   	; 6

.updatexpos
	lda	v_xpostate
	asl
	tax

	jmp	(.xposjmptbl,x)

.endit1
	jml	Endit


; --------------------------------------

.cmidtoleft
	lda	v_playerx
	cmp	#$1D0<<2
	bcc	.xitcxm

;	sta	v_plyrxsmpl
.changexmid
	lda	#1
	sta	v_bossbaron

;	lda	_dxmid
;	cmp	#64
;	bcs	.adjxmidm2l

	lda	#6
	sta	v_xpostate

	rts
.adjxmidm2l
;	sec
;	sbc	#1
;	sta	_dxmid
.xitcxm
	rts

.cmidtorght
	rts
.clefttomid
	rts
.crghttomid
	rts
.cleft2rght
	rts
.crght2left
	rts
.monitor
	lda	v_playerx	; Get current X
	cmp	v_plyrxsmpl	; Compare to last sample
	beq	.xitmonitor	; If =, then no scroll pos. update

;	sta	v_plyrxsmpl	; Otherwise, take a new sample

	bcc	.chklftlim	; If < then chk lower limit

	lda	_dxmid	; Otherwise, chk upper limit
	cmp	#192	; Right side of screen?
	bcs	.xitmonitor	; Yes - Then we're outta here

			; No  - Adjust dxmid
	bra	.xitmonitor

.chklftlim
	lda	_dxmid
	cmp	#64
	bcs	.adjxmidm2l


.xitmonitor
	rts


*****************************************************************************
*

Modeinit:
	bra	.findmode


.jmptbl
	.dw	.normal	; Null Level
	.dw	.normal	; Jungle 1
	.dw	.normal	; Tarzans Home
	.dw	.normal	; Apes Clearing
	.dw	.setswamp	; Swamp

	.dw	.normal	; Beach
	.dw	.normal	; Beach-Bay
	.dw	.normal	; Reef
	.dw	.normal	; Ship
	.dw	.holds	; Ship Hold A

	.dw	.holds	; Ship Hold B
	.dw	.normal	; Jungle 3
	.dw	.normal	; Downriver
	.dw	.normal	; Jungle 4
	.dw	.normal	; UpRiver

	.dw	.normal	; Waterfall
	.dw	.setcaves	; Caverns
	.dw	.normal	; Opar - Street
	.dw	.normal	; Opar - Wells
	.dw	.normal	; Opar - Temple

	.dw	.normal	; Opar - Shamon Boss

	.dw	.normal	; Bonus Level - Exterior
	.dw	.normal	; Bonus Level - Interior

.findmode
	lda	z_level
	and	#$FF
	tay
	ldx	z_levelword

	jmp	(.jmptbl,x)

.normal
	SAI

	lda	_bgmodes,y	; Get modes byte for this level
	sta	BGMODE	; Make it so

	lda	#BG1BASE/256
	ora	_scsize1,y
	sta	BG1SC

	lda	#BG2BASE/256
	ora	_scsize2,y
	sta	BG2SC

	lda	#BG3BASE/256
	ora	_scsize3,y
	sta	BG3SC

	lda	#BG3CHAR >> 12
	sta	BG34NBA

	lda	#$03
	sta	OBJSEL

	lda	#$30
	sta	CGSWSEL

	stz	CGADSUB	; enable add/sub
	stz	WOBJSEL

	lda	#$E0
	sta	COLDATA

	stz	WH0
	stz	WH1

	stz	W12SEL	; Disable Windows on BG1 & 2
	stz	W34SEL	; Disable Windows on BG3 & 4

	lda	_bgenables,y
	sta	_bgenable
	sta	TM
	sta	TMW
	sta	TS
	sta	TSW

	LAI

	rtl


*****************************************************************************

.setswamp
	SAI

	lda	_bgmodes,y	; Get modes byte for this level
	sta	BGMODE	; Make it so

	lda	#BG1BASE/256
	ora	_scsize1,y
	sta	BG1SC

	lda	#BG2BASE/256
	ora	_scsize2,y
	sta	BG2SC

	lda	#BG3BASE/256
;	ora	_scsize3,y
	sta	BG3SC

	lda	#BG3CHAR >> 12
	sta	BG34NBA

	lda	#$03
	sta	OBJSEL

	lda	#$13
	sta	TM
	sta	TMW

	lda	#$04
	sta	TS
	sta	TSW

	lda	#$11
	sta	CGADSUB	

	lda	#$02
	sta	CGSWSEL		

	lda	#$E0
	sta	COLDATA

	LAI

	rtl


	
*****************************************************************************

.setcaves
	SAI

	ldy	z_level	; Get byte level index
	lda	_bgmodes,y	; Get modes byte for this level
	sta	BGMODE	; Make it so

	lda	#BG1BASE/256
	ora	_scsize1,y
	sta	BG1SC

	lda	#BG2BASE/256
	ora	_scsize2,y
	sta	BG2SC

	lda	#BG3BASE/256
	ora	_scsize3,y
	sta	BG3SC

	lda	#BG3CHAR >> 12
	sta	BG34NBA

	lda	#$03
	sta	OBJSEL

	; set up color window

	lda	#$30
	sta	CGSWSEL

	stz	CGADSUB	; enable add/sub
	stz	WOBJSEL

	lda	#$E0
	sta	COLDATA

	stz	WH0
	stz	WH1

	stz	W34SEL	; Disable Windows on BG3

	lda	_bgenables,y
	sta	_bgenable
	sta	TM
	sta	TMW
	sta	TS
	sta	TSW

	lda	#$10
	sta	TM
	sta	TMW

	lda	#$03
	sta	TS
	sta	TSW

	lda	#$02
	sta	CGSWSEL		

	lda	#$30
	sta	CGADSUB	

	lda	#$E0
	sta	COLDATA

	lda	#$0
	sta	WOBJSEL

	lda	#0
	sta	WH0

	lda	#$FE
	sta	WH1
			      
	lda	#$31	      
	sta	BGMODE	      

	LAI

	rtl


*****************************************************************************

.holds
	SAI

	lda	_bgmodes,y	; Get modes byte for this level
	sta	BGMODE	; Make it so

	lda	#BG1BASE/256
	ora	_scsize1,y
	sta	BG1SC

	lda	#BG2BASE/256
	ora	_scsize2,y
	sta	BG2SC

	lda	#BG3BASE/256
	ora	_scsize3,y
	sta	BG3SC

	lda	#BG3CHAR >> 12
	sta	BG34NBA

	lda	#$03
	sta	OBJSEL

	lda	#$30
	sta	CGSWSEL

	stz	CGADSUB	; enable add/sub
	stz	WOBJSEL

	lda	#$E0
	sta	COLDATA

	stz	WH0
	stz	WH1

	stz	W12SEL	; Disable Windows on BG1 & 2
	stz	W34SEL	; Disable Windows on BG3 & 4

	lda	_bgenables,y
	sta	_bgenable
	sta	TM
	sta	TMW
	sta	TS
	sta	TSW

	lda	#$2
	sta	WOBJSEL	; Enable Window 1 on Objects

	lda	#0	; Set left side to 0
	sta	WH0
	sta	WH1

	LAI

	rtl


*****************************************************************************
*
* Core Routines Called by all Levels
*
*****************************************************************************

CCycletbl:
	.dw	-1	; Null Level
	.dw	-1	; Jungle 1
	.dw	-1	; Tarzans Home
	.dw	-1	; Apes Clearing
	.dw	-1	; Swamp

	.dw	PJCyclewaves	; Beach
	.dw	-1	; Beach-Bay
	.dw	PJCyclebaysurf	; Reef
	.dw	PJCycleship	; Ship
	.dw	PJCycleshphld	; Ship Hold A

	.dw	PJCycleshphld	; Ship Hold B
	.dw	-1	; Jungle 3
	.dw	PJCyclednriv	; Downriver
	.dw	-1	; Jungle 4
	.dw	PJCycleupriv	; UpRiver

	.dw	PJCyclefalls	; Waterfall
	.dw	PJCyclelamp	; Caverns
	.dw	-1	; Opar - Street
	.dw	-1	; Opar - Wells
	.dw	PJCycleflames	; Opar - Temple

	.dw	PJCycleflames	; Opar - Shamon Boss


*****************************************************************************

Corecalls:
	jsl	CheckPause	; Check for user pause
Corecalls1:
	jsl	PlotPause	; If in Pause mode

	jsl	PlotStats	; If debug start pause off

	jsl	Plotbossbar

	jsl	PlotXYs	; Debug conditional
	
	jsl	PlotItems	; Plot Coins
	
	jsl	PlotFrontFoes	; As it says

	jsl	PlotColBox	; Plot collision box around Tarzan

	jsl	PlotPlayer	; Plot Tarzan

	jsl	PlotBackFoes	; As it says

	jsl	PlotHoles	; Plot Holes

	jsl	Createoam	; Create OAM from ATTR

	jsl	Waitfornmi	; Wait for vblank

	jsl	Colxfer	; Transfer columns

	jsl	Offsetset	; Set offsets in SNES registers

  	jsl	Sendobj	; Send over the OAM

	jsl	Senddata	; Xfer the requested sprite images

	jsl	Cycleeol

	jsl	Palettehit	; Shimmer palette in/out red

	jsl	DoAmbients	; handle ambient sound effects

	lda	z_level
	and	#$FF
	cmp	#CAVERNS
	beq	.skipbgset

	cmp	#SWAMP
	beq	.skipbgset

	lda	_bgenable

	SA
	sta	TM
	sta	TMW
	sta	TS
	sta	TSW
	LA

.skipbgset
	ldx	z_levelword
	lda	CCycletbl,x
	cmp	#-1
	beq	.skipcycles

	jmp	(CCycletbl,x)
.skipcycles
	rtl

PJCyclewaves
	jsl	Cyclewaves

	rtl

PJCyclebaysurf
	jsl	Cyclebaysurf

	rtl

PJCycleship
	jsl	Cycleship	

	rtl

PJCycleshphld
	rtl
	jsl	Cycleshphld	

	rtl

PJCyclednriv
	jsl	Cyclednriv

	rtl

PJCycleupriv
	jsl	Cycleupriv

	rtl

PJCyclefalls
	lda	v_donedark
	bne	.exit

	jsl	Cyclefalls
.exit
	rtl

PJCyclelamp
	rtl
	jsl	Cyclelamp	

	rtl

PJCycleflames
	jsl	Cycleflames

	rtl


	
	
	
	

*****************************************************************************
*
*

Preload:
	jsl	InitStat	; Setup & Dwnld data for Health Stats

	lda	v_return
	bne	.skipitems

	jsl	InitItems	; Setup & Dwnld data for Powerups
.skipitems
	jsl	InitPlayer	; Setup & Dwnld data for Tarzan

	jsl	Statupdate	; Initial Status

	jsl	UpBossBar

	jsl	Senddata

	jsl	Doplayer	; Initial Tarzan

	jsl	Loadanims	; Load Foes into object tables

	jsl	Copypalall	; Copy full BG palette to palette2

	jsl	Resetattr	; Reset Objects

	jsl	PlotStats	; Plot the current health & lives

	jsl	Plotbossbar

	jsl	PlotItems	; Plot powerups & coins
	
	jsl	PlotFrontFoes

	jsl	PlotPlayer	; Plot Tarzan

	jsl	PlotBackFoes

	jsl	PlotHoles	; Plot Holes

	jsl	Createoam	; Build it

	jsl	Sendobj	; Send table to OAM

	jsl	Senddata	; Send any pending OBJ characters

	jsl	Scroll

	LAI

	stz	v_return
	stz	z_joy1
	stz	z_joy2

	BLANKOFF

	lda	z_level
	and	#$FF
;	cmp	#DOWNRIVER
;	beq	.exit

;	cmp	#UPRIVER
;	beq	.exit

;	cmp	#BEACH
;	beq	.exit

;	cmp	#SHIP
;	beq	.exit

	cmp	#SHIPHOLDA
	beq	.exit

	cmp	#SHIPHOLDB
	beq	.exit

	cmp	#WATERFALL
	beq	.exit

	jsl	Fadein
.exit
	rtl


*****************************************************************************
*
* Process Level Status & User Input
*
*****************************************************************************

ProcessLevel:
	LAI


	jsl	Resetattr	; Clear OAM shadow

	lda	v_donedark
	bne	dofoesmain

	lda	v_priestdead
	bne	.skipctrlread

	jsl	Readctrl	; Get user controller stats
.skipctrlread
	jsl	Doplayer	; Do Object Detect & Player Control 

	lda	v_levelfail
	bne	.exit

	lda	v_pause
	beq	.dofoesmain

	dec	v_playinvince	; Adjust invincible counter
	bpl	.skipmove

	stz	v_playinvince

	bra	.skipmove
@dofoesmain
	jsl	Dofoes	; Update Foe AI Logic
.skipmove
	jsl	Scroll	; Update Scroll Values

	jsl	Statupdate	; Update user interface & Stats

	jsl	UpBossBar
.exit
	rtl


