*****************************************************************************
*
* TZTITLES.ASM
*
*****************************************************************************

MainTitle:
	LAI

	stz	MDMAEN

	stz	.menustate	; Select "Start" menu item
	stz	_optselect
	stz	_acount

	jsr	.loadmtitle	; Download & fadein title art

	lda	#.titledly	; Delay count
	sta	z_transartdly	; Store it
	stz	_buttonpress	; No buttons yet

	lda	SLHV
	lda	OPHCT
	and	#$FF
	sta	z_seed0
.stayhere:
	jsl	Scanjoys	; Check controller
			
	lda	z_joy1	; Button pressed?
	bne	.chkcntrl	; Yes - Process delay

	stz	_acount

****************
.letswait:
	jsl	Waitfornmi	; No  - Wait

	lda	.bg1scshadow	; Get shadow BG1 screen pointer

	SA
	sta	BG1SC	; Load PPU register with BG1 ptr
	LA

	dec	z_transartdly	; Adjust delay value
	bmi	.fadetitle	; If count expired, then exit

	lda	z_transartdly	; Get it
	cmp	#640	; Have we been here at least a sec?
	bcs	.stayhere	; No  - No need to check other yet

	lda	z_tzstartup
	cmp	#$37
	beq	.stayhere

	lda	#$37	; Yes - Reset startup flag then
	sta	z_tzstartup

	jsr	.chgmenu	; Load BG1 shadow pointer var

	bra	.stayhere	; Continue logo display if not done

.chkcntrl:
	lda	z_tzstartup	; Yes - Get startup var
	cmp	#$FE	; Is this 1st time thru?
	beq	.letswait	; Yes - Then continue process delay

	lda	z_joy1	; No  - Get keypress back
	and	#JOYPADSTART	; Was it the "START" button?
	beq	.chkdwnb	; No  - Something else was pressed too

	stz	.menustate	; Select "Start" menu item

	jsr	.chgmenu	; Load BG1 shadow pointer var

	lda	#JOYPADSTART	; Yes - Get "START" bit pattern
	sta	_buttonpress	; Store it

	bra	.fadetitle	; Fade this logo out and exit

****************
.chkdwnb:
	lda	z_joy1	; Get keypress back
	and	#JOYPADDOWN	; Was it the "DOWN" button?
	beq	.chkupb	; No  -

	lda	_optselect	; Yes
	beq	.togopts

	bra	.xtogopts

****************
.chkupb:
	lda	z_joy1	; Get keypress back
	and	#JOYPADUP	; Was it the "UP" button?
	beq	.letswait	; No  - 

	lda	_optselect	; Yes
	beq	.xtogopts

****************
.togopts:
	lda	_acount
	bne	.stayhere

	lda	_optselect
	eor	#1
	sta	_optselect

	jsr	.chgmenu	; Load BG1 shadow pointer var

	lda	#.titledly	; Delay count
	sta	z_transartdly	; Store it
	stz	_buttonpress	; No buttons yet
.xtogopts:
	jmp	.letswait

.fadetitle:
	; Copy Screen Palette(s)
	lda	#tzantitlepal
	sta	z_temp0

	lda	#^tzantitlepal
	sta	z_temp1

	ldy	#$1FE 	;1fc
.palloop1:
	lda	[z_temp0],y
	sta	PALETTE,y

	dey
	dey
	bpl	.palloop1


	; Build Target Palette for Clearing

	ldx	#$1FE
.palclrloop:
	stz	PALETTE2,x

	dex
	dex
	bpl	.palclrloop

	jsl	Colfade	; Fade to Target Palette

	rtl


.bg1scshadow	.equ	_ScratchRam
.menustate	.equ	_ScratchRam+02
.keydly	.equ	_ScratchRam+04

.titledly	.equ	720

*****************************************************************************
*
*

.chgmenu:
	lda	_optselect
	beq	.startgameptr

	lda	#BG1BASE1B/256	; Preload "flashed" data pointer

	bra	.xitcm

.startgameptr:
	lda	#BG1BASE/256	; Preload "normal" data pointer
.xitcm:
	sta	.bg1scshadow	; Store to shadow variable for 
			; ...processing during VBLANK
	rts


*****************************************************************************
*
*

.loadmtitle:
	BLANKON
	NMIOFF


	; Unpack Screen Chars

	pea	|tzantitlechr	 ; Source
	pea	^tzantitlechr
	pea	|SCRATCH	 ; Destination
	pea	^SCRATCH

	jsl	UNPACK

	pla
	pla
	pla
	pla


	; VRAM Xfer for Screen Chars

	lda	#^SCRATCH
	sta	z_ztemp

	lda	#SCRATCH
	ldy	#$8000	
	ldx	#BG1CHAR

	jsl	Sendvram

	; Unpack Screen
	pea	|tzantitlescr	; push low word of packed file
	pea	^tzantitlescr	; push high word of packed file
	pea	|SCRATCH	; push low word of output buffer
	pea	^SCRATCH	; push high word of output buffer

	jsl	UNPACK	; unpack file to output buffer

	pla
	pla
	pla
	pla

	; Copy "Start Game" Screen
	ldx	#BG1BASE
	lda	#SCRATCH
	ldy	#^SCRATCH
	jsl	Sendbg


	; Copy "Options" Screen
	ldx	#BG1BASE+$400
	lda	#SCRATCH+1792
	ldy	#^SCRATCH
	jsl	Sendbg


	; Copy -Blank Menu- Screen
	ldx	#BG1BASE+$800
	lda	#SCRATCH+3584
	ldy	#^SCRATCH
	jsl	Sendbg


	; Copy Screen Palette(s)
	lda	#tzantitlepal
	sta	z_temp0

	lda	#^tzantitlepal
	sta	z_temp1

	ldy	#$1FE 	;1fc
.palloop:
	lda	[z_temp0],y
	sta	PALETTE2,y

	dey
	dey
	bpl	.palloop


	; Set Modes

	SA

	lda	#BG1BASE/256
	sta	.bg1scshadow	; Store to shadow variable for 
	sta	BG1SC

	lda	z_tzstartup	; Yes - Get startup var
	cmp	#$FE	; Is this 1st time thru?
	bne	.skip3rd	; Yes - Then continue process delay

	lda	#BG2BASE/256	; Preload "normal" data pointer
	sta	.bg1scshadow	; Store to shadow variable for 
	sta	BG1SC
.skip3rd
	lda	#1
	sta	BGMODE

	lda	#$1
	sta	TM

	sta	TS	

	lda	#$00
	sta	CGSWSEL		

	lda	#$00
	sta	CGADSUB

	lda	#$00
	sta	BG34NBA	

	lda	#$00
	sta	BG12NBA

	stz	BG1VOFS
	stz	BG1VOFS

	stz	BG1HOFS
	stz	BG1HOFS

	LAI

	NMION

	jsl	Fadein

	rts


*****************************************************************************

