*****************************************************************************
*
* TZUTILS.ASM
*
* This file contains programming utilities to facilitate program
* development. It also holds routines that are specific to and
* will be used in Tarzan.
*
* The following routines are available:
*
*     Objxy        - Uses the 1st "slot" in OBJECT space to
*                    display the x,y coordinates of the main
*                    characters position.
*
*     Ctrlremap    - Copys the selected controller key map from
*                    ROM to ZPAGE RAM variables.
*
*****************************************************************************

Objxy:
	LAI

	lda	v_pause2
	beq	.exitxy

	lda	v_playerx	; Get players x
	cmp	v_playeroldx
	bne	.updatexy

	lda	v_playery
	cmp	v_playeroldy
	bne	.updatexy
.exitxy:
	rtl

.updatexy:
	; 4th digit X

	lda	v_debugswitch1
	bne	.dotilenum1

	lda	v_playerx
	sta	v_playeroldx
	lsr
	lsr
	bra	.str1

.dotilenum1
	jsl	Chek4number
.str1
	sta	v_playerxglob

	xba		; Swap for MSB
	lsr		; Get upper nibble
	lsr
	lsr
	lsr
	and	#$F	; Isolate upper nibble data
	sta	v_xynumframe

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 3rd digit X

	lda	v_playerxglob	; Get players x
	xba		; Swap for MSB
	and	#$F	; Isolate lower nibble data
	sta	v_xynumframe+2

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$20	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 2nd digit X

	lda	v_playerxglob	; Get players x
	lsr		; Get upper nibble
	lsr
	lsr
	lsr
	and	#$F	; Isolate upper nibble data
	sta	v_xynumframe+4

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$40	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 1st digit X

	lda	v_playerxglob	; Get players x
	and	#$F	; Isolate lower nibble data
	sta	v_xynumframe+6

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$60	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 4th digit Y

	lda	v_debugswitch1
	bne	.dotilenum2

	lda	v_playery	; Get players x
	sta	v_playeroldy
	lsr
	lsr
	bra	.str2

.dotilenum2
	jsl	Chek4stat
.str2
	sta	v_playeryglob

	xba		; Swap for MSB
	lsr		; Get upper nibble
	lsr
	lsr
	lsr
	and	#$F	; Isolate upper nibble data
	sta	v_xynumframe+8

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$80	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 3rd digit Y

	lda	v_playeryglob	; Get players y
	xba		; Swap for MSB
	and	#$F	; Isolate lower nibble data
	sta	v_xynumframe+10

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$A0	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 2nd digit Y

	lda	v_playeryglob	; Get players y
	lsr		; Get upper nibble
	lsr
	lsr
	lsr
	and	#$F	; Isolate upper nibble data
	sta	v_xynumframe+12

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$C0	; Goes at the start

	jsl	Sendframe	; Send Frame


	; 1st digit Y

	lda	v_playeryglob	; Get players y
	and	#$F	; Isolate lower nibble data
	sta	v_xynumframe+14

	asl
	tay		; Store frame number

	lda	#^hexnums	; Bank
	sta	z_dreccount

	lda	#hexnums	; Relative offset
	sta	z_spridrec

	lda	#OBJBASE+$E0	; Goes at the start

	jsl	Sendframe	; Send Frame

	rtl

;Loadtilebox:
;	ldy	#0
;
;	lda	#^tilebox
;	sta	z_dreccount
;
;	lda	#tilebox
;	sta	z_spridrec
;
;	lda	#$4C	; Object name
;	asl
;	asl
;	asl
;	asl
;	adc	#OBJBASE	;
;
;	jsl	Sendframe	; Send Frame
;
;	jsl	Senddata
;
;	ldy	#0
;
;	lda	#^tilebox
;	sta	z_dreccount
;
;	lda	#tilebox
;	sta	z_spridrec
;
;	lda	#$4E	; Object name
;	asl
;	asl
;	asl
;	asl
;	adc	#OBJBASE	;
;
;	jsl	Sendframe	; Send Frame
;
;	jsl	Senddata
;
;	rtl

PlotColBox:
	lda	v_pause	; Pause Game?
	beq	.chksemi	; No  -

	lda	v_resettbos	; Reset oneshot indicated?
	beq	.inpausemode	; No  - Process normally

	stz	v_resettbos	; Yes -

.chksemi
	lda	v_plotcbox	; Plot tilebox semifore set?
	bne	.tbok	; Yes -
.inpausemode
	rtl
.tbok
	ldy	z_attrcount	; Get attr counter

	lda	v_playerleft	; Get players x1
	sta	ATTRX,y

	lda	v_playertop	; Get players y1
	sta	ATTRY,y

	lda	#0	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^tilebox	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#tilebox	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$324C	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	lda	v_playerrite	; Get players x2
	sec
	sbc	#16
	sta	ATTRX,y

	lda	v_playerbot	; Get players y2
	sec
	sbc	#16
	sta	ATTRY,y

	lda	#0	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$324E	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	sty	z_attrcount

	rtl


*****************************************************************************
*
*

PlotXYs:
	LAI

	lda	v_pause2
	bne	.oktoplot

	rtl

.oktoplot
	ldy	z_attrcount	; Get attr counter
	ldx	#0

	lda	#206
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#100
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$3200	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#216
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#100
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$3202	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#226
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#100
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$3204	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#236
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#100
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$3206	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#206
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#116
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$3208	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#216
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#116
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$320A	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#226
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#116
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$320C	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	inx
	inx

	lda	#236
	clc
	adc	z_levelx
	sta	ATTRX,y

	lda	#116
	clc
	adc	z_levely
	sta	ATTRY,y

	lda	v_xynumframe,x	; Which frame are we in?
	sta	ATTRFRAME,y

	lda	#^hexnums	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#hexnums	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$320E	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	lda	v_debugswitch1
	bne	.dotilenum1

	lda	v_playerx
	lsr
	lsr
	sta	ATTRX,y

	lda	v_playery
	lsr
	lsr
	sta	ATTRY,y

	bra	.str1

.dotilenum1
	lda	v_playerx
	lsr
	lsr
	and	#$FFF0
	sta	ATTRX,y

	lda	v_playery
	lsr
	lsr
	and	#$FFF0
	sta	ATTRY,y
.str1
	lda	#0
	sta	ATTRFRAME,y

	lda	#^tilebox	; Put anim bank in table
	sta	ATTRBANK,y

	lda	#tilebox	; Put anim pointer in table
	sta	ATTRDREC,y

	lda	#$324C	; Position Specific
	sta	ATTROFF,y

	iny
	iny

	sty	z_attrcount

	rtl


			

